@@ -46,6 +46,7 @@
 #include "FSAL/fsal_commonlib.h"
 #include "FSAL/fsal_config.h"
 #include "internal.h"
+#include "statx_compat.h"
 
 /**
  * @brief Clean up an export
@@ -108,7 +109,7 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
 	/* FSAL status structure */
 	fsal_status_t status = { ERR_FSAL_NO_ERROR, 0 };
 	/* The buffer in which to store stat info */
-	struct stat st;
+	struct ceph_statx stx;
 	/* Return code from Ceph */
 	int rc;
 	/* Find the actual path in the supplied path */
@@ -130,21 +131,35 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
 
 	*pub_handle = NULL;
 
+	/*
+	 * sanity check: ensure that this is the right export. realpath
+	 * must be a superset of the export fullpath, or the string
+	 * handling will be broken.
+	 */
+	if (strstr(realpath, op_ctx->ctx_export->fullpath) != realpath) {
+		status.major = ERR_FSAL_SERVERFAULT;
+		return status;
+	}
+
+	/* Advance past the export's fullpath */
+	realpath += strlen(op_ctx->ctx_export->fullpath);
+
+	/* special case the root */
 	if (strcmp(realpath, "/") == 0) {
 		assert(export->root);
 		*pub_handle = &export->root->handle;
 		return status;
 	}
 
-	rc = ceph_ll_walk(export->cmount, realpath, &i, &st);
+	rc = fsal_ceph_ll_walk(export->cmount, realpath, &i, &stx,
+				!!attrs_out, op_ctx->creds);
 	if (rc < 0)
 		return ceph2fsal_error(rc);
 
-	construct_handle(&st, i, export, &handle);
+	construct_handle(&stx, i, export, &handle);
 
-	if (attrs_out != NULL) {
-		posix2fsal_attributes(&st, attrs_out);
-	}
+	if (attrs_out != NULL)
+		ceph2fsal_attributes(&stx, attrs_out);
 
 	*pub_handle = &handle->handle;
 	return status;
@@ -159,10 +174,10 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
  * @param[in]  in_type  The type of digest being decoded
  * @param[out] fh_desc  Address and length of key
  */
-static fsal_status_t extract_handle(struct fsal_export *exp_hdl,
-				    fsal_digesttype_t in_type,
-				    struct gsh_buffdesc *fh_desc,
-				    int flags)
+static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
+				  fsal_digesttype_t in_type,
+				  struct gsh_buffdesc *fh_desc,
+				  int flags)
 {
 	switch (in_type) {
 		/* Digested Handles */
@@ -206,7 +221,7 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 	/* Ceph return code */
 	int rc = 0;
 	/* Stat buffer */
-	struct stat st;
+	struct ceph_statx stx;
 	/* Handle to be created */
 	struct handle *handle = NULL;
 	/* Inode pointer */
@@ -219,21 +234,33 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 		return status;
 	}
 
+	/* Check our local cache first */
 	i = ceph_ll_get_inode(export->cmount, *vi);
-	if (!i)
-		return ceph2fsal_error(-ESTALE);
+	if (!i) {
+		/*
+		 * Try the slow way, may not be in cache now.
+		 *
+		 * Currently, there is no interface for looking up a snapped
+		 * inode, so we just bail here in that case.
+		 */
+		if (vi->snapid.val != CEPH_NOSNAP)
+			return ceph2fsal_error(-ESTALE);
 
-	/* The ceph_ll_connectable_m should have populated libceph's
-	   cache with all this anyway */
-	rc = ceph_ll_getattr(export->cmount, i, &st, 0, 0);
+		rc = ceph_ll_lookup_inode(export->cmount, vi->ino, &i);
+		if (rc)
+			return ceph2fsal_error(rc);
+	}
+
+	rc = fsal_ceph_ll_getattr(export->cmount, i, &stx,
+		attrs_out ? CEPH_STATX_ATTR_MASK : CEPH_STATX_HANDLE_MASK,
+		op_ctx->creds);
 	if (rc < 0)
 		return ceph2fsal_error(rc);
 
-	construct_handle(&st, i, export, &handle);
+	construct_handle(&stx, i, export, &handle);
 
-	if (attrs_out != NULL) {
-		posix2fsal_attributes(&st, attrs_out);
-	}
+	if (attrs_out != NULL)
+		ceph2fsal_attributes(&stx, attrs_out);
 
 	*pub_handle = &handle->handle;
 	return status;
@@ -443,7 +470,7 @@ static fsal_aclsupp_t fs_acl_support(struct fsal_export *export_pub)
 
 static attrmask_t fs_supported_attrs(struct fsal_export *export_pub)
 {
-	return supported_attributes;
+	return CEPH_SUPPORTED_ATTRS;
 }
 
 /**
@@ -490,7 +517,7 @@ void export_ops_init(struct export_ops *ops)
 {
 	ops->release = release;
 	ops->lookup_path = lookup_path;
-	ops->extract_handle = extract_handle;
+	ops->wire_to_host = wire_to_host;
 	ops->create_handle = create_handle;
 	ops->get_fs_dynamic_info = get_fs_dynamic_info;
 	ops->fs_supports = fs_supports;
@@ -506,6 +533,7 @@ void export_ops_init(struct export_ops *ops)
 	ops->fs_umask = fs_umask;
 	ops->fs_xattr_access_rights = fs_xattr_access_rights;
 	ops->alloc_state = ceph_alloc_state;
+	ops->free_state = ceph_free_state;
 #ifdef CEPH_PNFS
 	export_ops_pnfs(ops);
 #endif				/* CEPH_PNFS */
