@@ -139,7 +139,7 @@ static fsal_status_t lookup_path(struct fsal_export *export_pub,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 			       GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -245,7 +245,7 @@ static fsal_status_t create_handle(struct fsal_export *export_pub,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 			       GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -304,7 +304,7 @@ fsal_status_t glfs2fsal_handle(struct glusterfs_export *glfs_export,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 				 GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -341,7 +341,7 @@ static fsal_status_t get_dynamic_info(struct fsal_export *exp_hdl,
 	rc = glfs_statvfs(glfs_export->gl_fs->fs, glfs_export->export_path,
 			  &vfssb);
 	if (rc != 0)
-		return gluster2fsal_error(rc);
+		return gluster2fsal_error(errno);
 
 	memset(infop, 0, sizeof(fsal_dynamicfsinfo_t));
 	infop->total_bytes = vfssb.f_frsize * vfssb.f_blocks;
@@ -382,6 +382,8 @@ struct state_t *glusterfs_alloc_state(struct fsal_export *exp_hdl,
 			      state)->glusterfs_fd;
 
 	my_fd->glfd = NULL;
+	my_fd->openflags = FSAL_O_CLOSED;
+	PTHREAD_RWLOCK_init(&my_fd->fdlock, NULL);
 
 	return state;
 }
@@ -397,6 +399,9 @@ void glusterfs_free_state(struct fsal_export *exp_hdl, struct state_t *state)
 {
 	struct glusterfs_state_fd *state_fd =
 		container_of(state, struct glusterfs_state_fd, state);
+	struct glusterfs_fd *my_fd = &state_fd->glusterfs_fd;
+
+	PTHREAD_RWLOCK_destroy(&my_fd->fdlock);
 
 	gsh_free(state_fd);
 }
@@ -626,11 +631,25 @@ void export_ops_init(struct export_ops *ops)
 	ops->free_state = glusterfs_free_state;
 }
 
+enum transport {
+	GLUSTER_TCP_VOL,
+	GLUSTER_RDMA_VOL
+};
+
 struct glexport_params {
 	char *glvolname;
 	char *glhostname;
 	char *glvolpath;
 	char *glfs_log;
+	uint64_t up_poll_usec;
+	bool enable_upcall;
+	enum transport gltransport;
+};
+
+static struct config_item_list transportformats[] = {
+	CONFIG_LIST_TOK("tcp", GLUSTER_TCP_VOL),
+	CONFIG_LIST_TOK("rdma", GLUSTER_RDMA_VOL),
+	CONFIG_LIST_EOL
 };
 
 static struct config_item export_params[] = {
@@ -643,6 +662,12 @@ static struct config_item export_params[] = {
 		      glexport_params, glvolpath),
 	CONF_ITEM_PATH("glfs_log", 1, MAXPATHLEN, GFAPI_LOG_LOCATION,
 		       glexport_params, glfs_log),
+	CONF_ITEM_UI64("up_poll_usec", 1, 60*1000*1000, 10,
+		       glexport_params, up_poll_usec),
+	CONF_ITEM_BOOL("enable_upcall", true, glexport_params,
+		       enable_upcall),
+	CONF_ITEM_TOKEN("transport", GLUSTER_TCP_VOL, transportformats,
+			glexport_params, gltransport),
 	CONFIG_EOL
 };
 
@@ -664,7 +689,6 @@ void
 glusterfs_free_fs(struct glusterfs_fs *gl_fs)
 {
 	int64_t refcnt;
-	int *retval = NULL;
 	int err     = 0;
 
 	PTHREAD_MUTEX_lock(&GlusterFS.lock);
@@ -686,9 +710,14 @@ glusterfs_free_fs(struct glusterfs_fs *gl_fs)
 
 	atomic_inc_int8_t(&gl_fs->destroy_mode);
 
+	if (!gl_fs->enable_upcall)
+		goto skip_upcall;
+
 	/* Cancel upcall readiness if not yet done */
 	up_ready_cancel((struct fsal_up_vector *)gl_fs->up_ops);
 
+#ifndef USE_GLUSTER_UPCALL_REGISTER
+	int *retval = NULL;
 	/* Wait for up_thread to exit */
 	err = pthread_join(gl_fs->up_thread, (void **)&retval);
 
@@ -698,11 +727,22 @@ glusterfs_free_fs(struct glusterfs_fs *gl_fs)
 	}
 
 	if (err) {
-		LogCrit(COMPONENT_FSAL, "Up_thread join failed (%s)",
+		LogWarn(COMPONENT_FSAL, "Up_thread join failed (%s)",
 			strerror(err));
-		return;
 	}
+#else
+	err = glfs_upcall_unregister(gl_fs->fs, GLFS_EVENT_ANY);
 
+	if ((err < 0) || (!(err & GLFS_EVENT_INODE_INVALIDATE))) {
+		/* Or can we ignore the error like in case of single
+		 * node ganesha server. */
+		LogWarn(COMPONENT_FSAL,
+			"Unable to unregister for upcalls. Volume: %s",
+			gl_fs->volname);
+	}
+#endif
+
+skip_upcall:
 	/* Gluster and memory cleanup */
 	glfs_fini(gl_fs->fs);
 	gsh_free(gl_fs->volname);
@@ -734,12 +774,6 @@ glusterfs_get_fs(struct glexport_params params,
 
 	gl_fs = gsh_calloc(1, sizeof(struct glusterfs_fs));
 
-	if (!gl_fs) {
-		LogCrit(COMPONENT_FSAL,
-			"Unable to allocate memory for glusterfs_fs object");
-		goto out;
-	}
-
 	glist_init(&gl_fs->fs_obj);
 
 	fs = glfs_new(params.glvolname);
@@ -750,7 +784,17 @@ glusterfs_get_fs(struct glexport_params params,
 		goto out;
 	}
 
-	rc = glfs_set_volfile_server(fs, "tcp", params.glhostname, 24007);
+	switch (params.gltransport) {
+	case GLUSTER_RDMA_VOL:
+		rc = glfs_set_volfile_server(fs, "rdma", params.glhostname,
+				24007);
+		break;
+	default:
+		rc = glfs_set_volfile_server(fs, "tcp", params.glhostname,
+				24007);
+		break;
+	}
+
 	if (rc != 0) {
 		LogCrit(COMPONENT_FSAL,
 			"Unable to set volume file. Volume: %s",
@@ -777,8 +821,16 @@ glusterfs_get_fs(struct glexport_params params,
 	gl_fs->fs = fs;
 	gl_fs->volname = strdup(params.glvolname);
 	gl_fs->destroy_mode = 0;
+	gl_fs->up_poll_usec = params.up_poll_usec;
 
 	gl_fs->up_ops = up_ops;
+
+	gl_fs->enable_upcall = params.enable_upcall;
+
+	if (!gl_fs->enable_upcall)
+		goto skip_upcall;
+
+#ifndef USE_GLUSTER_UPCALL_REGISTER
 	rc = initiate_up_thread(gl_fs);
 	if (rc != 0) {
 		LogCrit(COMPONENT_FSAL,
@@ -786,7 +838,24 @@ glusterfs_get_fs(struct glexport_params params,
 			params.glvolname);
 		goto out;
 	}
+#else
+	/* We are mainly interested in INODE_INVALIDATE for now. Still
+	 * register for all the events
+	 */
+	rc = glfs_upcall_register(fs, GLFS_EVENT_ANY, gluster_process_upcall,
+				  gl_fs);
 
+	if ((rc < 0) || (!(rc & GLFS_EVENT_INODE_INVALIDATE))) {
+		/* Or can we ignore the error like in case of single
+		 * node ganesha server. */
+		LogCrit(COMPONENT_FSAL,
+			"Unable to register for upcalls. Volume: %s",
+			params.glvolname);
+		goto out;
+	}
+#endif
+
+skip_upcall:
 	glist_add(&GlusterFS.fs_obj, &gl_fs->fs_obj);
 
 found:
@@ -829,6 +898,8 @@ fsal_status_t glusterfs_create_export(struct fsal_module *fsal_hdl,
 	LogDebug(COMPONENT_FSAL, "In args: export path = %s",
 		 op_ctx->ctx_export->fullpath);
 
+	glfsexport = gsh_calloc(1, sizeof(struct glusterfs_export));
+
 	rc = load_config_from_node(parse_node,
 				   &export_param,
 				   &params,
@@ -844,8 +915,6 @@ fsal_status_t glusterfs_create_export(struct fsal_module *fsal_hdl,
 	LogEvent(COMPONENT_FSAL, "Volume %s exported at : '%s'",
 		 params.glvolname, params.glvolpath);
 
-	glfsexport = gsh_calloc(1, sizeof(struct glusterfs_export));
-
 	fsal_export_init(&glfsexport->export);
 	export_ops_init(&glfsexport->export.exp_ops);
 
@@ -878,8 +947,9 @@ fsal_status_t glusterfs_create_export(struct fsal_module *fsal_hdl,
 	if (glfsexport->pnfs_ds_enabled) {
 		struct fsal_pnfs_ds *pds = NULL;
 
-		status = fsal_hdl->m_ops.
-				fsal_pnfs_ds(fsal_hdl, parse_node, &pds);
+		status =
+		    fsal_hdl->m_ops.fsal_pnfs_ds(fsal_hdl, parse_node, &pds);
+
 		if (status.major != ERR_FSAL_NO_ERROR)
 			goto out;
 
@@ -915,24 +985,19 @@ fsal_status_t glusterfs_create_export(struct fsal_module *fsal_hdl,
 	glfsexport->export.up_ops = up_ops;
 
  out:
-	if (params.glvolname)
-		gsh_free(params.glvolname);
-	if (params.glhostname)
-		gsh_free(params.glhostname);
-	if (params.glfs_log)
-		gsh_free(params.glfs_log);
+	gsh_free(params.glvolname);
+	gsh_free(params.glhostname);
+	gsh_free(params.glfs_log);
 
 	if (status.major != ERR_FSAL_NO_ERROR) {
-		if (params.glvolpath)
-			gsh_free(params.glvolpath);
+		gsh_free(params.glvolpath);
 
 		if (fsal_attached)
 			fsal_detach_export(fsal_hdl,
 					   &glfsexport->export.exports);
 		if (glfsexport->gl_fs)
 			glusterfs_free_fs(glfsexport->gl_fs);
-		if (glfsexport)
-			gsh_free(glfsexport);
+		gsh_free(glfsexport);
 	}
 
 	return status;
