@@ -78,6 +78,8 @@ void stat2fsal_attributes(const struct stat *buffstat,
 	 * other bits in the mask.
 	 */
 	fsalattr->valid_mask |= ATTRS_POSIX;
+	fsalattr->supported = op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+							op_ctx->fsal_export);
 
 	/* Fills the output struct */
 	fsalattr->type = posix2fsal_type(buffstat->st_mode);
@@ -198,7 +200,7 @@ bool fs_specific_has(const char *fs_specific, const char *key, char *val,
 		char *v = k;
 
 		strsep(&v, "=");
-		if (0 == strcmp(k, key)) {
+		if (strcmp(k, key) == 0) {
 			if (val)
 				strncpy(val, v, *max_val_bytes);
 			if (max_val_bytes)
@@ -214,8 +216,9 @@ bool fs_specific_has(const char *fs_specific, const char *key, char *val,
 	return ret;
 }
 
-int setglustercreds(struct glusterfs_export *glfs_export, uid_t *uid,
-		    gid_t *gid, unsigned int ngrps, gid_t *groups)
+void setglustercreds(struct glusterfs_export *glfs_export, uid_t *uid,
+		     gid_t *gid, unsigned int ngrps, gid_t *groups,
+		     char *file, int line, char *function)
 {
 	int rc = 0;
 
@@ -243,7 +246,12 @@ int setglustercreds(struct glusterfs_export *glfs_export, uid_t *uid,
 		rc = glfs_setfsgroups(0, NULL);
 
  out:
-	return rc;
+	if (rc != 0) {
+		DisplayLogComponentLevel(COMPONENT_FSAL, file, line, function,
+			 NIV_FATAL,
+			"Could not set Gluster credentials - uid(%d), gid(%d)",
+			*uid, *gid);
+	}
 }
 
 /*
@@ -502,8 +510,8 @@ void latency_dump(void)
 
 	for (; i < LATENCY_SLOTS; i++) {
 		LogCrit(COMPONENT_FSAL, "Op:%d:Count:%"PRIu64":nsecs:%"PRIu64,
-			i, glfsal_latencies[i].count, glfsal_latencies[i].
-			overall_time);
+			i, glfsal_latencies[i].count,
+			glfsal_latencies[i].overall_time);
 	}
 }
 #endif
