@@ -49,6 +49,7 @@ static void handle_release(struct fsal_obj_handle *obj_hdl)
 	int rc = 0;
 	struct glusterfs_handle *objhandle =
 	    container_of(obj_hdl, struct glusterfs_handle, handle);
+	struct glusterfs_fd *my_fd = &objhandle->globalfd;
 	fsal_status_t status = { ERR_FSAL_NO_ERROR, 0 };
 #ifdef GLTIMING
 	struct timespec s_time, e_time;
@@ -58,19 +59,34 @@ static void handle_release(struct fsal_obj_handle *obj_hdl)
 
 	fsal_obj_handle_fini(&objhandle->handle);
 
-	if (objhandle->globalfd.glfd) {
+	if (my_fd->glfd) {
 
-		/* Since handle gets released as part of internal
-		 * operation, we may not need to set credentials */
-		status = glusterfs_close_my_fd(&objhandle->globalfd);
-		if (status.major != ERR_FSAL_NO_ERROR) {
-			LogCrit(COMPONENT_FSAL,
-				"glfs_close returned %s(%d)",
-				strerror(errno), errno);
-			/* cleanup as much as possible */
+		/* During shutdown, the op_ctx is NULL */
+		if (op_ctx && op_ctx->fsal_export) {
+			/* Since handle gets released as part of internal
+			 * operation, we may not need to set credentials */
+			status = glusterfs_close_my_fd(my_fd);
+			if (FSAL_IS_ERROR(status)) {
+				LogCrit(COMPONENT_FSAL,
+					"glusterfs_close_my_fd returned %s",
+					fsal_err_txt(status));
+				/* cleanup as much as possible */
+			}
+		} else if (my_fd->openflags != FSAL_O_CLOSED) {
+			rc = glfs_close(my_fd->glfd);
+			if (rc) {
+				LogCrit(COMPONENT_FSAL,
+					"glfs_close returned %s(%d)",
+					strerror(errno), errno);
+			}
 		}
 	}
 
+	if (my_fd->creds.caller_garray) {
+		gsh_free(my_fd->creds.caller_garray);
+		my_fd->creds.caller_garray = NULL;
+	}
+
 	if (objhandle->glhandle) {
 		rc = glfs_h_close(objhandle->glhandle);
 		if (rc) {
@@ -130,7 +146,7 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 			       GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -237,6 +253,12 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		/* skip . and .. */
 		if ((strcmp(de.d_name, ".") == 0)
 		    || (strcmp(de.d_name, "..") == 0)) {
+#ifdef USE_GLUSTER_XREADDIRPLUS
+			if (xstat) {
+				glfs_free(xstat);
+				xstat = NULL;
+			}
+#endif
 			continue;
 		}
 		fsal_prepare_attrs(&attrs, attrmask);
@@ -310,83 +332,6 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	return status;
 }
 
-/**
- * @brief Implements GLUSTER FSAL objectoperation create
- */
-
-static fsal_status_t create(struct fsal_obj_handle *dir_hdl,
-			    const char *name, struct attrlist *attrib,
-			    struct fsal_obj_handle **handle,
-			    struct attrlist *attrs_out)
-{
-	int rc = 0;
-	fsal_status_t status = { ERR_FSAL_NO_ERROR, 0 };
-	struct stat sb;
-	struct glfs_object *glhandle = NULL;
-	unsigned char globjhdl[GFAPI_HANDLE_LENGTH] = {'\0'};
-	char vol_uuid[GLAPI_UUID_LENGTH] = {'\0'};
-	struct glusterfs_handle *objhandle = NULL;
-	struct glusterfs_export *glfs_export =
-	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
-	struct glusterfs_handle *parenthandle =
-	    container_of(dir_hdl, struct glusterfs_handle, handle);
-#ifdef GLTIMING
-	struct timespec s_time, e_time;
-
-	now(&s_time);
-#endif
-
-	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
-			  &op_ctx->creds->caller_gid,
-			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
-
-	/* FIXME: what else from attrib should we use? */
-	glhandle =
-	    glfs_h_creat(glfs_export->gl_fs->fs, parenthandle->glhandle, name,
-			 O_CREAT | O_EXCL, fsal2unix_mode(attrib->mode), &sb);
-
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
-
-	if (glhandle == NULL) {
-		status = gluster2fsal_error(errno);
-		goto out;
-	}
-
-	rc = glfs_h_extract_handle(glhandle, globjhdl, GFAPI_HANDLE_LENGTH);
-	if (rc < 0) {
-		status = gluster2fsal_error(errno);
-		goto out;
-	}
-
-	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
-			       GLAPI_UUID_LENGTH);
-	if (rc < 0) {
-		status = gluster2fsal_error(rc);
-		goto out;
-	}
-
-	construct_handle(glfs_export, &sb, glhandle, globjhdl,
-			 GLAPI_HANDLE_LENGTH, &objhandle, vol_uuid);
-
-	if (attrs_out != NULL) {
-		posix2fsal_attributes_all(&sb, attrs_out);
-	}
-
-	*handle = &objhandle->handle;
-
- out:
-	if (status.major != ERR_FSAL_NO_ERROR)
-		gluster_cleanup_vars(glhandle);
-
-#ifdef GLTIMING
-	now(&e_time);
-	latency_update(&s_time, &e_time, lat_create);
-#endif
-
-	return status;
-}
-
 /**
  * @brief Implements GLUSTER FSAL objectoperation mkdir
  */
@@ -438,7 +383,7 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 			       GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -564,7 +509,7 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 			       GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -665,7 +610,7 @@ static fsal_status_t makesymlink(struct fsal_obj_handle *dir_hdl,
 	rc = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 			       GLAPI_UUID_LENGTH);
 	if (rc < 0) {
-		status = gluster2fsal_error(rc);
+		status = gluster2fsal_error(errno);
 		goto out;
 	}
 
@@ -1242,31 +1187,44 @@ fsal_status_t find_fd(struct glusterfs_fd *my_fd,
 	struct glusterfs_handle *myself;
 	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
 	struct glusterfs_fd  tmp_fd = {0}, *tmp2_fd = &tmp_fd;
+	bool reusing_open_state_fd = false;
 
 	myself = container_of(obj_hdl, struct glusterfs_handle, handle);
 
 	/* Handle only regular files */
 	if (obj_hdl->type != REGULAR_FILE)
 		return fsalstat(posix2fsal_error(EINVAL), EINVAL);
-
 	status = fsal_find_fd((struct fsal_fd **)&tmp2_fd, obj_hdl,
-			      (struct fsal_fd *)&myself->globalfd,
-			      &myself->share, bypass, state,
-			      openflags, glusterfs_open_func,
-			      glusterfs_close_func,
-			      has_lock, closefd, open_for_locks);
-
+				(struct fsal_fd *)&myself->globalfd,
+				&myself->share, bypass, state,
+				openflags, glusterfs_open_func,
+				glusterfs_close_func,
+				has_lock, closefd, open_for_locks,
+				&reusing_open_state_fd);
 	/* since tmp2_fd is not accessed/closed outside
-	 * this routine, its safe to copy its variables into my_fd
-	 * without taking extra reference or allocating extra
-	 * memory.
-	 */
-	my_fd->glfd = tmp2_fd->glfd;
+	* this routine, its safe to copy its variables into my_fd
+	* without taking extra reference or allocating extra
+	* memory.
+	*/
+	if (reusing_open_state_fd) {
+		my_fd->glfd = glfs_dup(tmp2_fd->glfd);
+		my_fd->creds.caller_garray =
+			gsh_malloc(my_fd->creds.caller_glen *
+				   sizeof(gid_t));
+
+		memcpy(my_fd->creds.caller_garray,
+		       op_ctx->creds->caller_garray,
+		       op_ctx->creds->caller_glen *
+		       sizeof(gid_t));
+	} else {
+		my_fd->glfd = tmp2_fd->glfd;
+		my_fd->creds.caller_garray = tmp2_fd->creds.caller_garray;
+	}
+
 	my_fd->openflags = tmp2_fd->openflags;
 	my_fd->creds.caller_uid = tmp2_fd->creds.caller_uid;
 	my_fd->creds.caller_gid = tmp2_fd->creds.caller_gid;
 	my_fd->creds.caller_glen = tmp2_fd->creds.caller_glen;
-	my_fd->creds.caller_garray = tmp2_fd->creds.caller_garray;
 
 	return status;
 }
@@ -1416,6 +1374,10 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 			PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 		}
 
+		if (my_fd->openflags != FSAL_O_CLOSED) {
+			glusterfs_close_my_fd(my_fd);
+		}
+
 		/* truncate is set in p_flags */
 		status = glusterfs_open_my_fd(myself, openflags, p_flags,
 					      my_fd);
@@ -1454,6 +1416,10 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 				LogFullDebug(COMPONENT_FSAL,
 					     "New size = %" PRIx64,
 					     stat.st_size);
+				if (attrs_out) {
+					posix2fsal_attributes_all(&stat,
+								  attrs_out);
+				}
 			} else {
 				if (errno == EBADF)
 					errno = ESTALE;
@@ -1473,6 +1439,10 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 				status =
 				    fsalstat(posix2fsal_error(EEXIST), EEXIST);
 			}
+
+		} else if (attrs_out && attrs_out->request_mask &
+			   ATTR_RDATTR_ERR) {
+			attrs_out->valid_mask &= ATTR_RDATTR_ERR;
 		}
 
 		if (state == NULL) {
@@ -1494,6 +1464,7 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 		}
 
 		(void) glusterfs_close_my_fd(my_fd);
+
  undo_share:
 
 		/* Can only get here with state not NULL and an error */
@@ -1611,12 +1582,9 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 		glhandle =
 		    glfs_h_creat(glfs_export->gl_fs->fs, parenthandle->glhandle,
 				 name, p_flags, unix_mode, &sb);
-	} else if (!errno)
-                created = true;
-
-       /* preserve errno */
-	retval = errno;
-
+	} else if (!errno) {
+		created = true;
+	}
 
 	/* restore credentials */
 	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
@@ -1643,7 +1611,7 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 	 * file with a mode that prevented the open this caller was attempting.
 	 */
 
-        /* Do a permission check if we were not attempting to create. If we
+	/* Do a permission check if we were not attempting to create. If we
 	 * were attempting any sort of create, then the openat call was made
 	 * with the caller's credentials active and as such was permission
 	 * checked.
@@ -1662,13 +1630,15 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 	retval = glfs_get_volumeid(glfs_export->gl_fs->fs, vol_uuid,
 				   GLAPI_UUID_LENGTH);
 	if (retval < 0) {
-		status = gluster2fsal_error(retval);
+		status = gluster2fsal_error(errno);
 		goto direrr;
 	}
 
 	construct_handle(glfs_export, &sb, glhandle, globjhdl,
 			 GLAPI_HANDLE_LENGTH, &myself, vol_uuid);
 
+	*new_obj = &myself->handle;
+
 	/* If we didn't have a state above, use the global fd. At this point,
 	 * since we just created the global fd, no one else can have a
 	 * reference to it, and thus we can mamnipulate unlocked which is
@@ -1685,8 +1655,6 @@ open:
 	if (FSAL_IS_ERROR(status))
 		goto direrr;
 
-	*new_obj = &myself->handle;
-
 	if (created && attrib_set->valid_mask != 0) {
 		/* Set attributes using our newly opened file descriptor as the
 		 * share_fd if there are any left to set (mode and truncate
@@ -1700,14 +1668,8 @@ open:
 						      state,
 						      attrib_set);
 
-		if (FSAL_IS_ERROR(status)) {
-			/* Release the handle we just allocated. */
-			(*new_obj)->obj_ops.release(*new_obj);
-			/* We released handle at this point */
-			glhandle = NULL;
-			*new_obj = NULL;
+		if (FSAL_IS_ERROR(status))
 			goto fileerr;
-		}
 
 		if (attrs_out != NULL) {
 			status = (*new_obj)->obj_ops.getattrs(*new_obj,
@@ -1751,9 +1713,22 @@ open:
 
 
 fileerr:
+	/* Avoiding use after freed, make sure close my_fd before
+	 * obj_ops.release(), glfs_close is called depends on
+	 * FSAL_O_CLOSED flags, it's harmless of closing my_fd twice
+	 * in the floowing obj_ops.release().
+	 */
 	glusterfs_close_my_fd(my_fd);
 
 direrr:
+	/* Release the handle we just allocated. */
+	if (*new_obj) {
+		(*new_obj)->obj_ops.release(*new_obj);
+		/* We released handle at this point */
+		glhandle = NULL;
+		*new_obj = NULL;
+	}
+
 	/* Delete the file if we actually created it. */
 	if (created) {
 		SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
@@ -1770,7 +1745,6 @@ direrr:
 
 	if (status.major != ERR_FSAL_NO_ERROR)
 		gluster_cleanup_vars(glhandle);
-	return fsalstat(posix2fsal_error(retval), retval);
 
  out:
 #ifdef GLTIMING
@@ -1838,10 +1812,16 @@ static fsal_status_t glusterfs_reopen2(struct fsal_obj_handle *obj_hdl,
 
 	if (!FSAL_IS_ERROR(status)) {
 		/* Close the existing file descriptor and copy the new
-		 * one over.
+		 * one over. Make sure no one is using the fd that we are
+		 * about to close!
 		 */
+		PTHREAD_RWLOCK_wrlock(&my_share_fd->fdlock);
+
 		glusterfs_close_my_fd(my_share_fd);
-		*my_share_fd = fd;
+		my_share_fd->glfd = my_fd->glfd;
+		my_share_fd->openflags = my_fd->openflags;
+
+		PTHREAD_RWLOCK_unlock(&my_share_fd->fdlock);
 	} else {
 		/* We had a failure on open - we need to revert the share.
 		 * This can block over an I/O operation.
@@ -1878,6 +1858,7 @@ static fsal_status_t glusterfs_read2(struct fsal_obj_handle *obj_hdl,
 	bool closefd = false;
 	struct glusterfs_export *glfs_export =
 	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
+	struct glusterfs_fd *glusterfs_fd = NULL;
 
 	if (info != NULL) {
 		/* Currently we don't support READ_PLUS */
@@ -1894,6 +1875,16 @@ static fsal_status_t glusterfs_read2(struct fsal_obj_handle *obj_hdl,
 	}
 #endif
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		glusterfs_fd = &container_of(state, struct glusterfs_state_fd,
+					     state)->glusterfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&glusterfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
 	status = find_fd(&my_fd, obj_hdl, bypass, state, FSAL_O_READ,
 			 &has_lock, &closefd, false);
@@ -1940,6 +1931,9 @@ static fsal_status_t glusterfs_read2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
+	if (glusterfs_fd)
+		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
+
 	if (closefd)
 		glusterfs_close_my_fd(&my_fd);
 
@@ -1972,7 +1966,7 @@ static fsal_status_t glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 	fsal_openflags_t openflags = FSAL_O_WRITE;
 	struct glusterfs_export *glfs_export =
 	     container_of(op_ctx->fsal_export, struct glusterfs_export, export);
-
+	struct glusterfs_fd *glusterfs_fd = NULL;
 
 	if (info != NULL) {
 		/* Currently we don't support WRITE_PLUS */
@@ -1989,6 +1983,16 @@ static fsal_status_t glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 	}
 #endif
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		glusterfs_fd = &container_of(state, struct glusterfs_state_fd,
+					     state)->glusterfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&glusterfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
 	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
 			 &has_lock, &closefd, false);
@@ -2017,6 +2021,9 @@ static fsal_status_t glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
+	if (glusterfs_fd)
+		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
+
 	if (closefd)
 		glusterfs_close_my_fd(&my_fd);
 
@@ -2035,7 +2042,9 @@ static fsal_status_t glusterfs_commit2(struct fsal_obj_handle *obj_hdl,
 {
 	fsal_status_t status;
 	int retval;
-	struct glusterfs_fd tmp_fd = {0, NULL}, *out_fd = &tmp_fd;
+	struct glusterfs_fd tmp_fd = {
+			FSAL_O_CLOSED, PTHREAD_RWLOCK_INITIALIZER, NULL };
+	struct glusterfs_fd *out_fd = &tmp_fd;
 	struct glusterfs_handle *myself = NULL;
 	bool has_lock = false;
 	bool closefd = false;
@@ -2104,6 +2113,7 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	struct glusterfs_export *glfs_export =
 	    container_of(op_ctx->fsal_export,
 			 struct glusterfs_export, export);
+	struct glusterfs_fd *glusterfs_fd = NULL;
 
 #if 0
 	/** @todo: fsid work */
@@ -2178,22 +2188,42 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 		return fsalstat(ERR_FSAL_BAD_RANGE, 0);
 	}
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		glusterfs_fd = &container_of(state, struct glusterfs_state_fd,
+					     state)->glusterfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&glusterfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
 	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
-			 &has_lock, &closefd, true);
+			 &has_lock, &closefd,
+			 (state &&
+			  ((state->state_type == STATE_TYPE_NLM_LOCK) ||
+			  (state->state_type == STATE_TYPE_9P_FID))));
 
 	if (FSAL_IS_ERROR(status)) {
 		LogCrit(COMPONENT_FSAL, "Unable to find fd for lock operation");
 		return status;
 	}
 
-	/** @todo: setlkowner as well */
 	errno = 0;
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
 			  op_ctx->creds->caller_garray);
 
+	/* Convert lkowner ptr address to opaque string */
+	retval = glfs_fd_set_lkowner(my_fd.glfd, p_owner, sizeof(p_owner));
+	if (retval) {
+		LogCrit(COMPONENT_FSAL,
+			"Setting lkowner failed");
+		goto err;
+	}
+
 	retval = glfs_posix_lock(my_fd.glfd, fcntl_comm, &lock_args);
 
 	if (retval /* && lock_op == FSAL_OP_LOCK */) {
@@ -2206,6 +2236,16 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 
 		if (conflicting_lock != NULL) {
 			/* Get the conflicting lock */
+
+			rc = glfs_fd_set_lkowner(my_fd.glfd, p_owner,
+						 sizeof(p_owner));
+			if (rc) {
+				retval = errno; /* we losethe initial error */
+				LogCrit(COMPONENT_FSAL,
+					"Setting lkowner while trying to get conflicting lock failed");
+				goto err;
+			}
+
 			rc = glfs_posix_lock(my_fd.glfd, F_GETLK,
 						 &lock_args);
 
@@ -2243,6 +2283,9 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
  err:
 	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
 
+	if (glusterfs_fd)
+		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
+
 	if (closefd)
 		glusterfs_close_my_fd(&my_fd);
 
@@ -2283,7 +2326,7 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	glusterfs_fsal_xstat_t buffxstat;
 	int attr_valid = 0;
 	int mask = 0;
-
+	struct glusterfs_fd *glusterfs_fd = NULL;
 
 	/** @todo: Handle special file symblic links etc */
 	/* apply umask, if mode attribute is to be changed */
@@ -2315,6 +2358,17 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	/** TRUNCATE **/
 	if (FSAL_TEST_MASK(attrib_set->valid_mask, ATTR_SIZE) &&
 	    (obj_hdl->type == REGULAR_FILE)) {
+		/* Acquire state's fdlock to prevent OPEN upgrade closing the
+		 * file descriptor while we use it.
+		 */
+		if (state) {
+			glusterfs_fd = &container_of(state,
+						     struct glusterfs_state_fd,
+						     state)->glusterfs_fd;
+
+			PTHREAD_RWLOCK_rdlock(&glusterfs_fd->fdlock);
+		}
+
 		/* Get a usable file descriptor. Share conflict is only
 		 * possible if size is being set. For special files,
 		 * handle via handle.
@@ -2460,6 +2514,9 @@ out:
 			 strerror(status.minor));
 	}
 
+	if (glusterfs_fd)
+		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
+
 	if (closefd)
 		glusterfs_close_my_fd(&my_fd);
 
@@ -2702,7 +2759,6 @@ void handle_ops_init(struct fsal_obj_ops *ops)
 	ops->release = handle_release;
 	ops->merge = glusterfs_merge;
 	ops->lookup = lookup;
-	ops->create = create;
 	ops->mkdir = makedir;
 	ops->mknode = makenode;
 	ops->readdir = read_dirents;
