@@ -33,6 +33,7 @@
 #include <libgen.h>		/* used for 'dirname' */
 #include <pthread.h>
 #include <string.h>
+#include <sys/sysmacros.h>
 #include <sys/types.h>
 #include <mntent.h>
 #include <sys/statfs.h>
@@ -45,7 +46,6 @@
 #include "nfs_exports.h"
 #include "export_mgr.h"
 #include "pnfs_utils.h"
-#include "mdcache.h"
 #include "include/gpfs.h"
 
 /* export object methods
@@ -59,6 +59,7 @@ static void release(struct fsal_export *exp_hdl)
 	gpfs_unexport_filesystems(myself);
 	fsal_detach_export(exp_hdl->fsal, &exp_hdl->exports);
 	free_export_ops(exp_hdl);
+	close(myself->export_fd);
 
 	gsh_free(myself);		/* elvis has left the building */
 }
@@ -70,15 +71,15 @@ static fsal_status_t get_dynamic_info(struct fsal_export *exp_hdl,
 	fsal_status_t status;
 	struct statfs buffstatgpfs;
 	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
-	struct gpfs_filesystem *gpfs_fs;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	if (!infop) {
 		fsal_error = ERR_FSAL_FAULT;
 		goto out;
 	}
-	gpfs_fs = obj_hdl->fs->private_data;
-
-	status = GPFSFSAL_statfs(gpfs_fs->root_fd, obj_hdl, &buffstatgpfs);
+	status = GPFSFSAL_statfs(export_fd, obj_hdl, &buffstatgpfs);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
@@ -88,6 +89,8 @@ static fsal_status_t get_dynamic_info(struct fsal_export *exp_hdl,
 	infop->total_files = buffstatgpfs.f_files;
 	infop->free_files = buffstatgpfs.f_ffree;
 	infop->avail_files = buffstatgpfs.f_ffree;
+	infop->maxread = buffstatgpfs.f_bsize;
+	infop->maxwrite = buffstatgpfs.f_bsize;
 	infop->time_delta.tv_sec = 1;
 	infop->time_delta.tv_nsec = 0;
 
@@ -216,162 +219,131 @@ static uint32_t fs_xattr_access_rights(struct fsal_export *exp_hdl)
  * on linux, can map st_dev -> /proc/partitions name -> /dev/<name>
  */
 
-static fsal_status_t get_quota(struct fsal_export *exp_hdl,
-			       const char *filepath, int quota_type,
-			       int quota_id,
-			       fsal_quota_t *pquota)
+static fsal_status_t
+get_quota(struct fsal_export *exp_hdl, const char *filepath, int quota_type,
+	  int quota_id, fsal_quota_t *fsal_quota)
 {
-	struct gpfs_fsal_export *myself;
-	gpfs_quotaInfo_t fs_quota;
+	gpfs_quotaInfo_t gpfs_quota = {0};
 	struct stat path_stat;
-	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
-	int retval;
+	int retval = 0;
 	struct quotactl_arg args;
-	int errsv;
+	struct fsal_filesystem *fs = container_of(exp_hdl,
+						  struct gpfs_fsal_export,
+						  export)->root_fs;
 
-	myself = container_of(exp_hdl, struct gpfs_fsal_export, export);
-	retval = stat(filepath, &path_stat);
-	if (retval < 0) {
-		LogMajor(COMPONENT_FSAL,
-			 "GPFS get_quota, fstat: root_path: %s, errno=(%d) %s",
-			 myself->root_fs->path,
-			 errno, strerror(errno));
-		fsal_error = posix2fsal_error(errno);
+	if (stat(filepath, &path_stat) < 0) {
 		retval = errno;
-		goto out;
-	}
-	if ((major(path_stat.st_dev) != myself->root_fs->dev.major) ||
-	    (minor(path_stat.st_dev) != myself->root_fs->dev.minor)) {
 		LogMajor(COMPONENT_FSAL,
-			 "GPFS get_quota: crossed mount boundary! root_path: %s, quota path: %s",
-			 myself->root_fs->path, filepath);
-		fsal_error = ERR_FSAL_FAULT;	/* maybe a better error? */
-		retval = 0;
-		goto out;
+			 "GPFS get quota, stat: root_path: %s, errno=(%d) %s",
+			 fs->path, retval, strerror(retval));
+		return fsalstat(posix2fsal_error(retval), retval);
 	}
-	memset((void *)&fs_quota, 0, sizeof(gpfs_quotaInfo_t));
+
+	if ((major(path_stat.st_dev) != fs->dev.major) ||
+	    (minor(path_stat.st_dev) != fs->dev.minor)) {
+		LogMajor(COMPONENT_FSAL,
+			 "GPFS get quota: crossed mount boundary! root_path: %s, quota path: %s",
+			 fs->path, filepath);
+		return fsalstat(ERR_FSAL_FAULT, 0);  /* maybe a better error? */
+	}
+
 	args.pathname = filepath;
 	args.cmd = GPFS_QCMD(Q_GETQUOTA, quota_type);
 	args.qid = quota_id;
-	args.bufferP = (void *) &fs_quota;
+	args.bufferP = &gpfs_quota;
 
 	fsal_set_credentials(op_ctx->creds);
-	retval = gpfs_ganesha(OPENHANDLE_QUOTA, &args);
-	errsv = errno;
+	if (gpfs_ganesha(OPENHANDLE_QUOTA, &args) < 0)
+		retval = errno;
 	fsal_restore_ganesha_credentials();
 
-	if (retval < 0) {
-		fsal_error = posix2fsal_error(errsv);
-		retval = errsv;
-		goto out;
-	}
-	pquota->bhardlimit = fs_quota.blockHardLimit;
-	pquota->bsoftlimit = fs_quota.blockSoftLimit;
-	pquota->curblocks = fs_quota.blockUsage;
-	pquota->fhardlimit = fs_quota.inodeHardLimit;
-	pquota->fsoftlimit = fs_quota.inodeSoftLimit;
-	pquota->curfiles = fs_quota.inodeUsage;
-	pquota->btimeleft = fs_quota.blockGraceTime;
-	pquota->ftimeleft = fs_quota.inodeGraceTime;
-	pquota->bsize = 1024;
+	if (retval)
+		return fsalstat(posix2fsal_error(retval), retval);
 
- out:
-	return fsalstat(fsal_error, retval);
+	fsal_quota->bhardlimit = gpfs_quota.blockHardLimit;
+	fsal_quota->bsoftlimit = gpfs_quota.blockSoftLimit;
+	fsal_quota->curblocks = gpfs_quota.blockUsage;
+	fsal_quota->fhardlimit = gpfs_quota.inodeHardLimit;
+	fsal_quota->fsoftlimit = gpfs_quota.inodeSoftLimit;
+	fsal_quota->curfiles = gpfs_quota.inodeUsage;
+	fsal_quota->btimeleft = gpfs_quota.blockGraceTime;
+	fsal_quota->ftimeleft = gpfs_quota.inodeGraceTime;
+	fsal_quota->bsize = 1024;
+
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
 /* set_quota
  * same lower mount restriction applies
  */
 
-static fsal_status_t set_quota(struct fsal_export *exp_hdl,
-			       const char *filepath, int quota_type,
-			       int quota_id,
-			       fsal_quota_t *pquota, fsal_quota_t *presquota)
+static fsal_status_t
+set_quota(struct fsal_export *exp_hdl, const char *filepath, int quota_type,
+	  int quota_id, fsal_quota_t *fsal_quota, fsal_quota_t *res_quota)
 {
-	struct gpfs_fsal_export *myself;
-	gpfs_quotaInfo_t fs_quota;
+	gpfs_quotaInfo_t gpfs_quota = {0};
 	struct stat path_stat;
-	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
-	int retval;
+	int retval = 0;
 	struct quotactl_arg args;
-	int errsv;
+	struct fsal_filesystem *fs = container_of(exp_hdl,
+						  struct gpfs_fsal_export,
+						  export)->root_fs;
 
-	myself = container_of(exp_hdl, struct gpfs_fsal_export, export);
-	retval = stat(filepath, &path_stat);
-	if (retval < 0) {
-		LogMajor(COMPONENT_FSAL,
-			 "GPFS set_quota, fstat: root_path: %s, errno=(%d) %s",
-			 myself->root_fs->path,
-			 errno, strerror(errno));
-		fsal_error = posix2fsal_error(errno);
+	if (stat(filepath, &path_stat) < 0) {
 		retval = errno;
-		goto err;
-	}
-	if ((major(path_stat.st_dev) != myself->root_fs->dev.major) ||
-	    (minor(path_stat.st_dev) != myself->root_fs->dev.minor)) {
 		LogMajor(COMPONENT_FSAL,
-			 "GPFS set_quota: crossed mount boundary! root_path: %s, quota path: %s",
-			 myself->root_fs->path, filepath);
-		fsal_error = ERR_FSAL_FAULT;	/* maybe a better error? */
-		retval = 0;
-		goto err;
+			 "GPFS set quota, stat: root_path: %s, errno=(%d) %s",
+			 fs->path, retval, strerror(retval));
+		return fsalstat(posix2fsal_error(retval), retval);
 	}
-	memset((char *)&fs_quota, 0, sizeof(gpfs_quotaInfo_t));
-	if (pquota->bhardlimit != 0) {
-		fs_quota.blockHardLimit = pquota->bhardlimit;
-	}
-	if (pquota->bsoftlimit != 0) {
-		fs_quota.blockSoftLimit = pquota->bsoftlimit;
-	}
-	if (pquota->fhardlimit != 0) {
-		fs_quota.inodeHardLimit = pquota->fhardlimit;
-	}
-	if (pquota->fsoftlimit != 0) {
-		fs_quota.inodeSoftLimit = pquota->fsoftlimit;
-	}
-	if (pquota->btimeleft != 0) {
-		fs_quota.blockGraceTime = pquota->btimeleft;
-	}
-	if (pquota->ftimeleft != 0) {
-		fs_quota.inodeGraceTime = pquota->ftimeleft;
+
+	if ((major(path_stat.st_dev) != fs->dev.major) ||
+	    (minor(path_stat.st_dev) != fs->dev.minor)) {
+		LogMajor(COMPONENT_FSAL,
+			 "GPFS set quota: crossed mount boundary! root_path: %s, quota path: %s",
+			 fs->path, filepath);
+		return fsalstat(ERR_FSAL_FAULT, 0);  /* maybe a better error? */
 	}
+
+	gpfs_quota.blockHardLimit = fsal_quota->bhardlimit;
+	gpfs_quota.blockSoftLimit = fsal_quota->bsoftlimit;
+	gpfs_quota.inodeHardLimit = fsal_quota->fhardlimit;
+	gpfs_quota.inodeSoftLimit = fsal_quota->fsoftlimit;
+	gpfs_quota.blockGraceTime = fsal_quota->btimeleft;
+	gpfs_quota.inodeGraceTime = fsal_quota->ftimeleft;
+
 	args.pathname = filepath;
 	args.cmd = GPFS_QCMD(Q_SETQUOTA, quota_type);
 	args.qid = quota_id;
-	args.bufferP = (void *) &fs_quota;
+	args.bufferP = &gpfs_quota;
 
 	fsal_set_credentials(op_ctx->creds);
-	retval = gpfs_ganesha(OPENHANDLE_QUOTA, &args);
-	errsv = errno;
+	if (gpfs_ganesha(OPENHANDLE_QUOTA, &args) < 0)
+		retval = errno;
 	fsal_restore_ganesha_credentials();
 
-	if (retval < 0) {
-		fsal_error = posix2fsal_error(errsv);
-		retval = errsv;
-		goto err;
-	}
-	if (presquota != NULL)
-		return get_quota(exp_hdl, filepath,
-				 quota_type, quota_id, presquota);
+	if (retval)
+		return fsalstat(posix2fsal_error(retval), retval);
 
- err:
-	return fsalstat(fsal_error, retval);
+	if (res_quota != NULL)
+		return get_quota(exp_hdl, filepath,
+				 quota_type, quota_id, res_quota);
+
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
 /* extract a file handle from a buffer.
  * do verification checks and flag any and all suspicious bits.
  * Return an updated fh_desc into whatever was passed.  The most
- * common behavior, done here is to just reset the length.  There
- * is the option to also adjust the start pointer.
+ * common behavior, done here is to just reset the length.
  */
 
-static fsal_status_t gpfs_extract_handle(struct fsal_export *exp_hdl,
-					 fsal_digesttype_t in_type,
-					 struct gsh_buffdesc *fh_desc,
-					 int flags)
+static fsal_status_t
+gpfs_wire_to_host(struct fsal_export *exp_hdl, fsal_digesttype_t in_type,
+		  struct gsh_buffdesc *fh_desc, int flags)
 {
 	struct gpfs_file_handle *hdl;
-	size_t fh_size = 0;
+	size_t fh_size;
 
 	/* sanity checks */
 	if (!fh_desc || !fh_desc->addr)
@@ -400,34 +372,90 @@ static fsal_status_t gpfs_extract_handle(struct fsal_export *exp_hdl,
 	   hdl->handle_key_size, hdl->handle_fsid[0], hdl->handle_fsid[1],
 	   fh_size);
 
-	if (fh_desc->len != fh_size) {
+	/* Some older file handles include additional 16 bytes in fh_desc->len.
+	 * Honor those as well.
+	 */
+	if (fh_desc->len != fh_size &&
+	    fh_desc->len != fh_size + 16) {
 		LogMajor(COMPONENT_FSAL,
 			 "Size mismatch for handle.  should be %zu, got %zu",
 			 fh_size, fh_desc->len);
 		return fsalstat(ERR_FSAL_SERVERFAULT, 0);
 	}
-	fh_desc->len = hdl->handle_key_size;	/* pass back the key size */
+	fh_desc->len = hdl->handle_size;	/* pass back the size */
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/** \var GPFS_write_verifier
- *  @brief NFS V4 write verifier
- */
-verifier4 GPFS_write_verifier;
-
-static void gpfs_verifier(struct fsal_export *exp_hdl,
-			  struct gsh_buffdesc *verf_desc)
+/* Produce handle-key from a host-handle */
+static fsal_status_t
+gpfs_host_to_key(struct fsal_export *exp_hdl,
+		   struct gsh_buffdesc *fh_desc)
 {
-	memcpy(verf_desc->addr, &GPFS_write_verifier, verf_desc->len);
+	struct gpfs_file_handle *hdl;
+
+	if (fh_desc->len < offsetof(struct gpfs_file_handle, f_handle))
+		return fsalstat(ERR_FSAL_INVAL, 0);
+
+	hdl = (struct gpfs_file_handle *)fh_desc->addr;
+	fh_desc->len = hdl->handle_key_size;	/* pass back the key size */
+
+	LogFullDebug(COMPONENT_FSAL,
+		     "size %d type %d ver %d key_size %d FSID 0x%X:%X",
+		     hdl->handle_size, hdl->handle_type, hdl->handle_version,
+		     hdl->handle_key_size, hdl->handle_fsid[0],
+		     hdl->handle_fsid[1]);
+
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
 /**
- *  @brief set the global GPFS_write_verfier according to \a verifier
- *  @param verfier verifier4 type
+ * @brief Allocate a state_t structure
+ *
+ * Note that this is not expected to fail since memory allocation is
+ * expected to abort on failure.
+ *
+ * @param[in] exp_hdl               Export state_t will be associated with
+ * @param[in] state_type            Type of state to allocate
+ * @param[in] related_state         Related state if appropriate
+ *
+ * @returns a state structure.
  */
-void set_gpfs_verifier(verifier4 *verifier)
+struct state_t *
+gpfs_alloc_state(struct fsal_export *exp_hdl, enum state_type state_type,
+		 struct state_t *related_state)
 {
-	memcpy(&GPFS_write_verifier, verifier, sizeof(verifier4));
+	struct state_t *state;
+	struct gpfs_fd *my_fd;
+
+	state = init_state(gsh_calloc(1, sizeof(struct gpfs_state_fd)),
+			   exp_hdl, state_type, related_state);
+
+	my_fd = &container_of(state, struct gpfs_state_fd, state)->gpfs_fd;
+
+	my_fd->fd = -1;
+	my_fd->openflags = FSAL_O_CLOSED;
+	PTHREAD_RWLOCK_init(&my_fd->fdlock, NULL);
+
+	return state;
+}
+
+/**
+ * @brief free a gpfs_state_fd structure
+ *
+ * @param[in] exp_hdl  Export state_t will be associated with
+ * @param[in] state    Related state if appropriate
+ *
+ */
+void
+gpfs_free_state(struct fsal_export *exp_hdl, struct state_t *state)
+{
+	struct gpfs_state_fd *state_fd = container_of(state,
+						      struct gpfs_state_fd,
+						      state);
+	struct gpfs_fd *my_fd = &state_fd->gpfs_fd;
+
+	PTHREAD_RWLOCK_destroy(&my_fd->fdlock);
+	gsh_free(state_fd);
 }
 
 /**
@@ -438,7 +466,8 @@ void gpfs_export_ops_init(struct export_ops *ops)
 {
 	ops->release = release;
 	ops->lookup_path = gpfs_lookup_path;
-	ops->extract_handle = gpfs_extract_handle;
+	ops->wire_to_host = gpfs_wire_to_host;
+	ops->host_to_key = gpfs_host_to_key;
 	ops->create_handle = gpfs_create_handle;
 	ops->get_fs_dynamic_info = get_dynamic_info;
 	ops->fs_supports = fs_supports;
@@ -455,14 +484,15 @@ void gpfs_export_ops_init(struct export_ops *ops)
 	ops->fs_xattr_access_rights = fs_xattr_access_rights;
 	ops->get_quota = get_quota;
 	ops->set_quota = set_quota;
-	ops->get_write_verifier = gpfs_verifier;
 	ops->alloc_state = gpfs_alloc_state;
+	ops->free_state = gpfs_free_state;
 }
 
 static void free_gpfs_filesystem(struct gpfs_filesystem *gpfs_fs)
 {
 	if (gpfs_fs->root_fd >= 0)
 		close(gpfs_fs->root_fd);
+	PTHREAD_MUTEX_destroy(&gpfs_fs->upvector_mutex);
 	gsh_free(gpfs_fs);
 }
 
@@ -487,10 +517,7 @@ int open_root_fd(struct gpfs_filesystem *gpfs_fs)
 	struct fsal_fsid__ fsid;
 	int retval;
 	fsal_status_t status;
-	struct gpfs_file_handle *fh;
-
-	fh = alloca(sizeof(*fh));
-	memset(fh, 0, sizeof(*fh));
+	struct gpfs_file_handle fh = {0};
 
 	gpfs_fs->root_fd = open(gpfs_fs->fs->path, O_RDONLY | O_DIRECTORY);
 
@@ -501,10 +528,12 @@ int open_root_fd(struct gpfs_filesystem *gpfs_fs)
 			 gpfs_fs->fs->path, strerror(retval), retval);
 		return retval;
 	}
+	LogFullDebug(COMPONENT_FSAL, "root export_fd %d path %s",
+				gpfs_fs->root_fd, gpfs_fs->fs->path);
 
 	status = fsal_internal_get_handle_at(gpfs_fs->root_fd,
-					     gpfs_fs->fs->path, fh,
-					     0, &gpfs_fs->root_fd);
+					     gpfs_fs->fs->path, &fh,
+					     gpfs_fs->root_fd);
 
 	if (FSAL_IS_ERROR(status)) {
 		retval = status.minor;
@@ -514,22 +543,20 @@ int open_root_fd(struct gpfs_filesystem *gpfs_fs)
 		goto errout;
 	}
 
-	gpfs_extract_fsid(fh, &fsid);
+	gpfs_extract_fsid(&fh, &fsid);
 
 	retval = re_index_fs_fsid(gpfs_fs->fs, GPFS_FSID_TYPE, &fsid);
 
-	if (retval < 0) {
-		LogCrit(COMPONENT_FSAL,
-			"Could not re-index GPFS file system fsid for %s",
-			gpfs_fs->fs->path);
-		retval = -retval;
-		goto errout;
-	}
+	if (retval == 0)
+		return 0;
 
-	return retval;
+	LogCrit(COMPONENT_FSAL,
+		"Could not re-index GPFS file system fsid for %s, error:%d",
+		gpfs_fs->fs->path, retval);
+	assert(retval < 0);
+	retval = -retval;
 
 errout:
-
 	close(gpfs_fs->root_fd);
 	gpfs_fs->root_fd = -1;
 
@@ -544,51 +571,66 @@ errout:
  */
 int gpfs_claim_filesystem(struct fsal_filesystem *fs, struct fsal_export *exp)
 {
-	struct gpfs_filesystem *gpfs_fs = NULL;
+	struct gpfs_filesystem *gpfs_fs;
 	int retval;
-	struct gpfs_fsal_export *myself;
-	struct gpfs_filesystem_export_map *map = NULL;
+	struct gpfs_filesystem_export_map *map;
 	pthread_attr_t attr_thr;
 
-	myself = container_of(exp, struct gpfs_fsal_export, export);
-
 	if (strcmp(fs->type, "gpfs") != 0) {
-		LogInfo(COMPONENT_FSAL,
-			"Attempt to claim non-GPFS filesystem %s",
-			fs->path);
+		LogEvent(COMPONENT_FSAL,
+			"Attempt to claim non-GPFS filesystem %s", fs->path);
 		return ENXIO;
 	}
 
+	if (fs->fsal != NULL && fs->private_data == NULL)
+		LogFatal(COMPONENT_FSAL,
+			"Something wrong with export, fs %s appears already claimed but doesn't have private data",
+			fs->path);
+
+	if (fs->fsal == NULL && fs->private_data != NULL)
+		LogFatal(COMPONENT_FSAL,
+			"Something wrong with export, fs %s was not claimed but had non-NULL private",
+			fs->path);
+
+	gpfs_fs = fs->private_data;
+	if (!gpfs_fs) { /* first export */
+		gpfs_fs = gsh_calloc(1, sizeof(*gpfs_fs));
+		glist_init(&gpfs_fs->exports);
+		gpfs_fs->root_fd = -1;
+		gpfs_fs->fs = fs;
+		PTHREAD_MUTEX_init(&gpfs_fs->upvector_mutex, NULL);
+	}
+
+	/* Now map the file system and export */
 	map = gsh_calloc(1, sizeof(*map));
+	map->fs = gpfs_fs;
+	map->exp = container_of(exp, struct gpfs_fsal_export, export);
+	PTHREAD_MUTEX_lock(&gpfs_fs->upvector_mutex);
+	glist_add_tail(&gpfs_fs->exports, &map->on_exports);
+	glist_add_tail(&map->exp->filesystems, &map->on_filesystems);
+	PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
 
-	if (fs->fsal != NULL) {
-		gpfs_fs = fs->private_data;
-		if (gpfs_fs == NULL) {
-			LogCrit(COMPONENT_FSAL,
-				"Something wrong with export, fs %s appears already claimed but doesn't have private data",
-				fs->path);
-			retval = EINVAL;
-			goto errout;
-		}
-
-		goto already_claimed;
+	map->exp->export_fd = open(op_ctx->ctx_export->fullpath,
+						O_RDONLY | O_DIRECTORY);
+	if (map->exp->export_fd < 0) {
+		retval = errno;
+		LogMajor(COMPONENT_FSAL,
+			"Could not open GPFS export point %s: rc = %s (%d)",
+			op_ctx->ctx_export->fullpath, strerror(retval), retval);
+		goto errout;
 	}
 
-	if (fs->private_data != NULL) {
-			LogCrit(COMPONENT_FSAL,
-				"Something wrong with export, fs %s was not claimed but had non-NULL private",
-				fs->path);
-	}
+	LogFullDebug(COMPONENT_FSAL, "export_fd %d path %s",
+			map->exp->export_fd, op_ctx->ctx_export->fullpath);
 
-	gpfs_fs = gsh_calloc(1, sizeof(*gpfs_fs));
-
-	glist_init(&gpfs_fs->exports);
-	gpfs_fs->root_fd = -1;
-
-	gpfs_fs->fs = fs;
+	/* We have set up the export. If the file system is already claimed,
+	 * we are done.
+	 */
+	if (fs->private_data) /* file system is already claimed */
+		return 0;
 
+	/* Get an fd for the root and create an upcall thread */
 	retval = open_root_fd(gpfs_fs);
-
 	if (retval != 0) {
 		if (retval == ENOTTY) {
 			LogInfo(COMPONENT_FSAL,
@@ -599,8 +641,6 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs, struct fsal_export *exp)
 		goto errout;
 	}
 
-	memset(&attr_thr, 0, sizeof(attr_thr));
-
 	if (pthread_attr_init(&attr_thr) != 0)
 		LogCrit(COMPONENT_THREAD, "can't init pthread's attributes");
 
@@ -614,13 +654,8 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs, struct fsal_export *exp)
 	if (pthread_attr_setstacksize(&attr_thr, 2116488) != 0)
 		LogCrit(COMPONENT_THREAD, "can't set pthread's stack size");
 
-	gpfs_fs->up_ops = exp->up_ops;
-	retval = pthread_create(&gpfs_fs->up_thread,
-				&attr_thr,
-				GPFSFSAL_UP_Thread,
-				gpfs_fs);
-
-	if (retval != 0) {
+	if (pthread_create(&gpfs_fs->up_thread, &attr_thr, GPFSFSAL_UP_Thread,
+			   gpfs_fs)) {
 		retval = errno;
 		LogCrit(COMPONENT_THREAD,
 			"Could not create GPFSFSAL_UP_Thread, error = %d (%s)",
@@ -630,22 +665,19 @@ int gpfs_claim_filesystem(struct fsal_filesystem *fs, struct fsal_export *exp)
 
 	fs->private_data = gpfs_fs;
 
-already_claimed:
-
-	/* Now map the file system and export */
-	map->fs = gpfs_fs;
-	map->exp = myself;
-	glist_add_tail(&gpfs_fs->exports, &map->on_exports);
-	glist_add_tail(&myself->filesystems, &map->on_filesystems);
-
 	return 0;
 
 errout:
-
-	if (map != NULL)
-		gsh_free(map);
-
-	if (gpfs_fs != NULL)
+	if (map->exp->export_fd >= 0) {
+		close(map->exp->export_fd);
+		map->exp->export_fd = -1;
+	}
+	PTHREAD_MUTEX_lock(&gpfs_fs->upvector_mutex);
+	glist_del(&map->on_filesystems);
+	glist_del(&map->on_exports);
+	PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
+	gsh_free(map);
+	if (!fs->private_data)
 		free_gpfs_filesystem(gpfs_fs);
 
 	return retval;
@@ -660,49 +692,48 @@ void gpfs_unclaim_filesystem(struct fsal_filesystem *fs)
 	struct gpfs_filesystem *gpfs_fs = fs->private_data;
 	struct glist_head *glist, *glistn;
 	struct gpfs_filesystem_export_map *map;
-	struct callback_arg callback;
-	int reason;
-	int rc;
+	struct callback_arg callback = {0};
+	int reason = THREAD_STOP;
 
-	if (gpfs_fs != NULL) {
-		glist_for_each_safe(glist, glistn, &gpfs_fs->exports) {
-			map = glist_entry(glist,
-					  struct gpfs_filesystem_export_map,
-					  on_exports);
+	if (gpfs_fs == NULL)
+		goto out;
 
-			/* Remove this file system from mapping */
-			glist_del(&map->on_filesystems);
-			glist_del(&map->on_exports);
+	glist_for_each_safe(glist, glistn, &gpfs_fs->exports) {
+		map = glist_entry(glist, struct gpfs_filesystem_export_map,
+				  on_exports);
 
-			if (map->exp->root_fs == fs) {
-				LogInfo(COMPONENT_FSAL,
-					"Removing root_fs %s from GPFS export",
-					fs->path);
-			}
+		/* Remove this file system from mapping */
+		PTHREAD_MUTEX_lock(&map->fs->upvector_mutex);
+		glist_del(&map->on_filesystems);
+		glist_del(&map->on_exports);
+		PTHREAD_MUTEX_unlock(&map->fs->upvector_mutex);
 
-			/* And free it */
-			gsh_free(map);
-		}
+		if (map->exp->root_fs == fs)
+			LogInfo(COMPONENT_FSAL,
+				"Removing root_fs %s from GPFS export",
+				fs->path);
 
-		/* Terminate GPFS upcall thread */
-		callback.mountdirfd = gpfs_fs->root_fd;
-		reason = THREAD_STOP;
-		callback.reason = &reason;
-		rc = gpfs_ganesha(OPENHANDLE_THREAD_UPDATE, &callback);
-		if (rc)
-			LogCrit(COMPONENT_FSAL,
-				"Unable to stop upcall thread for %s, fd=%d, errno=%d",
-				fs->path, gpfs_fs->root_fd, errno);
-		else
-			LogFullDebug(COMPONENT_FSAL, "Thread STOP successful");
-		pthread_join(gpfs_fs->up_thread, NULL);
-		free_gpfs_filesystem(gpfs_fs);
-		fs->private_data = NULL;
+		/* And free it */
+		gsh_free(map);
 	}
 
-	LogInfo(COMPONENT_FSAL,
-		"GPFS Unclaiming %s",
-		fs->path);
+	/* Terminate GPFS upcall thread */
+	callback.mountdirfd = gpfs_fs->root_fd;
+	callback.reason = &reason;
+
+	if (gpfs_ganesha(OPENHANDLE_THREAD_UPDATE, &callback))
+		LogCrit(COMPONENT_FSAL,
+			"Unable to stop upcall thread for %s, fd=%d, errno=%d",
+			fs->path, gpfs_fs->root_fd, errno);
+	else
+		LogFullDebug(COMPONENT_FSAL, "Thread STOP successful");
+
+	pthread_join(gpfs_fs->up_thread, NULL);
+	free_gpfs_filesystem(gpfs_fs);
+	fs->private_data = NULL;
+
+out:
+	LogInfo(COMPONENT_FSAL, "GPFS Unclaiming %s", fs->path);
 }
 
 /**
@@ -717,13 +748,14 @@ void gpfs_unexport_filesystems(struct gpfs_fsal_export *exp)
 	PTHREAD_RWLOCK_wrlock(&fs_lock);
 
 	glist_for_each_safe(glist, glistn, &exp->filesystems) {
-		map = glist_entry(glist,
-				  struct gpfs_filesystem_export_map,
+		map = glist_entry(glist, struct gpfs_filesystem_export_map,
 				  on_filesystems);
 
 		/* Remove this export from mapping */
+		PTHREAD_MUTEX_lock(&map->fs->upvector_mutex);
 		glist_del(&map->on_filesystems);
 		glist_del(&map->on_exports);
+		PTHREAD_MUTEX_unlock(&map->fs->upvector_mutex);
 
 		if (glist_empty(&map->fs->exports)) {
 			LogInfo(COMPONENT_FSAL,
@@ -749,48 +781,45 @@ void gpfs_unexport_filesystems(struct gpfs_fsal_export *exp)
  *
  *  @return FSAL status
  */
-fsal_status_t gpfs_create_export(struct fsal_module *fsal_hdl,
-				 void *parse_node,
-				 struct config_error_type *err_type,
-				 const struct fsal_up_vector *up_ops)
+fsal_status_t
+gpfs_create_export(struct fsal_module *fsal_hdl, void *parse_node,
+		   struct config_error_type *err_type,
+		   const struct fsal_up_vector *up_ops)
 {
 	/* The status code to return */
 	fsal_status_t status = { ERR_FSAL_NO_ERROR, 0 };
-	struct gpfs_fsal_export *myself;
-	struct readlink_arg varg;
-	struct gpfs_filesystem *gpfs_fs;
-	gpfsfsal_xstat_t buffxstat;
-	int rc;
+	struct gpfs_fsal_export *gpfs_exp;
+	struct fsal_export *exp;
 
-	myself = gsh_calloc(1, sizeof(struct gpfs_fsal_export));
+	gpfs_exp = gsh_calloc(1, sizeof(struct gpfs_fsal_export));
+	exp = &gpfs_exp->export;
 
-	glist_init(&myself->filesystems);
+	glist_init(&gpfs_exp->filesystems);
 
 	status.minor = fsal_internal_version();
 	LogInfo(COMPONENT_FSAL, "GPFS get version is %d options 0x%X id %d",
 		status.minor,
-		op_ctx->export_perms ?
-		op_ctx->export_perms->options : 0,
+		op_ctx->export_perms ?  op_ctx->export_perms->options : 0,
 		op_ctx->ctx_export->export_id);
 
-	fsal_export_init(&myself->export);
-	gpfs_export_ops_init(&myself->export.exp_ops);
+	fsal_export_init(exp);
+	gpfs_export_ops_init(&exp->exp_ops);
 
-	status.minor = fsal_attach_export(fsal_hdl, &myself->export.exports);
+	status.minor = fsal_attach_export(fsal_hdl, &exp->exports);
 	if (status.minor != 0) {
 		status.major = posix2fsal_error(status.minor);
-		goto errout;	/* seriously bad */
+		goto free;	/* seriously bad */
 	}
-	myself->export.fsal = fsal_hdl;
-	op_ctx->fsal_export = &myself->export;
 
-	myself->export.up_ops = up_ops;
+	exp->fsal = fsal_hdl;
+	exp->up_ops = up_ops;
+	op_ctx->fsal_export = exp;
 
 	status.minor = resolve_posix_filesystem(op_ctx->ctx_export->fullpath,
-						fsal_hdl, &myself->export,
+						fsal_hdl, exp,
 						gpfs_claim_filesystem,
 						gpfs_unclaim_filesystem,
-						&myself->root_fs);
+						&gpfs_exp->root_fs);
 
 	if (status.minor != 0) {
 		LogCrit(COMPONENT_FSAL,
@@ -798,24 +827,16 @@ fsal_status_t gpfs_create_export(struct fsal_module *fsal_hdl,
 			op_ctx->ctx_export->fullpath,
 			strerror(status.minor), status.minor);
 		status.major = posix2fsal_error(status.minor);
-		goto uninit;
+		goto detach;
 	}
-	gpfs_fs = myself->root_fs->private_data;
-	gpfs_fs->root_fd = open_dir_by_path_walk(-1,
-			   op_ctx->ctx_export->fullpath, &buffxstat.buffstat);
-	varg.fd = gpfs_fs->root_fd;
-	varg.buffer = (char *)&GPFS_write_verifier;
-	rc = gpfs_ganesha(OPENHANDLE_GET_VERIFIER, &varg);
-	if (rc != 0)
-		LogCrit(COMPONENT_FSAL,
-		    "OPENHANDLE_GET_VERIFIER failed with rc = %d", rc);
 
 	/* if the nodeid has not been obtained, get it now */
 	if (!g_nodeid) {
+		struct gpfs_filesystem *gpfs_fs =
+						gpfs_exp->root_fs->private_data;
 		struct grace_period_arg gpa;
 		int nodeid;
 
-		gpfs_fs = myself->root_fs->private_data;
 		gpa.mountdirfd = gpfs_fs->root_fd;
 
 		nodeid = gpfs_ganesha(OPENHANDLE_GET_NODEID, &gpa);
@@ -826,19 +847,21 @@ fsal_status_t gpfs_create_export(struct fsal_module *fsal_hdl,
 			LogCrit(COMPONENT_FSAL,
 			    "OPENHANDLE_GET_NODEID failed rc %d", nodeid);
 	}
-	myself->pnfs_ds_enabled =
-	    myself->export.exp_ops.fs_supports(&myself->export,
-					    fso_pnfs_ds_supported);
-	myself->pnfs_mds_enabled =
-	    myself->export.exp_ops.fs_supports(&myself->export,
-					    fso_pnfs_mds_supported);
-	if (myself->pnfs_ds_enabled) {
+
+	gpfs_exp->pnfs_ds_enabled =
+	    exp->exp_ops.fs_supports(exp, fso_pnfs_ds_supported);
+
+	gpfs_exp->pnfs_mds_enabled =
+	    exp->exp_ops.fs_supports(exp, fso_pnfs_mds_supported);
+
+	if (gpfs_exp->pnfs_ds_enabled) {
 		struct fsal_pnfs_ds *pds = NULL;
 
-		status = fsal_hdl->m_ops.
-			fsal_pnfs_ds(fsal_hdl, parse_node, &pds);
+		status = fsal_hdl->m_ops.fsal_pnfs_ds(fsal_hdl, parse_node,
+						      &pds);
+
 		if (status.major != ERR_FSAL_NO_ERROR)
-			goto uninit;
+			goto unexport;
 
 		/* special case: server_id matches export_id */
 		pds->id_servers = op_ctx->ctx_export->export_id;
@@ -852,23 +875,25 @@ fsal_status_t gpfs_create_export(struct fsal_module *fsal_hdl,
 			status.major = ERR_FSAL_EXIST;
 			fsal_pnfs_ds_fini(pds);
 			gsh_free(pds);
-			goto uninit;
+			goto unexport;
 		}
 
 		LogInfo(COMPONENT_FSAL,
 			"gpfs_fsal_create: pnfs ds was enabled for [%s]",
 			op_ctx->ctx_export->fullpath);
-		export_ops_pnfs(&myself->export.exp_ops);
+		export_ops_pnfs(&exp->exp_ops);
 	}
-	myself->use_acl = !op_ctx_export_has_option(EXPORT_OPTION_DISABLE_ACL);
+	gpfs_exp->use_acl =
+		!op_ctx_export_has_option(EXPORT_OPTION_DISABLE_ACL);
 
 	return status;
 
-uninit:
-	mdcache_export_uninit();
-	fsal_detach_export(fsal_hdl, &myself->export.exports);
-errout:
-	free_export_ops(&myself->export);
-	gsh_free(myself);		/* elvis has left the building */
+unexport:
+	gpfs_unexport_filesystems(gpfs_exp);
+detach:
+	fsal_detach_export(fsal_hdl, &exp->exports);
+free:
+	free_export_ops(exp);
+	gsh_free(gpfs_exp);
 	return status;
 }
