@@ -43,6 +43,8 @@
 #include <fcntl.h>
 #include "gpfs_methods.h"
 
+#define STATE2FD(s) (&container_of(s, struct gpfs_state_fd, state)->gpfs_fd)
+
 static fsal_status_t
 gpfs_open_func(struct fsal_obj_handle *obj_hdl, fsal_openflags_t openflags,
 		struct fsal_fd *fd)
@@ -71,6 +73,7 @@ gpfs_close_func(struct fsal_obj_handle *obj_hdl, struct fsal_fd *fd)
 
 	status = fsal_internal_close(my_fd->fd, NULL, 0);
 	my_fd->fd = -1;
+	my_fd->openflags = FSAL_O_CLOSED;
 
 	return status;
 }
@@ -122,22 +125,6 @@ fsal_status_t gpfs_merge(struct fsal_obj_handle *orig_hdl,
 	return status;
 }
 
-/**
- * @brief get GPFS status
- *
- * @param obj_hdl FSAL object handle
- * @return FSAL status
- *
- * Let the caller peek into the file's open/close state.
- */
-fsal_openflags_t gpfs_status(struct fsal_obj_handle *obj_hdl)
-{
-	struct gpfs_fsal_obj_handle *myself =
-		container_of(obj_hdl, struct gpfs_fsal_obj_handle, obj_handle);
-
-	return myself->u.file.fd.openflags;
-}
-
 static fsal_status_t
 open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	       fsal_openflags_t openflags, int posix_flags,
@@ -149,7 +136,8 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	struct gpfs_filesystem *gpfs_fs = obj_hdl->fs->private_data;
 	fsal_status_t status;
 	const bool truncated = (posix_flags & O_TRUNC) != 0;
-	int *fd;
+	struct gpfs_fd *my_fd;
+	int fd;
 
 	/* This can block over an I/O operation. */
 	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
@@ -158,9 +146,9 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 				obj_handle);
 
 	if (state != NULL) {
-		struct gpfs_fd *my_fd = &container_of(state,
-						      struct gpfs_state_fd,
-						      state)->gpfs_fd;
+		my_fd = &container_of(state,
+				      struct gpfs_state_fd,
+				      state)->gpfs_fd;
 
 	       /* Prepare to take the share reservation, but only if we
 		* are called with a valid state (if state is NULL the
@@ -180,15 +168,13 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
-		fd = &my_fd->fd;
-		my_fd->openflags = openflags;
 	} else {
 		/* We need to use the global fd to continue. */
-		fd = &gpfs_hdl->u.file.fd.fd;
-		gpfs_hdl->u.file.fd.openflags = openflags;
+		my_fd = &gpfs_hdl->u.file.fd;
 	}
 
-	status = GPFSFSAL_open(obj_hdl, op_ctx, posix_flags, fd);
+	status = GPFSFSAL_open(obj_hdl, op_ctx, posix_flags, &fd);
+
 	if (FSAL_IS_ERROR(status)) {
 		if (state == NULL)
 			goto out;
@@ -196,6 +182,17 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 			goto undo_share;
 	}
 
+	/* Close any old open file descriptor and update with the new
+	 * one. There shouldn't be any old open for state based call.
+	 */
+	if (my_fd->openflags != FSAL_O_CLOSED) {
+		assert(my_fd->fd >= 0);
+		/* assert(state == NULL); */
+		(void)fsal_internal_close(my_fd->fd, NULL, 0);
+	}
+	my_fd->fd = fd;
+	my_fd->openflags = openflags;
+
 	if (attrs_out && (createmode >= FSAL_EXCLUSIVE || truncated)) {
 		/* Refresh the attributes */
 		status = GPFSFSAL_getattrs(export, gpfs_fs, op_ctx,
@@ -212,6 +209,8 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 				status = fsalstat(posix2fsal_error(EEXIST),
 						  EEXIST);
 		}
+	} else if (attrs_out && attrs_out->request_mask & ATTR_RDATTR_ERR) {
+		attrs_out->valid_mask &= ATTR_RDATTR_ERR;
 	}
 
 	if (state == NULL) {
@@ -230,7 +229,9 @@ open_by_handle(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 		return status;
 	}
 
-	(void) fsal_internal_close(*fd, state->state_owner, 0);
+	(void) fsal_internal_close(my_fd->fd, state->state_owner, 0);
+	my_fd->fd = -1;
+	my_fd->openflags = FSAL_O_CLOSED;
 
  undo_share:
 	/* On error we need to release our share reservation
@@ -616,13 +617,19 @@ gpfs_reopen2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	fsal2posix_openflags(openflags, &posix_flags);
 
 	status = GPFSFSAL_open(obj_hdl, op_ctx, posix_flags, &my_fd);
+
 	if (!FSAL_IS_ERROR(status)) {
 		/* Close the existing file descriptor and copy the new
-		 * one over.
+		 * one over. Make sure no one is using the fd that we are
+		 * about to close!
 		 */
+		PTHREAD_RWLOCK_wrlock(&my_share_fd->fdlock);
+
 		fsal_internal_close(my_share_fd->fd, NULL, 0);
 		my_share_fd->fd = my_fd;
 		my_share_fd->openflags = openflags;
+
+		PTHREAD_RWLOCK_unlock(&my_share_fd->fdlock);
 	} else {
 		/* We had a failure on open - we need to revert the share.
 		 * This can block over an I/O operation.
@@ -644,8 +651,11 @@ find_fd(int *fd, struct fsal_obj_handle *obj_hdl, bool bypass,
 {
 	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
 	struct gpfs_fsal_obj_handle *myself;
-	struct gpfs_fd temp_fd = {0, -1}, *out_fd = &temp_fd;
+	struct gpfs_fd temp_fd = {
+			FSAL_O_CLOSED, PTHREAD_RWLOCK_INITIALIZER, -1 };
+	struct gpfs_fd *out_fd = &temp_fd;
 	int posix_flags;
+	bool reusing_open_state_fd = false;
 
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle, obj_handle);
 
@@ -661,7 +671,8 @@ find_fd(int *fd, struct fsal_obj_handle *obj_hdl, bool bypass,
 				      &myself->u.file.share,
 				      bypass, state, openflags,
 				      gpfs_open_func, gpfs_close_func,
-				      has_lock, closefd, open_for_locks);
+				      has_lock, closefd, open_for_locks,
+				      &reusing_open_state_fd);
 
 		*fd = out_fd->fd;
 		return status;
@@ -748,6 +759,7 @@ gpfs_read2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
 					struct gpfs_fsal_export, export);
 	int export_fd = exp->export_fd;
+	struct gpfs_fd *gpfs_fd = NULL;
 
 	if (obj_hdl->fsal != obj_hdl->fs->fsal) {
 		LogDebug(COMPONENT_FSAL,
@@ -756,6 +768,14 @@ gpfs_read2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
 	}
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		gpfs_fd = STATE2FD(state);
+		PTHREAD_RWLOCK_rdlock(&gpfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
 	status = find_fd(&my_fd, obj_hdl, bypass, state, FSAL_O_READ,
 			 &has_lock, &closefd, false);
@@ -763,6 +783,8 @@ gpfs_read2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 	if (FSAL_IS_ERROR(status)) {
 		LogDebug(COMPONENT_FSAL,
 			 "find_fd failed %s", msg_fsal_err(status.major));
+		if (gpfs_fd)
+			PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
 		return status;
 	}
 
@@ -775,6 +797,9 @@ gpfs_read2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 					read_amount, end_of_file,
 					export_fd);
 
+	if (gpfs_fd)
+		PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
+
 	if (closefd) {
 		fsal_status_t status2;
 
@@ -830,6 +855,7 @@ gpfs_write2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
 					struct gpfs_fsal_export, export);
 	int export_fd = exp->export_fd;
+	struct gpfs_fd *gpfs_fd = NULL;
 
 	if (obj_hdl->fsal != obj_hdl->fs->fsal) {
 		LogDebug(COMPONENT_FSAL,
@@ -837,6 +863,15 @@ gpfs_write2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 			 obj_hdl->fsal->name, obj_hdl->fs->fsal->name);
 		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
 	}
+
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		gpfs_fd = STATE2FD(state);
+		PTHREAD_RWLOCK_rdlock(&gpfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
 	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
 			 &has_lock, &closefd, false);
@@ -844,6 +879,8 @@ gpfs_write2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 	if (FSAL_IS_ERROR(status)) {
 		LogDebug(COMPONENT_FSAL,
 			 "find_fd failed %s", msg_fsal_err(status.major));
+		if (gpfs_fd)
+			PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
 		return status;
 	}
 
@@ -856,6 +893,9 @@ gpfs_write2(struct fsal_obj_handle *obj_hdl, bool bypass, struct state_t *state,
 				wrote_amount, fsal_stable, op_ctx,
 				export_fd);
 
+	if (gpfs_fd)
+		PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
+
 	if (closefd) {
 		fsal_status_t status2;
 
@@ -923,7 +963,9 @@ gpfs_commit2(struct fsal_obj_handle *obj_hdl, off_t offset, size_t len)
 {
 	fsal_status_t status;
 	struct gpfs_fsal_obj_handle *myself;
-	struct gpfs_fd temp_fd = {0, -1}, *out_fd = &temp_fd;
+	struct gpfs_fd temp_fd = {
+			FSAL_O_CLOSED, PTHREAD_RWLOCK_INITIALIZER, -1 };
+	struct gpfs_fd *out_fd = &temp_fd;
 	bool has_lock = false;
 	bool closefd = false;
 
@@ -993,6 +1035,7 @@ gpfs_lock_op2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
 					struct gpfs_fsal_export, export);
 	int export_fd = exp->export_fd;
+	struct gpfs_fd *gpfs_fd = NULL;
 
 	LogFullDebug(COMPONENT_FSAL,
 		     "Locking: op:%d sle_type:%d type:%d start:%llu length:%llu owner:%p",
@@ -1077,6 +1120,14 @@ gpfs_lock_op2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 		return fsalstat(ERR_FSAL_NOTSUPP, 0);
 	}
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		gpfs_fd = STATE2FD(state);
+		PTHREAD_RWLOCK_rdlock(&gpfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
 	status = find_fd(&glock_args.lfd, obj_hdl, bypass, state,
 			 openflags, &has_lock, &closefd, true);
@@ -1084,6 +1135,8 @@ gpfs_lock_op2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	if (FSAL_IS_ERROR(status)) {
 		LogDebug(COMPONENT_FSAL,
 			 "find_fd failed %s", msg_fsal_err(status.major));
+		if (gpfs_fd)
+			PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
 		return status;
 	}
 
@@ -1099,6 +1152,9 @@ gpfs_lock_op2(struct fsal_obj_handle *obj_hdl, struct state_t *state,
 	status = GPFSFSAL_lock_op(export, lock_op, req_lock, conflicting_lock,
 				  &gpfs_sg_arg);
 
+	if (gpfs_fd)
+		PTHREAD_RWLOCK_unlock(&gpfs_fd->fdlock);
+
 	if (closefd) {
 		fsal_status_t status2;
 
@@ -1281,7 +1337,9 @@ gpfs_close2(struct fsal_obj_handle *obj_hdl, struct state_t *state)
 			     "state %p fd %d", state, my_fd->fd);
 		state_owner = state->state_owner;
 
-		return fsal_internal_close(my_fd->fd, state_owner, 0);
+		status = fsal_internal_close(my_fd->fd, state_owner, 0);
+		my_fd->fd = -1;
+		my_fd->openflags = FSAL_O_CLOSED;
 	}
 	return status;
 }
