@@ -62,6 +62,9 @@ GPFSFSAL_lookup(const struct req_op_context *op_ctx,
 	struct gpfs_fsal_obj_handle *parent_hdl;
 	struct gpfs_filesystem *gpfs_fs;
 	struct fsal_fsid__ fsid;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	if (!parent || !filename)
 		return fsalstat(ERR_FSAL_FAULT, 0);
@@ -71,12 +74,9 @@ GPFSFSAL_lookup(const struct req_op_context *op_ctx,
 	parent_hdl =
 	    container_of(parent, struct gpfs_fsal_obj_handle, obj_handle);
 	gpfs_fs = parent->fs->private_data;
+	status = fsal_internal_handle2fd(export_fd, parent_hdl->handle,
+					 &parent_fd, O_RDONLY);
 
-	status = fsal_internal_handle2fd_at(gpfs_fs->root_fd,
-					    parent_hdl->handle,
-					    &parent_fd,
-					    O_RDONLY,
-					    0);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
@@ -98,14 +98,14 @@ GPFSFSAL_lookup(const struct req_op_context *op_ctx,
 	}
 
 	status = fsal_internal_get_handle_at(parent_fd, filename, fh,
-					     gpfs_fs->root_fd, NULL);
+					     export_fd);
 	if (FSAL_IS_ERROR(status)) {
 		close(parent_fd);
 		return status;
 	}
 
 	/* In order to check XDEV, we need to get the fsid from the handle.
-	 * We need to do this before getting attributes in order to have tthe
+	 * We need to do this before getting attributes in order to have the
 	 * correct gpfs_fs to pass to GPFSFSAL_getattrs. We also return
 	 * the correct fs to the caller.
 	 */
