@@ -152,56 +152,6 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	return fsalstat(fsal_error, retval);
 }
 
-/* create
- * create a regular file and set its attributes
- */
-fsal_status_t create(struct fsal_obj_handle *dir_hdl,
-		     const char *name, struct attrlist *attr_in,
-		     struct fsal_obj_handle **handle,
-		     struct attrlist *attrs_out)
-{
-	struct gpfs_fsal_obj_handle *hdl;
-	fsal_status_t status;
-	/* Use a separate attrlist to getch the actual attributes into */
-	struct attrlist attrib;
-
-	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
-
-	*handle = NULL;		/* poison it */
-	if (!dir_hdl->obj_ops.handle_is(dir_hdl, DIRECTORY)) {
-		LogCrit(COMPONENT_FSAL,
-			"Parent handle is not a directory. hdl = 0x%p",
-			dir_hdl);
-		return fsalstat(ERR_FSAL_NOTDIR, 0);
-	}
-	memset(fh, 0, sizeof(struct gpfs_file_handle));
-	fh->handle_size = GPFS_MAX_FH_SIZE;
-
-	fsal_prepare_attrs(&attrib, ATTR_GPFS_ALLOC_HANDLE);
-
-	if (attrs_out != NULL)
-		attrib.request_mask |= attrs_out->request_mask;
-
-	status =
-	    GPFSFSAL_create(dir_hdl, name, op_ctx, attr_in->mode, fh, &attrib);
-	if (FSAL_IS_ERROR(status))
-		return status;
-
-	/* allocate an obj_handle and fill it up */
-	hdl = alloc_handle(fh, dir_hdl->fs, &attrib, NULL, op_ctx->fsal_export);
-
-	if (attrs_out != NULL) {
-		/* Copy the attributes to caller, passing ACL ref. */
-		fsal_copy_attrs(attrs_out, &attrib, true);
-	} else {
-		/* Done with the attrs */
-		fsal_release_attrs(&attrib);
-	}
-
-	*handle = &hdl->obj_handle;
-	return fsalstat(ERR_FSAL_NO_ERROR, 0);
-}
-
 static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 			     const char *name, struct attrlist *attr_in,
 			     struct fsal_obj_handle **handle,
@@ -272,7 +222,6 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 
 static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 			      const char *name, object_file_type_t nodetype,
-			      fsal_dev_t *dev,
 			      struct attrlist *attr_in,
 			      struct fsal_obj_handle **handle,
 			      struct attrlist *attrs_out)
@@ -300,8 +249,8 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 		attrib.request_mask |= attrs_out->request_mask;
 
 	status =
-	    GPFSFSAL_mknode(dir_hdl, name, op_ctx, attr_in->mode, nodetype, dev,
-			    fh, &attrib);
+	    GPFSFSAL_mknode(dir_hdl, name, op_ctx, attr_in->mode, nodetype,
+			    &attr_in->rawdev, fh, &attrib);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
@@ -421,52 +370,39 @@ static fsal_status_t readsymlink(struct fsal_obj_handle *obj_hdl,
 				 struct gsh_buffdesc *link_content,
 				 bool refresh)
 {
-	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
-	int retval = 0;
 	struct gpfs_fsal_obj_handle *myself = NULL;
 	fsal_status_t status;
 
-	if (obj_hdl->type != SYMBOLIC_LINK) {
-		fsal_error = ERR_FSAL_FAULT;
-		goto out;
-	}
+	if (obj_hdl->type != SYMBOLIC_LINK)
+		return fsalstat(ERR_FSAL_FAULT, 0);
+
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle, obj_handle);
 	if (refresh) {		/* lazy load or LRU'd storage */
-		size_t retlink;
 		char link_buff[PATH_MAX];
 
-		retlink = PATH_MAX - 1;
-
 		if (myself->u.symlink.link_content != NULL) {
 			gsh_free(myself->u.symlink.link_content);
 			myself->u.symlink.link_content = NULL;
 			myself->u.symlink.link_size = 0;
 		}
 
-		status =
-		    GPFSFSAL_readlink(obj_hdl, op_ctx, link_buff, &retlink);
+		status = GPFSFSAL_readlink(obj_hdl, op_ctx, link_buff,
+					   sizeof(link_buff));
+
 		if (FSAL_IS_ERROR(status))
 			return status;
 
-		myself->u.symlink.link_content = gsh_malloc(retlink + 1);
+		myself->u.symlink.link_content = gsh_strdup(link_buff);
+		myself->u.symlink.link_size = strlen(link_buff) + 1;
+	}
+
+	if (myself->u.symlink.link_content == NULL)
+		return fsalstat(ERR_FSAL_FAULT, 0);
 
-		memcpy(myself->u.symlink.link_content, link_buff, retlink);
-		myself->u.symlink.link_content[retlink] = '\0';
-		myself->u.symlink.link_size = retlink + 1;
-	}
-	if (myself->u.symlink.link_content == NULL) {
-		fsal_error = ERR_FSAL_FAULT;	/* probably a better error?? */
-		goto out;
-	}
 	link_content->len = myself->u.symlink.link_size;
-	link_content->addr = gsh_malloc(link_content->len);
+	link_content->addr = gsh_strdup(myself->u.symlink.link_content);
 
-	memcpy(link_content->addr, myself->u.symlink.link_content,
-	       link_content->len);
-
- out:
-
-	return fsalstat(fsal_error, retval);
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
 static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
@@ -508,17 +444,18 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	int bpos, cnt, nread;
 	struct dirent64 *dentry;
 	char buf[BUF_SIZE];
-	struct gpfs_filesystem *gpfs_fs;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	if (whence != NULL)
 		seekloc = (off_t) *whence;
 
 	myself = container_of(dir_hdl, struct gpfs_fsal_obj_handle, obj_handle);
-	gpfs_fs = dir_hdl->fs->private_data;
+	status = fsal_internal_handle2fd(export_fd, myself->handle,
+					 &dirfd, O_RDONLY | O_DIRECTORY);
 
-	status = fsal_internal_handle2fd_at(gpfs_fs->root_fd, myself->handle,
-					    &dirfd, O_RDONLY | O_DIRECTORY, 0);
-	if (dirfd < 0)
+	if (FSAL_IS_ERROR(status))
 		return status;
 
 	seekloc = lseek(dirfd, seekloc, SEEK_SET);
@@ -540,7 +477,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		for (bpos = 0; bpos < nread;) {
 			struct fsal_obj_handle *hdl;
 			struct attrlist attrs;
-			bool cb_rc;
+			enum fsal_dir_result cb_rc;
 
 			dentry = (struct dirent64 *)(buf + bpos);
 			if (strcmp(dentry->d_name, ".") == 0
@@ -561,7 +498,8 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 
 			fsal_release_attrs(&attrs);
 
-			if (!cb_rc)
+			/* Read ahead not supported by this FSAL. */
+			if (cb_rc >= DIR_READAHEAD)
 				goto done;
  skip:
 			bpos += dentry->d_reclen;
@@ -619,12 +557,14 @@ static fsal_status_t getxattrs(struct fsal_obj_handle *obj_hdl,
 	int errsv;
 	struct getxattr_arg gxarg;
 	struct gpfs_fsal_obj_handle *myself;
-	struct gpfs_filesystem *gpfs_fs = obj_hdl->fs->private_data;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle,
 				obj_handle);
 
-	gxarg.mountdirfd = gpfs_fs->root_fd;
+	gxarg.mountdirfd = export_fd;
 	gxarg.handle = myself->handle;
 	gxarg.name_len = xa_name->utf8string_len;
 	gxarg.name = xa_name->utf8string_val;
@@ -660,12 +600,14 @@ static fsal_status_t setxattrs(struct fsal_obj_handle *obj_hdl,
 	int errsv;
 	struct setxattr_arg sxarg;
 	struct gpfs_fsal_obj_handle *myself;
-	struct gpfs_filesystem *gpfs_fs = obj_hdl->fs->private_data;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle,
 				obj_handle);
 
-	sxarg.mountdirfd = gpfs_fs->root_fd;
+	sxarg.mountdirfd = export_fd;
 	sxarg.handle = myself->handle;
 	sxarg.name_len = xa_name->utf8string_len;
 	sxarg.name = xa_name->utf8string_val;
@@ -690,12 +632,14 @@ static fsal_status_t removexattrs(struct fsal_obj_handle *obj_hdl,
 	int errsv;
 	struct removexattr_arg rxarg;
 	struct gpfs_fsal_obj_handle *myself;
-	struct gpfs_filesystem *gpfs_fs = obj_hdl->fs->private_data;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle,
 				obj_handle);
 
-	rxarg.mountdirfd = gpfs_fs->root_fd;
+	rxarg.mountdirfd = export_fd;
 	rxarg.handle = myself->handle;
 	rxarg.name_len = xa_name->utf8string_len;
 	rxarg.name = xa_name->utf8string_val;
@@ -725,8 +669,10 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	char *buf = NULL;
 	struct listxattr_arg lxarg;
 	struct gpfs_fsal_obj_handle *myself;
-	struct gpfs_filesystem *gpfs_fs = obj_hdl->fs->private_data;
 	component4 *entry = lr_names->entries;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	val = (char *)entry + la_maxcount;
 	valstart = val;
@@ -736,7 +682,7 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	#define MAXCOUNT (1024*64)
 	buf = gsh_malloc(MAXCOUNT);
 
-	lxarg.mountdirfd = gpfs_fs->root_fd;
+	lxarg.mountdirfd = export_fd;
 	lxarg.handle = myself->handle;
 	lxarg.cookie = 0; /* For now gpfs doesn't support cookie */
 	lxarg.verifier = *((uint64_t *)la_cookieverf);
@@ -830,19 +776,6 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/*
- * NOTE: this is done under protection of the attributes rwlock in cache entry.
- */
-static fsal_status_t setattrs(struct fsal_obj_handle *obj_hdl,
-			      struct attrlist *attrs)
-{
-	fsal_status_t status;
-
-	status = GPFSFSAL_setattrs(obj_hdl, op_ctx, attrs);
-
-	return status;
-}
-
 /*
  * NOTE: this is done under protection of the attributes rwlock in cache entry.
  */
@@ -872,13 +805,13 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 	return status;
 }
 
-/* handle_digest
+/* handle_to_wire
  * fill in the opaque f/s file handle part.
  * we zero the buffer to length first.  This MAY already be done above
  * at which point, remove memset here because the caller is zeroing
  * the whole struct.
  */
-static fsal_status_t handle_digest(const struct fsal_obj_handle *obj_hdl,
+static fsal_status_t handle_to_wire(const struct fsal_obj_handle *obj_hdl,
 				   fsal_digesttype_t output_type,
 				   struct gsh_buffdesc *fh_desc)
 {
@@ -942,41 +875,32 @@ static void handle_to_key(struct fsal_obj_handle *obj_hdl,
 static void release(struct fsal_obj_handle *obj_hdl)
 {
 	struct gpfs_fsal_obj_handle *myself;
-	object_file_type_t type = obj_hdl->type;
-
-	LogFullDebug(COMPONENT_FSAL, "type %d", type);
-	if (type == REGULAR_FILE)
-		gpfs_close(obj_hdl);
+	const object_file_type_t type = obj_hdl->type;
 
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle, obj_handle);
 
+	LogFullDebug(COMPONENT_FSAL, "type %d", type);
+	if (type == REGULAR_FILE) {
+		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+
+		if (myself->u.file.fd.fd >= 0 &&
+		    myself->u.file.fd.openflags != FSAL_O_CLOSED) {
+			fsal_internal_close(myself->u.file.fd.fd, NULL, 0);
+			myself->u.file.fd.fd = -1;
+			myself->u.file.fd.openflags = FSAL_O_CLOSED;
+		}
+
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+	}
+
 	fsal_obj_handle_fini(obj_hdl);
 
 	if (type == SYMBOLIC_LINK) {
-		if (myself->u.symlink.link_content != NULL)
-			gsh_free(myself->u.symlink.link_content);
+		gsh_free(myself->u.symlink.link_content);
 	}
 	gsh_free(myself);
 }
 
-/* gpfs_share_op
- */
-static fsal_status_t share_op(struct fsal_obj_handle *obj_hdl,
-			      void *p_owner,
-			      fsal_share_param_t request_share)
-{
-	fsal_status_t status;
-	int fd, mntfd;
-	struct gpfs_fsal_obj_handle *myself;
-
-	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle, obj_handle);
-	mntfd = fd = myself->u.file.fd.fd;
-
-	status = GPFSFSAL_share_op(mntfd, fd, p_owner, request_share);
-
-	return status;
-}
-
 /* gpfs_fs_locations
  */
 static fsal_status_t gpfs_fs_locations(struct fsal_obj_handle *obj_hdl,
@@ -1005,31 +929,19 @@ void gpfs_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->release = release;
 	ops->lookup = lookup;
 	ops->readdir = read_dirents;
-	ops->create = create;
 	ops->mkdir = makedir;
 	ops->mknode = makenode;
 	ops->symlink = makesymlink;
 	ops->readlink = readsymlink;
 	ops->getattrs = getattrs;
-	ops->setattrs = setattrs;
 	ops->link = linkfile;
 	ops->rename = renamefile;
 	ops->unlink = file_unlink;
-	ops->open = gpfs_open;
-	ops->reopen = gpfs_reopen;
 	ops->fs_locations = gpfs_fs_locations;
-	ops->status = gpfs_status;
-	ops->read = gpfs_read;
-	ops->read_plus = gpfs_read_plus;
-	ops->write = gpfs_write;
-	ops->write_plus = gpfs_write_plus;
 	ops->seek = gpfs_seek;
 	ops->io_advise = gpfs_io_advise;
-	ops->commit = gpfs_commit;
-	ops->lock_op = gpfs_lock_op;
-	ops->share_op = share_op;
 	ops->close = gpfs_close;
-	ops->handle_digest = handle_digest;
+	ops->handle_to_wire = handle_to_wire;
 	ops->handle_to_key = handle_to_key;
 	handle_ops_pnfs(ops);
 	ops->getxattrs = getxattrs;
@@ -1060,11 +972,9 @@ fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 			       struct fsal_obj_handle **handle,
 			       struct attrlist *attrs_out)
 {
-	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
 	fsal_status_t fsal_status;
 	int retval = 0;
 	int dir_fd;
-	int exp_fd = 0;
 	struct fsal_filesystem *fs;
 	struct gpfs_fsal_obj_handle *hdl;
 	struct attrlist attributes;
@@ -1072,6 +982,10 @@ fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
 	struct fsal_fsid__ fsid;
 	struct gpfs_fsal_export *gpfs_export;
+	gpfs_acl_t *acl_buf;
+	unsigned int acl_buflen;
+	bool use_acl;
+	int retry;
 
 	memset(fh, 0, sizeof(struct gpfs_file_handle));
 	fh->handle_size = GPFS_MAX_FH_SIZE;
@@ -1087,32 +1001,67 @@ fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 
 	if (dir_fd < 0) {
 		LogCrit(COMPONENT_FSAL,
-			"Could not open directory for path %s",
-			path);
-		retval = -dir_fd;
+			"Could not open directory for path %s", path);
+		fsal_status = fsalstat(posix2fsal_error(-dir_fd), retval);
 		goto errout;
 	}
 
-	fsal_status = fsal_internal_fd2handle(dir_fd, fh, &exp_fd);
+	fsal_status = fsal_internal_fd2handle(dir_fd, fh);
 	if (FSAL_IS_ERROR(fsal_status))
 		goto fileerr;
 
 	gpfs_export = container_of(exp_hdl, struct gpfs_fsal_export, export);
 
-	fsal_status = fsal_get_xstat_by_handle(dir_fd, fh, &buffxstat,
-					       NULL, false,
-					       (attributes.request_mask &
-							ATTR_ACL) != 0);
+	/* Let us make the first request using the acl buffer that is
+	 * part of buffxstat itself. If that is not sufficient, we
+	 * allocate from heap and retry.
+	 */
+	use_acl = attributes.request_mask & ATTR_ACL;
+	for (retry = 0; retry < GPFS_ACL_MAX_RETRY; retry++) {
+		switch (retry) {
+		case 0: /* first attempt */
+			acl_buf = (gpfs_acl_t *)buffxstat.buffacl;
+			acl_buflen = GPFS_ACL_BUF_SIZE;
+			break;
+		case 1: /* first retry, don't free the old stack buffer */
+			acl_buflen = acl_buf->acl_len;
+			acl_buf = gsh_malloc(acl_buflen);
+			break;
+		default: /* second or later retry, free the old heap buffer */
+			acl_buflen = acl_buf->acl_len;
+			gsh_free(acl_buf);
+			acl_buf = gsh_malloc(acl_buflen);
+			break;
+		}
+
+		fsal_status = fsal_get_xstat_by_handle(dir_fd, fh, &buffxstat,
+				acl_buf, acl_buflen, NULL, false, use_acl);
+		if (FSAL_IS_ERROR(fsal_status) || !use_acl ||
+				acl_buflen >= acl_buf->acl_len)
+			break;
+	}
+
+	if (retry == GPFS_ACL_MAX_RETRY) { /* make up an error */
+		LogCrit(COMPONENT_FSAL, "unable to get ACLs");
+		fsal_status = fsalstat(ERR_FSAL_SERVERFAULT, 0);
+	}
+
 	if (FSAL_IS_ERROR(fsal_status))
-		goto fileerr;
+		goto xstat_err;
+
 	fsal_status = gpfsfsal_xstat_2_fsal_attributes(&buffxstat, &attributes,
-						       gpfs_export->use_acl);
+						acl_buf, gpfs_export->use_acl);
 	LogFullDebug(COMPONENT_FSAL,
 		     "fsid=0x%016"PRIx64".0x%016"PRIx64,
-		     attributes.fsid.major,
-		     attributes.fsid.minor);
+		     attributes.fsid.major, attributes.fsid.minor);
+
 	if (FSAL_IS_ERROR(fsal_status))
-		goto fileerr;
+		goto xstat_err;
+
+	if (acl_buflen != GPFS_ACL_BUF_SIZE) {
+		assert(acl_buf != (gpfs_acl_t *)buffxstat.buffacl);
+		gsh_free(acl_buf);
+	}
 
 	close(dir_fd);
 
@@ -1124,14 +1073,14 @@ fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 		LogInfo(COMPONENT_FSAL,
 			"Could not find file system for path %s",
 			path);
-		retval = ENOENT;
+		fsal_status = fsalstat(posix2fsal_error(ENOENT), ENOENT);
 		goto errout;
 	}
 	if (fs->fsal != exp_hdl->fsal) {
 		LogInfo(COMPONENT_FSAL,
 			"File system for path %s did not belong to FSAL %s",
 			path, exp_hdl->fsal->name);
-		retval = EACCES;
+		fsal_status = fsalstat(posix2fsal_error(EACCES), EACCES);
 		goto errout;
 	}
 
@@ -1153,17 +1102,20 @@ fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 	*handle = &hdl->obj_handle;
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 
- fileerr:
-	retval = errno;
+xstat_err:
+	/* free if the acl buffer is from the heap */
+	if (acl_buflen != GPFS_ACL_BUF_SIZE) {
+		assert(acl_buf != (gpfs_acl_t *)buffxstat.buffacl);
+		gsh_free(acl_buf);
+	}
+
+fileerr:
 	close(dir_fd);
 
- errout:
-
+errout:
 	/* Done with attributes */
 	fsal_release_attrs(&attributes);
-
-	fsal_error = posix2fsal_error(retval);
-	return fsalstat(fsal_error, retval);
+	return fsal_status;
 }
 
 /**
@@ -1189,17 +1141,17 @@ fsal_status_t gpfs_create_handle(struct fsal_export *exp_hdl,
 				 struct fsal_obj_handle **handle,
 				 struct attrlist *attrs_out)
 {
-	int retval = 0;
 	fsal_status_t status;
 	struct gpfs_fsal_obj_handle *hdl;
 	struct gpfs_file_handle *fh;
 	struct attrlist attrib;
-	char *link_content = NULL;
-	ssize_t retlink = PATH_MAX - 1;
 	char link_buff[PATH_MAX];
 	struct fsal_fsid__ fsid;
 	struct fsal_filesystem *fs;
 	struct gpfs_filesystem *gpfs_fs;
+	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
+					struct gpfs_fsal_export, export);
+	int export_fd = exp->export_fd;
 
 	*handle = NULL;		/* poison it first */
 	if ((hdl_desc->len > (sizeof(struct gpfs_file_handle))))
@@ -1240,23 +1192,13 @@ fsal_status_t gpfs_create_handle(struct fsal_export *exp_hdl,
 		return status;
 
 	if (attrib.type == SYMBOLIC_LINK) {	/* I could lazy eval this... */
-
-		status = fsal_readlink_by_handle(gpfs_fs->root_fd, fh,
-						 link_buff, &retlink);
+		status = fsal_readlink_by_handle(export_fd, fh,
+						 link_buff, sizeof(link_buff));
 		if (FSAL_IS_ERROR(status))
 			return status;
-
-		if (retlink < 0 || retlink == PATH_MAX) {
-			retval = errno;
-			if (retlink == PATH_MAX)
-				retval = ENAMETOOLONG;
-			return fsalstat(posix2fsal_error(retval), retval);
-		}
-		link_buff[retlink] = '\0';
-		link_content = link_buff;
 	}
 
-	hdl = alloc_handle(fh, fs, &attrib, link_content, exp_hdl);
+	hdl = alloc_handle(fh, fs, &attrib, link_buff, exp_hdl);
 
 	if (attrs_out != NULL) {
 		/* Copy the attributes to caller, passing ACL ref. */
