@@ -32,7 +32,6 @@
 #include "pxy_fsal_methods.h"
 #include "nfs_exports.h"
 #include "export_mgr.h"
-#include "mdcache.h"
 
 static void pxy_release(struct fsal_export *exp_hdl)
 {
@@ -134,7 +133,7 @@ void pxy_export_ops_init(struct export_ops *ops)
 {
 	ops->release = pxy_release;
 	ops->lookup_path = pxy_lookup_path;
-	ops->extract_handle = pxy_extract_handle;
+	ops->wire_to_host = pxy_wire_to_host;
 	ops->create_handle = pxy_create_handle;
 	ops->get_fs_dynamic_info = pxy_get_dynamic_info;
 	ops->fs_supports = pxy_get_supports;
@@ -149,6 +148,8 @@ void pxy_export_ops_init(struct export_ops *ops)
 	ops->fs_supported_attrs = pxy_get_supported_attrs;
 	ops->fs_umask = pxy_get_umask;
 	ops->fs_xattr_access_rights = pxy_get_xattr_access_rights;
+	ops->alloc_state = pxy_alloc_state;
+	ops->free_state = pxy_free_state;
 };
 
 /* Here and not static because proxy.c needs this function
