@@ -37,14 +37,7 @@
 #include "FSAL/fsal_init.h"
 #include "pxy_fsal_methods.h"
 
-/* defined the set of attributes supported with POSIX */
-#define SUPPORTED_ATTRIBUTES (                                       \
-		ATTR_TYPE     | ATTR_SIZE     |			     \
-		ATTR_FSID     | ATTR_FILEID   |			     \
-		ATTR_MODE     | ATTR_NUMLINKS | ATTR_OWNER     |     \
-		ATTR_GROUP    | ATTR_ATIME    | ATTR_RAWDEV    |     \
-		ATTR_CTIME    | ATTR_MTIME    | ATTR_SPACEUSED |     \
-		ATTR_CHGTIME)
+#define PROXY_SUPPORTED_ATTRS ((const attrmask_t) (ATTRS_POSIX))
 
 /* filesystem info for PROXY */
 static struct fsal_staticfsinfo_t proxy_info = {
@@ -61,7 +54,7 @@ static struct fsal_staticfsinfo_t proxy_info = {
 	.lease_time = {10, 0},
 	.acl_support = FSAL_ACLSUPPORT_ALLOW,
 	.homogenous = true,
-	.supported_attrs = SUPPORTED_ATTRIBUTES,
+	.supported_attrs = PROXY_SUPPORTED_ATTRS,
 	.link_supports_permission_checks = true,
 };
 
@@ -74,6 +67,11 @@ static struct config_item_list sec_types[] = {
 };
 #endif
 
+/*512 bytes to store header*/
+#define SEND_RECV_HEADER_SPACE 512
+/*1MB of default maxsize*/
+#define DEFAULT_MAX_WRITE_READ 1048576
+
 static struct config_item proxy_remote_params[] = {
 	CONF_ITEM_UI32("Retry_SleepTime", 0, 60, 10,
 		       pxy_client_params, retry_sleeptime),
@@ -81,13 +79,17 @@ static struct config_item proxy_remote_params[] = {
 			  pxy_client_params, srv_addr),
 	CONF_ITEM_UI32("NFS_Service", 0, UINT32_MAX, 100003,
 		       pxy_client_params, srv_prognum),
-	CONF_ITEM_UI32("NFS_SendSize", 512, FSAL_MAXIOSIZE, 32768,
+	CONF_ITEM_UI32("NFS_SendSize", 512 + SEND_RECV_HEADER_SPACE,
+		       FSAL_MAXIOSIZE,
+		       DEFAULT_MAX_WRITE_READ + SEND_RECV_HEADER_SPACE,
 		       pxy_client_params, srv_sendsize),
-	CONF_ITEM_UI32("NFS_RecvSize", 512, FSAL_MAXIOSIZE, 32768,
+	CONF_ITEM_UI32("NFS_RecvSize", 512 + SEND_RECV_HEADER_SPACE,
+		       FSAL_MAXIOSIZE,
+		       DEFAULT_MAX_WRITE_READ + SEND_RECV_HEADER_SPACE,
 		       pxy_client_params, srv_recvsize),
-	CONF_ITEM_INET_PORT("NFS_Port", 0, UINT16_MAX, 2049,
-			    pxy_client_params, srv_port),
-	CONF_ITEM_BOOL("Use_Privileged_Client_Port", false,
+	CONF_ITEM_UI16("NFS_Port", 0, UINT16_MAX, 2049,
+		       pxy_client_params, srv_port),
+	CONF_ITEM_BOOL("Use_Privileged_Client_Port", true,
 		       pxy_client_params, use_privileged_client_port),
 	CONF_ITEM_UI32("RPC_Client_Timeout", 1, 60*4, 60,
 		       pxy_client_params, srv_timeout),
@@ -124,8 +126,21 @@ static int remote_commit(void *node, void *link_mem, void *self_struct,
 		       struct config_error_type *err_type)
 {
 	/* struct pxy_client_params *pcpi = self_struct; */
+	struct pxy_fsal_module *pxy;
 
-	/* Verifications/parameter checking to be added here */
+	pxy = container_of(link_mem, struct pxy_fsal_module, special);
+
+	if (pxy->fsinfo.maxwrite + SEND_RECV_HEADER_SPACE >
+			pxy->special.srv_sendsize ||
+	    pxy->fsinfo.maxread + SEND_RECV_HEADER_SPACE >
+			pxy->special.srv_recvsize) {
+		LogCrit(COMPONENT_CONFIG,
+"FSAL_PROXY CONF : maxwrite/maxread + header > Max_SendSize/Max_RecvSize");
+		err_type->invalid = true;
+		return 1;
+	}
+
+	/* Other verifications/parameter checking to be added here */
 
 	return 0;
 }
@@ -146,9 +161,13 @@ static struct config_item proxy_params[] = {
 		       pxy_fsal_module, fsinfo.symlink_support),
 	CONF_ITEM_BOOL("cansettime", true,
 		       pxy_fsal_module, fsinfo.cansettime),
-	CONF_ITEM_UI64("maxread", 512, FSAL_MAXIOSIZE, FSAL_MAXIOSIZE,
+	CONF_ITEM_UI64("maxread", 512,
+		       FSAL_MAXIOSIZE - SEND_RECV_HEADER_SPACE,
+		       DEFAULT_MAX_WRITE_READ,
 		       pxy_fsal_module, fsinfo.maxread),
-	CONF_ITEM_UI64("maxwrite", 512, FSAL_MAXIOSIZE, FSAL_MAXIOSIZE,
+	CONF_ITEM_UI64("maxwrite", 512,
+		       FSAL_MAXIOSIZE - SEND_RECV_HEADER_SPACE,
+		       DEFAULT_MAX_WRITE_READ,
 		       pxy_fsal_module, fsinfo.maxwrite),
 	CONF_ITEM_MODE("umask", 0,
 		       pxy_fsal_module, fsinfo.umask),
@@ -217,8 +236,17 @@ MODULE_FINI void pxy_unload(void)
 	int retval;
 
 	retval = unregister_fsal(&PROXY.module);
+
 	if (retval != 0) {
-		fprintf(stderr, "PROXY module failed to unregister");
+		fprintf(stderr, "PROXY module failed to unregister : %d",
+			retval);
+		return;
+	}
+
+	retval = pxy_close_thread();
+	if (retval != 0) {
+		fprintf(stderr, "PROXY module failed to wait threads : %d",
+			retval);
 		return;
 	}
 }
