@@ -1,5 +1,5 @@
 /*
- * vim:expandtab:shiftwidth=8:tabstop=8:
+ * vim:shiftwidth=8:tabstop=8:
  *
  * Copyright (C) Panasas Inc., 2011
  * Author: Jim Lieb jlieb@panasas.com
@@ -239,14 +239,13 @@ static fsal_status_t set_quota(struct fsal_export *exp_hdl,
 /* extract a file handle from a buffer.
  * do verification checks and flag any and all suspicious bits.
  * Return an updated fh_desc into whatever was passed.  The most
- * common behavior, done here is to just reset the length.  There
- * is the option to also adjust the start pointer.
+ * common behavior, done here is to just reset the length.
  */
 
-static fsal_status_t extract_handle(struct fsal_export *exp_hdl,
-				    fsal_digesttype_t in_type,
-				    struct gsh_buffdesc *fh_desc,
-				    int flags)
+static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
+				  fsal_digesttype_t in_type,
+				  struct gsh_buffdesc *fh_desc,
+				  int flags)
 {
 	size_t fh_min;
 	uint64_t *hashkey;
@@ -284,7 +283,7 @@ void pseudofs_export_ops_init(struct export_ops *ops)
 {
 	ops->release = release;
 	ops->lookup_path = pseudofs_lookup_path;
-	ops->extract_handle = extract_handle;
+	ops->wire_to_host = wire_to_host;
 	ops->create_handle = pseudofs_create_handle;
 	ops->get_fs_dynamic_info = get_dynamic_info;
 	ops->fs_supports = fs_supports;
@@ -317,7 +316,6 @@ fsal_status_t pseudofs_create_export(struct fsal_module *fsal_hdl,
 {
 	struct pseudofs_fsal_export *myself;
 	int retval = 0;
-	fsal_status_t status = {0, 0};
 
 	myself = gsh_calloc(1, sizeof(struct pseudofs_fsal_export));
 
@@ -350,13 +348,6 @@ fsal_status_t pseudofs_create_export(struct fsal_module *fsal_hdl,
 	myself->export_path = gsh_strdup(op_ctx->ctx_export->fullpath);
 	op_ctx->fsal_export = &myself->export;
 
-	/* Stack MDCACHE on top */
-	status = mdcache_export_init(up_ops, &myself->export.up_ops);
-	if (FSAL_IS_ERROR(status)) {
-		LogDebug(COMPONENT_FSAL, "MDCACHE creation failed for PSEUDO");
-		return status;
-	}
-
 	LogDebug(COMPONENT_FSAL,
 		 "Created exp %p - %s",
 		 myself, myself->export_path);
