@@ -78,23 +78,10 @@ pseudofs_i_cmpf(const struct avltree_node *lhs,
 }
 
 static inline struct avltree_node *
-avltree_inline_name_lookup(
-	const struct avltree_node *key,
-	const struct avltree *tree)
+avltree_inline_name_lookup(const struct avltree_node *key,
+			   const struct avltree *tree)
 {
-	struct avltree_node *node = tree->root;
-	int res = 0;
-
-	while (node) {
-		res = pseudofs_n_cmpf(node, key);
-		if (res == 0)
-			return node;
-		if (res > 0)
-			node = node->left;
-		else
-			node = node->right;
-	}
-	return NULL;
+	return avltree_inline_lookup(key, tree, pseudofs_n_cmpf);
 }
 
 /**
@@ -190,7 +177,7 @@ static struct pseudo_fsal_obj_handle
 			struct attrlist *attrs)
 {
 	struct pseudo_fsal_obj_handle *hdl;
-	char path[MAXPATHLEN];
+	char path[MAXPATHLEN] = "\0";
 	struct display_buffer pathbuf = {sizeof(path), path, path};
 	int rc;
 
@@ -275,7 +262,8 @@ static struct pseudo_fsal_obj_handle
 	hdl->attributes.rawdev.minor = 0;
 
 	/* Set the mask at the end. */
-	hdl->attributes.valid_mask = ATTRS_POSIX;
+	hdl->attributes.valid_mask = PSEUDO_SUPPORTED_ATTRS;
+	hdl->attributes.supported = PSEUDO_SUPPORTED_ATTRS;
 
 	fsal_obj_handle_init(&hdl->obj_handle, exp_hdl, DIRECTORY);
 	pseudofs_handle_ops_init(&hdl->obj_handle.obj_ops);
@@ -285,12 +273,12 @@ static struct pseudo_fsal_obj_handle
 	hdl->next_i = 2;
 	if (parent != NULL) {
 		/* Attach myself to my parent */
-		PTHREAD_RWLOCK_wrlock(&parent->obj_handle.lock);
+		PTHREAD_RWLOCK_wrlock(&parent->obj_handle.obj_lock);
 		avltree_insert(&hdl->avl_n, &parent->avl_name);
 		hdl->index = (parent->next_i)++;
 		avltree_insert(&hdl->avl_i, &parent->avl_index);
 		hdl->inavl = true;
-		PTHREAD_RWLOCK_unlock(&parent->obj_handle.lock);
+		PTHREAD_RWLOCK_unlock(&parent->obj_handle.obj_lock);
 	}
 	return hdl;
 
@@ -328,7 +316,7 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	 * this directory.
 	 */
 	if (op_ctx->fsal_private != parent)
-		PTHREAD_RWLOCK_rdlock(&parent->lock);
+		PTHREAD_RWLOCK_rdlock(&parent->obj_lock);
 	else
 		LogFullDebug(COMPONENT_FSAL,
 			     "Skipping lock for %s",
@@ -363,7 +351,7 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 out:
 
 	if (op_ctx->fsal_private != parent)
-		PTHREAD_RWLOCK_unlock(&parent->lock);
+		PTHREAD_RWLOCK_unlock(&parent->obj_lock);
 
 	if (error == ERR_FSAL_NO_ERROR && attrs_out != NULL) {
 		/* This is unlocked, however, for the most part, attributes
@@ -461,7 +449,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	struct avltree_node *node;
 	fsal_cookie_t seekloc;
 	struct attrlist attrs;
-	bool cb_rc;
+	enum fsal_dir_result cb_rc;
 
 	if (whence != NULL)
 		seekloc = *whence;
@@ -478,7 +466,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		 "hdl=%p, name=%s",
 		 myself, myself->name);
 
-	PTHREAD_RWLOCK_rdlock(&dir_hdl->lock);
+	PTHREAD_RWLOCK_rdlock(&dir_hdl->obj_lock);
 
 	/* Use fsal_private to signal to lookup that we hold
 	 * the lock.
@@ -499,11 +487,12 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		fsal_copy_attrs(&attrs, &hdl->attributes, false);
 
 		cb_rc = cb(hdl->name, &hdl->obj_handle, &attrs,
-			   dir_state, hdl->index);
+			   dir_state, hdl->index + 1);
 
 		fsal_release_attrs(&attrs);
 
-		if (!cb_rc) {
+		/* Read ahead not supported by this FSAL. */
+		if (cb_rc >= DIR_READAHEAD) {
 			*eof = false;
 			break;
 		}
@@ -511,7 +500,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 
 	op_ctx->fsal_private = NULL;
 
-	PTHREAD_RWLOCK_unlock(&dir_hdl->lock);
+	PTHREAD_RWLOCK_unlock(&dir_hdl->obj_lock);
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
@@ -565,7 +554,7 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 			      struct pseudo_fsal_obj_handle,
 			      obj_handle);
 
-	PTHREAD_RWLOCK_wrlock(&dir_hdl->lock);
+	PTHREAD_RWLOCK_wrlock(&dir_hdl->obj_lock);
 
 	/* Check if directory is empty */
 	numlinks = atomic_fetch_uint32_t(&hdl->numlinks);
@@ -591,21 +580,21 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 	error = ERR_FSAL_NO_ERROR;
 
 unlock:
-	PTHREAD_RWLOCK_unlock(&dir_hdl->lock);
+	PTHREAD_RWLOCK_unlock(&dir_hdl->obj_lock);
 
 	return fsalstat(error, 0);
 }
 
-/* handle_digest
+/* handle_to_wire
  * fill in the opaque f/s file handle part.
  * we zero the buffer to length first.  This MAY already be done above
  * at which point, remove memset here because the caller is zeroing
  * the whole struct.
  */
 
-static fsal_status_t handle_digest(const struct fsal_obj_handle *obj_hdl,
-				   fsal_digesttype_t output_type,
-				   struct gsh_buffdesc *fh_desc)
+static fsal_status_t handle_to_wire(const struct fsal_obj_handle *obj_hdl,
+				    fsal_digesttype_t output_type,
+				    struct gsh_buffdesc *fh_desc)
 {
 	const struct pseudo_fsal_obj_handle *myself;
 
@@ -696,7 +685,7 @@ void pseudofs_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->mkdir = makedir;
 	ops->getattrs = getattrs;
 	ops->unlink = file_unlink;
-	ops->handle_digest = handle_digest;
+	ops->handle_to_wire = handle_to_wire;
 	ops->handle_to_key = handle_to_key;
 }
 
