@@ -109,20 +109,18 @@ struct rgw_open_state {
 
 /**
  * The attributes this FSAL can interpret or supply.
+ * Currently FSAL_RGW uses posix2fsal_attributes, so we should indicate support
+ * for at least those attributes.
  */
-#define rgw_supported_attributes (const attrmask_t) (			\
-	ATTR_TYPE      | ATTR_SIZE     | ATTR_FSID  | ATTR_FILEID |\
-	ATTR_MODE      | ATTR_NUMLINKS | ATTR_OWNER | ATTR_GROUP  |\
-	ATTR_ATIME     | ATTR_RAWDEV   | ATTR_CTIME | ATTR_MTIME  |\
-	ATTR_SPACEUSED | ATTR_CHGTIME)
+#define RGW_SUPPORTED_ATTRIBUTES ((const attrmask_t) (ATTRS_POSIX))
 
 /**
  * The attributes this FSAL can set.
  */
-#define rgw_settable_attributes (const attrmask_t) (			\
+#define RGW_SETTABLE_ATTRIBUTES ((const attrmask_t) (			\
 	ATTR_MODE  | ATTR_OWNER | ATTR_GROUP | ATTR_ATIME	 |\
 	ATTR_CTIME | ATTR_MTIME | ATTR_SIZE  | ATTR_MTIME_SERVER |\
-	ATTR_ATIME_SERVER)
+	ATTR_ATIME_SERVER))
 
 /**
  * Linux supports a stripe pattern with no more than 4096 stripes, but
@@ -149,8 +147,8 @@ void deconstruct_handle(struct rgw_handle *obj);
 fsal_status_t rgw2fsal_error(const int errorcode);
 void export_ops_init(struct export_ops *ops);
 void handle_ops_init(struct fsal_obj_ops *ops);
-struct state_t *alloc_state(struct fsal_export *exp_hdl,
-			enum state_type state_type,
-			struct state_t *related_state);
+struct state_t *rgw_alloc_state(struct fsal_export *exp_hdl,
+				enum state_type state_type,
+				struct state_t *related_state);
 void rgw_fs_invalidate(void *handle, struct rgw_fh_hk fh_hk);
 #endif				/* !FSAL_RGW_INTERNAL_INTERNAL */
