@@ -38,7 +38,6 @@
 #include "fsal_convert.h"
 #include <unistd.h>
 #include <fcntl.h>
-#include "FSAL/fsal_commonlib.h"
 #include "vfs_methods.h"
 #include "os/subr.h"
 #include "sal_data.h"
@@ -91,6 +90,7 @@ fsal_status_t vfs_close_my_fd(struct vfs_fd *my_fd)
 	int retval = 0;
 
 	if (my_fd->fd >= 0 && my_fd->openflags != FSAL_O_CLOSED) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", my_fd->fd);
 		retval = close(my_fd->fd);
 		if (retval < 0) {
 			retval = errno;
@@ -162,14 +162,17 @@ fsal_status_t vfs_close(struct fsal_obj_handle *obj_hdl)
 		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
 	}
 
+	if (myself->u.file.fd.openflags == FSAL_O_CLOSED)
+		return fsalstat(ERR_FSAL_NOT_OPENED, 0);
+
 	/* Take write lock on object to protect file descriptor.
 	 * This can block over an I/O operation.
 	 */
-	PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
+	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
 	status = vfs_close_my_fd(&myself->u.file.fd);
 
-	PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	return status;
 }
@@ -194,17 +197,36 @@ struct state_t *vfs_alloc_state(struct fsal_export *exp_hdl,
 	struct state_t *state;
 	struct vfs_fd *my_fd;
 
-	state = init_state(gsh_calloc(1, sizeof(struct state_t)
-					 + sizeof(struct vfs_fd)),
+	state = init_state(gsh_calloc(1, sizeof(struct vfs_state_fd)),
 			   exp_hdl, state_type, related_state);
 
-	my_fd = (struct vfs_fd *)(state + 1);
+	my_fd = &container_of(state, struct vfs_state_fd, state)->vfs_fd;
 
 	my_fd->fd = -1;
+	my_fd->openflags = FSAL_O_CLOSED;
+	PTHREAD_RWLOCK_init(&my_fd->fdlock, NULL);
 
 	return state;
 }
 
+/**
+ * @brief free a vfs_state_fd structure
+ *
+ * @param[in] exp_hdl  Export state_t will be associated with
+ * @param[in] state    Related state if appropriate
+ *
+ */
+void vfs_free_state(struct fsal_export *exp_hdl, struct state_t *state)
+{
+	struct vfs_state_fd *state_fd = container_of(state, struct vfs_state_fd,
+						     state);
+	struct vfs_fd *my_fd = &state_fd->vfs_fd;
+
+	PTHREAD_RWLOCK_destroy(&my_fd->fdlock);
+
+	gsh_free(state_fd);
+}
+
 /**
  * @brief Merge a duplicate handle with an original handle
  *
@@ -242,16 +264,247 @@ fsal_status_t vfs_merge(struct fsal_obj_handle *orig_hdl,
 				    obj_handle);
 
 		/* This can block over an I/O operation. */
-		PTHREAD_RWLOCK_wrlock(&orig_hdl->lock);
+		PTHREAD_RWLOCK_wrlock(&orig_hdl->obj_lock);
 
 		status = merge_share(&orig->u.file.share, &dupe->u.file.share);
 
-		PTHREAD_RWLOCK_unlock(&orig_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&orig_hdl->obj_lock);
 	}
 
 	return status;
 }
 
+static fsal_status_t fetch_attrs(struct vfs_fsal_obj_handle *myself,
+				 int my_fd, struct attrlist *attrs)
+{
+	struct stat stat;
+	int retval = 0;
+	fsal_status_t status = {0, 0};
+	const char *func = "unknown";
+
+	/* Now stat the file as appropriate */
+	switch (myself->obj_handle.type) {
+	case SOCKET_FILE:
+	case CHARACTER_FILE:
+	case BLOCK_FILE:
+		retval = fstatat(my_fd, myself->u.unopenable.name, &stat,
+				 AT_SYMLINK_NOFOLLOW);
+		func = "fstatat";
+		break;
+
+	case REGULAR_FILE:
+		retval = fstat(my_fd, &stat);
+		func = "fstat";
+		break;
+
+	case SYMBOLIC_LINK:
+	case FIFO_FILE:
+	case DIRECTORY:
+		retval = vfs_stat_by_handle(my_fd, &stat);
+		func = "vfs_stat_by_handle";
+		break;
+
+	case NO_FILE_TYPE:
+	case EXTENDED_ATTR:
+		/* Caught during open with EINVAL */
+		break;
+	}
+
+	if (retval < 0) {
+		if (errno == ENOENT)
+			retval = ESTALE;
+		else
+			retval = errno;
+
+		LogDebug(COMPONENT_FSAL, "%s failed with %s", func,
+			 strerror(retval));
+
+		if (attrs->request_mask & ATTR_RDATTR_ERR) {
+			/* Caller asked for error to be visible. */
+			attrs->valid_mask = ATTR_RDATTR_ERR;
+		}
+
+		return fsalstat(posix2fsal_error(retval), retval);
+	}
+
+	posix2fsal_attributes_all(&stat, attrs);
+	attrs->fsid = myself->obj_handle.fs->fsid;
+
+	if (myself->sub_ops && myself->sub_ops->getattrs) {
+		status =
+		   myself->sub_ops->getattrs(myself, my_fd, attrs->request_mask,
+					     attrs);
+
+		if (FSAL_IS_ERROR(status) &&
+		    (attrs->request_mask & ATTR_RDATTR_ERR) != 0) {
+			/* Caller asked for error to be visible. */
+			attrs->valid_mask = ATTR_RDATTR_ERR;
+		}
+	}
+
+	return status;
+}
+
+static fsal_status_t vfs_open2_by_handle(struct fsal_obj_handle *obj_hdl,
+					 struct state_t *state,
+					 fsal_openflags_t openflags,
+					 enum fsal_create_mode createmode,
+					 struct attrlist *attrib_set,
+					 fsal_verifier_t verifier,
+					 struct attrlist *attrs_out,
+					 bool *caller_perm_check)
+{
+	fsal_status_t status = {0, 0};
+	struct vfs_fd *my_fd = NULL;
+	int posix_flags = 0;
+	bool truncated;
+	struct vfs_fsal_obj_handle *myself = container_of(obj_hdl, struct
+					  vfs_fsal_obj_handle, obj_handle);
+
+	if (state != NULL)
+		my_fd = &container_of(state, struct vfs_state_fd,
+				      state)->vfs_fd;
+
+	fsal2posix_openflags(openflags, &posix_flags);
+	truncated = (posix_flags & O_TRUNC) != 0;
+
+	LogFullDebug(COMPONENT_FSAL,
+		     truncated ? "Truncate" : "No truncate");
+
+	/* This is an open by handle */
+	if (obj_hdl->fsal != obj_hdl->fs->fsal) {
+		LogDebug(COMPONENT_FSAL,
+			 "FSAL %s operation for handle belonging to FSAL %s, return EXDEV",
+			 obj_hdl->fsal->name, obj_hdl->fs->fsal->name);
+		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
+	}
+
+	if (state != NULL) {
+		/* Prepare to take the share reservation, but only if we are
+		 * called with a valid state (if state is NULL the caller is a
+		 * stateless create such as NFS v3 CREATE).
+		 */
+
+		/* This can block over an I/O operation. */
+		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+
+		/* Check share reservation conflicts. */
+		status = check_share_conflict(&myself->u.file.share,
+					      openflags,
+					      false);
+
+		if (FSAL_IS_ERROR(status)) {
+			PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+			return status;
+		}
+
+		/* Take the share reservation now by updating the
+		 * counters.
+		 */
+		update_share_counters(&myself->u.file.share,
+				      FSAL_O_CLOSED,
+				      openflags);
+
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+	} else {
+		/* We need to use the global fd to continue, and take
+		 * the lock to protect it.
+		 */
+		my_fd = &myself->u.file.fd;
+		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+	}
+
+	/* Close if there was any proir fd */
+	if (my_fd->openflags != FSAL_O_CLOSED) {
+		vfs_close_my_fd(my_fd);
+	}
+	status = vfs_open_my_fd(myself, openflags, posix_flags, my_fd);
+
+	if (FSAL_IS_ERROR(status)) {
+		if (state == NULL) {
+			/* Release the lock taken above, and return
+			 * since there is nothing to undo.
+			 */
+			PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+			return status;
+		} else {
+			/* Error - need to release the share */
+			goto undo_share;
+		}
+	}
+
+	if (createmode >= FSAL_EXCLUSIVE || truncated) {
+		/* Refresh the attributes */
+		struct attrlist attrs;
+		attrmask_t attrs_mask = ATTR_ATIME | ATTR_MTIME;
+
+		if (attrs_out)
+			attrs_mask |= attrs_out->request_mask;
+
+		fsal_prepare_attrs(&attrs, attrs_mask);
+
+		status = fetch_attrs(myself, my_fd->fd, &attrs);
+		if (FSAL_IS_SUCCESS(status)) {
+			LogFullDebug(COMPONENT_FSAL,
+				     "New size = %" PRIx64,
+				     attrs.filesize);
+
+			if (createmode >= FSAL_EXCLUSIVE &&
+			    createmode != FSAL_EXCLUSIVE_9P &&
+			    !check_verifier_attrlist(&attrs, verifier)) {
+				/* Verifier didn't match, return EEXIST */
+				status = fsalstat(posix2fsal_error(EEXIST),
+						  EEXIST);
+			} else if (attrs_out) {
+				fsal_copy_attrs(attrs_out, &attrs, true);
+			}
+		}
+
+		fsal_release_attrs(&attrs);
+	} else if (attrs_out && attrs_out->request_mask & ATTR_RDATTR_ERR) {
+		attrs_out->valid_mask &= ATTR_RDATTR_ERR;
+	}
+
+	if (state == NULL) {
+		/* If no state, release the lock taken above and return
+		 * status. If success, we haven't done any permission
+		 * check so ask the caller to do so.
+		 */
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+		*caller_perm_check = !FSAL_IS_ERROR(status);
+		return status;
+	}
+
+	if (!FSAL_IS_ERROR(status)) {
+		/* Return success. We haven't done any permission
+		 * check so ask the caller to do so.
+		 */
+		*caller_perm_check = true;
+		return status;
+	}
+
+	(void) vfs_close_my_fd(my_fd);
+
+undo_share:
+
+	/* Can only get here with state not NULL and an error */
+
+	/* On error we need to release our share reservation
+	 * and undo the update of the share counters.
+	 * This can block over an I/O operation.
+	 */
+	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+
+	update_share_counters(&myself->u.file.share,
+			      openflags,
+			      FSAL_O_CLOSED);
+
+	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+
+	return status;
+}
+
+
 /**
  * @brief Open a file descriptor for read or write and possibly create
  *
@@ -327,17 +580,18 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 	int posix_flags = 0;
 	int fd, dir_fd;
 	int retval = 0;
-	mode_t unix_mode;
+	mode_t unix_mode = 0000;
 	fsal_status_t status = {0, 0};
 	struct vfs_fd *my_fd = NULL;
 	struct vfs_fsal_obj_handle *myself, *hdl = NULL;
 	struct stat stat;
 	vfs_file_handle_t *fh = NULL;
-	bool truncated;
 	bool created = false;
 
 	if (state != NULL)
-		my_fd = (struct vfs_fd *)(state + 1);
+		my_fd = &container_of(state, struct vfs_state_fd,
+				      state)->vfs_fd;
+
 
 	myself = container_of(obj_hdl, struct vfs_fsal_obj_handle, obj_handle);
 
@@ -346,148 +600,15 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 
 	fsal2posix_openflags(openflags, &posix_flags);
 
-	truncated = (posix_flags & O_TRUNC) != 0;
-
-	LogFullDebug(COMPONENT_FSAL,
-		     truncated ? "Truncate" : "No truncate");
-
 	if (createmode >= FSAL_EXCLUSIVE) {
 		/* Now fixup attrs for verifier if exclusive create */
 		set_common_verifier(attrib_set, verifier);
 	}
 
 	if (name == NULL) {
-		/* This is an open by handle */
-		struct vfs_fsal_obj_handle *myself;
-
-		myself = container_of(obj_hdl,
-				      struct vfs_fsal_obj_handle,
-				      obj_handle);
-
-		if (obj_hdl->fsal != obj_hdl->fs->fsal) {
-			LogDebug(COMPONENT_FSAL,
-				 "FSAL %s operation for handle belonging to FSAL %s, return EXDEV",
-				 obj_hdl->fsal->name, obj_hdl->fs->fsal->name);
-			return fsalstat(posix2fsal_error(EXDEV), EXDEV);
-		}
-
-		if (state != NULL) {
-			/* Prepare to take the share reservation, but only if we
-			 * are called with a valid state (if state is NULL the
-			 * caller is a stateless create such as NFS v3 CREATE).
-			 */
-
-			/* This can block over an I/O operation. */
-			PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
-
-			/* Check share reservation conflicts. */
-			status = check_share_conflict(&myself->u.file.share,
-						      openflags,
-						      false);
-
-			if (FSAL_IS_ERROR(status)) {
-				PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
-				return status;
-			}
-
-			/* Take the share reservation now by updating the
-			 * counters.
-			 */
-			update_share_counters(&myself->u.file.share,
-					      FSAL_O_CLOSED,
-					      openflags);
-
-			PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
-		} else {
-			/* We need to use the global fd to continue, and take
-			 * the lock to protect it.
-			 */
-			my_fd = &hdl->u.file.fd;
-			PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
-		}
-
-		status = vfs_open_my_fd(myself, openflags, posix_flags, my_fd);
-
-		if (FSAL_IS_ERROR(status)) {
-			if (state == NULL) {
-				/* Release the lock taken above, and return
-				 * since there is nothing to undo.
-				 */
-				PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
-				return status;
-			} else {
-				/* Error - need to release the share */
-				goto undo_share;
-			}
-		}
-
-		if (createmode >= FSAL_EXCLUSIVE || truncated) {
-			/* Refresh the attributes */
-			struct stat stat;
-
-			retval = fstat(my_fd->fd, &stat);
-
-			if (retval == 0) {
-				LogFullDebug(COMPONENT_FSAL,
-					     "New size = %" PRIx64,
-					     stat.st_size);
-			} else {
-				if (errno == EBADF)
-					errno = ESTALE;
-				status = fsalstat(posix2fsal_error(errno),
-						  errno);
-			}
-
-			/* Now check verifier for exclusive, but not for
-			 * FSAL_EXCLUSIVE_9P.
-			 */
-			if (!FSAL_IS_ERROR(status) &&
-			    createmode >= FSAL_EXCLUSIVE &&
-			    createmode != FSAL_EXCLUSIVE_9P &&
-			    !check_verifier_stat(&stat, verifier)) {
-				/* Verifier didn't match, return EEXIST */
-				status =
-				    fsalstat(posix2fsal_error(EEXIST), EEXIST);
-			}
-		}
-
-		if (state == NULL) {
-			/* If no state, release the lock taken above and return
-			 * status. If success, we haven't done any permission
-			 * check so ask the caller to do so.
-			 */
-			PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
-			*caller_perm_check = !FSAL_IS_ERROR(status);
-			return status;
-		}
-
-		if (!FSAL_IS_ERROR(status)) {
-			/* Return success. We haven't done any permission
-			 * check so ask the caller to do so.
-			 */
-			*caller_perm_check = true;
-			return status;
-		}
-
-		(void) vfs_close_my_fd(my_fd);
-
- undo_share:
-
-		/* Can only get here with state not NULL and an error */
-
-		/* On error we need to release our share reservation
-		 * and undo the update of the share counters.
-		 * This can block over an I/O operation.
-		 */
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
-
-		update_share_counters(&myself->u.file.share,
-				      openflags,
-				      FSAL_O_CLOSED);
-
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
-
-		return status;
+		return vfs_open2_by_handle(obj_hdl, state, openflags,
+					   createmode, attrib_set, verifier,
+					   attrs_out, caller_perm_check);
 	}
 
 	/* In this path where we are opening by name, we can't check share
@@ -612,6 +733,9 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 		goto direrr;
 	}
 
+	LogFullDebug(COMPONENT_FSAL,
+		     "Opened fd=%d for file %s", fd, name);
+
 	/* Remember if we were responsible for creating the file.
 	 * Note that in an UNCHECKED retry we MIGHT have re-created the
 	 * file and won't remember that. Oh well, so in that rare case we
@@ -685,8 +809,10 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 	 * handy since we can then call setattr2 which WILL take the lock
 	 * without a double locking deadlock.
 	 */
-	if (my_fd == NULL)
+	if (my_fd == NULL) {
+		LogFullDebug(COMPONENT_FSAL, "Using global fd");
 		my_fd = &hdl->u.file.fd;
+	}
 
 	my_fd->fd = fd;
 	my_fd->openflags = openflags;
@@ -709,12 +835,8 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 						      state,
 						      attrib_set);
 
-		if (FSAL_IS_ERROR(status)) {
-			/* Release the handle we just allocated. */
-			(*new_obj)->obj_ops.release(*new_obj);
-			*new_obj = NULL;
+		if (FSAL_IS_ERROR(status))
 			goto fileerr;
-		}
 
 		if (attrs_out != NULL) {
 			status = (*new_obj)->obj_ops.getattrs(*new_obj,
@@ -733,9 +855,11 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 		 * on create (if we even created), just use the stat results
 		 * we used to create the fsal_obj_handle.
 		 */
-		posix2fsal_attributes(&stat, attrs_out);
+		posix2fsal_attributes_all(&stat, attrs_out);
+		attrs_out->fsid = myself->obj_handle.fs->fsid;
 	}
 
+	LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", dir_fd);
 	close(dir_fd);
 
 	if (state != NULL) {
@@ -745,21 +869,31 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 		 */
 
 		/* This can block over an I/O operation. */
-		PTHREAD_RWLOCK_wrlock(&(*new_obj)->lock);
+		PTHREAD_RWLOCK_wrlock(&(*new_obj)->obj_lock);
 
 		/* Take the share reservation now by updating the counters. */
 		update_share_counters(&hdl->u.file.share,
 				      FSAL_O_CLOSED,
 				      openflags);
 
-		PTHREAD_RWLOCK_unlock(&(*new_obj)->lock);
+		PTHREAD_RWLOCK_unlock(&(*new_obj)->obj_lock);
 	}
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 
  fileerr:
 
-	close(fd);
+	/* hdl->u.file.fd will be close in obj_ops.release */
+	if (my_fd == &hdl->u.file.fd) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", fd);
+		close(fd);
+	}
+
+	if (*new_obj) {
+		/* Release the handle we just allocated. */
+		(*new_obj)->obj_ops.release(*new_obj);
+		*new_obj = NULL;
+	}
 
 	/* Delete the file if we actually created it. */
 	if (created)
@@ -767,6 +901,7 @@ fsal_status_t vfs_open2(struct fsal_obj_handle *obj_hdl,
 
  direrr:
 
+	LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", dir_fd);
 	close(dir_fd);
 	return fsalstat(posix2fsal_error(retval), retval);
 }
@@ -797,7 +932,8 @@ fsal_status_t vfs_reopen2(struct fsal_obj_handle *obj_hdl,
 	int posix_flags = 0;
 	fsal_openflags_t old_openflags;
 
-	my_share_fd = (struct vfs_fd *)(state + 1);
+	my_share_fd = &container_of(state, struct vfs_state_fd,
+				    state)->vfs_fd;
 
 	fsal2posix_openflags(openflags, &posix_flags);
 
@@ -819,7 +955,7 @@ fsal_status_t vfs_reopen2(struct fsal_obj_handle *obj_hdl,
 	}
 
 	/* This can block over an I/O operation. */
-	PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
+	PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
 	old_openflags = my_share_fd->openflags;
 
@@ -827,7 +963,7 @@ fsal_status_t vfs_reopen2(struct fsal_obj_handle *obj_hdl,
 	status = check_share_conflict(&myself->u.file.share, openflags, false);
 
 	if (FSAL_IS_ERROR(status)) {
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 		return status;
 	}
@@ -837,27 +973,33 @@ fsal_status_t vfs_reopen2(struct fsal_obj_handle *obj_hdl,
 	 */
 	update_share_counters(&myself->u.file.share, old_openflags, openflags);
 
-	PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	status = vfs_open_my_fd(myself, openflags, posix_flags, my_fd);
 
 	if (!FSAL_IS_ERROR(status)) {
 		/* Close the existing file descriptor and copy the new
-		 * one over.
+		 * one over. Make sure no one is using the fd that we are
+		 * about to close!
 		 */
+		PTHREAD_RWLOCK_wrlock(&my_share_fd->fdlock);
+
 		vfs_close_my_fd(my_share_fd);
-		*my_share_fd = fd;
+		my_share_fd->fd = my_fd->fd;
+		my_share_fd->openflags = my_fd->openflags;
+
+		PTHREAD_RWLOCK_unlock(&my_share_fd->fdlock);
 	} else {
 		/* We had a failure on open - we need to revert the share.
 		 * This can block over an I/O operation.
 		 */
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
 		update_share_counters(&myself->u.file.share,
 				      openflags,
 				      old_openflags);
 
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
 	return status;
@@ -869,15 +1011,17 @@ fsal_status_t find_fd(int *fd,
 		      struct state_t *state,
 		      fsal_openflags_t openflags,
 		      bool *has_lock,
-		      bool *need_fsync,
 		      bool *closefd,
 		      bool open_for_locks)
 {
 	struct vfs_fsal_obj_handle *myself;
 	struct vfs_filesystem *vfs_fs;
-	struct vfs_fd temp_fd = {0, -1}, *out_fd = &temp_fd;
+	struct vfs_fd temp_fd = {
+			FSAL_O_CLOSED, PTHREAD_RWLOCK_INITIALIZER, -1 };
+	struct vfs_fd *out_fd = &temp_fd;
 	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
 	int rc, posix_flags;
+	bool reusing_open_state_fd = false;
 
 	myself = container_of(obj_hdl, struct vfs_fsal_obj_handle, obj_handle);
 	vfs_fs = myself->obj_handle.fs->private_data;
@@ -901,6 +1045,10 @@ fsal_status_t find_fd(int *fd,
 		}
 		*fd = rc;
 		*closefd = true;
+		LogFullDebug(COMPONENT_FSAL,
+			     "Opened fd=%d for file %p of type %s",
+			     rc, myself,
+			     object_file_type_to_str(obj_hdl->type));
 		return status;
 
 	case REGULAR_FILE:
@@ -909,10 +1057,14 @@ fsal_status_t find_fd(int *fd,
 				      &myself->u.file.share,
 				      bypass, state, openflags,
 				      vfs_open_func, vfs_close_func,
-				      has_lock, need_fsync,
-				      closefd, open_for_locks);
+				      has_lock, closefd, open_for_locks,
+				      &reusing_open_state_fd);
 
 		*fd = out_fd->fd;
+		LogFullDebug(COMPONENT_FSAL,
+			     "Found fd=%d for file %p of type %s",
+			     out_fd->fd, myself,
+			     object_file_type_to_str(obj_hdl->type));
 		return status;
 
 	case SYMBOLIC_LINK:
@@ -942,8 +1094,8 @@ fsal_status_t find_fd(int *fd,
 	}
 
 	LogFullDebug(COMPONENT_FSAL,
-		     "Opened fd=%d for file of type %s",
-		     rc, object_file_type_to_str(obj_hdl->type));
+		     "Opened fd=%d for file %p of type %s",
+		     rc, myself, object_file_type_to_str(obj_hdl->type));
 
 	*fd = rc;
 	*closefd = true;
@@ -987,8 +1139,8 @@ fsal_status_t vfs_read2(struct fsal_obj_handle *obj_hdl,
 	fsal_status_t status;
 	int retval = 0;
 	bool has_lock = false;
-	bool need_fsync = false;
 	bool closefd = false;
+	struct vfs_fd *vfs_fd = NULL;
 
 	if (info != NULL) {
 		/* Currently we don't support READ_PLUS */
@@ -1002,9 +1154,20 @@ fsal_status_t vfs_read2(struct fsal_obj_handle *obj_hdl,
 		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
 	}
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		vfs_fd = &container_of(state, struct vfs_state_fd,
+				       state)->vfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&vfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
+	LogFullDebug(COMPONENT_FSAL, "Calling find_fd, state = %p", state);
 	status = find_fd(&my_fd, obj_hdl, bypass, state, FSAL_O_READ,
-			 &has_lock, &need_fsync, &closefd, false);
+			 &has_lock, &closefd, false);
 
 	if (FSAL_IS_ERROR(status))
 		goto out;
@@ -1039,11 +1202,13 @@ fsal_status_t vfs_read2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
-	if (closefd)
+	if (closefd) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", my_fd);
 		close(my_fd);
+	}
 
 	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	return status;
 }
@@ -1088,9 +1253,9 @@ fsal_status_t vfs_write2(struct fsal_obj_handle *obj_hdl,
 	int retval = 0;
 	int my_fd = -1;
 	bool has_lock = false;
-	bool need_fsync = false;
 	bool closefd = false;
 	fsal_openflags_t openflags = FSAL_O_WRITE;
+	struct vfs_fd *vfs_fd = NULL;
 
 	if (info != NULL) {
 		/* Currently we don't support WRITE_PLUS */
@@ -1104,12 +1269,20 @@ fsal_status_t vfs_write2(struct fsal_obj_handle *obj_hdl,
 		return fsalstat(posix2fsal_error(EXDEV), EXDEV);
 	}
 
-	if (*fsal_stable)
-		openflags |= FSAL_O_SYNC;
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		vfs_fd = &container_of(state, struct vfs_state_fd,
+				       state)->vfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&vfs_fd->fdlock);
+	}
 
 	/* Get a usable file descriptor */
+	LogFullDebug(COMPONENT_FSAL, "Calling find_fd, state = %p", state);
 	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
-			 &has_lock, &need_fsync, &closefd, false);
+			 &has_lock, &closefd, false);
 
 	if (FSAL_IS_ERROR(status)) {
 		LogDebug(COMPONENT_FSAL,
@@ -1129,8 +1302,7 @@ fsal_status_t vfs_write2(struct fsal_obj_handle *obj_hdl,
 
 	*wrote_amount = nb_written;
 
-	/* attempt stability if we aren't using an O_SYNC fd */
-	if (need_fsync) {
+	if (*fsal_stable) {
 		retval = fsync(my_fd);
 		if (retval == -1) {
 			retval = errno;
@@ -1140,11 +1312,16 @@ fsal_status_t vfs_write2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
-	if (closefd)
+	if (vfs_fd)
+		PTHREAD_RWLOCK_unlock(&vfs_fd->fdlock);
+
+	if (closefd) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", my_fd);
 		close(my_fd);
+	}
 
 	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	fsal_restore_ganesha_credentials();
 	return status;
@@ -1174,7 +1351,9 @@ fsal_status_t vfs_commit2(struct fsal_obj_handle *obj_hdl,
 	struct vfs_fsal_obj_handle *myself;
 	fsal_status_t status;
 	int retval;
-	struct vfs_fd temp_fd = {0, -1}, *out_fd = &temp_fd;
+	struct vfs_fd temp_fd = {
+			FSAL_O_CLOSED, PTHREAD_RWLOCK_INITIALIZER, -1 };
+	struct vfs_fd *out_fd = &temp_fd;
 	bool has_lock = false;
 	bool closefd = false;
 
@@ -1204,11 +1383,14 @@ fsal_status_t vfs_commit2(struct fsal_obj_handle *obj_hdl,
 		fsal_restore_ganesha_credentials();
 	}
 
-	if (closefd)
+	if (closefd) {
+		LogFullDebug(COMPONENT_FSAL,
+			     "Closing Opened fd %d", out_fd->fd);
 		close(out_fd->fd);
+	}
 
 	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	return status;
 }
@@ -1248,10 +1430,10 @@ fsal_status_t vfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	int retval = 0;
 	int my_fd = -1;
 	bool has_lock = false;
-	bool need_fsync = false;
 	bool closefd = false;
 	bool bypass = false;
 	fsal_openflags_t openflags = FSAL_O_RDWR;
+	struct vfs_fd *vfs_fd = NULL;
 
 	if (obj_hdl->fsal != obj_hdl->fs->fsal) {
 		LogDebug(COMPONENT_FSAL,
@@ -1323,9 +1505,20 @@ fsal_status_t vfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 		return fsalstat(ERR_FSAL_BAD_RANGE, 0);
 	}
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		vfs_fd = &container_of(state, struct vfs_state_fd,
+				       state)->vfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&vfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor */
+	LogFullDebug(COMPONENT_FSAL, "Calling find_fd, state = %p", state);
 	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
-			 &has_lock, &need_fsync, &closefd, true);
+			 &has_lock, &closefd, true);
 
 	if (FSAL_IS_ERROR(status)) {
 		LogCrit(COMPONENT_FSAL, "Unable to find fd for lock operation");
@@ -1381,87 +1574,21 @@ fsal_status_t vfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 
  err:
 
-	if (closefd)
+	if (vfs_fd)
+		PTHREAD_RWLOCK_unlock(&vfs_fd->fdlock);
+
+	if (closefd) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", my_fd);
 		close(my_fd);
+	}
 
 	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	return fsalstat(posix2fsal_error(retval), retval);
 }
 #endif
 
-fsal_status_t fetch_attrs(struct vfs_fsal_obj_handle *myself,
-			  int my_fd, struct attrlist *attrs)
-{
-	struct stat stat;
-	int retval = 0;
-	fsal_status_t status = {0, 0};
-	const char *func = "unknown";
-
-	/* Now stat the file as appropriate */
-	switch (myself->obj_handle.type) {
-	case SOCKET_FILE:
-	case CHARACTER_FILE:
-	case BLOCK_FILE:
-		retval = fstatat(my_fd, myself->u.unopenable.name, &stat,
-				 AT_SYMLINK_NOFOLLOW);
-		func = "fstatat";
-		break;
-
-	case REGULAR_FILE:
-		retval = fstat(my_fd, &stat);
-		func = "fstat";
-		break;
-
-	case SYMBOLIC_LINK:
-	case FIFO_FILE:
-	case DIRECTORY:
-		retval = vfs_stat_by_handle(my_fd, &stat);
-		func = "vfs_stat_by_handle";
-		break;
-
-	case NO_FILE_TYPE:
-	case EXTENDED_ATTR:
-		/* Caught during open with EINVAL */
-		break;
-	}
-
-	if (retval < 0) {
-		if (errno == ENOENT)
-			retval = ESTALE;
-		else
-			retval = errno;
-
-		LogDebug(COMPONENT_FSAL, "%s failed with %s", func,
-			 strerror(retval));
-
-		if (attrs->request_mask & ATTR_RDATTR_ERR) {
-			/* Caller asked for error to be visible. */
-			attrs->valid_mask = ATTR_RDATTR_ERR;
-		}
-
-		return fsalstat(posix2fsal_error(retval), retval);
-	}
-
-	posix2fsal_attributes(&stat, attrs);
-	attrs->fsid = myself->obj_handle.fs->fsid;
-
-	if (myself->sub_ops && myself->sub_ops->getattrs) {
-		status =
-		   myself->sub_ops->getattrs(myself, my_fd, attrs->request_mask,
-					     attrs);
-
-		if (FSAL_IS_ERROR(status) &&
-		    (attrs->request_mask & ATTR_RDATTR_ERR) != 0) {
-			/* Caller asked for error to be visible. */
-			attrs->valid_mask = ATTR_RDATTR_ERR;
-		}
-	}
-
-	return status;
-}
-
 /**
  * @brief Get attributes
  *
@@ -1480,7 +1607,6 @@ fsal_status_t vfs_getattr2(struct fsal_obj_handle *obj_hdl,
 	struct vfs_fsal_obj_handle *myself;
 	fsal_status_t status = {0, 0};
 	bool has_lock = false;
-	bool need_fsync = false;
 	bool closefd = false;
 	int my_fd;
 
@@ -1499,8 +1625,12 @@ fsal_status_t vfs_getattr2(struct fsal_obj_handle *obj_hdl,
 	/* Get a usable file descriptor (don't need to bypass - FSAL_O_ANY
 	 * won't conflict with any share reservation).
 	 */
+	LogFullDebug(COMPONENT_FSAL, "Calling find_fd, state = NULL");
 	status = find_fd(&my_fd, obj_hdl, false, NULL, FSAL_O_ANY,
-			 &has_lock, &need_fsync, &closefd, false);
+			 &has_lock, &closefd, false);
+
+	LogFullDebug(COMPONENT_FSAL, "Got fd %d closefd = %s",
+		     my_fd, closefd ? "true" : "false");
 
 	if (FSAL_IS_ERROR(status)) {
 		if (obj_hdl->type == SYMBOLIC_LINK &&
@@ -1524,11 +1654,13 @@ fsal_status_t vfs_getattr2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
-	if (closefd)
+	if (closefd) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", my_fd);
 		close(my_fd);
+	}
 
 	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	return status;
 }
@@ -1556,10 +1688,10 @@ fsal_status_t vfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	int retval = 0;
 	fsal_openflags_t openflags = FSAL_O_ANY;
 	bool has_lock = false;
-	bool need_fsync = false;
 	bool closefd = false;
 	int my_fd;
-	const char *func;
+	const char *func = "none";
+	struct vfs_fd *vfs_fd = NULL;
 
 	/* apply umask, if mode attribute is to be changed */
 	if (FSAL_TEST_MASK(attrib_set->valid_mask, ATTR_MODE))
@@ -1634,11 +1766,22 @@ fsal_status_t vfs_setattr2(struct fsal_obj_handle *obj_hdl,
 		openflags = FSAL_O_RDWR;
 	}
 
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		vfs_fd = &container_of(state, struct vfs_state_fd,
+				       state)->vfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&vfs_fd->fdlock);
+	}
+
 	/* Get a usable file descriptor. Share conflict is only possible if
 	 * size is being set.
 	 */
+	LogFullDebug(COMPONENT_FSAL, "Calling find_fd, state = %p", state);
 	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
-			 &has_lock, &need_fsync, &closefd, false);
+			 &has_lock, &closefd, false);
 
 	if (FSAL_IS_ERROR(status)) {
 		if (obj_hdl->type == SYMBOLIC_LINK &&
@@ -1799,11 +1942,16 @@ fsal_status_t vfs_setattr2(struct fsal_obj_handle *obj_hdl,
 
  out:
 
-	if (closefd)
+	if (vfs_fd)
+		PTHREAD_RWLOCK_unlock(&vfs_fd->fdlock);
+
+	if (closefd) {
+		LogFullDebug(COMPONENT_FSAL, "Closing Opened fd %d", my_fd);
 		close(my_fd);
+	}
 
 	if (has_lock)
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
 	return status;
 }
@@ -1825,8 +1973,9 @@ fsal_status_t vfs_setattr2(struct fsal_obj_handle *obj_hdl,
 fsal_status_t vfs_close2(struct fsal_obj_handle *obj_hdl,
 			 struct state_t *state)
 {
-	struct vfs_fd *my_fd = (struct vfs_fd *)(state + 1);
 	struct vfs_fsal_obj_handle *myself = NULL;
+	struct vfs_fd *my_fd = &container_of(state, struct vfs_state_fd,
+					     state)->vfs_fd;
 
 	myself = container_of(obj_hdl,
 			      struct vfs_fsal_obj_handle,
@@ -1838,13 +1987,13 @@ fsal_status_t vfs_close2(struct fsal_obj_handle *obj_hdl,
 		/* This is a share state, we must update the share counters */
 
 		/* This can block over an I/O operation. */
-		PTHREAD_RWLOCK_wrlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
 
 		update_share_counters(&myself->u.file.share,
 				      my_fd->openflags,
 				      FSAL_O_CLOSED);
 
-		PTHREAD_RWLOCK_unlock(&obj_hdl->lock);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 	}
 
 	return vfs_close_my_fd(my_fd);
