@@ -104,7 +104,7 @@ void display_vfs_handle(struct display_buffer *dspbuf,
 #define LogVFSHandle(fh)						\
 	do {								\
 		if (isMidDebug(COMPONENT_FSAL)) {			\
-			char buf[256];					\
+			char buf[256] = "\0";				\
 			struct display_buffer dspbuf =			\
 					{sizeof(buf), buf, buf};	\
 									\
@@ -241,7 +241,7 @@ bool vfs_valid_handle(struct gsh_buffdesc *desc)
 		return false;
 
 	if (isMidDebug(COMPONENT_FSAL)) {
-		char buf[256];
+		char buf[256] = "\0";
 		struct display_buffer dspbuf = {sizeof(buf), buf, buf};
 		int b_left;
 
