@@ -30,6 +30,7 @@
 
 #include "fsal_handle_syscalls.h"
 #include "fsal_api.h"
+#include "FSAL/fsal_commonlib.h"
 
 struct vfs_fsal_obj_handle;
 struct vfs_fsal_export;
@@ -119,10 +120,17 @@ struct vfs_subfsal_obj_ops {
 struct vfs_fd {
 	/** The open and share mode etc. */
 	fsal_openflags_t openflags;
+	/* rw lock to protect the file descriptor */
+	pthread_rwlock_t fdlock;
 	/** The kernel file descriptor. */
 	int fd;
 };
 
+struct vfs_state_fd {
+	struct state_t state;
+	struct vfs_fd vfs_fd;
+};
+
 /*
  * VFS internal object handle
  * handle is a pointer because
@@ -151,6 +159,10 @@ struct vfs_fsal_obj_handle {
 			struct fsal_share share;
 			struct vfs_fd fd;
 		} file;
+		struct {
+			char *path;
+			char *fs_location;
+		} directory;
 		struct {
 			unsigned char *link_content;
 			int link_size;
@@ -255,6 +267,7 @@ fsal_status_t vfs_close(struct fsal_obj_handle *obj_hdl);
 struct state_t *vfs_alloc_state(struct fsal_export *exp_hdl,
 				enum state_type state_type,
 				struct state_t *related_state);
+void vfs_free_state(struct fsal_export *exp_hdl, struct state_t *state);
 
 fsal_status_t vfs_merge(struct fsal_obj_handle *orig_hdl,
 			struct fsal_obj_handle *dupe_hdl);
