@@ -44,13 +44,11 @@
  */
 
 /* defined the set of attributes supported with POSIX */
-#define XFS_SUPPORTED_ATTRIBUTES (                                       \
-		ATTR_TYPE     | ATTR_SIZE     |				\
-		ATTR_FSID     | ATTR_FILEID   |				\
-		ATTR_MODE     | ATTR_NUMLINKS | ATTR_OWNER     |	\
-		ATTR_GROUP    | ATTR_ATIME    | ATTR_RAWDEV    |	\
-		ATTR_CTIME    | ATTR_MTIME    | ATTR_SPACEUSED |	\
-		ATTR_CHGTIME)
+#ifndef ENABLE_VFS_DEBUG_ACL
+#define XFS_SUPPORTED_ATTRIBUTES ((const attrmask_t) (ATTRS_POSIX))
+#else
+#define XFS_SUPPORTED_ATTRIBUTES ((const attrmask_t) (ATTRS_POSIX | ATTR_ACL))
+#endif
 
 struct xfs_fsal_module {
 	struct fsal_module fsal;
@@ -71,7 +69,6 @@ static struct fsal_staticfsinfo_t default_posix_info = {
 	.case_insensitive = false,
 	.case_preserving = true,
 	.lock_support = false,
-	.lock_support_owner = true,
 	.lock_support_async_block = false,
 	.named_attr = true,
 	.unique_handles = true,
@@ -191,7 +188,7 @@ static fsal_status_t init_config(struct fsal_module *fsal_hdl,
 	display_fsinfo(&xfs_me->fs_info);
 	LogFullDebug(COMPONENT_FSAL,
 		     "Supported attributes constant = 0x%" PRIx64,
-		     (uint64_t) XFS_SUPPORTED_ATTRIBUTES);
+		     XFS_SUPPORTED_ATTRIBUTES);
 	LogFullDebug(COMPONENT_FSAL,
 		     "Supported attributes default = 0x%" PRIx64,
 		     default_posix_info.supported_attrs);
@@ -209,17 +206,6 @@ fsal_status_t vfs_create_export(struct fsal_module *fsal_hdl,
 				struct config_error_type *err_type,
 				const struct fsal_up_vector *up_ops);
 
-/**
- * @brief Indicate support for extended operations.
- *
- * @retval true if extended operations are supported.
- */
-
-bool vfs_support_ex(struct fsal_obj_handle *obj)
-{
-	return true;
-}
-
 /* Module initialization.
  * Called by dlopen() to register the module
  * keep a private pointer to me in myself
@@ -246,7 +232,6 @@ MODULE_INIT void xfs_init(void)
 	}
 	myself->m_ops.create_export = vfs_create_export;
 	myself->m_ops.init_config = init_config;
-	myself->m_ops.support_ex = vfs_support_ex;
 }
 
 MODULE_FINI void xfs_unload(void)
