@@ -181,82 +181,6 @@ static fsal_status_t mdcache_lookup(struct fsal_obj_handle *parent,
 	return status;
 }
 
-/**
- * @brief Create a file
- *
- * @param[in] dir_hdl	Handle of parent directory
- * @param[in] name	Name of file to create
- * @param[in] attrib	Attributes to set on new file
- * @param[out] handle	Newly created file
- *
- * @note This returns an INITIAL ref'd entry on success
- * @return FSAL status
- */
-static fsal_status_t mdcache_create(struct fsal_obj_handle *dir_hdl,
-				    const char *name, struct attrlist *attrs_in,
-				    struct fsal_obj_handle **new_obj,
-				    struct attrlist *attrs_out)
-{
-	mdcache_entry_t *parent =
-		container_of(dir_hdl, mdcache_entry_t,
-			     obj_handle);
-	struct mdcache_fsal_export *export = mdc_cur_export();
-	struct fsal_obj_handle *sub_handle;
-	fsal_status_t status;
-	struct attrlist attrs;
-	bool invalidate = true;
-
-	/* Ask for all supported attributes except ACL (we defer fetching ACL
-	 * until asked for it (including a permission check).
-	 */
-	fsal_prepare_attrs(&attrs,
-			   op_ctx->fsal_export->exp_ops.
-				   fs_supported_attrs(op_ctx->fsal_export)
-				   & ~ATTR_ACL);
-
-	subcall_raw(export,
-		status = parent->sub_handle->obj_ops.create(
-			parent->sub_handle, name, attrs_in, &sub_handle, &attrs)
-	       );
-
-	if (unlikely(FSAL_IS_ERROR(status))) {
-		LogDebug(COMPONENT_CACHE_INODE,
-			 "create %s failed with %s",
-			 name, fsal_err_txt(status));
-		if (status.major == ERR_FSAL_STALE) {
-			/* If we got ERR_FSAL_STALE, the previous FSAL call
-			 * must have failed with a bad parent.
-			 */
-			LogEvent(COMPONENT_CACHE_INODE,
-				 "FSAL returned STALE on create");
-			mdcache_kill_entry(parent);
-		}
-		*new_obj = NULL;
-		fsal_release_attrs(&attrs);
-		return status;
-	}
-
-	PTHREAD_RWLOCK_wrlock(&parent->content_lock);
-
-	status = mdcache_alloc_and_check_handle(export, sub_handle, new_obj,
-						false, &attrs, attrs_out,
-						"create ", parent, name,
-						&invalidate, NULL);
-
-	PTHREAD_RWLOCK_unlock(&parent->content_lock);
-
-	fsal_release_attrs(&attrs);
-
-	if (!invalidate) {
-		/* Refresh destination directory attributes without
-		 * invalidating dirents.
-		 */
-		mdcache_refresh_attrs_no_invalidate(parent);
-	}
-
-	return status;
-}
-
 /**
  * @brief Make a directory
  *
@@ -288,9 +212,8 @@ static fsal_status_t mdcache_mkdir(struct fsal_obj_handle *dir_hdl,
 	 * until asked for it (including a permission check).
 	 */
 	fsal_prepare_attrs(&attrs,
-			   op_ctx->fsal_export->exp_ops.
-				   fs_supported_attrs(op_ctx->fsal_export)
-				   & ~ATTR_ACL);
+			   op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+					op_ctx->fsal_export) & ~ATTR_ACL);
 
 	subcall_raw(export,
 		status = parent->sub_handle->obj_ops.mkdir(
@@ -368,9 +291,8 @@ static fsal_status_t mdcache_mknode(struct fsal_obj_handle *dir_hdl,
 	 * until asked for it (including a permission check).
 	 */
 	fsal_prepare_attrs(&attrs,
-			   op_ctx->fsal_export->exp_ops.
-				   fs_supported_attrs(op_ctx->fsal_export)
-				   & ~ATTR_ACL);
+			   op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+					op_ctx->fsal_export) & ~ATTR_ACL);
 
 	subcall_raw(export,
 		status = parent->sub_handle->obj_ops.mknode(
@@ -449,9 +371,8 @@ static fsal_status_t mdcache_symlink(struct fsal_obj_handle *dir_hdl,
 	 * until asked for it (including a permission check).
 	 */
 	fsal_prepare_attrs(&attrs,
-			   op_ctx->fsal_export->exp_ops.
-				   fs_supported_attrs(op_ctx->fsal_export)
-				   & ~ATTR_ACL);
+			   op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+					op_ctx->fsal_export) & ~ATTR_ACL);
 
 	subcall_raw(export,
 		status = parent->sub_handle->obj_ops.symlink(
@@ -718,7 +639,7 @@ static fsal_status_t mdcache_readdir(struct fsal_obj_handle *dir_hdl,
 	}
 
 	LogFullDebugAlt(COMPONENT_NFS_READDIR, COMPONENT_CACHE_INODE,
-			"About to readdir in mdcache_readdir: directory=%p cookie=%"
+			"About to readdir directory=%p cookie=%"
 			PRIu64 " collisions %d",
 			directory, *whence,
 			directory->fsobj.fsdir.avl.collisions);
@@ -869,7 +790,9 @@ static fsal_status_t mdcache_rename(struct fsal_obj_handle *obj_hdl,
 	mdcache_entry_t *mdc_obj =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 	mdcache_entry_t *mdc_lookup_dst = NULL;
+	struct fsal_export *sub_export = op_ctx->fsal_export->sub_export;
 	bool refresh = false;
+	bool rename_change_key;
 	fsal_status_t status;
 
 	/* Now update cached dirents.  Must take locks in the correct order */
@@ -942,8 +865,11 @@ static fsal_status_t mdcache_rename(struct fsal_obj_handle *obj_hdl,
 	}
 
 
-	if (op_ctx->fsal_export->sub_export->exp_ops.fs_supports(
-		op_ctx->fsal_export->sub_export, fso_rename_changes_key)) {
+	subcall(
+		rename_change_key = sub_export->exp_ops.fs_supports(
+				sub_export, fso_rename_changes_key)
+	       );
+	if (rename_change_key) {
 		LogDebug(COMPONENT_CACHE_INODE,
 			 "Rename (%p,%s)->(%p,%s) : key changing", mdc_olddir,
 			 old_name, mdc_newdir, new_name);
@@ -1110,8 +1036,9 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 	/* We always ask for all regular attributes, even if the caller was
 	 * only interested in the ACL.
 	 */
-	fsal_prepare_attrs(&attrs, op_ctx->fsal_export->exp_ops.
-		fs_supported_attrs(op_ctx->fsal_export) | ATTR_RDATTR_ERR);
+	fsal_prepare_attrs(&attrs,
+			   op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+					op_ctx->fsal_export) | ATTR_RDATTR_ERR);
 
 	if (!need_acl) {
 		/* Don't request the ACL if not necessary. */
@@ -1219,64 +1146,6 @@ unlock_no_attrs:
 	return status;
 }
 
-/**
- * @brief Set attributes on an object
- *
- * @param[in] obj_hdl	Object to set attributes on
- * @param[in] attrs	Attributes to set
- * @return FSAL status
- */
-static fsal_status_t mdcache_setattrs(struct fsal_obj_handle *obj_hdl,
-				      struct attrlist *attrs)
-{
-	mdcache_entry_t *entry =
-		container_of(obj_hdl, mdcache_entry_t, obj_handle);
-	fsal_status_t status;
-	uint64_t change;
-
-	PTHREAD_RWLOCK_wrlock(&entry->attr_lock);
-
-	change = entry->attrs.change;
-
-	subcall(
-		status = entry->sub_handle->obj_ops.setattrs(
-			entry->sub_handle, attrs)
-	       );
-
-	if (FSAL_IS_ERROR(status)) {
-		LogDebug(COMPONENT_CACHE_INODE,
-			 "sub_handle setattrs returned %s",
-			 fsal_err_txt(status));
-		goto unlock;
-	}
-
-	status = mdcache_refresh_attrs(
-			entry, (attrs->valid_mask & ATTR_ACL) != 0, false);
-
-	if (!FSAL_IS_ERROR(status) && change == entry->attrs.change) {
-		LogDebug(COMPONENT_CACHE_INODE,
-			 "setattrs did not change change attribute before %lld after = %lld",
-			 (long long int) change,
-			 (long long int) entry->attrs.change);
-		entry->attrs.change = change + 1;
-	}
-
-	if (FSAL_IS_ERROR(status)) {
-		LogDebug(COMPONENT_CACHE_INODE,
-			 "sub_handle getattrs returned %s",
-			 fsal_err_txt(status));
-	}
-
-unlock:
-
-	PTHREAD_RWLOCK_unlock(&entry->attr_lock);
-
-	if (FSAL_IS_ERROR(status) && (status.major == ERR_FSAL_STALE))
-		mdcache_kill_entry(entry);
-
-	return status;
-}
-
 /**
  * @brief Set attributes on an object (new style)
  *
@@ -1531,13 +1400,15 @@ static void mdcache_handle_to_key(struct fsal_obj_handle *obj_hdl,
 static bool mdcache_handle_cmp(struct fsal_obj_handle *obj_hdl1,
 			       struct fsal_obj_handle *obj_hdl2)
 {
-	mdcache_entry_t *entry =
+	mdcache_entry_t *entry1 =
 		container_of(obj_hdl1, mdcache_entry_t, obj_handle);
+	mdcache_entry_t *entry2 =
+		container_of(obj_hdl2, mdcache_entry_t, obj_handle);
 	bool status;
 
 	subcall(
-		status = entry->sub_handle->obj_ops.handle_cmp(
-			entry->sub_handle, obj_hdl2)
+		status = entry1->sub_handle->obj_ops.handle_cmp(
+			entry1->sub_handle, entry2->sub_handle)
 	       );
 
 	return status;
@@ -1728,30 +1599,18 @@ void mdcache_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->merge = mdcache_merge;
 	ops->lookup = mdcache_lookup;
 	ops->readdir = mdcache_readdir;
-	ops->create = mdcache_create;
 	ops->mkdir = mdcache_mkdir;
 	ops->mknode = mdcache_mknode;
 	ops->symlink = mdcache_symlink;
 	ops->readlink = mdcache_readlink;
 	ops->test_access = mdcache_test_access;
 	ops->getattrs = mdcache_getattrs;
-	ops->setattrs = mdcache_setattrs;
 	ops->link = mdcache_link;
 	ops->rename = mdcache_rename;
 	ops->unlink = mdcache_unlink;
-	ops->open = mdcache_open;
-	ops->reopen = mdcache_reopen;
 	ops->fs_locations = mdcache_fs_locations;
-	ops->status = mdcache_status;
-	ops->read = mdcache_read;
-	ops->read_plus = mdcache_read_plus;
-	ops->write = mdcache_write;
-	ops->write_plus = mdcache_write_plus;
 	ops->seek = mdcache_seek;
 	ops->io_advise = mdcache_io_advise;
-	ops->commit = mdcache_commit;
-	ops->lock_op = mdcache_lock_op;
-	ops->share_op = mdcache_share_op;
 	ops->close = mdcache_close;
 	ops->handle_is = mdcache_handle_is;
 	ops->handle_to_wire = mdcache_handle_to_wire;
@@ -1774,6 +1633,7 @@ void mdcache_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->io_advise2 = mdcache_io_advise2;
 	ops->commit2 = mdcache_commit2;
 	ops->lock_op2 = mdcache_lock_op2;
+	ops->lease_op2 = mdcache_lease_op2;
 	ops->setattr2 = mdcache_setattr2;
 	ops->close2 = mdcache_close2;
 
@@ -1817,8 +1677,8 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 {
 	struct fsal_obj_handle *sub_handle = NULL;
 	struct mdcache_fsal_export *export =
-		container_of(exp_hdl, struct mdcache_fsal_export, export);
-	struct fsal_export *sub_export = export->export.sub_export;
+		container_of(exp_hdl, struct mdcache_fsal_export, mfe_exp);
+	struct fsal_export *sub_export = export->mfe_exp.sub_export;
 	fsal_status_t status;
 	struct attrlist attrs;
 	mdcache_entry_t *new_entry;
@@ -1829,9 +1689,8 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 	 * until asked for it (including a permission check).
 	 */
 	fsal_prepare_attrs(&attrs,
-			   op_ctx->fsal_export->exp_ops.
-				   fs_supported_attrs(op_ctx->fsal_export)
-				   & ~ATTR_ACL);
+			   op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+					op_ctx->fsal_export) & ~ATTR_ACL);
 
 	subcall_raw(export,
 		status = sub_export->exp_ops.lookup_path(sub_export, path,
@@ -1890,7 +1749,7 @@ fsal_status_t mdcache_create_handle(struct fsal_export *exp_hdl,
 				   struct attrlist *attrs_out)
 {
 	struct mdcache_fsal_export *export =
-		container_of(exp_hdl, struct mdcache_fsal_export, export);
+		container_of(exp_hdl, struct mdcache_fsal_export, mfe_exp);
 	mdcache_entry_t *entry;
 	fsal_status_t status;
 
