@@ -67,6 +67,8 @@
 struct lru_state {
 	uint64_t entries_hiwat;
 	uint64_t entries_used;
+	uint64_t chunks_hiwat;
+	uint64_t chunks_used;
 	uint32_t fds_system_imposed;
 	uint32_t fds_hard_limit;
 	uint32_t fds_hiwat;
@@ -138,20 +140,22 @@ fsal_status_t mdcache_lru_pkgshutdown(void);
 
 extern size_t open_fd_count;
 
-fsal_status_t mdcache_lru_get(mdcache_entry_t **entry);
-fsal_status_t mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags);
+mdcache_entry_t *mdcache_lru_get(void);
+void mdcache_lru_insert(mdcache_entry_t *entry);
+#define mdcache_lru_ref(e, f) _mdcache_lru_ref(e, f, __func__, __LINE__)
+fsal_status_t _mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags,
+			       const char *func, int line);
 
 /* XXX */
 void mdcache_lru_kill(mdcache_entry_t *entry);
 void mdcache_lru_cleanup_push(mdcache_entry_t *entry);
 void mdcache_lru_cleanup_try_push(mdcache_entry_t *entry);
 
-bool mdcache_lru_unref(mdcache_entry_t *entry, uint32_t flags);
-void mdcache_lru_putback(mdcache_entry_t *entry, uint32_t flags);
+#define mdcache_lru_unref(e) _mdcache_lru_unref(e, LRU_FLAG_NONE, \
+						__func__, __LINE__)
+bool _mdcache_lru_unref(mdcache_entry_t *entry, uint32_t flags,
+			const char *func, int line);
 void lru_wake_thread(void);
-fsal_status_t mdcache_inc_noscan_ref(mdcache_entry_t *entry);
-void mdcache_dec_noscan_ref(mdcache_entry_t *entry);
-bool mdcache_is_noscan(mdcache_entry_t *entry);
 void mdcache_lru_kill_for_shutdown(mdcache_entry_t *entry);
 
 /**
@@ -183,32 +187,7 @@ static inline fsal_status_t mdcache_get(mdcache_entry_t *entry)
  */
 static inline void mdcache_put(mdcache_entry_t *entry)
 {
-	mdcache_lru_unref(entry, LRU_FLAG_NONE);
-}
-
-/**
- * Return true if there are FDs available to serve open requests,
- * false otherwise.  This function also wakes the LRU thread if the
- * current FD count is above the high water mark.
- */
-
-static inline bool mdcache_lru_fds_available(void)
-{
-	if ((atomic_fetch_size_t(&open_fd_count) >= lru_state.fds_hard_limit)
-	    && lru_state.caching_fds) {
-		LogCrit(COMPONENT_CACHE_INODE_LRU,
-			"FD Hard Limit Exceeded.  Disabling FD Cache and waking LRU thread.");
-		lru_state.caching_fds = false;
-		lru_wake_thread();
-		return false;
-	}
-	if (atomic_fetch_size_t(&open_fd_count) >= lru_state.fds_hiwat) {
-		LogInfo(COMPONENT_CACHE_INODE_LRU,
-			"FDs above high water mark, waking LRU thread.");
-		lru_wake_thread();
-	}
-
-	return true;
+	mdcache_lru_unref(entry);
 }
 
 /**
@@ -219,5 +198,10 @@ static inline bool mdcache_lru_caching_fds(void)
 {
 	return lru_state.caching_fds;
 }
+
+void lru_remove_chunk(struct dir_chunk *chunk);
+struct dir_chunk *mdcache_get_chunk(mdcache_entry_t *parent);
+void lru_bump_chunk(struct dir_chunk *chunk);
+
 #endif				/* MDCACHE_LRU_H */
 /** @} */
