@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2016 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2017 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -53,15 +53,6 @@ pool_t *mdcache_entry_pool;
 /* MDCACHE FSAL module private storage
  */
 
-/* defined the set of attributes supported with POSIX */
-#define MDCACHE_SUPPORTED_ATTRIBUTES (                    \
-	ATTR_TYPE     | ATTR_SIZE     |                  \
-	ATTR_FSID     | ATTR_FILEID   |                  \
-	ATTR_MODE     | ATTR_NUMLINKS | ATTR_OWNER     | \
-	ATTR_GROUP    | ATTR_ATIME    | ATTR_RAWDEV    | \
-	ATTR_CTIME    | ATTR_MTIME    | ATTR_SPACEUSED | \
-	ATTR_CHGTIME)
-
 struct mdcache_fsal_module {
 	struct fsal_module fsal;
 	struct fsal_staticfsinfo_t fs_info;
@@ -92,7 +83,6 @@ static struct fsal_staticfsinfo_t default_posix_info = {
 	.link_support = true,
 	.symlink_support = true,
 	.lock_support = true,
-	.lock_support_owner = false,
 	.lock_support_async_block = false,
 	.named_attr = true,
 	.unique_handles = true,
@@ -100,7 +90,7 @@ static struct fsal_staticfsinfo_t default_posix_info = {
 	.acl_support = FSAL_ACLSUPPORT_ALLOW,
 	.cansettime = true,
 	.homogenous = true,
-	.supported_attrs = MDCACHE_SUPPORTED_ATTRIBUTES,
+	.supported_attrs = ALL_ATTRIBUTES,
 	.maxread = FSAL_MAXIOSIZE,
 	.maxwrite = FSAL_MAXIOSIZE,
 	.umask = 0,
@@ -151,9 +141,6 @@ mdcache_fsal_init_config(struct fsal_module *fsal_hdl,
 	 */
 
 	display_fsinfo(&mdcache_me->fs_info);
-	LogFullDebug(COMPONENT_FSAL,
-		     "Supported attributes constant = 0x%" PRIx64,
-		     (uint64_t) MDCACHE_SUPPORTED_ATTRIBUTES);
 	LogFullDebug(COMPONENT_FSAL,
 		     "Supported attributes default = 0x%" PRIx64,
 		     default_posix_info.supported_attrs);
@@ -163,45 +150,6 @@ mdcache_fsal_init_config(struct fsal_module *fsal_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/**
- * @brief Enable caching for a FSAL export
- *
- * This is the API to call to enable caching on an export.  The sub-FSAL calls
- * this with with the up_ops that were passed to it, and is wrapped in a
- * FSAL_MDCACHE instance to do caching.  @ref op_ctx should already be
- * initialized by the sub-FSAL.  On success, @a mdc_up_ops will contain
- * the up_ops for the MDCACHE instance, that the sub-FSAL can the specialize.
- *
- * @see mdcache_fsal_create_export
- *
- * @param[in] super_up_ops	FSAL_UP ops for the super-FSAL
- * @param[out] mdc_up_ops	FSAL_UP ops for MDCACHE
- * @return FSAL status
- */
-fsal_status_t mdcache_export_init(const struct fsal_up_vector *super_up_ops,
-				  const struct fsal_up_vector **mdc_up_ops)
-{
-	struct mdcache_fsal_export *exp;
-	struct fsal_up_vector my_up_ops;
-	fsal_status_t status;
-
-	/* Get ref on FSAL MDCACHE for sub FSAL. Done before mdc_init_export */
-	fsal_get(&MDCACHE.fsal);
-
-	*mdc_up_ops = NULL;
-	mdcache_export_up_ops_init(&my_up_ops, super_up_ops);
-	status =  mdc_init_export(&MDCACHE.fsal, &my_up_ops, super_up_ops);
-	if (FSAL_IS_ERROR(status)) {
-		fsal_put(&MDCACHE.fsal);
-		return status;
-	}
-
-	exp = mdc_cur_export();
-	*mdc_up_ops = &exp->up_ops;
-
-	return fsalstat(ERR_FSAL_NO_ERROR, 0);
-}
-
 /**
  * @brief Clean up caching for a FSAL export on error
  *
@@ -213,10 +161,15 @@ fsal_status_t mdcache_export_init(const struct fsal_up_vector *super_up_ops,
 void mdcache_export_uninit(void)
 {
 	struct mdcache_fsal_export *exp = mdc_cur_export();
-	struct fsal_export *sub_export = exp->export.sub_export;
+	struct fsal_export *sub_export = exp->mfe_exp.sub_export;
 
 	fsal_put(sub_export->fsal);
 
+	LogFullDebug(COMPONENT_FSAL,
+		     "FSAL %s refcount %"PRIu32,
+		     sub_export->fsal->name,
+		     atomic_fetch_int32_t(&sub_export->fsal->refcount));
+
 	fsal_detach_export(op_ctx->fsal_export->fsal,
 			   &op_ctx->fsal_export->exports);
 	free_export_ops(op_ctx->fsal_export);
@@ -240,10 +193,10 @@ void mdcache_export_uninit(void)
  * There is currently no config; FSALs that want caching should call @ref
  * mdcache_export_init
  *
- * @param[in] fsal_hdl		Sub-FSAL module handle
+ * @param[in] sub_fsal		Sub-FSAL module handle
  * @param[in] parse_node	Config node for export
  * @param[out] err_type		Parse errors
- * @param[in] up_ops		Upcall ops for export
+ * @param[in] super_up_ops	Upcall ops for export
  * @return FSAL status
  */
 fsal_status_t
@@ -261,14 +214,15 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 	myself->name = gsh_calloc(1, namelen);
 	snprintf(myself->name, namelen, "%s/MDC", sub_fsal->name);
 
-	fsal_export_init(&myself->export);
-	mdcache_export_ops_init(&myself->export.exp_ops);
+	fsal_export_init(&myself->mfe_exp);
+	mdcache_export_ops_init(&myself->mfe_exp.exp_ops);
 
 	myself->super_up_ops = *super_up_ops; /* Struct copy */
 	mdcache_export_up_ops_init(&myself->up_ops, super_up_ops);
-	myself->up_ops.up_export = &myself->export;
-	myself->export.up_ops = &myself->up_ops;
-	myself->export.fsal = &MDCACHE.fsal;
+	myself->up_ops.up_gsh_export = op_ctx->ctx_export;
+	myself->up_ops.up_fsal_export = &myself->mfe_exp;
+	myself->mfe_exp.up_ops = &myself->up_ops;
+	myself->mfe_exp.fsal = &MDCACHE.fsal;
 
 	glist_init(&myself->entry_list);
 	pthread_rwlockattr_init(&attrs);
@@ -286,19 +240,29 @@ mdcache_fsal_create_export(struct fsal_module *sub_fsal, void *parse_node,
 		LogMajor(COMPONENT_FSAL,
 			 "Failed to call create_export on underlying FSAL %s",
 			 sub_fsal->name);
-		fsal_put(sub_fsal);
+		gsh_free(myself->name);
+		gsh_free(myself);
 		return status;
 	}
 
 	/* Get ref for sub-FSAL */
-	fsal_get(myself->export.fsal);
-	fsal_export_stack(op_ctx->fsal_export, &myself->export);
+	fsal_get(myself->mfe_exp.fsal);
+
+	LogFullDebug(COMPONENT_FSAL,
+		     "FSAL %s refcount %"PRIu32,
+		     myself->mfe_exp.fsal->name,
+		     atomic_fetch_int32_t(&myself->mfe_exp.fsal->refcount));
+
+	fsal_export_stack(op_ctx->fsal_export, &myself->mfe_exp);
 
 
 	/* Set up op_ctx */
-	op_ctx->fsal_export = &myself->export;
+	op_ctx->fsal_export = &myself->mfe_exp;
 	op_ctx->fsal_module = &MDCACHE.fsal;
 
+	/* Stacking is setup and ready to take upcalls now */
+	up_ready_set(&myself->up_ops);
+
 	return status;
 }
 
@@ -337,23 +301,6 @@ mdcache_fsal_unload(struct fsal_module *fsal_hdl)
 	return retval;
 }
 
- /**
- * @brief Get the support_ex for the handle
- *
- * Just pass through to the underlying FSAL
- *
- * @param[in] obj_hdl	Handle to digest
- * @param[out] fh_desc	Buffer to write key into
- * @return True if supported, false otherwise
- */
-static bool mdcache_support_ex(struct fsal_obj_handle *obj_hdl)
-{
-	mdcache_entry_t *entry =
-		container_of(obj_hdl, mdcache_entry_t, obj_handle);
-
-	return entry->sub_handle->fsal->m_ops.support_ex(entry->sub_handle);
-}
-
 void mdcache_fsal_init(void)
 {
 	int retval;
@@ -368,7 +315,6 @@ void mdcache_fsal_init(void)
 	/*myself->m_ops.create_export = mdcache_fsal_create_export;*/
 	myself->m_ops.init_config = mdcache_fsal_init_config;
 	myself->m_ops.unload = mdcache_fsal_unload;
-	myself->m_ops.support_ex = mdcache_support_ex;
 }
 
 /**
@@ -401,6 +347,35 @@ fsal_status_t mdcache_pkginit(void)
 	return status;
 }
 
+/**
+ * @brief Check if FDs are available.
+ *
+ * This function checks if FDs are available to serve open
+ * requests. This function also wakes the LRU thread if the
+ * current FD count is above the high water mark.
+ *
+ * @return true if there are FDs available to serve open requests,
+ * false otherwise.
+ */
+bool mdcache_lru_fds_available(void)
+{
+	if ((atomic_fetch_size_t(&open_fd_count) >= lru_state.fds_hard_limit)
+		&& lru_state.caching_fds) {
+		LogCrit(COMPONENT_CACHE_INODE_LRU,
+			"FD Hard Limit Exceeded.  Disabling FD Cache and waking LRU thread.");
+		lru_state.caching_fds = false;
+		lru_wake_thread();
+		return false;
+	}
+	if (atomic_fetch_size_t(&open_fd_count) >= lru_state.fds_hiwat) {
+		LogInfo(COMPONENT_CACHE_INODE_LRU,
+			"FDs above high water mark, waking LRU thread.");
+		lru_wake_thread();
+	}
+
+	return true;
+}
+
 #ifdef USE_DBUS
 void mdcache_dbus_show(DBusMessageIter *iter)
 {
