@@ -69,6 +69,11 @@ static void release(struct fsal_export *exp_hdl)
 	myself->export.sub_export->exp_ops.release(myself->export.sub_export);
 	fsal_put(sub_fsal);
 
+	LogFullDebug(COMPONENT_FSAL,
+		     "FSAL %s refcount %"PRIu32,
+		     sub_fsal->name,
+		     atomic_fetch_int32_t(&sub_fsal->refcount));
+
 	fsal_detach_export(exp_hdl->fsal, &exp_hdl->exports);
 	free_export_ops(exp_hdl);
 
@@ -343,15 +348,29 @@ static void nullfs_free_state(struct fsal_export *exp_hdl,
 	op_ctx->fsal_export = &exp->export;
 }
 
+static bool nullfs_is_superuser(struct fsal_export *exp_hdl,
+				const struct user_cred *creds)
+{
+	struct nullfs_fsal_export *exp = container_of(exp_hdl,
+					struct nullfs_fsal_export, export);
+	bool rv;
+
+	op_ctx->fsal_export = exp->export.sub_export;
+	rv = exp->export.sub_export->exp_ops.is_superuser(
+					exp->export.sub_export, creds);
+	op_ctx->fsal_export = &exp->export;
+
+	return rv;
+}
+
 
 /* extract a file handle from a buffer.
  * do verification checks and flag any and all suspicious bits.
  * Return an updated fh_desc into whatever was passed.  The most
- * common behavior, done here is to just reset the length.  There
- * is the option to also adjust the start pointer.
+ * common behavior, done here is to just reset the length.
  */
 
-static fsal_status_t extract_handle(struct fsal_export *exp_hdl,
+static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
 				    fsal_digesttype_t in_type,
 				    struct gsh_buffdesc *fh_desc,
 				    int flags)
@@ -361,13 +380,28 @@ static fsal_status_t extract_handle(struct fsal_export *exp_hdl,
 
 	op_ctx->fsal_export = exp->export.sub_export;
 	fsal_status_t result =
-		exp->export.sub_export->exp_ops.extract_handle(
+		exp->export.sub_export->exp_ops.wire_to_host(
 			exp->export.sub_export, in_type, fh_desc, flags);
 	op_ctx->fsal_export = &exp->export;
 
 	return result;
 }
 
+static fsal_status_t nullfs_host_to_key(struct fsal_export *exp_hdl,
+					  struct gsh_buffdesc *fh_desc)
+{
+	struct nullfs_fsal_export *exp =
+		container_of(exp_hdl, struct nullfs_fsal_export, export);
+
+	op_ctx->fsal_export = exp->export.sub_export;
+	fsal_status_t result =
+		exp->export.sub_export->exp_ops.host_to_key(
+			exp->export.sub_export, fh_desc);
+	op_ctx->fsal_export = &exp->export;
+
+	return result;
+}
+
 /* nullfs_export_ops_init
  * overwrite vector entries with the methods that we support
  */
@@ -376,7 +410,8 @@ void nullfs_export_ops_init(struct export_ops *ops)
 {
 	ops->release = release;
 	ops->lookup_path = nullfs_lookup_path;
-	ops->extract_handle = extract_handle;
+	ops->wire_to_host = wire_to_host;
+	ops->host_to_key = nullfs_host_to_key;
 	ops->create_handle = nullfs_create_handle;
 	ops->get_fs_dynamic_info = get_dynamic_info;
 	ops->fs_supports = fs_supports;
@@ -395,6 +430,7 @@ void nullfs_export_ops_init(struct export_ops *ops)
 	ops->set_quota = set_quota;
 	ops->alloc_state = nullfs_alloc_state;
 	ops->free_state = nullfs_free_state;
+	ops->is_superuser = nullfs_is_superuser;
 }
 
 struct nullfsal_args {
@@ -455,7 +491,7 @@ fsal_status_t nullfs_create_export(struct fsal_module *fsal_hdl,
 	fsal_stack = lookup_fsal(nullfsal.subfsal.name);
 	if (fsal_stack == NULL) {
 		LogMajor(COMPONENT_FSAL,
-			 "nullfs_create_export: failed to lookup for FSAL %s",
+			 "nullfs create export failed to lookup for FSAL %s",
 			 nullfsal.subfsal.name);
 		return fsalstat(ERR_FSAL_INVAL, EINVAL);
 	}
@@ -466,6 +502,12 @@ fsal_status_t nullfs_create_export(struct fsal_module *fsal_hdl,
 						 err_type,
 						 up_ops);
 	fsal_put(fsal_stack);
+
+	LogFullDebug(COMPONENT_FSAL,
+		     "FSAL %s refcount %"PRIu32,
+		     fsal_stack->name,
+		     atomic_fetch_int32_t(&fsal_stack->refcount));
+
 	if (FSAL_IS_ERROR(expres)) {
 		LogMajor(COMPONENT_FSAL,
 			 "Failed to call create_export on underlying FSAL %s",
