@@ -80,6 +80,8 @@ static struct nullfs_fsal_obj_handle *nullfs_alloc_handle(
 	result->obj_handle.fsid = sub_handle->fsid;
 	result->obj_handle.fileid = sub_handle->fileid;
 	result->obj_handle.fs = fs;
+	result->obj_handle.state_hdl = sub_handle->state_hdl;
+	result->refcnt = 1;
 
 	return result;
 }
@@ -99,7 +101,7 @@ static struct nullfs_fsal_obj_handle *nullfs_alloc_handle(
  *
  * @return An error code for the function.
  */
-static fsal_status_t nullfs_alloc_and_check_handle(
+fsal_status_t nullfs_alloc_and_check_handle(
 		struct nullfs_fsal_export *export,
 		struct fsal_obj_handle *sub_handle,
 		struct fsal_filesystem *fs,
@@ -152,36 +154,6 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 					     handle, status);
 }
 
-static fsal_status_t create(struct fsal_obj_handle *dir_hdl,
-			    const char *name, struct attrlist *attrs_in,
-			    struct fsal_obj_handle **new_obj,
-			    struct attrlist *attrs_out)
-{
-	/** Parent directory nullfs handle. */
-	struct nullfs_fsal_obj_handle *nullfs_dir =
-		container_of(dir_hdl, struct nullfs_fsal_obj_handle,
-			     obj_handle);
-	/** Current nullfs export. */
-	struct nullfs_fsal_export *export =
-		container_of(op_ctx->fsal_export, struct nullfs_fsal_export,
-			     export);
-
-	/** Subfsal handle of the new file.*/
-	struct fsal_obj_handle *sub_handle;
-
-	*new_obj = NULL;
-
-	/* creating the file with a subfsal handle. */
-	op_ctx->fsal_export = export->export.sub_export;
-	fsal_status_t status = nullfs_dir->sub_handle->obj_ops.create(
-		nullfs_dir->sub_handle, name, attrs_in, &sub_handle, attrs_out);
-	op_ctx->fsal_export = &export->export;
-
-	/* wraping the subfsal handle in a nullfs handle. */
-	return nullfs_alloc_and_check_handle(export, sub_handle, dir_hdl->fs,
-					     new_obj, status);
-}
-
 static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 			     const char *name, struct attrlist *attrs_in,
 			     struct fsal_obj_handle **new_obj,
@@ -214,7 +186,6 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 			      const char *name,
 			      object_file_type_t nodetype,
-			      fsal_dev_t *dev,
 			      struct attrlist *attrs_in,
 			      struct fsal_obj_handle **new_obj,
 			      struct attrlist *attrs_out)
@@ -236,7 +207,7 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 	/* Creating the node with a subfsal handle. */
 	op_ctx->fsal_export = export->export.sub_export;
 	fsal_status_t status = nullfs_dir->sub_handle->obj_ops.mknode(
-		nullfs_dir->sub_handle, name, nodetype, dev, attrs_in,
+		nullfs_dir->sub_handle, name, nodetype, attrs_in,
 		&sub_handle, attrs_out);
 	op_ctx->fsal_export = &export->export;
 
@@ -338,15 +309,24 @@ static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
  *
  * @return Result coming from the upper layer.
  */
-static bool nullfs_readdir_cb(const char *name, struct fsal_obj_handle *obj,
-			      struct attrlist *attrs,
-			      void *dir_state, fsal_cookie_t cookie)
+static enum fsal_dir_result nullfs_readdir_cb(
+					const char *name,
+					struct fsal_obj_handle *sub_handle,
+					struct attrlist *attrs,
+					void *dir_state, fsal_cookie_t cookie)
 {
 	struct nullfs_readdir_state *state =
 		(struct nullfs_readdir_state *) dir_state;
+	struct fsal_obj_handle *new_obj;
+
+	if (FSAL_IS_ERROR(nullfs_alloc_and_check_handle(state->exp, sub_handle,
+		sub_handle->fs, &new_obj, fsalstat(ERR_FSAL_NO_ERROR, 0)))) {
+		return false;
+	}
 
 	op_ctx->fsal_export = &state->exp->export;
-	bool result = state->cb(name, obj, attrs, state->dir_state, cookie);
+	enum fsal_dir_result result = state->cb(name, new_obj, attrs,
+						state->dir_state, cookie);
 
 	op_ctx->fsal_export = state->exp->export.sub_export;
 
@@ -393,6 +373,90 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	return status;
 }
 
+/**
+ * @brief Compute the readdir cookie for a given filename.
+ *
+ * Some FSALs are able to compute the cookie for a filename deterministically
+ * from the filename. They also have a defined order of entries in a directory
+ * based on the name (could be strcmp sort, could be strict alpha sort, could
+ * be deterministic order based on cookie - in any case, the dirent_cmp method
+ * will also be provided.
+ *
+ * The returned cookie is the cookie that can be passed as whence to FIND that
+ * directory entry. This is different than the cookie passed in the readdir
+ * callback (which is the cookie of the NEXT entry).
+ *
+ * @param[in]  parent  Directory file name belongs to.
+ * @param[in]  name    File name to produce the cookie for.
+ *
+ * @retval 0 if not supported.
+ * @returns The cookie value.
+ */
+
+fsal_cookie_t compute_readdir_cookie(struct fsal_obj_handle *parent,
+				     const char *name)
+{
+	fsal_cookie_t cookie;
+	struct nullfs_fsal_obj_handle *handle =
+		container_of(parent, struct nullfs_fsal_obj_handle,
+			     obj_handle);
+
+	struct nullfs_fsal_export *export =
+		container_of(op_ctx->fsal_export, struct nullfs_fsal_export,
+			     export);
+
+	/* calling subfsal method */
+	op_ctx->fsal_export = export->export.sub_export;
+	cookie = handle->sub_handle->obj_ops.compute_readdir_cookie(
+						handle->sub_handle, name);
+	op_ctx->fsal_export = &export->export;
+	return cookie;
+}
+
+/**
+ * @brief Help sort dirents.
+ *
+ * For FSALs that are able to compute the cookie for a filename
+ * deterministically from the filename, there must also be a defined order of
+ * entries in a directory based on the name (could be strcmp sort, could be
+ * strict alpha sort, could be deterministic order based on cookie).
+ *
+ * Although the cookies could be computed, the caller will already have them
+ * and thus will provide them to save compute time.
+ *
+ * @param[in]  parent   Directory entries belong to.
+ * @param[in]  name1    File name of first dirent
+ * @param[in]  cookie1  Cookie of first dirent
+ * @param[in]  name2    File name of second dirent
+ * @param[in]  cookie2  Cookie of second dirent
+ *
+ * @retval < 0 if name1 sorts before name2
+ * @retval == 0 if name1 sorts the same as name2
+ * @retval >0 if name1 sorts after name2
+ */
+
+int dirent_cmp(struct fsal_obj_handle *parent,
+	       const char *name1, fsal_cookie_t cookie1,
+	       const char *name2, fsal_cookie_t cookie2)
+{
+	int rc;
+	struct nullfs_fsal_obj_handle *handle =
+		container_of(parent, struct nullfs_fsal_obj_handle,
+			     obj_handle);
+
+	struct nullfs_fsal_export *export =
+		container_of(op_ctx->fsal_export, struct nullfs_fsal_export,
+			     export);
+
+	/* calling subfsal method */
+	op_ctx->fsal_export = export->export.sub_export;
+	rc = handle->sub_handle->obj_ops.dirent_cmp(handle->sub_handle,
+						    name1, cookie1,
+						    name2, cookie2);
+	op_ctx->fsal_export = &export->export;
+	return rc;
+}
+
 static fsal_status_t renamefile(struct fsal_obj_handle *obj_hdl,
 				struct fsal_obj_handle *olddir_hdl,
 				const char *old_name,
@@ -444,13 +508,10 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 	return status;
 }
 
-/*
- * NOTE: this is done under protection of the
- * attributes rwlock in the cache entry.
- */
-
-static fsal_status_t setattrs(struct fsal_obj_handle *obj_hdl,
-			      struct attrlist *attrs)
+static fsal_status_t nullfs_setattr2(struct fsal_obj_handle *obj_hdl,
+				     bool bypass,
+				     struct state_t *state,
+				     struct attrlist *attrs)
 {
 	struct nullfs_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct nullfs_fsal_obj_handle,
@@ -462,8 +523,8 @@ static fsal_status_t setattrs(struct fsal_obj_handle *obj_hdl,
 
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
-	fsal_status_t status = handle->sub_handle->obj_ops.setattrs(
-		handle->sub_handle, attrs);
+	fsal_status_t status = handle->sub_handle->obj_ops.setattr2(
+		handle->sub_handle, bypass, state, attrs);
 	op_ctx->fsal_export = &export->export;
 
 	return status;
@@ -496,16 +557,16 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 	return status;
 }
 
-/* handle_digest
+/* handle_to_wire
  * fill in the opaque f/s file handle part.
  * we zero the buffer to length first.  This MAY already be done above
  * at which point, remove memset here because the caller is zeroing
  * the whole struct.
  */
 
-static fsal_status_t handle_digest(const struct fsal_obj_handle *obj_hdl,
-				   fsal_digesttype_t output_type,
-				   struct gsh_buffdesc *fh_desc)
+static fsal_status_t handle_to_wire(const struct fsal_obj_handle *obj_hdl,
+				    fsal_digesttype_t output_type,
+				    struct gsh_buffdesc *fh_desc)
 {
 	struct nullfs_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct nullfs_fsal_obj_handle,
@@ -517,7 +578,7 @@ static fsal_status_t handle_digest(const struct fsal_obj_handle *obj_hdl,
 
 	/* calling subfsal method */
 	op_ctx->fsal_export = export->export.sub_export;
-	fsal_status_t status = handle->sub_handle->obj_ops.handle_digest(
+	fsal_status_t status = handle->sub_handle->obj_ops.handle_to_wire(
 		handle->sub_handle, output_type, fh_desc);
 	op_ctx->fsal_export = &export->export;
 
@@ -550,7 +611,7 @@ static void handle_to_key(struct fsal_obj_handle *obj_hdl,
 
 /*
  * release
- * release our export first so they know we are gone
+ * release our handle first so they know we are gone
  */
 
 static void release(struct fsal_obj_handle *obj_hdl)
@@ -578,26 +639,34 @@ void nullfs_handle_ops_init(struct fsal_obj_ops *ops)
 	ops->release = release;
 	ops->lookup = lookup;
 	ops->readdir = read_dirents;
-	ops->create = create;
+	ops->compute_readdir_cookie = compute_readdir_cookie,
+	ops->dirent_cmp = dirent_cmp,
 	ops->mkdir = makedir;
 	ops->mknode = makenode;
 	ops->symlink = makesymlink;
 	ops->readlink = readsymlink;
 	ops->getattrs = getattrs;
-	ops->setattrs = setattrs;
 	ops->link = linkfile;
 	ops->rename = renamefile;
 	ops->unlink = file_unlink;
-	ops->open = nullfs_open;
-	ops->status = nullfs_status;
-	ops->read = nullfs_read;
-	ops->write = nullfs_write;
-	ops->commit = nullfs_commit;
-	ops->lock_op = nullfs_lock_op;
 	ops->close = nullfs_close;
-	ops->handle_digest = handle_digest;
+	ops->handle_to_wire = handle_to_wire;
 	ops->handle_to_key = handle_to_key;
 
+	/* Multi-FD */
+	ops->open2 = nullfs_open2;
+	ops->check_verifier = nullfs_check_verifier;
+	ops->status2 = nullfs_status2;
+	ops->reopen2 = nullfs_reopen2;
+	ops->read2 = nullfs_read2;
+	ops->write2 = nullfs_write2;
+	ops->seek2 = nullfs_seek2;
+	ops->io_advise2 = nullfs_io_advise2;
+	ops->commit2 = nullfs_commit2;
+	ops->lock_op2 = nullfs_lock_op2;
+	ops->setattr2 = nullfs_setattr2;
+	ops->close2 = nullfs_close2;
+
 	/* xattr related functions */
 	ops->list_ext_attrs = nullfs_list_ext_attrs;
 	ops->getextattr_id_by_name = nullfs_getextattr_id_by_name;
