@@ -418,8 +418,6 @@ void display_fsinfo(struct fsal_staticfsinfo_t *info)
 	LogDebug(COMPONENT_FSAL, "  symlink_support  = %d  ",
 		 info->symlink_support);
 	LogDebug(COMPONENT_FSAL, "  lock_support  = %d  ", info->lock_support);
-	LogDebug(COMPONENT_FSAL, "  lock_support_owner  = %d  ",
-		 info->lock_support_owner);
 	LogDebug(COMPONENT_FSAL, "  lock_support_async_block  = %d  ",
 		 info->lock_support_async_block);
 	LogDebug(COMPONENT_FSAL, "  named_attr  = %d  ", info->named_attr);
@@ -437,10 +435,6 @@ void display_fsinfo(struct fsal_staticfsinfo_t *info)
 		 info->auth_exportpath_xdev);
 	LogDebug(COMPONENT_FSAL, "  xattr_access_rights = %#o ",
 		 info->xattr_access_rights);
-	LogDebug(COMPONENT_FSAL, "  share_support  = %d  ",
-		 info->share_support);
-	LogDebug(COMPONENT_FSAL, "  share_support_owner  = %d  ",
-		 info->share_support_owner);
 	LogDebug(COMPONENT_FSAL, "  delegations = %d  ",
 		 info->delegations);
 	LogDebug(COMPONENT_FSAL, "  pnfs_mds = %d  ",
@@ -668,23 +662,25 @@ fsal_fs_cmpf_fsid(const struct avltree_node *lhs,
 	lk = avltree_container_of(lhs, struct fsal_filesystem, avl_fsid);
 	rk = avltree_container_of(rhs, struct fsal_filesystem, avl_fsid);
 
+	if (lk->fsid_type < rk->fsid_type)
+		return -1;
+
+	if (lk->fsid_type > rk->fsid_type)
+		return 1;
+
 	if (lk->fsid.major < rk->fsid.major)
 		return -1;
 
 	if (lk->fsid.major > rk->fsid.major)
 		return 1;
 
-	if (lk->fsid_type == FSID_MAJOR_64 &&
-	    rk->fsid_type == FSID_MAJOR_64)
+	/* No need to compare minors as they should be
+	 * zeros if the type is FSID_MAJOR_64
+	 */
+	if (lk->fsid_type == FSID_MAJOR_64) {
+		assert(rk->fsid_type == FSID_MAJOR_64);
 		return 0;
-
-	/* Treat no minor as strictly less than any minor */
-	if (lk->fsid_type == FSID_MAJOR_64)
-		return -1;
-
-	/* Treat no minor as strictly less than any minor */
-	if (rk->fsid_type == FSID_MAJOR_64)
-		return 1;
+	}
 
 	if (lk->fsid.minor < rk->fsid.minor)
 		return -1;
@@ -1031,11 +1027,11 @@ out:
 #endif
 	fs->fsid_type = FSID_TWO_UINT32;
 #if __FreeBSD__
-	fs->fsid.major = (unsigned) stat_fs.f_fsid.val[0];
-	fs->fsid.minor = (unsigned) stat_fs.f_fsid.val[1];
+	fs->fsid.major = (unsigned int) stat_fs.f_fsid.val[0];
+	fs->fsid.minor = (unsigned int) stat_fs.f_fsid.val[1];
 #else
-	fs->fsid.major = (unsigned) stat_fs.f_fsid.__val[0];
-	fs->fsid.minor = (unsigned) stat_fs.f_fsid.__val[1];
+	fs->fsid.major = (unsigned int) stat_fs.f_fsid.__val[0];
+	fs->fsid.minor = (unsigned int) stat_fs.f_fsid.__val[1];
 #endif
 	if ((fs->fsid.major == 0) && (fs->fsid.minor == 0)) {
 		fs->fsid.major = fs->dev.major;
@@ -2675,6 +2671,8 @@ again:
  * @param[out]    has_lock       Indicates that obj_hdl->obj_lock is held read
  * @param[out]    closefd        Indicates that file descriptor must be closed
  * @param[in]     open_for_locks Indicates file is open for locks
+ * @param[out]    reusing_open_state_fd Indicates whether already opened fd
+ *					can be reused
  *
  * @return FSAL status.
  */
@@ -2690,7 +2688,8 @@ fsal_status_t fsal_find_fd(struct fsal_fd **out_fd,
 			   fsal_close_func close_func,
 			   bool *has_lock,
 			   bool *closefd,
-			   bool open_for_locks)
+			   bool open_for_locks,
+			   bool *reusing_open_state_fd)
 {
 	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
 	struct fsal_fd *state_fd;
@@ -2774,8 +2773,16 @@ fsal_status_t fsal_find_fd(struct fsal_fd **out_fd,
 			 */
 			LogFullDebug(COMPONENT_FSAL,
 				     "Use related_fd %p", related_fd);
-			if (out_fd)
+			if (out_fd) {
 				*out_fd = related_fd;
+				/* The associated open state has an open fd,
+				 * however some FSALs can not use it and must
+				 * need to dup the fd into the lock state
+				 * instead. So to signal this to the caller
+				 * function the following flag
+				 */
+				*reusing_open_state_fd = true;
+			}
 
 			*has_lock = false;
 			return status;
