@@ -220,15 +220,26 @@ static void fsal_pnfs_ds_ops(struct fsal_pnfs_ds_ops *ops)
 }
 
 /**
- * @brief Indicate support for extended operations.
+ * @brief Provides function to extract FSAL stats
  *
- * @param[in]  obj	Object being operated on
- *
- * @retval true if extended operations are supported.
+ * @param[in] fsal_hdl          FSAL module
+ * @param[in] iter              opaque pointer to DBusMessageIter
  */
-static bool support_ex(struct fsal_obj_handle *obj)
+static void fsal_extract_stats(struct fsal_module *const fsal_hdl, void *iter)
 {
-	return false;
+	LogDebug(COMPONENT_FSAL,
+		"Invoking unsupported FSAL operation");
+}
+
+/**
+ * @brief FSAL function to reset FSAL stats
+ *
+ * @param[in] fsal_hdl          FSAL module
+ */
+static void fsal_reset_stats(struct fsal_module *const fsal_hdl)
+{
+	LogDebug(COMPONENT_FSAL,
+		"Invoking unsupported FSAL operation");
 }
 
 /* Default fsal module method vector.
@@ -245,7 +256,8 @@ struct fsal_ops def_fsal_ops = {
 	.fs_da_addr_size = fs_da_addr_size,
 	.fsal_pnfs_ds = fsal_pnfs_ds,
 	.fsal_pnfs_ds_ops = fsal_pnfs_ds_ops,
-	.support_ex = support_ex,
+	.fsal_extract_stats = fsal_extract_stats,
+	.fsal_reset_stats = fsal_reset_stats,
 };
 
 /* get_name
@@ -297,20 +309,30 @@ static fsal_status_t lookup_junction(struct fsal_export *exp_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/* extract_handle
+/* wire_to_host
  * default case is not supported
  */
 
-static fsal_status_t extract_handle(struct fsal_export *exp_hdl,
-				    fsal_digesttype_t in_type,
-				    struct gsh_buffdesc *fh_desc,
-				    int flags)
+static fsal_status_t wire_to_host(struct fsal_export *exp_hdl,
+				  fsal_digesttype_t in_type,
+				  struct gsh_buffdesc *fh_desc,
+				  int flags)
 {
 	LogCrit(COMPONENT_FSAL,
 		"Invoking unsupported FSAL operation");
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
+/* exp_host_to_key  Produce handle-key from host-handle
+ *
+ * default case is that "handle-key" is same as host-handle!
+ */
+static fsal_status_t exp_host_to_key(struct fsal_export *exp_hdl,
+				       struct gsh_buffdesc *fh_desc)
+{
+	return fsalstat(ERR_FSAL_NO_ERROR, 0);
+}
+
 /* create_handle
  * default case is not supported
  */
@@ -566,9 +588,9 @@ static void global_verifier(struct fsal_export *exp_hdl,
  * @returns a state structure.
  */
 
-struct state_t *alloc_state(struct fsal_export *exp_hdl,
-			    enum state_type state_type,
-			    struct state_t *related_state)
+static struct state_t *alloc_state(struct fsal_export *exp_hdl,
+				   enum state_type state_type,
+				   struct state_t *related_state)
 {
 	return init_state(gsh_calloc(1, sizeof(struct state_t)),
 			  exp_hdl, state_type, related_state);
@@ -587,6 +609,20 @@ void free_state(struct fsal_export *exp_hdl, struct state_t *state)
 	gsh_free(state);
 }
 
+/**
+ * @brief Check to see if a user is superuser
+ *
+ * @param[in] exp_hdl               Export state_t is associated with
+ * @param[in] creds                 Credentials to check for superuser
+ *
+ * @returns NULL on failure otherwise a state structure.
+ */
+
+bool is_superuser(struct fsal_export *exp_hdl, const struct user_cred *creds)
+{
+	return (creds->caller_uid == 0);
+}
+
 /* Default fsal export method vector.
  * copied to allocated vector at register time
  */
@@ -597,7 +633,8 @@ struct export_ops def_export_ops = {
 	.release = export_release,
 	.lookup_path = lookup_path,
 	.lookup_junction = lookup_junction,
-	.extract_handle = extract_handle,
+	.wire_to_host = wire_to_host,
+	.host_to_key = exp_host_to_key,
 	.create_handle = create_handle,
 	.get_fs_dynamic_info = get_dynamic_info,
 	.fs_supports = fs_supports,
@@ -623,6 +660,7 @@ struct export_ops def_export_ops = {
 	.get_write_verifier = global_verifier,
 	.alloc_state = alloc_state,
 	.free_state = free_state,
+	.is_superuser = is_superuser,
 };
 
 /* fsal_obj_handle common methods
@@ -720,18 +758,27 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
-/* create
- * default case not supported
+/* compute_readdir_cookie
+ * default is to return 0 which indicates not supported
  */
 
-static fsal_status_t create(struct fsal_obj_handle *dir_hdl,
-			    const char *name, struct attrlist *attrib,
-			    struct fsal_obj_handle **handle,
-			    struct attrlist *attrs_out)
+fsal_cookie_t compute_readdir_cookie(struct fsal_obj_handle *parent,
+				     const char *name)
 {
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
+	return 0;
+}
+
+/* dirent_cmp
+ * Sort dirents by name.
+ */
+
+int dirent_cmp(struct fsal_obj_handle *parent,
+	       const char *name1, fsal_cookie_t cookie1,
+	       const char *name2, fsal_cookie_t cookie2)
+{
+	/* Not supported by default. */
+	assert(false);
+	return 0;
 }
 
 /* makedir
@@ -754,7 +801,7 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 
 static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 			      const char *name, object_file_type_t nodetype,
-			      fsal_dev_t *dev, struct attrlist *attrib,
+			      struct attrlist *attrib,
 			      struct fsal_obj_handle **handle,
 			      struct attrlist *attrs_out)
 {
@@ -858,18 +905,6 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
-/* setattrs
- * default case not supported
- */
-
-static fsal_status_t setattrs(struct fsal_obj_handle *obj_hdl,
-			      struct attrlist *attrs)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
 /* linkfile
  * default case not supported
  */
@@ -911,30 +946,6 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
-/* file_open
- * default case not supported
- */
-
-static fsal_status_t file_open(struct fsal_obj_handle *obj_hdl,
-			       fsal_openflags_t openflags)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
-/* file_reopen
- * default case not supported
- */
-
-static fsal_status_t file_reopen(struct fsal_obj_handle *obj_hdl,
-				 fsal_openflags_t openflags)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
 /* fs_locations
  * default case not supported
  */
@@ -945,71 +956,6 @@ static fsal_status_t fs_locations(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
-/* file_status
- * default case file always closed
- */
-
-static fsal_openflags_t file_status(struct fsal_obj_handle *obj_hdl)
-{
-	return FSAL_O_CLOSED;
-}
-
-/* file_read
- * default case not supported
- */
-
-static fsal_status_t file_read(struct fsal_obj_handle *obj_hdl,
-			       uint64_t seek_descriptor, size_t buffer_size,
-			       void *buffer, size_t *read_amount,
-			       bool *end_of_file)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
-/* file_read_plus
- * default case not supported
- */
-
-static fsal_status_t file_read_plus(struct fsal_obj_handle *obj_hdl,
-			       uint64_t seek_descriptor, size_t buffer_size,
-			       void *buffer, size_t *read_amount,
-			       bool *end_of_file, struct io_info *info)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
-/* file_write
- * default case not supported
- */
-
-static fsal_status_t file_write(struct fsal_obj_handle *obj_hdl,
-				uint64_t seek_descriptor, size_t buffer_size,
-				void *buffer, size_t *write_amount,
-				bool *fsal_stable)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
-/* file_write_plus
- * default case not supported
- */
-
-static fsal_status_t file_write_plus(struct fsal_obj_handle *obj_hdl,
-				uint64_t seek_descriptor, size_t buffer_size,
-				void *buffer, size_t *write_amount,
-				bool *fsal_stable, struct io_info *info)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
 /* seek
  * default case not supported
  */
@@ -1034,46 +980,6 @@ static fsal_status_t file_io_advise(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/* commit
- * default case not supported
- */
-
-static fsal_status_t commit(struct fsal_obj_handle *obj_hdl,	/* sync */
-			    off_t offset, size_t len)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
-/* lock_op
- * default case not supported
- */
-
-static fsal_status_t lock_op(struct fsal_obj_handle *obj_hdl,
-			     void *p_owner,
-			     fsal_lock_op_t lock_op,
-			     fsal_lock_param_t *request_lock,
-			     fsal_lock_param_t *conflicting_lock)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
-/* share_op
- * default case not supported
- */
-
-static fsal_status_t share_op(struct fsal_obj_handle *obj_hdl,
-			      void *p_owner,
-			      fsal_share_param_t request_share)
-{
-	LogCrit(COMPONENT_FSAL,
-		"Invoking unsupported FSAL operation");
-	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
-}
-
 /* file_close
  * default case not supported
  */
@@ -1196,13 +1102,13 @@ static fsal_status_t remove_extattr_by_name(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
-/* handle_digest
+/* handle_to_wire
  * default case server fault
  */
 
-static fsal_status_t handle_digest(const struct fsal_obj_handle *obj_hdl,
-				   fsal_digesttype_t output_type,
-				   struct gsh_buffdesc *fh_desc)
+static fsal_status_t handle_to_wire(const struct fsal_obj_handle *obj_hdl,
+				    fsal_digesttype_t output_type,
+				    struct gsh_buffdesc *fh_desc)
 {
 	return fsalstat(ERR_FSAL_SERVERFAULT, 0);
 }
@@ -1339,7 +1245,6 @@ static fsal_status_t open2(struct fsal_obj_handle *obj_hdl,
 static bool check_verifier(struct fsal_obj_handle *obj_hdl,
 			   fsal_verifier_t verifier)
 {
-	uint32_t verf_hi = 0, verf_lo = 0;
 	struct attrlist attrs;
 	bool result;
 
@@ -1348,22 +1253,7 @@ static bool check_verifier(struct fsal_obj_handle *obj_hdl,
 	if (FSAL_IS_ERROR(obj_hdl->obj_ops.getattrs(obj_hdl, &attrs)))
 		return false;
 
-	memcpy(&verf_hi,
-	       verifier,
-	       sizeof(uint32_t));
-	memcpy(&verf_lo,
-	       verifier + sizeof(uint32_t),
-	       sizeof(uint32_t));
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "Passed verifier %"PRIx32" %"PRIx32
-		     " file verifier %"PRIx32" %"PRIx32,
-		     verf_hi, verf_lo,
-		     (uint32_t) attrs.atime.tv_sec,
-		     (uint32_t) attrs.mtime.tv_sec);
-
-	result = attrs.atime.tv_sec == verf_hi &&
-		 attrs.mtime.tv_sec == verf_lo;
+	result = check_verifier_attrlist(&attrs, verifier);
 
 	/* Done with the attrs */
 	fsal_release_attrs(&attrs);
@@ -1488,6 +1378,20 @@ static fsal_status_t lock_op2(struct fsal_obj_handle *obj_hdl,
 	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 }
 
+/* lease_op2
+ * default case not supported
+ */
+
+static fsal_status_t lease_op2(struct fsal_obj_handle *obj_hdl,
+				  struct state_t *state,
+				  void *owner,
+				  fsal_deleg_t deleg)
+{
+	LogCrit(COMPONENT_FSAL,
+		"Invoking unsupported FSAL operation");
+	return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
+}
+
 /* setattr2
  * default case not supported
  */
@@ -1525,30 +1429,20 @@ struct fsal_obj_ops def_handle_ops = {
 	.merge = handle_merge,
 	.lookup = lookup,
 	.readdir = read_dirents,
-	.create = create,
+	.compute_readdir_cookie = compute_readdir_cookie,
+	.dirent_cmp = dirent_cmp,
 	.mkdir = makedir,
 	.mknode = makenode,
 	.symlink = makesymlink,
 	.readlink = readsymlink,
 	.test_access = fsal_test_access,	/* default is use common test */
 	.getattrs = getattrs,
-	.setattrs = setattrs,
 	.link = linkfile,
 	.rename = renamefile,
 	.unlink = file_unlink,
-	.open = file_open,
-	.reopen = file_reopen,
 	.fs_locations = fs_locations,
-	.status = file_status,
-	.read = file_read,
-	.read_plus = file_read_plus,
-	.write = file_write,
-	.write_plus = file_write_plus,
 	.seek = file_seek,
 	.io_advise = file_io_advise,
-	.commit = commit,
-	.lock_op = lock_op,
-	.share_op = share_op,
 	.close = file_close,
 	.list_ext_attrs = list_ext_attrs,
 	.getextattr_id_by_name = getextattr_id_by_name,
@@ -1559,7 +1453,7 @@ struct fsal_obj_ops def_handle_ops = {
 	.remove_extattr_by_id = remove_extattr_by_id,
 	.remove_extattr_by_name = remove_extattr_by_name,
 	.handle_is = handle_is,
-	.handle_digest = handle_digest,
+	.handle_to_wire = handle_to_wire,
 	.handle_cmp = handle_cmp,
 	.handle_to_key = handle_to_key,
 	.layoutget = layoutget,
@@ -1579,6 +1473,7 @@ struct fsal_obj_ops def_handle_ops = {
 	.io_advise2 = io_advise2,
 	.commit2 = commit2,
 	.lock_op2 = lock_op2,
+	.lease_op2 = lease_op2,
 	.setattr2 = setattr2,
 	.close2 = close2,
 };
