@@ -47,6 +47,7 @@
 #include "nfs_exports.h"
 #include "nfs4_acls.h"
 #include "sal_data.h"
+#include "sal_functions.h"
 #include "FSAL/fsal_commonlib.h"
 
 /**
@@ -59,20 +60,7 @@
  * them as to usage or instruct them to close them.
  */
 
-size_t open_fd_count = 0;
-
-/* XXX dang locking
- * - FD locking (open, close, is_open) - was content lock
- *   will be fixed in conversion to support_ex
- */
-
-
-bool fsal_is_open(struct fsal_obj_handle *obj)
-{
-	if ((obj == NULL) || (obj->type != REGULAR_FILE))
-		return false;
-	return (obj->obj_ops.status(obj) != FSAL_O_CLOSED);
-}
+size_t open_fd_count;
 
 static bool fsal_not_in_group_list(gid_t gid)
 {
@@ -160,7 +148,7 @@ static fsal_status_t check_open_permission(struct fsal_obj_handle *obj,
 	 *       because somehow the owner changed.
 	 *
 	 */
-	status = fsal_access(obj, FSAL_EXECUTE_ACCESS, NULL, NULL);
+	status = fsal_access(obj, FSAL_EXECUTE_ACCESS);
 
 	if (!FSAL_IS_ERROR(status))
 		*reason = "";
@@ -194,14 +182,15 @@ static fsal_status_t fsal_check_setattr_perms(struct fsal_obj_handle *obj,
 
 	/* Shortcut, if current user is root, then we can just bail out with
 	 * success. */
-	if (creds->caller_uid == 0) {
+	if (op_ctx->fsal_export->exp_ops.is_superuser(op_ctx->fsal_export,
+						      creds)) {
 		note = " (Ok for root user)";
 		goto out;
 	}
 
 	fsal_prepare_attrs(current,
-			   op_ctx->fsal_export->
-				exp_ops.fs_supported_attrs(op_ctx->fsal_export)
+			   op_ctx->fsal_export->exp_ops.fs_supported_attrs(
+							op_ctx->fsal_export)
 			   & (ATTRS_CREDS | ATTR_MODE | ATTR_ACL));
 
 	status = obj->obj_ops.getattrs(obj, current);
@@ -229,7 +218,12 @@ static fsal_status_t fsal_check_setattr_perms(struct fsal_obj_handle *obj,
 				    "Change OWNER requires FSAL_ACE_PERM_WRITE_OWNER");
 		}
 	}
-	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_GROUP)) {
+	/* Check if we are changing the owner_group, if owner_group is passed,
+	 * but is the current owner_group, then that will be considered a
+	 * NO-OP and allowed IF the caller is the owner of the file.
+	 */
+	if (FSAL_TEST_MASK(attr->valid_mask, ATTR_GROUP) &&
+	    (attr->group != current->group || not_owner)) {
 		/* non-root is only allowed to change group_owner to a group
 		 * user is a member of. */
 		int not_in_group = fsal_not_in_group_list(attr->group);
@@ -392,7 +386,7 @@ fsal_status_t open2_by_name(struct fsal_obj_handle *in_obj,
 	}
 
 	/* Check directory permission for LOOKUP */
-	status = fsal_access(in_obj, FSAL_EXECUTE_ACCESS, NULL, NULL);
+	status = fsal_access(in_obj, FSAL_EXECUTE_ACCESS);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
@@ -415,6 +409,10 @@ fsal_status_t open2_by_name(struct fsal_obj_handle *in_obj,
 		return status;
 	}
 
+	if (!state) {
+		(void) atomic_inc_size_t(&open_fd_count);
+	}
+
 	LogFullDebug(COMPONENT_FSAL,
 		     "Created entry %p FSAL %s for %s",
 		     *obj, (*obj)->fsal->name, name);
@@ -436,7 +434,7 @@ fsal_status_t open2_by_name(struct fsal_obj_handle *in_obj,
 	if (state != NULL)
 		close_status = (*obj)->obj_ops.close2(*obj, state);
 	else
-		close_status = (*obj)->obj_ops.close(*obj);
+		close_status = fsal_close(*obj);
 
 	if (FSAL_IS_ERROR(close_status)) {
 		/* Just log but don't return this error (we want to
@@ -471,6 +469,7 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 	fsal_status_t status = { 0, 0 };
 	const struct user_cred *creds = op_ctx->creds;
 	struct attrlist current;
+	bool is_superuser;
 
 	if ((attr->valid_mask & (ATTR_SIZE | ATTR4_SPACE_RESERVED))
 	     && (obj->type != REGULAR_FILE)) {
@@ -496,6 +495,8 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 	if (FSAL_IS_ERROR(status))
 		return status;
 
+	is_superuser = op_ctx->fsal_export->exp_ops.is_superuser(
+					op_ctx->fsal_export, creds);
 	/* Test for the following condition from chown(2):
 	 *
 	 *     When the owner or group of an executable file are changed by an
@@ -507,7 +508,7 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 	 *     mandatory locking, and is not cleared by a chown().
 	 *
 	 */
-	if (creds->caller_uid != 0 &&
+	if (!is_superuser &&
 	    (FSAL_TEST_MASK(attr->valid_mask, ATTR_OWNER) ||
 	     FSAL_TEST_MASK(attr->valid_mask, ATTR_GROUP)) &&
 	    ((current.mode & (S_IXOTH | S_IXUSR | S_IXGRP)) != 0) &&
@@ -545,7 +546,7 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 	 * We test the actual mode being set before testing for group
 	 * membership since that is a bit more expensive.
 	 */
-	if (creds->caller_uid != 0 &&
+	if (!is_superuser &&
 	    FSAL_TEST_MASK(attr->valid_mask, ATTR_MODE) &&
 	    (attr->mode & S_ISGID) != 0 &&
 	    fsal_not_in_group_list(current.group)) {
@@ -553,27 +554,16 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 		attr->mode &= ~S_ISGID;
 	}
 
-	if (obj->fsal->m_ops.support_ex(obj)) {
-		status = obj->obj_ops.setattr2(obj, bypass, state, attr);
-		if (FSAL_IS_ERROR(status)) {
-			if (status.major == ERR_FSAL_STALE) {
-				LogEvent(COMPONENT_FSAL,
-					 "FSAL returned STALE from setattr2");
-			}
-			return status;
-		}
-	} else {
-		status = obj->obj_ops.setattrs(obj, attr);
-		if (FSAL_IS_ERROR(status)) {
-			if (status.major == ERR_FSAL_STALE) {
-				LogEvent(COMPONENT_FSAL,
-					 "FSAL returned STALE from setattrs");
-			}
-			return status;
+	status = obj->obj_ops.setattr2(obj, bypass, state, attr);
+	if (FSAL_IS_ERROR(status)) {
+		if (status.major == ERR_FSAL_STALE) {
+			LogEvent(COMPONENT_FSAL,
+				 "FSAL returned STALE from setattr2");
 		}
+		return status;
 	}
 
-	if (creds->caller_uid != 0) {
+	if (!is_superuser)  {
 		/* Done with the current attrs */
 		fsal_release_attrs(&current);
 	}
@@ -581,28 +571,6 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/**
- *
- * @brief Checks the permissions on an object
- *
- * This function returns success if the supplied credentials possess
- * permission required to meet the specified access.
- *
- * @param[in]  obj         The object to be checked
- * @param[in]  access_type The kind of access to be checked
- *
- * @return FSAL status
- *
- */
-fsal_status_t fsal_access(struct fsal_obj_handle *obj,
-			  fsal_accessflags_t access_type,
-			  fsal_accessflags_t *allowed,
-			  fsal_accessflags_t *denied)
-{
-	return
-	    obj->obj_ops.test_access(obj, access_type, allowed, denied, false);
-}
-
 /**
  * @brief Read the contents of a symlink
  *
@@ -648,6 +616,10 @@ fsal_status_t fsal_link(struct fsal_obj_handle *obj,
 	if (dest_dir->type != DIRECTORY)
 		return fsalstat(ERR_FSAL_NOTDIR, 0);
 
+	/* Must be the same FS */
+	if (obj->fs != dest_dir->fs)
+		return fsalstat(ERR_FSAL_XDEV, 0);
+
 	if (!op_ctx->fsal_export->exp_ops.fs_supports(
 			op_ctx->fsal_export,
 			fso_link_supports_permission_checks)) {
@@ -655,8 +627,7 @@ fsal_status_t fsal_link(struct fsal_obj_handle *obj,
 			FSAL_MODE_MASK_SET(FSAL_W_OK) |
 			FSAL_MODE_MASK_SET(FSAL_X_OK) |
 			FSAL_ACE4_MASK_SET(FSAL_ACE_PERM_EXECUTE) |
-			FSAL_ACE4_MASK_SET(FSAL_ACE_PERM_ADD_FILE),
-			NULL, NULL);
+			FSAL_ACE4_MASK_SET(FSAL_ACE_PERM_ADD_FILE));
 
 		if (FSAL_IS_ERROR(status))
 			return status;
@@ -697,14 +668,14 @@ fsal_status_t fsal_lookup(struct fsal_obj_handle *parent,
 		return fsalstat(ERR_FSAL_NOTDIR, 0);
 	}
 
-	fsal_status = fsal_access(parent, access_mask, NULL, NULL);
+	fsal_status = fsal_access(parent, access_mask);
 	if (FSAL_IS_ERROR(fsal_status))
 		return fsal_status;
 
 	if (strcmp(name, ".") == 0) {
 		parent->obj_ops.get_ref(parent);
 		*obj = parent;
-		return fsalstat(ERR_FSAL_NO_ERROR, 0);
+		return get_optional_attrs(*obj, attrs_out);
 	} else if (strcmp(name, "..") == 0)
 		return fsal_lookupp(parent, obj, attrs_out);
 
@@ -732,7 +703,9 @@ fsal_status_t fsal_lookupp(struct fsal_obj_handle *obj,
 		fsal_status_t status = {0, 0};
 		struct fsal_obj_handle *root_obj = NULL;
 
-		status = nfs_export_get_root_entry(op_ctx->ctx_export, &root_obj);
+		status = nfs_export_get_root_entry(op_ctx->ctx_export,
+						   &root_obj);
+
 		if (FSAL_IS_ERROR(status))
 			return status;
 
@@ -820,9 +793,6 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 {
 	fsal_status_t status = { 0, 0 };
 	attrmask_t orig_mask = attrs->valid_mask;
-	uint64_t owner = attrs->owner;
-	uint64_t group = attrs->group;
-	bool support_ex = parent->fsal->m_ops.support_ex(parent);
 
 	if ((type != REGULAR_FILE) && (type != DIRECTORY)
 	    && (type != SYMBOLIC_LINK) && (type != SOCKET_FILE)
@@ -836,22 +806,16 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 		goto out;
 	}
 
-	if (!support_ex) {
-		/* For old API, make sure owner/group attr matches op_ctx */
-		attrs->valid_mask |= ATTR_OWNER | ATTR_GROUP;
-		attrs->owner = op_ctx->creds->caller_uid;
-		attrs->group = op_ctx->creds->caller_gid;
-	} else {
-		/* For support_ex API, turn off owner and/or group attr
-		 * if they are the same as the credentials.
-		 */
-		if ((attrs->valid_mask & ATTR_OWNER) &&
-		    attrs->owner == op_ctx->creds->caller_uid)
-			FSAL_UNSET_MASK(attrs->valid_mask, ATTR_OWNER);
-		if ((attrs->valid_mask & ATTR_GROUP) &&
-		    attrs->group == op_ctx->creds->caller_gid)
-			FSAL_UNSET_MASK(attrs->valid_mask, ATTR_GROUP);
-	}
+	/* For support_ex API, turn off owner and/or group attr
+	 * if they are the same as the credentials.
+	 */
+	if ((attrs->valid_mask & ATTR_OWNER) &&
+	    attrs->owner == op_ctx->creds->caller_uid)
+		FSAL_UNSET_MASK(attrs->valid_mask, ATTR_OWNER);
+
+	if ((attrs->valid_mask & ATTR_GROUP) &&
+	    attrs->group == op_ctx->creds->caller_gid)
+		FSAL_UNSET_MASK(attrs->valid_mask, ATTR_GROUP);
 
 	/* Permission checking will be done by the FSAL operation. */
 
@@ -859,9 +823,12 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 
 	switch (type) {
 	case REGULAR_FILE:
-		/* NOTE: will not be called here if support_ex */
-		status = parent->obj_ops.create(parent, name, attrs,
-						obj, attrs_out);
+		status = fsal_open2(parent, NULL, FSAL_O_RDWR, FSAL_UNCHECKED,
+				    name, attrs, NULL, obj, attrs_out);
+		if (FSAL_IS_SUCCESS(status)) {
+			/* Close it again; this is just a create */
+			(void)fsal_close(*obj);
+		}
 		break;
 
 	case DIRECTORY:
@@ -876,15 +843,9 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 
 	case SOCKET_FILE:
 	case FIFO_FILE:
-		status = parent->obj_ops.mknode(parent, name, type,
-						NULL, /* dev_t !needed */
-						attrs, obj, attrs_out);
-		break;
-
 	case BLOCK_FILE:
 	case CHARACTER_FILE:
 		status = parent->obj_ops.mknode(parent, name, type,
-						&attrs->rawdev,
 						attrs, obj, attrs_out);
 		break;
 
@@ -920,7 +881,7 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 				    (attrs->valid_mask & ATTR_SIZE) &&
 				    attrs->filesize == 0) {
 					attrs->valid_mask &= ATTR_SIZE;
-					goto setattrs;
+					goto out;
 				}
 			}
 		} else {
@@ -929,42 +890,6 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 		goto out;
 	}
 
-setattrs:
-	if (!support_ex) {
-		/* Handle setattr for old API */
-		attrs->valid_mask = orig_mask;
-		attrs->owner = owner;
-		attrs->group = group;
-
-		/* We already handled mode */
-		FSAL_UNSET_MASK(attrs->valid_mask, ATTR_MODE);
-
-		/* Check if attrs->owner was same as creds */
-		if ((attrs->valid_mask & ATTR_OWNER) &&
-		    (op_ctx->creds->caller_uid == attrs->owner))
-			FSAL_UNSET_MASK(attrs->valid_mask, ATTR_OWNER);
-
-		/* Check if attrs->group was same as creds */
-		if ((attrs->valid_mask & ATTR_GROUP) &&
-		    (op_ctx->creds->caller_gid == attrs->group))
-			FSAL_UNSET_MASK(attrs->valid_mask, ATTR_GROUP);
-
-		/* Setting uid/gid works only for root. AIX or Irix NFS
-                 * clients send gid on create if the parent directory has
-                 * setgid bit. Clear the OWNER and GROUP attributes for
-                 * create requests for non-root users.
-		 */
-		if (type != SYMBOLIC_LINK && op_ctx->creds->caller_uid != 0) {
-			FSAL_UNSET_MASK(attrs->valid_mask,
-					ATTR_OWNER|ATTR_GROUP);
-		}
-
-		if (attrs->valid_mask) {
-			/* If any attributes were left to set, set them now. */
-			status = fsal_setattr(*obj, false, NULL, attrs);
-		}
-	}
-
  out:
 
 	/* Restore original mask so caller isn't bamboozled... */
@@ -1061,6 +986,24 @@ fsal_status_t fsal_read2(struct fsal_obj_handle *obj,
 		     "inode/direct: io_size=%zu, bytes_moved=%zu, offset=%"
 		     PRIu64, io_size, *bytes_moved, offset);
 
+	if (!*eof) {
+		/** @todo FSF: add a config option for this behavior?
+		 */
+		/*
+		 * NFS requires to set the EOF flag for all reads that
+		 * reach the EOF, i.e., even the ones returning data.
+		 * Most FSALs don't set the flag in this case. The only
+		 * client that cares about this is ESXi. Other clients
+		 * will just see a short read and continue reading and then
+		 * get the EOF flag as 0 bytes are returned.
+		 */
+		struct attrlist attrs;
+
+		fsal_prepare_attrs(&attrs, ATTR_SIZE);
+		if (FSAL_IS_SUCCESS(obj->obj_ops.getattrs(obj, &attrs)))
+			*eof = (offset + *bytes_moved) >= attrs.filesize;
+		fsal_release_attrs(&attrs);
+	}
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
@@ -1130,181 +1073,18 @@ fsal_status_t fsal_write2(struct fsal_obj_handle *obj,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
-/**
- * @brief Read/Write
- *
- * @param[in]     obj          File to be read or written
- * @param[in]     io_direction Whether this is a read or a write
- * @param[in]     offset       Absolute file position for I/O
- * @param[in]     io_size      Amount of data to be read or written
- * @param[out]    bytes_moved  The length of data successfuly read or written
- * @param[in,out] buffer       Where in memory to read or write data
- * @param[out]    eof          Whether a READ encountered the end of file.  May
- *                             be NULL for writes.
- * @param[in]     sync         Whether the write is synchronous or not
- *
- * @return FSAL status
- */
-
-fsal_status_t fsal_rdwr(struct fsal_obj_handle *obj,
-		      fsal_io_direction_t io_direction,
-		      uint64_t offset, size_t io_size,
-		      size_t *bytes_moved, void *buffer,
-		      bool *eof,
-		      bool *sync, struct io_info *info)
-{
-	/* Error return from FSAL calls */
-	fsal_status_t fsal_status = { 0, 0 };
-	/* Required open mode to successfully read or write */
-	fsal_openflags_t openflags = FSAL_O_CLOSED;
-	fsal_openflags_t loflags;
-	/* TRUE if we opened a previously closed FD */
-	bool opened = false;
-
-	/* Set flags for a read or write, as appropriate */
-	if (io_direction == FSAL_IO_READ ||
-	    io_direction == FSAL_IO_READ_PLUS) {
-		openflags = FSAL_O_READ;
-	} else {
-		/* Pretent that the caller requested sync (stable write)
-		 * if the export has COMMIT option. Note that
-		 * FSAL_O_SYNC is not always honored, so just setting
-		 * FSAL_O_SYNC has no guaranty that this write will be
-		 * a stable write.
-		 */
-		if (op_ctx->export_perms->options & EXPORT_OPTION_COMMIT)
-			*sync = true;
-		openflags = FSAL_O_WRITE;
-		if (*sync)
-			openflags |= FSAL_O_SYNC;
-	}
-
-	assert(obj != NULL);
-
-	/* IO is done only on REGULAR_FILEs */
-	if (obj->type != REGULAR_FILE) {
-		fsal_status = fsalstat(
-		    obj->type ==
-		    DIRECTORY ? ERR_FSAL_ISDIR :
-		    ERR_FSAL_BADTYPE, 0);
-		goto out;
-	}
-
-	loflags = obj->obj_ops.status(obj);
-	while ((!fsal_is_open(obj))
-	       || (loflags && loflags != FSAL_O_RDWR && loflags != openflags)) {
-		loflags = obj->obj_ops.status(obj);
-		if ((!fsal_is_open(obj))
-		    || (loflags && loflags != FSAL_O_RDWR
-			&& loflags != openflags)) {
-			fsal_status = fsal_open(obj, openflags);
-			if (FSAL_IS_ERROR(fsal_status))
-				goto out;
-			opened = true;
-		}
-		loflags = obj->obj_ops.status(obj);
-	}
-
-	/* Call FSAL_read or FSAL_write */
-	if (io_direction == FSAL_IO_READ) {
-		fsal_status = obj->obj_ops.read(obj, offset, io_size, buffer,
-						bytes_moved, eof);
-	} else if (io_direction == FSAL_IO_READ_PLUS) {
-		fsal_status = obj->obj_ops.read_plus(obj, offset, io_size,
-						     buffer, bytes_moved, eof,
-						     info);
-	} else {
-		bool fsal_sync = *sync;
-
-		if (io_direction == FSAL_IO_WRITE)
-			fsal_status = obj->obj_ops.write(obj, offset, io_size,
-							 buffer, bytes_moved,
-							 &fsal_sync);
-		else
-			fsal_status = obj->obj_ops.write_plus(obj, offset,
-							      io_size, buffer,
-							      bytes_moved,
-							      &fsal_sync, info);
-		/* Alright, the unstable write is complete. Now if it was
-		   supposed to be a stable write we can sync to the hard
-		   drive. */
-
-		if (*sync && !(loflags & FSAL_O_SYNC) && !fsal_sync &&
-		    !FSAL_IS_ERROR(fsal_status)) {
-			fsal_status = obj->obj_ops.commit(obj, offset, io_size);
-		} else {
-			*sync = fsal_sync;
-		}
-	}
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "fsal_rdwr_plus: FSAL IO operation returned %s, asked_size=%zu, effective_size=%zu",
-		     fsal_err_txt(fsal_status), io_size, *bytes_moved);
-
-	if (FSAL_IS_ERROR(fsal_status)) {
-		if (fsal_status.major == ERR_FSAL_DELAY) {
-			LogEvent(COMPONENT_FSAL,
-				 "fsal_rdwr_plus: FSAL_write  returned EBUSY");
-		} else {
-			LogDebug(COMPONENT_FSAL,
-				 "fsal_rdwr_plus: fsal_status = %s",
-				 fsal_err_txt(fsal_status));
-		}
-
-		*bytes_moved = 0;
-
-		if (fsal_status.major == ERR_FSAL_STALE)
-			goto out;
-
-		if ((fsal_status.major != ERR_FSAL_NOT_OPENED)
-		    && (obj->obj_ops.status(obj) != FSAL_O_CLOSED)) {
-			LogFullDebug(COMPONENT_FSAL,
-				     "fsal_rdwr_plus: CLOSING file %p",
-				     obj);
-
-			fsal_status = obj->obj_ops.close(obj);
-			if (FSAL_IS_ERROR(fsal_status)) {
-				LogCrit(COMPONENT_FSAL,
-					"Error closing file in fsal_rdwr_plus: %s.",
-					fsal_err_txt(fsal_status));
-			}
-		}
-
-		goto out;
-	}
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "fsal_rdwr_plus: inode/direct: io_size=%zu, bytes_moved=%zu, offset=%"
-		     PRIu64, io_size, *bytes_moved, offset);
-
-	if (opened) {
-		fsal_status = obj->obj_ops.close(obj);
-		if (FSAL_IS_ERROR(fsal_status)) {
-			LogEvent(COMPONENT_FSAL,
-				 "fsal_rdwr_plus: close = %s",
-				 fsal_err_txt(fsal_status));
-			goto out;
-		}
-	}
-
-	fsal_status = fsalstat(0, 0);
-
- out:
-
-	return fsal_status;
-}
-
 struct fsal_populate_cb_state {
 	struct fsal_obj_handle *directory;
 	fsal_status_t *status;
-	fsal_getattr_cb_t cb;
-	void *opaque;
+	helper_readdir_cb cb;
+	fsal_cookie_t last_cookie;
 	enum cb_state cb_state;
 	unsigned int *cb_nfound;
 	attrmask_t attrmask;
+	struct fsal_readdir_cb_parms cb_parms;
 };
 
-static bool
+static enum fsal_dir_result
 populate_dirent(const char *name,
 		struct fsal_obj_handle *obj,
 		struct attrlist *attrs,
@@ -1313,12 +1093,13 @@ populate_dirent(const char *name,
 {
 	struct fsal_populate_cb_state *state =
 	    (struct fsal_populate_cb_state *)dir_state;
-	struct fsal_readdir_cb_parms cb_parms = { state->opaque, name,
-		true, true };
 	fsal_status_t status = {0, 0};
-	bool retval = true;
+	enum fsal_dir_result retval;
 
-	status.major = state->cb(&cb_parms, obj, attrs, attrs->fileid,
+	retval = DIR_CONTINUE;
+	state->cb_parms.name = name;
+
+	status.major = state->cb(&state->cb_parms, obj, attrs, attrs->fileid,
 				 cookie, state->cb_state);
 
 	if (status.major == ERR_FSAL_CROSS_JUNCTION) {
@@ -1353,9 +1134,11 @@ populate_dirent(const char *name,
 					 fsal_err_txt(status));
 				/* Need to signal problem to callback */
 				state->cb_state = CB_PROBLEM;
-				(void) state->cb(&cb_parms, NULL, NULL, 0,
-						 cookie, state->cb_state);
-				retval = false;
+				(void) state->cb(&state->cb_parms, NULL, NULL,
+						 0, cookie, state->cb_state);
+				/* Protocol layers NEVER do readahead. */
+				retval = DIR_TERMINATE;
+				put_gsh_export(junction_export);
 				goto out;
 			}
 		} else {
@@ -1363,9 +1146,10 @@ populate_dirent(const char *name,
 				 "A junction became stale");
 			/* Need to signal problem to callback */
 			state->cb_state = CB_PROBLEM;
-			(void) state->cb(&cb_parms, NULL, NULL, 0, cookie,
-					 state->cb_state);
-			retval = false;
+			(void) state->cb(&state->cb_parms, NULL, NULL, 0,
+					 cookie, state->cb_state);
+			/* Protocol layers NEVER do readahead. */
+			retval = DIR_TERMINATE;
 			goto out;
 		}
 
@@ -1383,7 +1167,7 @@ populate_dirent(const char *name,
 		if (!FSAL_IS_ERROR(status)) {
 			/* Now call the callback again with that. */
 			state->cb_state = CB_JUNCTION;
-			status.major = state->cb(&cb_parms,
+			status.major = state->cb(&state->cb_parms,
 						 junction_obj,
 						 &attrs2,
 						 junction_export
@@ -1403,8 +1187,9 @@ populate_dirent(const char *name,
 		put_gsh_export(junction_export);
 	}
 
-	if (!cb_parms.in_result) {
-		retval = false;
+	if (!state->cb_parms.in_result) {
+		/* Protocol layers NEVER do readahead. */
+		retval = DIR_TERMINATE;
 		goto out;
 	}
 
@@ -1442,11 +1227,10 @@ fsal_status_t fsal_readdir(struct fsal_obj_handle *directory,
 		    unsigned int *nbfound,
 		    bool *eod_met,
 		    attrmask_t attrmask,
-		    fsal_getattr_cb_t cb,
+		    helper_readdir_cb cb,
 		    void *opaque)
 {
 	fsal_status_t fsal_status = {0, 0};
-	fsal_status_t attr_status = {0, 0};
 	fsal_status_t cb_status = {0, 0};
 	struct fsal_populate_cb_state state;
 
@@ -1464,8 +1248,7 @@ fsal_status_t fsal_readdir(struct fsal_obj_handle *directory,
 
 	/* readdir can be done only with a directory */
 	if (directory->type != DIRECTORY) {
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Not a directory");
+		LogDebug(COMPONENT_NFS_READDIR, "Not a directory");
 		return fsalstat(ERR_FSAL_NOTDIR, 0);
 	}
 
@@ -1477,27 +1260,34 @@ fsal_status_t fsal_readdir(struct fsal_obj_handle *directory,
 		access_mask_attr |= FSAL_ACE4_MASK_SET(FSAL_ACE_PERM_READ_ACL);
 	}
 
-	fsal_status = fsal_access(directory, access_mask, NULL, NULL);
+	fsal_status = fsal_access(directory, access_mask);
 	if (FSAL_IS_ERROR(fsal_status)) {
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "permission check for directory status=%s",
-			     fsal_err_txt(fsal_status));
+		LogDebug(COMPONENT_NFS_READDIR,
+			 "permission check for directory status=%s",
+			 fsal_err_txt(fsal_status));
 		return fsal_status;
 	}
 	if (attrmask != 0) {
 		/* Check for access permission to get attributes */
-		attr_status = fsal_access(directory, access_mask_attr, NULL,
-					  NULL);
+		fsal_status_t attr_status = fsal_access(directory,
+							access_mask_attr);
 		if (FSAL_IS_ERROR(attr_status))
-			LogFullDebug(COMPONENT_NFS_READDIR,
-				     "permission check for attributes status=%s",
-				     fsal_err_txt(fsal_status));
+			LogDebug(COMPONENT_NFS_READDIR,
+				 "permission check for attributes status=%s",
+				 fsal_err_txt(attr_status));
+		state.cb_parms.attr_allowed = !FSAL_IS_ERROR(attr_status);
+	} else {
+		/* No attributes requested. */
+		state.cb_parms.attr_allowed = false;
 	}
 
 	state.directory = directory;
 	state.status = &cb_status;
 	state.cb = cb;
-	state.opaque = opaque;
+	state.last_cookie = 0;
+	state.cb_parms.opaque = opaque;
+	state.cb_parms.in_result = true;
+	state.cb_parms.name = NULL;
 	state.cb_state = CB_ORIGINAL;
 	state.cb_nfound = nbfound;
 	state.attrmask = attrmask;
@@ -1526,9 +1316,6 @@ fsal_remove(struct fsal_obj_handle *parent, const char *name)
 {
 	struct fsal_obj_handle *to_remove_obj = NULL;
 	fsal_status_t status = { 0, 0 };
-#ifdef ENABLE_RFC_ACL
-	bool isdir = false;
-#endif /* ENABLE_RFC_ACL */
 
 	if (parent->type != DIRECTORY) {
 		status = fsalstat(ERR_FSAL_NOTDIR, 0);
@@ -1544,26 +1331,10 @@ fsal_remove(struct fsal_obj_handle *parent, const char *name)
 	}
 
 	/* Do not remove a junction node or an export root. */
-	if (to_remove_obj->type == DIRECTORY) {
-#ifdef ENABLE_RFC_ACL
-		isdir = true;
-#endif /* ENABLE_RFC_ACL */
-
-		PTHREAD_RWLOCK_rdlock(&to_remove_obj->state_hdl->state_lock);
-		if (to_remove_obj->state_hdl->dir.junction_export != NULL ||
-		    atomic_fetch_int32_t(
-			&to_remove_obj->state_hdl->dir.exp_root_refcount)
-		    != 0) {
-			/* Trying to remove an export mount point */
-			LogCrit(COMPONENT_FSAL, "Attempt to remove export %s",
-				name);
-
-			PTHREAD_RWLOCK_unlock(
-					&to_remove_obj->state_hdl->state_lock);
-			status = fsalstat(ERR_FSAL_NOTEMPTY, 0);
-			goto out;
-		}
-		PTHREAD_RWLOCK_unlock(&to_remove_obj->state_hdl->state_lock);
+	if (obj_is_junction(to_remove_obj)) {
+		LogCrit(COMPONENT_FSAL, "Attempt to remove export %s", name);
+		status = fsalstat(ERR_FSAL_NOTEMPTY, 0);
+		goto out;
 	}
 
 	LogFullDebug(COMPONENT_FSAL, "%s", name);
@@ -1581,7 +1352,8 @@ fsal_remove(struct fsal_obj_handle *parent, const char *name)
 	}
 
 #ifdef ENABLE_RFC_ACL
-	status = fsal_remove_access(parent, to_remove_obj, isdir);
+	status = fsal_remove_access(parent, to_remove_obj,
+				    (to_remove_obj->type == DIRECTORY));
 	if (FSAL_IS_ERROR(status))
 		goto out;
 #endif /* ENABLE_RFC_ACL */
@@ -1644,21 +1416,16 @@ fsal_status_t fsal_rename(struct fsal_obj_handle *dir_src,
 	}
 
 	/* Do not rename a junction node or an export root. */
-	if (lookup_src->type == DIRECTORY) {
-		PTHREAD_RWLOCK_rdlock(&lookup_src->state_hdl->state_lock);
+	if (obj_is_junction(lookup_src)) {
+		LogCrit(COMPONENT_FSAL, "Attempt to rename export %s", oldname);
+		fsal_status = fsalstat(ERR_FSAL_NOTEMPTY, 0);
+		goto out;
+	}
 
-		if ((lookup_src->state_hdl->dir.junction_export != NULL ||
-		     atomic_fetch_int32_t(
-			&lookup_src->state_hdl->dir.exp_root_refcount) != 0)) {
-			/* Trying to rename an export mount point */
-			PTHREAD_RWLOCK_unlock(
-					&lookup_src->state_hdl->state_lock);
-			LogCrit(COMPONENT_FSAL, "Attempt to rename export %s",
-				oldname);
-			fsal_status = fsalstat(ERR_FSAL_NOTEMPTY, 0);
-			goto out;
-		}
-		PTHREAD_RWLOCK_unlock(&lookup_src->state_hdl->state_lock);
+	/* Don't allow rename of an object as parent of itself */
+	if (dir_dest == lookup_src) {
+		fsal_status = fsalstat(ERR_FSAL_INVAL, 0);
+		goto out;
 	}
 
 	LogFullDebug(COMPONENT_FSAL, "about to call FSAL rename");
@@ -1678,75 +1445,17 @@ fsal_status_t fsal_rename(struct fsal_obj_handle *dir_src,
 	}
 
 out:
-	if (lookup_src)
+	if (lookup_src) {
+		/* Note that even with a junction, this object is in the same
+		 * export since that would be the junction node, NOT the export
+		 * root node on the other side of the junction.
+		 */
 		lookup_src->obj_ops.put_ref(lookup_src);
+	}
 
 	return fsal_status;
 }
 
-/**
- * @brief Open a file
- *
- * @param[in] obj	File to open
- * @param[in] openflags	The type of access for which to open
- * @return FSAL status
- */
-fsal_status_t fsal_open(struct fsal_obj_handle *obj_hdl,
-			fsal_openflags_t openflags)
-{
-	fsal_openflags_t current_flags;
-	fsal_status_t status = {0, 0};
-
-	if (obj_hdl->type != REGULAR_FILE)
-		return fsalstat(ERR_FSAL_BADTYPE, 0);
-
-	current_flags = obj_hdl->obj_ops.status(obj_hdl);
-
-	/* Filter out overloaded FSAL_O_RECLAIM */
-	openflags &= ~FSAL_O_RECLAIM;
-
-	/* Make sure current state meet requirements */
-	if ((current_flags != FSAL_O_RDWR) && (current_flags != FSAL_O_CLOSED)
-	    && (current_flags != openflags)) {
-		bool closed;
-		/* Flags are insufficient; need to re-open */
-		if (op_ctx->fsal_export->exp_ops.fs_supports(
-			op_ctx->fsal_export, fso_reopen_method)) {
-			/* FSAL has re-open; use that */
-			status = obj_hdl->obj_ops.reopen(obj_hdl,
-							   openflags);
-			closed = false;
-		} else {
-			status = obj_hdl->obj_ops.close(obj_hdl);
-			closed = true;
-		}
-		if (FSAL_IS_ERROR(status)
-		    && (status.major != ERR_FSAL_NOT_OPENED))
-			return status;
-		if (!FSAL_IS_ERROR(status) && closed)
-			(void) atomic_dec_size_t(&open_fd_count);
-
-		/* Potentially force re-openning */
-		current_flags = obj_hdl->obj_ops.status(obj_hdl);
-	}
-
-	if (current_flags == FSAL_O_CLOSED) {
-		status = obj_hdl->obj_ops.open(obj_hdl, openflags);
-		if (FSAL_IS_ERROR(status))
-			return status;
-
-		(void) atomic_inc_size_t(&open_fd_count);
-
-		LogDebug(COMPONENT_FSAL,
-			 "obj %p: openflags = %d, open_fd_count = %zd",
-			 obj_hdl, openflags,
-			 atomic_fetch_size_t(&open_fd_count));
-	}
-	status = fsalstat(ERR_FSAL_NO_ERROR, 0);
-
-	return status;
-}
-
 /**
  * @brief Opens a file by name or by handle.
  *
@@ -1919,7 +1628,7 @@ fsal_status_t fsal_statfs(struct fsal_obj_handle *obj,
 	fsal_status =
 	    export->exp_ops.get_fs_dynamic_info(export, obj, dynamicinfo);
 	LogFullDebug(COMPONENT_FSAL,
-		     "fsal_statfs: dynamicinfo: {total_bytes = %" PRIu64
+		     "dynamicinfo: {total_bytes = %" PRIu64
 		     ", free_bytes = %" PRIu64 ", avail_bytes = %" PRIu64
 		     ", total_files = %" PRIu64 ", free_files = %" PRIu64
 		     ", avail_files = %" PRIu64 "}", dynamicinfo->total_bytes,
@@ -1929,43 +1638,6 @@ fsal_status_t fsal_statfs(struct fsal_obj_handle *obj,
 	return fsal_status;
 }
 
-/**
- * @brief Commit a section of a file to storage
- *
- * @param[in] obj	File to commit
- * @param[in] offset	Offset for start of commit
- * @param[in] len	Length of commit
- * @return FSAL status
- */
-fsal_status_t fsal_commit(struct fsal_obj_handle *obj, off_t offset,
-			 size_t len)
-{
-	/* Error return from FSAL calls */
-	fsal_status_t fsal_status = { 0, 0 };
-	bool opened = false;
-
-	if ((uint64_t) len > ~(uint64_t) offset)
-		return fsalstat(ERR_FSAL_INVAL, 0);
-
-	if (obj->fsal->m_ops.support_ex(obj))
-		return obj->obj_ops.commit2(obj, offset, len);
-
-	if (!fsal_is_open(obj)) {
-		LogFullDebug(COMPONENT_FSAL, "need to open");
-		fsal_status = fsal_open(obj, FSAL_O_WRITE);
-		if (FSAL_IS_ERROR(fsal_status))
-			return fsal_status;
-		opened = true;
-	}
-
-	fsal_status = obj->obj_ops.commit(obj, offset, len);
-
-	if (opened)
-		obj->obj_ops.close(obj);
-
-	return fsal_status;
-}
-
 /**
  * @brief Verify an exclusive create replay when the file is already open.
  *
@@ -1989,4 +1661,42 @@ fsal_status_t fsal_verify2(struct fsal_obj_handle *obj,
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
+/**
+ * @brief Fetch optional attributes
+ *
+ * The request_mask should be set in attrs_out indicating which attributes
+ * are desired. If ATTR_RDATTR_ERR is set, and the getattrs fails,
+ * the error ERR_FSAL_NO_ERROR will be returned, however the attributes
+ * valid_mask will be set to ATTR_RDATTR_ERR. Otherwise, if
+ * ATTR_RDATTR_ERR is not set and the getattrs fails, the error returned
+ * by getattrs will be returned.
+ *
+ * @param[in]     obj_hdl   Object to get attributes for.
+ * @param[in,out] attrs_out Optional attributes for the object
+ *
+ * @return FSAL status.
+ **/
+fsal_status_t get_optional_attrs(struct fsal_obj_handle *obj_hdl,
+				 struct attrlist *attrs_out)
+{
+	fsal_status_t status;
+
+	if (attrs_out == NULL)
+		return fsalstat(ERR_FSAL_NO_ERROR, 0);
+
+	status = obj_hdl->obj_ops.getattrs(obj_hdl, attrs_out);
+
+	if (FSAL_IS_ERROR(status)) {
+		if (attrs_out->request_mask & ATTR_RDATTR_ERR) {
+			/* Indicate the failure of requesting attributes by
+			 * marking the ATTR_RDATTR_ERR in the mask.
+			 */
+			attrs_out->valid_mask = ATTR_RDATTR_ERR;
+			status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+		} /* otherwise let the error stand. */
+	}
+
+	return status;
+}
+
 /** @} */
