@@ -72,7 +72,7 @@ static int schedule_delegrecall_task(struct delegrecall_context *ctx,
  * operations, instead, the FSAL may directly close the file as
  * necessary.
  *
- * @param[in] export The export
+ * @param[in] vec    Up ops vector
  * @param[in] handle Handle being invalidated
  * @param[in] flags  Flags governing invalidation
  *
@@ -80,7 +80,7 @@ static int schedule_delegrecall_task(struct delegrecall_context *ctx,
  *
  */
 
-static fsal_status_t invalidate_close(struct fsal_export *export,
+static fsal_status_t invalidate_close(const struct fsal_up_vector *vec,
 				      struct gsh_buffdesc *handle,
 				      uint32_t flags)
 {
@@ -89,7 +89,7 @@ static fsal_status_t invalidate_close(struct fsal_export *export,
 
 /** Invalidate some or all of a cache entry
  *
- * @param[in] export The export
+ * @param[in] vec    Up ops vector
  * @param[in] handle Handle being invalidated
  * @param[in] flags  Flags governing invalidation
  *
@@ -97,7 +97,7 @@ static fsal_status_t invalidate_close(struct fsal_export *export,
  *
  */
 
-fsal_status_t invalidate(struct fsal_export *export,
+fsal_status_t invalidate(const struct fsal_up_vector *vec,
 			 struct gsh_buffdesc *handle,
 			 uint32_t flags)
 {
@@ -108,7 +108,7 @@ fsal_status_t invalidate(struct fsal_export *export,
 /**
  * @brief Update cached attributes
  *
- * @param[in] export The export
+ * @param[in] vec    Up ops vector
  * @param[in] obj    Key to specify object
  * @param[in] attr   New attributes
  * @param[in] flags  Flags to govern update
@@ -116,7 +116,7 @@ fsal_status_t invalidate(struct fsal_export *export,
  * @return FSAL status
  */
 
-static fsal_status_t update(struct fsal_export *export,
+static fsal_status_t update(const struct fsal_up_vector *vec,
 			    struct gsh_buffdesc *obj,
 			    struct attrlist *attr, uint32_t flags)
 {
@@ -134,13 +134,14 @@ static fsal_status_t update(struct fsal_export *export,
  * @return STATE_SUCCESS or errors.
  */
 
-static state_status_t lock_grant(struct fsal_export *export,
+static state_status_t lock_grant(const struct fsal_up_vector *vec,
 				 struct gsh_buffdesc *file,
 				 void *owner,
 				 fsal_lock_param_t *lock_param)
 {
 	struct fsal_obj_handle *obj;
 	fsal_status_t status;
+	struct fsal_export *export = vec->up_fsal_export;
 
 	status = export->exp_ops.create_handle(export, file, &obj, NULL);
 	if (FSAL_IS_ERROR(status))
@@ -161,13 +162,14 @@ static state_status_t lock_grant(struct fsal_export *export,
  * @return STATE_SUCCESS or errors.
  */
 
-static state_status_t lock_avail(struct fsal_export *export,
+static state_status_t lock_avail(const struct fsal_up_vector *vec,
 				 struct gsh_buffdesc *file,
 				 void *owner,
 				 fsal_lock_param_t *lock_param)
 {
 	struct fsal_obj_handle *obj;
 	fsal_status_t status;
+	struct fsal_export *export = vec->up_fsal_export;
 
 	status = export->exp_ops.create_handle(export, file, &obj, NULL);
 	if (FSAL_IS_ERROR(status))
@@ -388,7 +390,7 @@ struct layoutrecall_cb_data {
  * @retval STATE_MALLOC_ERROR if there was insufficient memory to construct the
  *         recall state.
  */
-state_status_t layoutrecall(struct fsal_export *export,
+state_status_t layoutrecall(const struct fsal_up_vector *vec,
 			    struct gsh_buffdesc *handle,
 			    layouttype4 layout_type, bool changed,
 			    const struct pnfs_segment *segment, void *cookie,
@@ -404,6 +406,7 @@ state_status_t layoutrecall(struct fsal_export *export,
 	struct glist_head *wi = NULL;
 	struct gsh_export *exp = NULL;
 	state_owner_t *owner = NULL;
+	struct fsal_export *export = vec->up_fsal_export;
 
 	rc = state_error_convert(export->exp_ops.create_handle(export, handle,
 							       &obj, NULL));
@@ -455,7 +458,6 @@ state_status_t layoutrecall(struct fsal_export *export,
 			/* The export, owner, or state_t has gone stale,
 			 * skip this entry
 			 */
-			gsh_free(layout->lor_fh.nfs_fh4_val);
 			gsh_free(cb_data);
 			continue;
 		}
@@ -499,8 +501,10 @@ state_status_t layoutrecall(struct fsal_export *export,
 
 static void free_layoutrec(nfs_cb_argop4 *op)
 {
-	gsh_free(op->nfs_cb_argop4_u.opcblayoutrecall.clora_recall.
-		 layoutrecall4_u.lor_layout.lor_fh.nfs_fh4_val);
+	layoutrecall4 *clora_recall =
+		&op->nfs_cb_argop4_u.opcblayoutrecall.clora_recall;
+
+	gsh_free(clora_recall->layoutrecall4_u.lor_layout.lor_fh.nfs_fh4_val);
 }
 
 /**
@@ -518,17 +522,11 @@ static void free_layoutrec(nfs_cb_argop4 *op)
  * period of delay has surpassed the lease period.
  *
  * @param[in] call  The RPC call being completed
- * @param[in] hook  The hook itself
- * @param[in] arg   Supplied argument (the callback data)
- * @param[in] flags There are no flags.
- *
- * @return 0, constantly.
  */
 
-static int32_t layoutrec_completion(rpc_call_t *call, rpc_call_hook hook,
-				    void *arg, uint32_t flags)
+static void layoutrec_completion(rpc_call_t *call)
 {
-	struct layoutrecall_cb_data *cb_data = arg;
+	struct layoutrecall_cb_data *cb_data = call->call_arg;
 	bool deleted = false;
 	state_t *state = NULL;
 	struct root_op_context root_op_context;
@@ -545,7 +543,7 @@ static int32_t layoutrec_completion(rpc_call_t *call, rpc_call_hook hook,
 		     call->cbt.v_u.v4.res.status, cb_data);
 
 	/* Get this out of the way up front */
-	if (hook != RPC_CALL_COMPLETE)
+	if (call->states & NFS_CB_CALL_ABORTED)
 		goto revoke;
 
 	if (call->cbt.v_u.v4.res.status == NFS4_OK) {
@@ -563,7 +561,7 @@ static int32_t layoutrec_completion(rpc_call_t *call, rpc_call_hook hook,
 		 * the clientid in cb_data.
 		 */
 		free_layoutrec(&call->cbt.v_u.v4.args.argarray.argarray_val[1]);
-		nfs41_complete_single(call, hook, cb_data, flags);
+		nfs41_release_single(call);
 		gsh_free(cb_data);
 		goto out;
 	} else if (call->cbt.v_u.v4.res.status == NFS4ERR_DELAY) {
@@ -588,7 +586,7 @@ static int32_t layoutrec_completion(rpc_call_t *call, rpc_call_hook hook,
 
 		/* We don't free the argument here, because we'll be
 		   re-using that to make the queued call. */
-		nfs41_complete_single(call, hook, cb_data, flags);
+		nfs41_release_single(call);
 		delayed_submit(layoutrecall_one_call, cb_data, delay);
 		goto out;
 	}
@@ -607,9 +605,8 @@ static int32_t layoutrec_completion(rpc_call_t *call, rpc_call_hook hook,
 	if (ok) {
 		enum fsal_layoutreturn_circumstance circumstance;
 
-		if (hook == RPC_CALL_COMPLETE &&
-		    call->cbt.v_u.v4.res.status ==
-		    NFS4ERR_NOMATCHING_LAYOUT)
+		if (!(call->states & NFS_CB_CALL_ABORTED)
+		 && call->cbt.v_u.v4.res.status == NFS4ERR_NOMATCHING_LAYOUT)
 			circumstance = circumstance_client;
 		else
 			circumstance = circumstance_revoke;
@@ -649,7 +646,7 @@ static int32_t layoutrec_completion(rpc_call_t *call, rpc_call_hook hook,
 	}
 
 	free_layoutrec(&call->cbt.v_u.v4.args.argarray.argarray_val[1]);
-	nfs41_complete_single(call, hook, cb_data, flags);
+	nfs41_release_single(call);
 	gsh_free(cb_data);
 
 out:
@@ -665,8 +662,6 @@ out:
 		/* Release the owner */
 		dec_state_owner_ref(owner);
 	}
-
-	return 0;
 }
 
 /**
@@ -772,10 +767,10 @@ static void layoutrecall_one_call(void *arg)
 		root_op_context.req_ctx.ctx_export = export;
 		root_op_context.req_ctx.fsal_export = export->fsal_export;
 
-		code = nfs_rpc_v41_single(cb_data->client, &cb_data->arg,
+		code = nfs_rpc_cb_single(cb_data->client, &cb_data->arg,
 					  &state->state_refer,
 					  layoutrec_completion,
-					  cb_data, free_layoutrec);
+					  cb_data);
 
 		if (code != 0) {
 			/**
@@ -816,6 +811,7 @@ static void layoutrecall_one_call(void *arg)
 						      circumstance_revoke,
 						      state, cb_data->segment,
 						      0, NULL, &deleted);
+				free_layoutrec(&cb_data->arg);
 				gsh_free(cb_data);
 			}
 		} else {
@@ -862,20 +858,13 @@ struct cb_notify {
  * @brief Handle CB_NOTIFY_DEVICE response
  *
  * @param[in] call  The RPC call being completed
- * @param[in] hook  The hook itself
- * @param[in] arg   Supplied argument (the callback data)
- * @param[in] flags There are no flags.
- *
- * @return 0, constantly.
  */
 
-static int32_t notifydev_completion(rpc_call_t *call, rpc_call_hook hook,
-				    void *arg, uint32_t flags)
+static void notifydev_completion(rpc_call_t *call)
 {
 	LogFullDebug(COMPONENT_NFS_CB, "status %d arg %p",
-		     call->cbt.v_u.v4.res.status, arg);
-	gsh_free(arg);
-	return 0;
+		     call->cbt.v_u.v4.res.status, call->call_arg);
+	gsh_free(call->call_arg);
 }
 
 /**
@@ -933,9 +922,8 @@ static bool devnotify_client_callback(nfs_client_id_t *clientid,
 	memcpy(arg->notify_del.ndd_deviceid,
 	       &devicenotify->devid,
 	       sizeof(arg->notify_del.ndd_deviceid));
-	code =
-	    nfs_rpc_v41_single(clientid, &arg->arg, NULL, notifydev_completion,
-			       &arg->arg, NULL);
+	code = nfs_rpc_cb_single(clientid, &arg->arg, NULL,
+				 notifydev_completion, &arg->arg);
 	if (code != 0)
 		gsh_free(arg);
 
@@ -1028,7 +1016,7 @@ static enum recall_resp_action handle_recall_response(
 				rpc_call_t *call)
 {
 	enum recall_resp_action resp_action;
-	char str[DISPLAY_STATEID_OTHER_SIZE];
+	char str[DISPLAY_STATEID_OTHER_SIZE] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
 
@@ -1101,28 +1089,27 @@ free_delegrecall_context(struct delegrecall_context *deleg_ctx)
  * @brief Handle the reply to a CB_RECALL
  *
  * @param[in] call  The RPC call being completed
- * @param[in] hook  The hook itself
- * @param[in] arg   Supplied argument (the callback data)
- * @param[in] flags There are no flags.
  *
  * @return 0, constantly.
  */
 
-static int32_t delegrecall_completion_func(rpc_call_t *call,
-					   rpc_call_hook hook, void *arg,
-					   uint32_t flags)
+static void delegrecall_completion_func(rpc_call_t *call)
 {
-	char *fh = NULL;
 	enum recall_resp_action resp_act;
 	nfsstat4 rc = NFS4_OK;
-	struct delegrecall_context *deleg_ctx = arg;
+	struct delegrecall_context *deleg_ctx = call->call_arg;
+	uint32_t minorversion = deleg_ctx->drc_clid->cid_minorversion;
 	struct state_t *state;
 	struct fsal_obj_handle *obj = NULL;
-	char str[LOG_BUFF_LEN];
+	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
+	CB_RECALL4args *opcbrecall;
+	struct req_op_context *save_ctx = op_ctx, req_ctx = {0};
+	struct gsh_export *export = NULL;
+	bool ret = false;
 
 	LogDebug(COMPONENT_NFS_CB, "%p %s", call,
-		 (hook == RPC_CALL_COMPLETE) ? "Success" : "Failed");
+		 !(call->states & NFS_CB_CALL_ABORTED) ? "Success" : "Failed");
 
 	state = nfs4_State_Get_Pointer(deleg_ctx->drc_stateid.other);
 
@@ -1131,44 +1118,47 @@ static int32_t delegrecall_completion_func(rpc_call_t *call,
 		goto out_free_drc;
 	}
 
-	obj = get_state_obj_ref(state);
+	ret = get_state_obj_export_owner_refs(state, &obj,
+					     &export,
+					     NULL);
 
-	if (obj == NULL) {
+	if (!ret || obj == NULL) {
 		LogDebug(COMPONENT_NFS_CB, "Stale file");
 		goto out_free_drc;
 	}
 
+	op_ctx = &req_ctx;
+	op_ctx->ctx_export = export;
+	op_ctx->fsal_export = export->fsal_export;
+
 	if (isDebug(COMPONENT_NFS_CB)) {
-		char str[LOG_BUFF_LEN];
+		char str[LOG_BUFF_LEN] = "\0";
 		struct display_buffer dspbuf = {sizeof(str), str, str};
 
 		display_stateid(&dspbuf, state);
 		LogDebug(COMPONENT_NFS_CB, "deleg_entry %s", str);
 	}
 
-	switch (hook) {
-	case RPC_CALL_COMPLETE:
-		LogMidDebug(COMPONENT_NFS_CB, "call result: %d", call->stat);
-		fh = call->cbt.v_u.v4.args.argarray.argarray_val->
-				nfs_cb_argop4_u.opcbrecall.fh.nfs_fh4_val;
-		if (call->stat != RPC_SUCCESS) {
+	if (!(call->states & NFS_CB_CALL_ABORTED)) {
+		LogMidDebug(COMPONENT_NFS_CB, "call result: %d",
+			    call->call_req.cc_error.re_status);
+		if (call->call_req.cc_error.re_status != RPC_SUCCESS) {
 			LogEvent(COMPONENT_NFS_CB,
-				 "Call stat: %d, marking CB channel down",
-				 call->stat);
+				 "call result: %d, marking CB channel down",
+				 call->call_req.cc_error.re_status);
 			set_cb_chan_down(deleg_ctx->drc_clid, true);
 			resp_act = DELEG_RECALL_SCHED;
 		} else
 			resp_act = handle_recall_response(deleg_ctx,
 							  state,
 							  call);
-		break;
-	default:
+	} else {
 		LogEvent(COMPONENT_NFS_CB,
-			 "Unknown hook %d, marking CB channel down", hook);
+			 "Aborted: %d, marking CB channel down",
+			 call->call_req.cc_error.re_status);
 		set_cb_chan_down(deleg_ctx->drc_clid, true);
 		/* Mark the recall as failed */
 		resp_act = DELEG_RECALL_SCHED;
-		break;
 	}
 	switch (resp_act) {
 	case DELEG_RECALL_SCHED:
@@ -1213,16 +1203,28 @@ out_free_drc:
 	free_delegrecall_context(deleg_ctx);
 
 out_free:
-
-	fh = call->cbt.v_u.v4.args.argarray.argarray_val->
-				nfs_cb_argop4_u.opcbrecall.fh.nfs_fh4_val;
-	gsh_free(fh);
-	free_rpc_call(call);
+	if (minorversion == 0) {
+		opcbrecall = &call->cbt.v_u.v4.args.argarray.argarray_val[0]
+				.nfs_cb_argop4_u.opcbrecall;
+		nfs4_freeFH(&opcbrecall->fh);
+	} else {
+		opcbrecall = &call->cbt.v_u.v4.args.argarray.argarray_val[1]
+				.nfs_cb_argop4_u.opcbrecall;
+		nfs4_freeFH(&opcbrecall->fh);
+		nfs41_release_single(call);
+	}
 
 	if (state != NULL)
 		dec_state_t_ref(state);
 
-	return 0; /*Always return zero, the delegation is recalled or revoked */
+	/* Release the obj ref and export ref. */
+	if (obj != NULL)
+		obj->obj_ops.put_ref(obj);
+
+	if (export != NULL)
+		put_gsh_export(export);
+
+	op_ctx = save_ctx;
 }
 
 /**
@@ -1240,11 +1242,10 @@ void delegrecall_one(struct fsal_obj_handle *obj,
 		     struct state_t *state,
 		     struct delegrecall_context *p_cargs)
 {
-	rpc_call_channel_t *chan;
-	rpc_call_t *call = NULL;
-	nfs_cb_argop4 argop[1];
+	int ret;
+	nfs_cb_argop4 argop;
 	struct cf_deleg_stats *clfl_stats;
-	char str[LOG_BUFF_LEN];
+	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
 
@@ -1264,66 +1265,27 @@ void delegrecall_one(struct fsal_obj_handle *obj,
 
 	inc_recalls(p_cargs->drc_clid->gsh_client);
 
-	/* Attempt a recall only if channel state is UP */
-	if (get_cb_chan_down(p_cargs->drc_clid)) {
-		LogCrit(COMPONENT_NFS_CB,
-			"Call back channel down, not issuing a recall");
-		goto out;
-	}
-
-	chan = nfs_rpc_get_chan(p_cargs->drc_clid, NFS_RPC_FLAG_NONE);
-	if (!chan) {
-		LogCrit(COMPONENT_NFS_CB, "nfs_rpc_get_chan failed");
-		/* TODO: move this to nfs_rpc_get_chan ? */
-		set_cb_chan_down(p_cargs->drc_clid, true);
-		goto out;
-	}
-	if (!chan->clnt) {
-		LogCrit(COMPONENT_NFS_CB, "nfs_rpc_get_chan failed (no clnt)");
-		set_cb_chan_down(p_cargs->drc_clid, true);
-		goto out;
-	}
-	/* allocate a new call--freed in completion hook */
-	call = alloc_rpc_call();
-
-	call->chan = chan;
-
-	/* setup a compound */
-	cb_compound_init_v4(&call->cbt, 1, 0,
-			    p_cargs->drc_clid->cid_cb.v40.cb_callback_ident,
-			    "brrring!!!", 10);
-
-	argop->argop = NFS4_OP_CB_RECALL;
-	COPY_STATEID(&argop->nfs_cb_argop4_u.opcbrecall.stateid, state);
-	argop->nfs_cb_argop4_u.opcbrecall.truncate = false;
+	argop.argop = NFS4_OP_CB_RECALL;
+	COPY_STATEID(&argop.nfs_cb_argop4_u.opcbrecall.stateid, state);
+	argop.nfs_cb_argop4_u.opcbrecall.truncate = false;
 
 	/* Convert it to a file handle */
-	if (!nfs4_FSALToFhandle(true, &argop->nfs_cb_argop4_u.opcbrecall.fh,
+	if (!nfs4_FSALToFhandle(true, &argop.nfs_cb_argop4_u.opcbrecall.fh,
 				obj, p_cargs->drc_exp)) {
 		LogCrit(COMPONENT_FSAL_UP,
 			"nfs4_FSALToFhandle failed, can not process recall");
 		goto out;
 	}
 
-	/* add ops, till finished */
-	cb_compound_add_op(&call->cbt, argop);
-
-	/* set completion hook */
-	call->call_hook = delegrecall_completion_func;
-
-	/* call it (here, in current thread context)
-	   ret is always 0 for async calls, might change in future */
-	if (nfs_rpc_submit_call(call, p_cargs, NFS_RPC_CALL_NONE) == 0)
+	ret = nfs_rpc_cb_single(p_cargs->drc_clid, &argop, &state->state_refer,
+				delegrecall_completion_func, p_cargs);
+	if (ret == 0)
 		return;
+	LogDebug(COMPONENT_FSAL_UP, "nfs_rpc_cb_single returned %d", ret);
 
 out:
-
 	inc_failed_recalls(p_cargs->drc_clid->gsh_client);
-
-	nfs4_freeFH(&argop->nfs_cb_argop4_u.opcbrecall.fh);
-
-	if (call)
-		free_rpc_call(call);
+	nfs4_freeFH(&argop.nfs_cb_argop4_u.opcbrecall.fh);
 
 	if (!eval_deleg_revoke(state) &&
 	    !schedule_delegrecall_task(p_cargs, 1)) {
@@ -1337,8 +1299,7 @@ out:
 	if (!str_valid)
 		display_stateid(&dspbuf, state);
 
-	LogCrit(COMPONENT_STATE, "Delegation will be revoked for %s",
-		str);
+	LogCrit(COMPONENT_STATE, "Delegation will be revoked for %s", str);
 
 	p_cargs->drc_clid->num_revokes++;
 	inc_revokes(p_cargs->drc_clid->gsh_client);
@@ -1367,9 +1328,12 @@ static void delegrevoke_check(void *ctx)
 	struct fsal_obj_handle *obj = NULL;
 	struct state_t *state = NULL;
 	bool free_drc = true;
-	char str[LOG_BUFF_LEN];
+	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
+	struct req_op_context *save_ctx = op_ctx, req_ctx = {0};
+	struct gsh_export *export = NULL;
+	bool   ret = false;
 
 	state = nfs4_State_Get_Pointer(deleg_ctx->drc_stateid.other);
 
@@ -1383,13 +1347,20 @@ static void delegrevoke_check(void *ctx)
 		str_valid = true;
 	}
 
-	obj = get_state_obj_ref(state);
+	ret = get_state_obj_export_owner_refs(state, &obj,
+					     &export,
+					     NULL);
 
-	if (obj == NULL) {
+	if (!ret || obj == NULL) {
 		LogDebug(COMPONENT_NFS_CB, "Stale file");
 		goto out;
 	}
 
+	/* op_ctx may be used by state_del_locked and others */
+	op_ctx = &req_ctx;
+	op_ctx->ctx_export = export;
+	op_ctx->fsal_export = export->fsal_export;
+
 	if (eval_deleg_revoke(state)) {
 		if (str_valid)
 			LogDebug(COMPONENT_STATE,
@@ -1427,6 +1398,15 @@ static void delegrevoke_check(void *ctx)
 
 	if (state != NULL)
 		dec_state_t_ref(state);
+
+	/* Release the obj ref and export ref. */
+	if (obj != NULL)
+		obj->obj_ops.put_ref(obj);
+
+	if (export != NULL)
+		put_gsh_export(export);
+
+	op_ctx = save_ctx;
 }
 
 static void delegrecall_task(void *ctx)
@@ -1434,6 +1414,9 @@ static void delegrecall_task(void *ctx)
 	struct delegrecall_context *deleg_ctx = ctx;
 	struct state_t *state;
 	struct fsal_obj_handle *obj;
+	bool   ret = false;
+	struct req_op_context *save_ctx, req_ctx = {0};
+	struct gsh_export *export = NULL;
 
 	state = nfs4_State_Get_Pointer(deleg_ctx->drc_stateid.other);
 
@@ -1441,15 +1424,30 @@ static void delegrecall_task(void *ctx)
 		LogDebug(COMPONENT_NFS_CB, "Delgation is already returned");
 		free_delegrecall_context(deleg_ctx);
 	} else {
-		obj = get_state_obj_ref(state);
+		ret = get_state_obj_export_owner_refs(state, &obj,
+						     &export,
+						     NULL);
 
-		if (obj != NULL) {
-			delegrecall_one(obj, state, deleg_ctx);
-
-		} else {
+		if (!ret || obj == NULL) {
 			LogDebug(COMPONENT_NFS_CB,
 				 "Delgation recall skipped due to stale file");
+			goto out;
 		}
+
+		/* op_ctx may be used by state_del_locked and others */
+		save_ctx = op_ctx;
+		op_ctx = &req_ctx;
+		op_ctx->ctx_export = export;
+		op_ctx->fsal_export = export->fsal_export;
+		delegrecall_one(obj, state, deleg_ctx);
+
+		/* Release the obj ref and export ref. */
+		obj->obj_ops.put_ref(obj);
+		put_gsh_export(export);
+
+		op_ctx = save_ctx;
+
+out:
 		dec_state_t_ref(state);
 	}
 }
@@ -1492,6 +1490,7 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 	struct state_t *state;
 	state_owner_t *owner;
 	struct delegrecall_context *drc_ctx;
+	struct req_op_context *save_ctx = op_ctx, req_ctx = {0};
 
 	LogDebug(COMPONENT_FSAL_UP,
 		 "FSAL_UP_DELEG: obj %p type %u",
@@ -1506,7 +1505,7 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 			continue;
 
 		if (isDebug(COMPONENT_NFS_CB)) {
-			char str[LOG_BUFF_LEN];
+			char str[LOG_BUFF_LEN] = "\0";
 			struct display_buffer dspbuf = {sizeof(str), str, str};
 
 			display_stateid(&dspbuf, state);
@@ -1537,6 +1536,11 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 			continue;
 		}
 
+		/* op_ctx may be used by state_del_locked and others */
+		op_ctx = &req_ctx;
+		op_ctx->ctx_export = drc_ctx->drc_exp;
+		op_ctx->fsal_export = drc_ctx->drc_exp->fsal_export;
+
 		drc_ctx->drc_clid = owner->so_owner.so_nfs4_owner.so_clientrec;
 		COPY_STATEID(&drc_ctx->drc_stateid, state);
 		inc_client_id_ref(drc_ctx->drc_clid);
@@ -1563,6 +1567,8 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 		delegrecall_one(obj, state, drc_ctx);
 	}
 	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+
+	op_ctx = save_ctx;
 	return rc;
 }
 
@@ -1573,9 +1579,10 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
  *
  * @return STATE_SUCCESS or errors.
  */
-state_status_t delegrecall(struct fsal_export *export,
+state_status_t delegrecall(const struct fsal_up_vector *vec,
 			   struct gsh_buffdesc *handle)
 {
+	struct fsal_export *export = vec->up_fsal_export;
 	struct fsal_obj_handle *obj = NULL;
 	state_status_t rc = 0;
 
@@ -1600,7 +1607,40 @@ state_status_t delegrecall(struct fsal_export *export,
 	return rc;
 }
 
+void up_ready_init(struct fsal_up_vector *up_ops)
+{
+	up_ops->up_ready = false;
+	up_ops->up_cancel = false;
+	PTHREAD_MUTEX_init(&up_ops->up_mutex, NULL);
+	PTHREAD_COND_init(&up_ops->up_cond, NULL);
+}
 
+/* This should be called when a module is ready to take upcalls */
+void up_ready_set(struct fsal_up_vector *up_ops)
+{
+	PTHREAD_MUTEX_lock(&up_ops->up_mutex);
+	up_ops->up_ready = true;
+	pthread_cond_broadcast(&up_ops->up_cond);
+	PTHREAD_MUTEX_unlock(&up_ops->up_mutex);
+}
+
+/* This is to cancel waiting and resume Upcall threads */
+void up_ready_cancel(struct fsal_up_vector *up_ops)
+{
+	PTHREAD_MUTEX_lock(&up_ops->up_mutex);
+	up_ops->up_cancel = true;
+	pthread_cond_broadcast(&up_ops->up_cond);
+	PTHREAD_MUTEX_unlock(&up_ops->up_mutex);
+}
+
+/* Upcall threads should call this to wait for upcall readiness */
+void up_ready_wait(struct fsal_up_vector *up_ops)
+{
+	PTHREAD_MUTEX_lock(&up_ops->up_mutex);
+	while (!up_ops->up_ready && !up_ops->up_cancel)
+		pthread_cond_wait(&up_ops->up_cond, &up_ops->up_mutex);
+	PTHREAD_MUTEX_unlock(&up_ops->up_mutex);
+}
 
 /**
  * @brief The top level vector of operations
@@ -1611,7 +1651,8 @@ state_status_t delegrecall(struct fsal_export *export,
  */
 
 struct fsal_up_vector fsal_up_top = {
-	.up_export = NULL,
+	.up_gsh_export = NULL,
+	.up_fsal_export = NULL,
 	.lock_grant = lock_grant,
 	.lock_avail = lock_avail,
 	.invalidate = invalidate,
