@@ -82,7 +82,11 @@
 struct nfs_init nfs_init;
 
 /* global information exported to all layers (as extern vars) */
+pool_t *request_pool;
 nfs_parameter_t nfs_param;
+struct _nfs_health health;
+
+static struct _nfs_health healthstats;
 
 /* ServerEpoch is ServerBootTime unless overriden by -E command line option */
 struct timespec ServerBootTime;
@@ -92,13 +96,25 @@ verifier4 NFS4_write_verifier;	/* NFS V4 write verifier */
 writeverf3 NFS3_write_verifier;	/* NFS V3 write verifier */
 
 /* node ID used to identify an individual node in a cluster */
-int g_nodeid = 0;
+int g_nodeid;
 
 nfs_start_info_t nfs_start_info;
 
 pthread_t admin_thrid;
 pthread_t sigmgr_thrid;
 
+tirpc_pkg_params ntirpc_pp = {
+	TIRPC_DEBUG_FLAG_DEFAULT,
+	0,
+	SetNameFunction,
+	(mem_format_t)rpc_warnx,
+	gsh_free_size,
+	gsh_malloc__,
+	gsh_malloc_aligned__,
+	gsh_calloc__,
+	gsh_realloc__,
+};
+
 #ifdef _USE_9P
 pthread_t _9p_dispatcher_thrid;
 #endif
@@ -289,6 +305,9 @@ static void init_crash_handlers(void)
 void nfs_prereq_init(char *program_name, char *host_name, int debug_level,
 		     char *log_path, bool dump_trace)
 {
+	healthstats.enqueued_reqs = health.enqueued_reqs = 0;
+	healthstats.enqueued_reqs = health.dequeued_reqs = 0;
+
 	/* Initialize logging */
 	SetNamePgm(program_name);
 	SetNameFunction("main");
@@ -298,6 +317,11 @@ void nfs_prereq_init(char *program_name, char *host_name, int debug_level,
 	if (dump_trace) {
 		init_crash_handlers();
 	}
+
+	/* Redirect TI-RPC allocators, log channel */
+	if (!tirpc_control(TIRPC_PUT_PARAMETERS, &ntirpc_pp)) {
+		LogFatal(COMPONENT_INIT, "Setting nTI-RPC parameters failed");
+	}
 }
 
 /**
@@ -330,10 +354,6 @@ void nfs_print_param_config(void)
 	printf("\tDRC_UDP_Hiwat = %u ;\n", nfs_param.core_param.drc.udp.hiwat);
 	printf("\tDRC_UDP_Checksum = %u ;\n",
 	       nfs_param.core_param.drc.udp.checksum);
-	printf("\tDecoder_Fridge_Expiration_Delay = %" PRIu64 " ;\n",
-	       (uint64_t) nfs_param.core_param.decoder_fridge_expiration_delay);
-	printf("\tDecoder_Fridge_Block_Timeout = %" PRIu64 " ;\n",
-	       (uint64_t) nfs_param.core_param.decoder_fridge_block_timeout);
 	printf("\tBlocked_Lock_Poller_Interval = %" PRIu64 " ;\n",
 	       (uint64_t) nfs_param.core_param.blocked_lock_poller_interval);
 
@@ -531,16 +551,13 @@ static void nfs_Start_threads(void)
 	}
 	LogDebug(COMPONENT_THREAD, "sigmgr thread started");
 
-	rc = worker_init();
+#ifdef _USE_9P
+	rc = _9p_worker_init();
 	if (rc != 0) {
 		LogFatal(COMPONENT_THREAD, "Could not start worker threads: %d",
 			 errno);
 	}
 
-	/* Start event channel service threads */
-	nfs_rpc_dispatch_threads(&attr_thr);
-
-#ifdef _USE_9P
 	/* Starting the 9P/TCP dispatcher thread */
 	if (nfs_param.core_param.core_options & CORE_OPTION_9P) {
 		rc = pthread_create(&_9p_dispatcher_thrid, &attr_thr,
@@ -570,20 +587,6 @@ static void nfs_Start_threads(void)
 	}
 #endif
 
-#ifdef _USE_NFS_RDMA
-	/* Starting the NFS/RDMA dispatcher thread */
-	rc = pthread_create(&nfs_rdma_dispatcher_thrid, &attr_thr,
-			    nfs_rdma_dispatcher_thread, NULL);
-
-	if (rc != 0) {
-		LogFatal(COMPONENT_THREAD,
-			 "Could not create NFS/RDMA dispatcher, error = %d (%s)",
-			 errno, strerror(errno));
-	}
-	LogEvent(COMPONENT_THREAD,
-		 "NFS/RDMA dispatcher thread was started successfully");
-#endif
-
 #ifdef USE_DBUS
 	/* DBUS event thread */
 	rc = pthread_create(&gsh_dbus_thrid, &attr_thr, gsh_dbus_thread, NULL);
@@ -668,10 +671,8 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 		OM_uint32 gss_status = GSS_S_COMPLETE;
 
 		if (*nfs_param.krb5_param.keytab != '\0')
-			gss_status =
-			    krb5_gss_register_acceptor_identity(nfs_param.
-								krb5_param.
-								keytab);
+			gss_status = krb5_gss_register_acceptor_identity(
+						nfs_param.krb5_param.keytab);
 
 		if (gss_status != GSS_S_COMPLETE) {
 			log_sperror_gss(GssError, gss_status, 0);
@@ -721,14 +722,6 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 	}			/*  if( nfs_param.krb5_param.active_krb5 ) */
 #endif				/* HAVE_KRB5 */
 #endif				/* _HAVE_GSSAPI */
-
-	/* RPC Initialisation - exits on failure */
-	nfs_Init_svc();
-	LogInfo(COMPONENT_INIT, "RPC resources successfully initialized");
-
-	/* Admin initialisation */
-	nfs_Init_admin_thread();
-
 	/* Init the NFSv4 Clientid cache */
 	LogDebug(COMPONENT_INIT, "Now building NFSv4 clientid cache");
 	if (nfs_Init_client_id() !=
@@ -827,11 +820,15 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 	 */
 	nfs4_recovery_init();
 
-	/* read in the client IDs */
-	nfs4_recovery_load_clids(NULL);
-
 	/* Start grace period */
-	nfs4_start_grace(NULL);
+	nfs_start_grace(NULL);
+
+	/* RPC Initialisation - exits on failure */
+	nfs_Init_svc();
+	LogInfo(COMPONENT_INIT, "RPC resources successfully initialized");
+
+	/* Admin initialisation */
+	nfs_Init_admin_thread();
 
 	/* callback dispatch */
 	nfs_rpc_cb_pkginit();
@@ -861,6 +858,11 @@ static void lower_my_caps(void)
 	char *cap_text;
 	int capsz;
 
+	if (!nfs_start_info.drop_caps) {
+		/* Skip dropping caps by request */
+		return;
+	}
+
 	(void) capget(&caphdr, NULL);
 	switch (caphdr.version) {
 	case _LINUX_CAPABILITY_VERSION_1:
@@ -974,9 +976,8 @@ void nfs_start(nfs_start_info_t *p_start_info)
 	/* Regular exit */
 	LogEvent(COMPONENT_MAIN, "NFS EXIT: regular exit");
 
-	/* if not in grace period, clean up the old state directory */
-	if (!nfs_in_grace())
-		nfs4_recovery_cleanup();
+	/* Try to lift the grace period */
+	nfs_try_lift_grace();
 
 	Cleanup();
 
@@ -1006,3 +1007,34 @@ void nfs_init_wait(void)
 	}
 	PTHREAD_MUTEX_unlock(&nfs_init.init_mutex);
 }
+
+bool nfs_health(void)
+{
+	uint64_t newenq, newdeq;
+	uint64_t dequeue_diff, enqueue_diff;
+	bool healthy;
+
+	newenq = health.enqueued_reqs;
+	newdeq = health.dequeued_reqs;
+	enqueue_diff = newenq - healthstats.enqueued_reqs;
+	dequeue_diff = newdeq - healthstats.dequeued_reqs;
+
+	/* Consider healthy and making progress if we have dequeued some
+	 * requests or there is nothing to dequeue.
+	 */
+	healthy = dequeue_diff > 0 || enqueue_diff == 0;
+
+	if (!healthy) {
+		LogWarn(COMPONENT_DBUS,
+			"Health status is unhealthy. "
+			"enq new: %" PRIu64 ", old: %" PRIu64 "; "
+			"deq new: %" PRIu64 ", old: %" PRIu64,
+			newenq, healthstats.enqueued_reqs,
+			newdeq, healthstats.dequeued_reqs);
+	}
+
+	healthstats.enqueued_reqs = newenq;
+	healthstats.dequeued_reqs = newdeq;
+
+	return healthy;
+}
