@@ -39,9 +39,12 @@
 #include <errno.h>
 #include "fsal.h"
 #include "log.h"
+#include "gsh_rpc.h"
 #include "nfs_init.h"
 #include "nfs_exports.h"
 #include "pnfs_utils.h"
+#include "config_parsing.h"
+#include "conf_url.h"
 
 /**
  * @brief LTTng trace enabling magic
@@ -63,26 +66,31 @@
 #define TRACEPOINT_PROBE_DYNAMIC_LINKAGE
 
 #include "gsh_lttng/logger.h"
+#include "gsh_lttng/mdcache.h"
 #include "gsh_lttng/nfs_rpc.h"
+#include "gsh_lttng/state.h"
+#include "gsh_lttng/fsal_mem.h"
 #endif /* USE_LTTNG */
 
 /* parameters for NFSd startup and default values */
 
 nfs_start_info_t my_nfs_start_info = {
 	.dump_default_config = false,
-	.lw_mark_trigger = false
+	.lw_mark_trigger = false,
+	.drop_caps = true
 };
 
 config_file_t config_struct;
-char *log_path = NULL;
+char *log_path;
 char *exec_name = "nfs-ganesha";
 char *host_name = "localhost";
 int debug_level = -1;
 int detach_flag = true;
+bool dump_trace;
 
 /* command line syntax */
 
-char options[] = "v@L:N:f:p:FRTE:h";
+char options[] = "v@L:N:f:p:FRTE:Ch";
 char usage[] =
 	"Usage: %s [-hd][-L <logfile>][-N <dbg_lvl>][-f <config_file>]\n"
 	"\t[-v]                display version information\n"
@@ -94,6 +102,7 @@ char usage[] =
 	"\t[-R]                daemon will manage RPCSEC_GSS (default is no RPCSEC_GSS)\n"
 	"\t[-T]                dump the default configuration on stdout\n"
 	"\t[-E] <epoch<]       overrides ServerBootTime for ServerEpoch\n"
+	"\t[-C]                dump trace when segfault\n"
 	"\t[-h]                display this help\n"
 	"----------------- Signals ----------------\n"
 	"SIGUSR1    : Enable/Disable File Content Cache forced flush\n"
@@ -145,6 +154,7 @@ int main(int argc, char *argv[])
 	/* Set the server's boot time and epoch */
 	now(&ServerBootTime);
 	ServerEpoch = (time_t) ServerBootTime.tv_sec;
+	srand(ServerEpoch);
 
 	tempo_exec_name = strrchr(argv[0], '/');
 	if (tempo_exec_name != NULL)
@@ -228,6 +238,10 @@ int main(int argc, char *argv[])
 			my_nfs_start_info.dump_default_config = true;
 			break;
 
+		case 'C':
+			dump_trace = true;
+			break;
+
 		case 'E':
 			ServerEpoch = (time_t) atoll(optarg);
 			break;
@@ -243,7 +257,8 @@ int main(int argc, char *argv[])
 	}
 
 	/* initialize memory and logging */
-	nfs_prereq_init(exec_name, host_name, debug_level, log_path);
+	nfs_prereq_init(exec_name, host_name, debug_level, log_path,
+			dump_trace);
 #if GANESHA_BUILD_RELEASE
 	LogEvent(COMPONENT_MAIN, "%s Starting: Ganesha Version %s",
 		 exec_name, GANESHA_VERSION);
@@ -254,6 +269,8 @@ int main(int argc, char *argv[])
 		 __DATE__ " " __TIME__ " on " BUILD_HOST);
 #endif
 
+	/* initialize nfs_init */
+	nfs_init_init();
 
 	nfs_check_malloc();
 
@@ -387,6 +404,9 @@ int main(int argc, char *argv[])
 		LogFatal(COMPONENT_MAIN,
 			 "Could not start nfs daemon, pthread_sigmask failed");
 
+	/* init URL package */
+	config_url_init();
+
 	/* Create a memstream for parser+processing error messages */
 	if (!init_error_type(&err_type))
 		goto fatal_die;
@@ -408,6 +428,8 @@ int main(int argc, char *argv[])
 				 "Error %s while parsing (%s)",
 				 errstr != NULL ? errstr : "unknown",
 				 config_path);
+			if (errstr != NULL)
+				gsh_free(errstr);
 			goto fatal_die;
 		} else
 			LogWarn(COMPONENT_INIT,
