@@ -324,8 +324,7 @@ void _9p_openflags2FSAL(u32 *inflags, fsal_openflags_t *outflags)
 void free_fid(struct _9p_fid *pfid)
 {
 	if (pfid->state != NULL) {
-		if ((pfid->pentry->type == REGULAR_FILE) && pfid->opens &&
-		    pfid->pentry->fsal->m_ops.support_ex(pfid->pentry)) {
+		if ((pfid->pentry->type == REGULAR_FILE) && pfid->opens) {
 			/* We need to close the state before freeing the state.
 			 */
 			(void) pfid->pentry->obj_ops.close2(
@@ -349,7 +348,7 @@ void free_fid(struct _9p_fid *pfid)
 	if (pfid->ucred != NULL)
 		release_9p_user_cred_ref(pfid->ucred);
 
-	gsh_free(pfid->specdata.xattr.xattr_content);
+	gsh_free(pfid->xattr);
 	gsh_free(pfid);
 }
 
@@ -370,47 +369,30 @@ int _9p_tools_clunk(struct _9p_fid *pfid)
 	uid2grp_unref(pfid->gdata);
 
 	/* If the fid is related to a xattr, free the related memory */
-	if (pfid->specdata.xattr.xattr_content != NULL &&
-	    pfid->specdata.xattr.xattr_write == true) {
+	if (pfid->xattr != NULL &&
+	    pfid->xattr->xattr_write == _9P_XATTR_DID_WRITE) {
 		/* Check size give at TXATTRCREATE with
 		 * the one resulting from the writes */
-		if (pfid->specdata.xattr.xattr_size !=
-		    pfid->specdata.xattr.xattr_offset) {
+		if (pfid->xattr->xattr_size != pfid->xattr->xattr_offset) {
 			free_fid(pfid);
 			return EINVAL;
 		}
 
-		/* Do we handle system.posix_acl_access */
-		if (pfid->specdata.xattr.xattr_id == ACL_ACCESS_XATTR_ID) {
-			fsal_status =
-			    pfid->pentry->obj_ops.setextattr_value(
-					pfid->pentry,
-					"system.posix_acl_access",
-					pfid->specdata.xattr.xattr_content,
-					pfid->specdata.xattr.xattr_size,
-					false);
-			if (FSAL_IS_ERROR(fsal_status)) {
-				free_fid(pfid);
-				return _9p_tools_errno(fsal_status);
-			}
-		} else {
-			/* Write the xattr content */
-			fsal_status =
-				pfid->pentry->obj_ops.setextattr_value_by_id(
-					pfid->pentry,
-					pfid->specdata.xattr.xattr_id,
-					pfid->specdata.xattr.xattr_content,
-					pfid->specdata.xattr.xattr_size);
-			if (FSAL_IS_ERROR(fsal_status)) {
-				free_fid(pfid);
-				return _9p_tools_errno(fsal_status);
-			}
+		fsal_status =
+		    pfid->pentry->obj_ops.setextattr_value(
+				pfid->pentry,
+				pfid->xattr->xattr_name,
+				pfid->xattr->xattr_content,
+				pfid->xattr->xattr_size,
+				false);
+		if (FSAL_IS_ERROR(fsal_status)) {
+			free_fid(pfid);
+			return _9p_tools_errno(fsal_status);
 		}
 	}
 
 	/* If object is an opened file, close it */
 	if ((pfid->pentry->type == REGULAR_FILE) && pfid->opens) {
-		pfid->pentry->obj_ops.put_ref(pfid->pentry);
 		pfid->opens = 0;	/* dead */
 
 		LogDebug(COMPONENT_9P,
@@ -418,22 +400,12 @@ int _9p_tools_clunk(struct _9p_fid *pfid)
 			 object_file_type_to_str(pfid->pentry->type),
 			 pfid->pentry);
 
-		if (pfid->pentry->fsal->m_ops.support_ex(pfid->pentry)) {
-			fsal_status =
-			    pfid->pentry->obj_ops.close2(pfid->pentry,
-							 pfid->state);
+		fsal_status = pfid->pentry->obj_ops.close2(pfid->pentry,
+							   pfid->state);
 
-			if (FSAL_IS_ERROR(fsal_status)) {
-				free_fid(pfid);
-				return _9p_tools_errno(fsal_status);
-			}
-		} else {
-			/* Under this flag, pin ref is still checked */
-			fsal_status = fsal_close(pfid->pentry);
-			if (FSAL_IS_ERROR(fsal_status)) {
-				free_fid(pfid);
-				return _9p_tools_errno(fsal_status);
-			}
+		if (FSAL_IS_ERROR(fsal_status)) {
+			free_fid(pfid);
+			return _9p_tools_errno(fsal_status);
 		}
 	}
 
