@@ -72,10 +72,11 @@ int nfs3_fsstat(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	if (isDebug(COMPONENT_NFSPROTO)) {
 		char str[LEN_FH_STR];
 
-		nfs_FhandleToStr(req->rq_vers, &(arg->arg_fsstat3.fsroot), NULL,
-				 str);
+		nfs_FhandleToStr(req->rq_msg.cb_vers,
+				 &(arg->arg_fsstat3.fsroot),
+				 NULL, str);
 		LogDebug(COMPONENT_NFSPROTO,
-			 "REQUEST PROCESSING: Calling nfs3_fsstat handle: %s",
+			 "REQUEST PROCESSING: Calling NFS3_FSSTAT handle: %s",
 			 str);
 	}
 
