@@ -131,6 +131,10 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	struct attrlist attrs_dir, attrs_parent;
 	bool use_cookie_verifier = op_ctx_export_has_option(
 					EXPORT_OPTION_USE_COOKIE_VERIFIER);
+	READDIRPLUS3resfail *resfail =
+			&res->res_readdirplus3.READDIRPLUS3res_u.resfail;
+	READDIRPLUS3resok *resok =
+			&res->res_readdirplus3.READDIRPLUS3res_u.resok;
 
 	/* We have the option of not sending attributes, so set ATTR_RDATTR_ERR.
 	 */
@@ -139,28 +143,21 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	if (isDebug(COMPONENT_NFSPROTO) || isDebug(COMPONENT_NFS_READDIR)) {
 		char str[LEN_FH_STR];
-		log_components_t component;
 
 		sprint_fhandle3(str, &(arg->arg_readdirplus3.dir));
 
-		if (isDebug(COMPONENT_NFSPROTO))
-			component = COMPONENT_NFSPROTO;
-		else
-			component = COMPONENT_NFS_READDIR;
-
-		LogDebug(component,
-			 "REQUEST PROCESSING: Calling nfs3_readdirplus handle: %s",
-			 str);
+		LogDebugAlt(COMPONENT_NFSPROTO, COMPONENT_NFS_READDIR,
+			    "REQUEST PROCESSING: Calling NFS3_READDIRPLUS handle: %s",
+			    str);
 	}
 
 	/* to avoid setting it on each error case */
-	res->res_readdir3.READDIR3res_u.resfail.dir_attributes.
-	    attributes_follow = FALSE;
+	res->res_readdir3.READDIR3res_u.resfail.dir_attributes.attributes_follow
+		= FALSE;
 
 	if (op_ctx_export_has_option(EXPORT_OPTION_NO_READDIR_PLUS)) {
 		res->res_readdirplus3.status = NFS3ERR_NOTSUPP;
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Request not supported");
+		LogDebug(COMPONENT_NFS_READDIR, "Request not supported");
 		goto out;
 	}
 
@@ -179,11 +176,11 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	tracker.total_entries = estimated_num_entries;
 
-	LogFullDebug(COMPONENT_NFS_READDIR,
-		     "nfs3_readdirplus: dircount=%u begin_cookie=%" PRIu64
-		     " estimated_num_entries=%lu, mem_left=%zd",
-		     arg->arg_readdirplus3.dircount, begin_cookie,
-		     estimated_num_entries, tracker.mem_left);
+	LogDebug(COMPONENT_NFS_READDIR,
+		 "NFS3_READDIRPLUS: dircount=%u begin_cookie=%" PRIu64
+		 " estimated_num_entries=%lu, mem_left=%zd",
+		 arg->arg_readdirplus3.dircount, begin_cookie,
+		 estimated_num_entries, tracker.mem_left);
 
 	/* Convert file handle into a vnode */
 	dir_obj = nfs3_FhandleToCache(&(arg->arg_readdirplus3.dir),
@@ -211,9 +208,9 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res->res_readdirplus3.status = nfs3_Errno_status(fsal_status);
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Error %s fetching attributes",
-			     fsal_err_txt(fsal_status));
+		LogDebug(COMPONENT_NFS_READDIR,
+			 "Error %s fetching attributes",
+			 fsal_err_txt(fsal_status));
 		goto out;
 	}
 
@@ -227,8 +224,8 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	if (use_cookie_verifier) {
 		if (attrs_dir.valid_mask == ATTR_RDATTR_ERR) {
 			res->res_readdir3.status = NFS3ERR_SERVERFAULT;
-			LogFullDebug(COMPONENT_NFS_READDIR,
-				     "Could not fetch ctime");
+			LogDebug(COMPONENT_NFS_READDIR,
+				 "Could not fetch ctime");
 			goto out_fail;
 		}
 
@@ -249,8 +246,8 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		}
 	}
 
-	res->res_readdirplus3.READDIRPLUS3res_u.resok.reply.entries = NULL;
-	res->res_readdirplus3.READDIRPLUS3res_u.resok.reply.eof = FALSE;
+	resok->reply.entries = NULL;
+	resok->reply.eof = FALSE;
 
 	/* Fudge cookie for "." and "..", if necessary */
 	if (begin_cookie > 2)
@@ -305,6 +302,10 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		}
 	}
 
+	LogDebug(COMPONENT_NFS_READDIR,
+		 "Readdirplus3 -> Call to fsal_readdir, cookie=%"
+		 PRIu64, fsal_cookie);
+
 	/* Call readdir */
 	fsal_status = fsal_readdir(dir_obj, fsal_cookie, &num_entries, &eod_met,
 				   ATTRS_NFS3, nfs3_readdirplus_callback,
@@ -326,30 +327,18 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		goto out_fail;
 	}
 
-	LogFullDebug(COMPONENT_NFS_READDIR,
-		     "Readdirplus3 -> Call to fsal_readdir( cookie=%"
-		     PRIu64 ")", fsal_cookie);
-
 	if ((num_entries == 0) && (begin_cookie > 1)) {
 		res->res_readdirplus3.status = NFS3_OK;
-		res->res_readdirplus3.READDIRPLUS3res_u.resok.reply.entries =
-		    NULL;
-		res->res_readdirplus3.READDIRPLUS3res_u.resok.reply.eof = TRUE;
+		resok->reply.entries = NULL;
+		resok->reply.eof = TRUE;
 	} else {
-		res->res_readdirplus3.READDIRPLUS3res_u.resok.reply.entries =
-		    tracker.entries;
-		res->res_readdirplus3.READDIRPLUS3res_u.resok.reply.eof =
-		    eod_met;
+		resok->reply.entries = tracker.entries;
+		resok->reply.eof = eod_met;
 	}
 
-	nfs_SetPostOpAttr(dir_obj,
-			  &res->res_readdirplus3.READDIRPLUS3res_u.resok.
-				dir_attributes,
-			  &attrs_dir);
+	nfs_SetPostOpAttr(dir_obj, &resok->dir_attributes, &attrs_dir);
 
-	memcpy(res->res_readdirplus3.READDIRPLUS3res_u.resok.cookieverf,
-	       cookie_verifier,
-	       sizeof(cookieverf3));
+	memcpy(resok->cookieverf, cookie_verifier, sizeof(cookieverf3));
 
 	res->res_readdirplus3.status = NFS3_OK;
 
@@ -360,10 +349,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
  out_fail:
 
 	/* Set the post op attributes as we have found them. */
-	nfs_SetPostOpAttr(dir_obj,
-			  &res->res_readdirplus3.READDIRPLUS3res_u.
-				resfail.dir_attributes,
-			  &attrs_dir);
+	nfs_SetPostOpAttr(dir_obj, &resfail->dir_attributes, &attrs_dir);
 
  out:
 
@@ -431,6 +417,7 @@ fsal_errors_t nfs3_readdirplus_callback(void *opaque,
 		cb_parms->in_result = false;
 		return ERR_FSAL_NO_ERROR;
 	}
+
 	/* This is a pessimistic check, which assumes that we're going
 	 * to send attributes and full size handle - if it fails then
 	 * we're close enough to the buffer size limit and t's time to
@@ -444,6 +431,10 @@ fsal_errors_t nfs3_readdirplus_callback(void *opaque,
 		return ERR_FSAL_NO_ERROR;
 	}
 
+	LogDebug(COMPONENT_NFS_READDIR,
+		"Callback for %s cookie %"PRIu64,
+		cb_parms->name, cookie);
+
 	ep3->fileid = obj->fileid;
 	ep3->name = gsh_strdup(cb_parms->name);
 	ep3->cookie = cookie;
@@ -468,9 +459,10 @@ fsal_errors_t nfs3_readdirplus_callback(void *opaque,
 		tracker->mem_left -=
 		    ep3->name_handle.post_op_fh3_u.handle.data.data_len + 12;
 
-		ep3->name_attributes.attributes_follow = true;
-
-		nfs3_FSALattr_To_Fattr(
+		/* Check if attributes follow and then place the attributes
+		 * that follow
+		 */
+		ep3->name_attributes.attributes_follow = nfs3_FSALattr_To_Fattr(
 			obj, attr,
 			&ep3->name_attributes.post_op_attr_u.attributes);
 	} else {
