@@ -98,22 +98,21 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 	EXCHANGE_ID4res * const res_EXCHANGE_ID4 =
 	    &resp->nfs_resop4_u.opexchange_id;
 	EXCHANGE_ID4resok * const res_EXCHANGE_ID4_ok =
-	    (&resp->nfs_resop4_u.opexchange_id.EXCHANGE_ID4res_u.eir_resok4);
+	    &resp->nfs_resop4_u.opexchange_id.EXCHANGE_ID4res_u.eir_resok4;
 
 	resp->resop = NFS4_OP_EXCHANGE_ID;
 
 	if (data->minorversion == 0)
 		return res_EXCHANGE_ID4->eir_status = NFS4ERR_INVAL;
 
-	if ((arg_EXCHANGE_ID4->
-	     eia_flags & ~(EXCHGID4_FLAG_SUPP_MOVED_REFER |
-			   EXCHGID4_FLAG_SUPP_MOVED_MIGR |
-			   EXCHGID4_FLAG_BIND_PRINC_STATEID |
-			   EXCHGID4_FLAG_USE_NON_PNFS |
-			   EXCHGID4_FLAG_USE_PNFS_MDS |
-			   EXCHGID4_FLAG_USE_PNFS_DS |
-			   EXCHGID4_FLAG_UPD_CONFIRMED_REC_A |
-			   EXCHGID4_FLAG_CONFIRMED_R)) != 0)
+	if ((arg_EXCHANGE_ID4->eia_flags & ~(EXCHGID4_FLAG_SUPP_MOVED_REFER |
+					     EXCHGID4_FLAG_SUPP_MOVED_MIGR |
+					     EXCHGID4_FLAG_BIND_PRINC_STATEID |
+					     EXCHGID4_FLAG_USE_NON_PNFS |
+					     EXCHGID4_FLAG_USE_PNFS_MDS |
+					     EXCHGID4_FLAG_USE_PNFS_DS |
+					     EXCHGID4_FLAG_UPD_CONFIRMED_REC_A)
+	     ) != 0)
 		return res_EXCHANGE_ID4->eir_status = NFS4ERR_INVAL;
 
 	/* If client did not ask for pNFS related server roles than just set
@@ -149,13 +148,10 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 	server_addr = get_raddr(data->req->rq_xprt);
 
 	/* Do we already have one or more records for client id (x)? */
-	client_record =
-	    get_client_record(arg_EXCHANGE_ID4->eia_clientowner.co_ownerid.
-			      co_ownerid_val,
-			      arg_EXCHANGE_ID4->eia_clientowner.co_ownerid.
-			      co_ownerid_len,
-			      pnfs_flags,
-			      server_addr);
+	client_record = get_client_record(
+		arg_EXCHANGE_ID4->eia_clientowner.co_ownerid.co_ownerid_val,
+		arg_EXCHANGE_ID4->eia_clientowner.co_ownerid. co_ownerid_len,
+		pnfs_flags, server_addr);
 
 	if (client_record == NULL) {
 		/* Some major failure */
@@ -248,12 +244,16 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 				res_EXCHANGE_ID4->eir_status = NFS4ERR_PERM;
 			} else {
 				/* CASE 6, Update */
-				/** @todo: this is not implemented,
-				    the things it updates aren't even
-				    tracked */
-				LogMajor(COMPONENT_CLIENTID,
-					 "EXCHANGE_ID Update not supported");
-				res_EXCHANGE_ID4->eir_status = NFS4ERR_NOTSUPP;
+				/** @todo: we don't track or handle the things
+				 *         that are updated, but we can still
+				 *         allow the update.
+				 */
+				LogDebug(COMPONENT_CLIENTID,
+					 "EXCHANGE_ID Update ignored");
+				unconf = conf;
+				res_EXCHANGE_ID4_ok->eir_flags |=
+				    EXCHGID4_FLAG_CONFIRMED_R;
+				goto return_ok;
 			}
 		} else {
 			/* CASE 8, Update but wrong verifier */
@@ -265,6 +265,7 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 
 		goto out;
 	} else if (conf == NULL && update) {
+		/* CASE 7, Update but No Confirmed Record */
 		res_EXCHANGE_ID4->eir_status = NFS4ERR_NOENT;
 		goto out;
 	}
@@ -301,6 +302,7 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 	}
 
 	unconf->cid_create_session_sequence = 1;
+	unconf->cid_create_session_slot.csr_status = NFS4ERR_SEQ_MISORDERED;
 
 	glist_init(&unconf->cid_cb.v41.cb_session_list);
 
@@ -365,6 +367,13 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 
 	res_EXCHANGE_ID4->eir_status = NFS4_OK;
 
+	if (unconf == conf) {
+		/* We just updated a confirmed clientid, release the refcount
+		 * now.
+		 */
+		dec_client_id_ref(conf);
+	}
+
  out:
 
 	PTHREAD_MUTEX_unlock(&client_record->cr_mutex);
@@ -383,19 +392,11 @@ int nfs4_op_exchange_id(struct nfs_argop4 *op, compound_data_t *data,
 void nfs4_op_exchange_id_Free(nfs_resop4 *res)
 {
 	EXCHANGE_ID4res *resp = &res->nfs_resop4_u.opexchange_id;
+	EXCHANGE_ID4resok *resok = &resp->EXCHANGE_ID4res_u.eir_resok4;
 
 	if (resp->eir_status == NFS4_OK) {
-		if (resp->EXCHANGE_ID4res_u.eir_resok4.eir_server_scope.
-		    eir_server_scope_val != NULL)
-			gsh_free(resp->EXCHANGE_ID4res_u.eir_resok4.
-				 eir_server_scope.eir_server_scope_val);
-		if (resp->EXCHANGE_ID4res_u.eir_resok4.eir_server_owner.
-		    so_major_id.so_major_id_val != NULL)
-			gsh_free(resp->EXCHANGE_ID4res_u.eir_resok4.
-				 eir_server_owner.so_major_id.so_major_id_val);
-		if (resp->EXCHANGE_ID4res_u.eir_resok4.eir_server_impl_id.
-		    eir_server_impl_id_val != NULL)
-			gsh_free(resp->EXCHANGE_ID4res_u.eir_resok4.
-				 eir_server_impl_id.eir_server_impl_id_val);
+		gsh_free(resok->eir_server_scope.eir_server_scope_val);
+		gsh_free(resok->eir_server_owner.so_major_id.so_major_id_val);
+		gsh_free(resok->eir_server_impl_id.eir_server_impl_id_val);
 	}
 }
