@@ -60,6 +60,8 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 {
 	/* Shorter alias for arguments */
 	LOCK4args * const arg_LOCK4 = &op->nfs_argop4_u.oplock;
+	open_to_lock_owner4 *arg_open_owner =
+		&arg_LOCK4->locker.locker4_u.open_owner;
 	/* Shorter alias for response */
 	LOCK4res * const res_LOCK4 = &resp->nfs_resop4_u.oplock;
 	/* Status code from state calls */
@@ -97,6 +99,8 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	/* Indicate if we let FSAL to handle requests during grace. */
 	bool_t fsal_grace = false;
 	int rc;
+	struct fsal_obj_handle *obj = data->current_obj;
+	bool state_lock_held = false;
 
 	LogDebug(COMPONENT_NFS_V4_LOCK,
 		 "Entering NFS v4 LOCK handler ----------------------");
@@ -156,12 +160,12 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	if (arg_LOCK4->locker.new_lock_owner) {
 		/* Check stateid correctness and get pointer to state */
 		nfs_status = nfs4_Check_Stateid(
-			&arg_LOCK4->locker.locker4_u.open_owner.open_stateid,
-			data->current_obj,
+			&arg_open_owner->open_stateid,
+			obj,
 			&state_open,
 			data,
 			STATEID_SPECIAL_FOR_LOCK,
-			arg_LOCK4->locker.locker4_u.open_owner.open_seqid,
+			arg_open_owner->open_seqid,
 			data->minorversion == 0,
 			lock_tag);
 
@@ -200,16 +204,16 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		lock_state = NULL;
 		lock_owner = NULL;
 		resp_owner = open_owner;
-		seqid = arg_LOCK4->locker.locker4_u.open_owner.open_seqid;
+		seqid = arg_open_owner->open_seqid;
 
 		LogLock(COMPONENT_NFS_V4_LOCK, NIV_FULL_DEBUG,
 			"LOCK New lock owner from open owner",
-			data->current_obj, open_owner, &lock_desc);
+			obj, open_owner, &lock_desc);
 
 		/* Check is the clientid is known or not */
 		rc = nfs_client_id_get_confirmed(
-			data->minorversion == 0 ? arg_LOCK4->locker.
-				  locker4_u.open_owner.lock_owner.clientid
+			data->minorversion == 0
+				? arg_open_owner->lock_owner.clientid
 				: data->session->clientid,
 			&clientid);
 
@@ -250,8 +254,8 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		}
 
 		/* Is this lock_owner known ? */
-		convert_nfs4_lock_owner(&arg_LOCK4->locker.locker4_u.open_owner.
-					lock_owner, &owner_name);
+		convert_nfs4_lock_owner(&arg_open_owner->lock_owner,
+					&owner_name);
 		LogStateOwner("Lock: ", lock_owner);
 	} else {
 		/* Existing lock owner Find the lock stateid From
@@ -267,7 +271,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		 */
 		nfs_status = nfs4_Check_Stateid(
 			&arg_LOCK4->locker.locker4_u.lock_owner.lock_stateid,
-			data->current_obj,
+			obj,
 			&lock_state,
 			data,
 			STATEID_SPECIAL_FOR_LOCK,
@@ -343,7 +347,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		seqid = arg_LOCK4->locker.locker4_u.lock_owner.lock_seqid;
 
 		LogLock(COMPONENT_NFS_V4_LOCK, NIV_FULL_DEBUG,
-			"LOCK Existing lock owner", data->current_obj,
+			"LOCK Existing lock owner", obj,
 			lock_owner, &lock_desc);
 
 		/* Get the client for this open owner */
@@ -358,7 +362,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		if (!Check_nfs4_seqid(resp_owner,
 				      seqid,
 				      op,
-				      data->current_obj,
+				      obj,
 				      resp,
 				      lock_tag)) {
 			/* Response is all setup for us and LogDebug
@@ -408,7 +412,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		 */
 		LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
 			"LOCK failed, SHARE doesn't allow access",
-			data->current_obj, lock_owner, &lock_desc);
+			obj, lock_owner, &lock_desc);
 
 		res_LOCK4->status = NFS4ERR_OPENMODE;
 
@@ -421,14 +425,14 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	 * the other and non-NULL at this point - so makes for a better log).
 	 */
 	if (nfs_in_grace()) {
-		if (op_ctx->fsal_export->exp_ops.
-			fs_supports(op_ctx->fsal_export, fso_grace_method))
-				fsal_grace = true;
+		if (op_ctx->fsal_export->exp_ops.fs_supports(
+					op_ctx->fsal_export, fso_grace_method))
+			fsal_grace = true;
 
 		if (!fsal_grace && !arg_LOCK4->reclaim) {
 			LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
 			"LOCK failed, non-reclaim while in grace",
-				data->current_obj, resp_owner, &lock_desc);
+				obj, resp_owner, &lock_desc);
 			res_LOCK4->status = NFS4ERR_GRACE;
 			goto out;
 		}
@@ -436,7 +440,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		    && !clientid->cid_allow_reclaim) {
 			LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
 				"LOCK failed, invalid reclaim while in grace",
-				data->current_obj, resp_owner, &lock_desc);
+				obj, resp_owner, &lock_desc);
 			res_LOCK4->status = NFS4ERR_NO_GRACE;
 			goto out;
 		}
@@ -444,7 +448,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		if (arg_LOCK4->reclaim) {
 			LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
 				"LOCK failed, reclaim while not in grace",
-				data->current_obj, resp_owner, &lock_desc);
+				obj, resp_owner, &lock_desc);
 			res_LOCK4->status = NFS4ERR_NO_GRACE;
 			goto out;
 		}
@@ -466,7 +470,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 					       open_owner,
 					       0,
 					       &isnew,
-					       CARE_ALWAYS);
+					       CARE_ALWAYS, true);
 
 		LogStateOwner("Lock: ", lock_owner);
 
@@ -474,7 +478,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			res_LOCK4->status = NFS4ERR_RESOURCE;
 			LogLock(COMPONENT_NFS_V4_LOCK, NIV_EVENT,
 				"LOCK failed to create new lock owner",
-				data->current_obj, open_owner, &lock_desc);
+				obj, open_owner, &lock_desc);
 			goto out2;
 		}
 
@@ -484,15 +488,14 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			if (!glist_empty(&lock_owner->so_lock_list)
 			    && (data->minorversion == 0)
 			    && !Check_nfs4_seqid(lock_owner,
-						 arg_LOCK4->locker.locker4_u.
-						     open_owner.lock_seqid,
+						 arg_open_owner->lock_seqid,
 						 op,
-						 data->current_obj,
+						 obj,
 						 resp,
 						 lock_tag)) {
 				LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
 					"LOCK failed to create new lock owner, re-use",
-					data->current_obj,
+					obj,
 					open_owner, &lock_desc);
 				dump_all_locks(
 					"All locks (re-use of lock owner)");
@@ -511,15 +514,18 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			 * there to be such a state if the lock owner was
 			 * previously unknown.
 			 * This handles 4.0 replay locks with an open stateid
-			 * and new_lock_owner == true. It's not required for
-			 * 4.1 (the replay will be caught by the SEQUENCE op),
-			 * and 4.1 can get a concurrent FREE STATEID call
-			 * which would false-positive here and mess it up.
+			 * and new_lock_owner == true. It also handles
+			 * situations where all locks have been released and
+			 * the client is claiming a new lock owner to get a
+			 * new stateid, we will attempt to recycle.
 			 */
-			if (data->minorversion == 0)
-				lock_state = nfs4_State_Get_Obj(
-						data->current_obj,
-						lock_owner);
+			PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+			state_lock_held = true;
+			lock_state = nfs4_State_Get_Obj(obj, lock_owner);
+		} else {
+			/* Take the state_lock now */
+			PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+			state_lock_held = true;
 		}
 
 		if (lock_state == NULL) {
@@ -528,20 +534,20 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			candidate_data.lock.openstate = state_open;
 
 			/* Add the lock state to the lock table */
-			state_status = state_add(data->current_obj,
-						 STATE_TYPE_LOCK,
-						 &candidate_data,
-						 lock_owner,
-						 &lock_state,
-						 data->minorversion > 0 ?
-							&refer : NULL);
+			state_status = state_add_impl(obj,
+						      STATE_TYPE_LOCK,
+						      &candidate_data,
+						      lock_owner,
+						      &lock_state,
+						      data->minorversion > 0 ?
+								&refer : NULL);
 
 			if (state_status != STATE_SUCCESS) {
 				res_LOCK4->status = NFS4ERR_RESOURCE;
 
 				LogLock(COMPONENT_NFS_V4_LOCK, NIV_DEBUG,
 					"LOCK failed to add new stateid",
-					data->current_obj, lock_owner,
+					obj, lock_owner,
 					&lock_desc);
 
 				goto out2;
@@ -556,6 +562,10 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 				&lock_state->state_data.lock.state_sharelist);
 
 		}
+	} else {
+		/* Take the state_lock now */
+		PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+		state_lock_held = true;
 	}
 
 	if (data->minorversion == 0) {
@@ -565,7 +575,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 
 	/* Now we have a lock owner and a stateid.  Go ahead and push
 	 * lock into SAL (and FSAL). */
-	state_status = state_lock(data->current_obj,
+	state_status = state_lock(obj,
 				  lock_owner,
 				  lock_state,
 				  blocking,
@@ -595,15 +605,16 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 			Copy_nfs4_state_req(resp_owner,
 					    seqid,
 					    op,
-					    data->current_obj,
+					    obj,
 					    resp,
 					    lock_tag);
 		}
 
 		if (arg_LOCK4->locker.new_lock_owner) {
 			/* Need to destroy new state */
-			state_del(lock_state);
+			state_del_locked(lock_state);
 		}
+
 		goto out2;
 	}
 
@@ -621,10 +632,9 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	if (arg_LOCK4->locker.new_lock_owner) {
 		/* Also save the response in the lock owner */
 		Copy_nfs4_state_req(lock_owner,
-				    arg_LOCK4->locker.locker4_u.open_owner.
-				    lock_seqid,
+				    arg_open_owner->lock_seqid,
 				    op,
-				    data->current_obj,
+				    obj,
 				    resp,
 				    lock_tag);
 
@@ -640,7 +650,7 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 	}
 
 	LogLock(COMPONENT_NFS_V4_LOCK, NIV_FULL_DEBUG, "LOCK applied",
-		data->current_obj, lock_owner, &lock_desc);
+		obj, lock_owner, &lock_desc);
 
  out:
 
@@ -649,13 +659,18 @@ int nfs4_op_lock(struct nfs_argop4 *op, compound_data_t *data,
 		Copy_nfs4_state_req(resp_owner,
 				    seqid,
 				    op,
-				    data->current_obj,
+				    obj,
 				    resp,
 				    lock_tag);
 	}
 
  out2:
 
+	if (state_lock_held) {
+		/* Now release the state_lock */
+		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+	}
+
 	if (state_open != NULL)
 		dec_state_t_ref(state_open);
 
