@@ -60,238 +60,6 @@ void nfs4_op_open_CopyRes(OPEN4res *res_dst, OPEN4res *res_src)
 	res_dst->OPEN4res_u.resok4.attrset = res_src->OPEN4res_u.resok4.attrset;
 }
 
-/**
- * @brief Perform the open operation
- *
- * This function performs the actual open operation in cache_inode and
- * the State Abstraction layer.
- *
- * @note state_lock MUST be held for write
- *
- * @param[in]     op        Arguments to the OPEN operation
- * @param[in,out] data      Compound's data
- * @param[in]     owner     The open owner
- * @param[out]    state     The created or found open state
- * @param[out]    new_state True if the state was newly created
- * @param[in]     openflags Open flags for the FSAL
- *
- * @retval NFS4_OK on success.
- * @retval Valid errors for NFS4_OP_OPEN.
- */
-
-static nfsstat4 open4_do_open(struct nfs_argop4 *op, compound_data_t *data,
-			      state_owner_t *owner, state_t **state,
-			      bool *new_state, fsal_openflags_t openflags,
-			      bool skip_permission)
-{
-	/* The arguments to the open operation */
-	OPEN4args *args = &op->nfs_argop4_u.opopen;
-	/* The state to be added */
-	union state_data candidate_data;
-	/* Return value of state operations */
-	state_status_t state_status = STATE_SUCCESS;
-	/* Return value of Cache inode operations */
-	fsal_status_t status = {0, 0};
-	/* The open state for the file */
-	state_t *file_state = NULL;
-	/* Tracking data for the open state */
-	struct state_refer refer;
-	fsal_accessflags_t access_mask = 0;
-	struct state_hdl *ostate;
-
-	*state = NULL;
-	*new_state = true;
-
-	/* Record the sequence info */
-	if (data->minorversion > 0) {
-		memcpy(refer.session,
-		       data->session->session_id,
-		       sizeof(sessionid4));
-		refer.sequence = data->sequence;
-		refer.slot = data->slot;
-	}
-
-	if (args->share_access & OPEN4_SHARE_ACCESS_WRITE)
-		access_mask |= FSAL_WRITE_ACCESS;
-
-	if (args->share_access & OPEN4_SHARE_ACCESS_READ)
-		access_mask |= FSAL_READ_ACCESS;
-
-	if (!skip_permission) {
-		status = fsal_access(data->current_obj, access_mask);
-
-		if (FSAL_IS_ERROR(status)) {
-			/* If non-permission error, return it. */
-			if (status.major != ERR_FSAL_ACCESS) {
-				LogDebug(COMPONENT_STATE,
-					 "fsal_access returned %s",
-					 msg_fsal_err(status.major));
-				return nfs4_Errno_status(status);
-			}
-
-			/* If WRITE access is requested, return permission
-			 * error
-			 */
-			if (args->share_access & OPEN4_SHARE_ACCESS_WRITE) {
-				LogDebug(COMPONENT_STATE,
-					 "fsal_access returned %s with ACCESS_WRITE",
-					 msg_fsal_err(status.major));
-				return nfs4_Errno_status(status);
-			}
-
-			/* If just a permission error and file was opened read
-			 * only, try execute permission.
-			 */
-			status = fsal_access(data->current_obj,
-				FSAL_MODE_MASK_SET(FSAL_X_OK) |
-				FSAL_ACE4_MASK_SET(FSAL_ACE_PERM_EXECUTE));
-
-			if (FSAL_IS_ERROR(status)) {
-				LogDebug(COMPONENT_STATE,
-					 "fsal_access returned %s after checking for executer permission",
-					 msg_fsal_err(status.major));
-				return nfs4_Errno_status(status);
-			}
-		}
-	}
-
-	ostate = data->current_obj->state_hdl;
-	if (!ostate)
-		return NFS4ERR_SERVERFAULT;
-
-	candidate_data.share.share_access =
-	    args->share_access & OPEN4_SHARE_ACCESS_BOTH;
-	candidate_data.share.share_deny = args->share_deny;
-	candidate_data.share.share_access_prev = 0;
-	candidate_data.share.share_deny_prev = 0;
-
-	state_status =
-	    state_share_check_conflict(ostate,
-				       candidate_data.share.share_access,
-				       candidate_data.share.share_deny,
-				       SHARE_BYPASS_NONE);
-
-	/* Quick exit if there is any share conflict */
-	if (state_status != STATE_SUCCESS)
-		return nfs4_Errno_state(state_status);
-
-	/* Check if any existing delegations conflict with this open.
-	 * Delegation recalls will be scheduled if there is a conflict.
-	 */
-	if (state_deleg_conflict(data->current_obj, candidate_data.share.
-				 share_access & OPEN4_SHARE_ACCESS_WRITE)) {
-		return NFS4ERR_DELAY;
-	}
-
-	/* Try to find if the same open_owner already has acquired a
-	 * stateid for this file
-	 */
-	file_state = nfs4_State_Get_Obj(data->current_obj, owner);
-
-	if (file_state != NULL) {
-		*new_state = false;
-
-		if (isFullDebug(COMPONENT_STATE)) {
-			char str[LOG_BUFF_LEN] = "\0";
-			struct display_buffer dspbuf = {sizeof(str), str, str};
-
-			display_stateid(&dspbuf, file_state);
-
-			LogFullDebug(COMPONENT_STATE,
-				     "Found existing state %s",
-				     str);
-		}
-	}
-
-	if (*new_state) {
-		state_status = state_add_impl(data->current_obj,
-					      STATE_TYPE_SHARE,
-					      &candidate_data,
-					      owner,
-					      &file_state,
-					      data->minorversion > 0 ?
-							&refer : NULL);
-
-		if (state_status != STATE_SUCCESS)
-			return nfs4_Errno_state(state_status);
-
-		glist_init(&(file_state->state_data.share.share_lockstates));
-	} else {
-		/* Check if open from another export */
-		if (!state_same_export(file_state, op_ctx->ctx_export)) {
-			LogEvent(COMPONENT_STATE,
-				 "Lock Owner Export Conflict, Lock held for export %"
-				 PRIu16" request for export %"PRIu16,
-				 state_export_id(file_state),
-				 op_ctx->ctx_export->export_id);
-			dec_state_t_ref(file_state);
-			return NFS4ERR_INVAL;
-		}
-	}
-
-	/* Fill in the clientid for NFSv4.0 */
-	if (data->minorversion == 0) {
-		op_ctx->clientid =
-		    &owner->so_owner.so_nfs4_owner.so_clientid;
-	}
-
-	status = fsal_open(data->current_obj, openflags);
-	if (FSAL_IS_ERROR(status))
-		return nfs4_Errno_status(status);
-
-	/* Clear the clientid for NFSv4.0 */
-
-	if (data->minorversion == 0)
-		op_ctx->clientid = NULL;
-
-	/* Push share state to SAL (and FSAL) and update the union of
-	   file share state. */
-
-	if (*new_state) {
-		state_status = state_share_add(data->current_obj,
-					       owner,
-					       file_state,
-					       (openflags & FSAL_O_RECLAIM));
-
-		if (state_status != STATE_SUCCESS) {
-			status = fsal_close(data->current_obj);
-			if (FSAL_IS_ERROR(status))
-				/* Log bad close and continue. */
-				LogEvent(COMPONENT_STATE,
-					 "Failed to close file: status=%s",
-					 msg_fsal_err(status.major));
-
-			return nfs4_Errno_state(state_status);
-		}
-	} else {
-		/* If we find the previous share state, update share state. */
-		LogFullDebug(COMPONENT_STATE,
-			     "Update existing share state");
-		state_status = state_share_upgrade(data->current_obj,
-						   &candidate_data,
-						   owner,
-						   file_state,
-						   openflags & FSAL_O_RECLAIM);
-
-		if (state_status != STATE_SUCCESS) {
-			status = fsal_close(data->current_obj);
-			if (FSAL_IS_ERROR(status))
-				/* Log bad close and continue. */
-				LogEvent(COMPONENT_STATE,
-					 "Failed to close file: status=%s",
-					 msg_fsal_err(status.major));
-
-			LogEvent(COMPONENT_STATE,
-				 "Failed to update existing share state");
-			dec_state_t_ref(file_state);
-			return nfs4_Errno_state(state_status);
-		}
-	}
-
-	*state = file_state;
-	return NFS4_OK;
-}
-
 /**
  * @brief Create an NFSv4 filehandle
  *
@@ -384,9 +152,9 @@ static nfsstat4 open4_validate_claim(compound_data_t *data,
 		if (data->minorversion == 0)
 			status = NFS4ERR_NOTSUPP;
 
-		if (op_ctx->fsal_export->exp_ops.
-			fs_supports(op_ctx->fsal_export, fso_grace_method))
-				fsal_grace = true;
+		if (op_ctx->fsal_export->exp_ops.fs_supports(
+					op_ctx->fsal_export, fso_grace_method))
+			fsal_grace = true;
 		if (!fsal_grace && nfs_in_grace())
 			status = NFS4ERR_GRACE;
 		break;
@@ -396,7 +164,7 @@ static nfsstat4 open4_validate_claim(compound_data_t *data,
 		break;
 
 	case CLAIM_PREVIOUS:
-		if ((clientid->cid_allow_reclaim != 1) || !nfs_in_grace()
+		if (!clientid->cid_allow_reclaim || !nfs_in_grace()
 		    || ((data->minorversion > 0)
 		    && clientid->cid_cb.v41.cid_reclaim_complete))
 			status = NFS4ERR_NO_GRACE;
@@ -447,8 +215,6 @@ bool open4_open_owner(struct nfs_argop4 *op, compound_data_t *data,
 	bool_t isnew;
 	/* Return value of FSAL operations */
 	fsal_status_t status = {0, 0};
-	/* The filename to create */
-	char *filename = NULL;
 	struct fsal_obj_handle *obj_lookup = NULL;
 
 	/* Is this open_owner known? If so, get it so we can use
@@ -482,343 +248,66 @@ bool open4_open_owner(struct nfs_argop4 *op, compound_data_t *data,
 		LogDebug(COMPONENT_STATE,
 			 "Previously known open_owner is used with seqid=0, ask the client to confirm it again");
 		(*owner)->so_owner.so_nfs4_owner.so_confirmed = false;
-	} else {
-		/* Check for replay */
-		if (!Check_nfs4_seqid(*owner,
-				      arg_OPEN4->seqid,
-				      op,
-				      data->current_obj,
-				      res,
-				      open_tag)) {
-			/* Response is setup for us and LogDebug told what was
-			 * wrong.
-			 *
-			 * Or if this is a seqid replay, find the file entry
-			 * and update currentFH
-			 */
-			if (res_OPEN4->status == NFS4_OK) {
-				utf8string *utfile;
-				open_claim4 *oc = &arg_OPEN4->claim;
+		return true;
+	}
 
-				switch (oc->claim) {
-				case CLAIM_NULL:
-					utfile = &oc->open_claim4_u.file;
-					break;
-				case CLAIM_DELEGATE_CUR:
-					utfile = &oc->open_claim4_u.
-							delegate_cur_info.file;
-					break;
-				case CLAIM_DELEGATE_PREV:
-				default:
-					return false;
-				}
-				/* Check if filename is correct */
-				res_OPEN4->status = nfs4_utf8string2dynamic(
-				     utfile, UTF8_SCAN_ALL, &filename);
+	/* Check for replay */
+	if (Check_nfs4_seqid(*owner,
+			     arg_OPEN4->seqid,
+			     op,
+			     data->current_obj,
+			     res,
+			     open_tag)) {
+		/* No replay */
+		return true;
+	}
 
-				if (res_OPEN4->status != NFS4_OK)
-					return false;
+	/* Response is setup for us and LogDebug told what was
+	 * wrong.
+	 *
+	 * Or if this is a seqid replay, find the file entry
+	 * and update currentFH
+	 */
+	if (res_OPEN4->status == NFS4_OK) {
+		utf8string *utfile;
+		open_claim4 *oc = &arg_OPEN4->claim;
+		char *filename;
 
-				status = fsal_lookup(data->current_obj,
-						     filename,
-						     &obj_lookup,
-						     NULL);
-				if (filename) {
-					gsh_free(filename);
-					filename = NULL;
-				}
-
-				if (obj_lookup == NULL) {
-					res_OPEN4->status =
-					    nfs4_Errno_status(status);
-					return false;
-				}
-				res_OPEN4->status =
-				    open4_create_fh(data, obj_lookup, false);
-			}
+		/* Load up CLAIM_DELEGATE_CUR file */
 
+		switch (oc->claim) {
+		case CLAIM_NULL:
+			utfile = &oc->open_claim4_u.file;
+			break;
+		case CLAIM_DELEGATE_CUR:
+			utfile = &oc->open_claim4_u.delegate_cur_info.file;
+			break;
+		case CLAIM_DELEGATE_PREV:
+		default:
 			return false;
 		}
-	}
+		/* Check if filename is correct */
+		res_OPEN4->status = nfs4_utf8string2dynamic(
+					utfile, UTF8_SCAN_ALL, &filename);
 
-	return true;
-}
+		if (res_OPEN4->status != NFS4_OK)
+			return false;
 
-/**
- * @brief Create a named file
- *
- * This function implements the OPEN4_CREATE alternative of
- * CLAIM_NULL.
- *
- * @param[in]      arg      OPEN4 arguments
- * @param[in,out]  data     Comopund's data
- * @param[out]     res      OPEN4 response
- * @param[in]      parent   Directory in which to create the file
- * @param[out]     obj	    File to be opened
- * @param[in]      filename filename
- */
+		status = fsal_lookup(data->current_obj,
+				     filename,
+				     &obj_lookup,
+				     NULL);
 
-static nfsstat4 open4_create(OPEN4args *arg, compound_data_t *data,
-			     OPEN4res *res, struct fsal_obj_handle *parent,
-			     struct fsal_obj_handle **obj, const char *filename,
-			     bool *created)
-{
-	/* Newly created file */
-	struct fsal_obj_handle *obj_newfile = NULL;
-	/* Return code from calls made directly to the FSAL. */
-	fsal_status_t fsal_status = { 0, 0 };
-	/* Convertedattributes to set */
-	struct attrlist sattr;
-	/* Whether the client supplied any attributes */
-	bool sattr_provided = false;
-	/* Return from FSAL calls */
-	fsal_status_t status = {0, 0};
-	/* True if a verifier has been specified and we are
-	   performing exclusive creation semantics. */
-	bool verf_provided = false;
-	/* Client provided verifier, split into two piees */
-	uint32_t verf_hi = 0, verf_lo = 0;
-
-	memset(&sattr, 0, sizeof(sattr));
-
-	*obj = NULL;
-	*created = false;
-
-	/* if quota support is active, then we should check is
-	   the FSAL allows inode creation or not */
-	fsal_status = op_ctx->fsal_export->exp_ops.check_quota(
-						op_ctx->fsal_export,
-						op_ctx->ctx_export->fullpath,
-						FSAL_QUOTA_INODES);
-
-	if (FSAL_IS_ERROR(fsal_status))
-		return NFS4ERR_DQUOT;
-
-	/* Check if asked attributes are correct */
-	if (arg->openhow.openflag4_u.how.mode == GUARDED4
-	    || arg->openhow.openflag4_u.how.mode == UNCHECKED4) {
-		if (!nfs4_Fattr_Supported(
-		     &arg->openhow.openflag4_u.how.createhow4_u.createattrs))
-			return NFS4ERR_ATTRNOTSUPP;
-
-		if (!nfs4_Fattr_Check_Access(
-		     &arg->openhow.openflag4_u.how.createhow4_u.createattrs,
-		     FATTR4_ATTR_WRITE)) {
-			return NFS4ERR_INVAL;
-		}
-		if (arg->openhow.openflag4_u.how.createhow4_u.createattrs.
-		    attrmask.bitmap4_len != 0) {
-			/* Convert fattr4 so nfs4_sattr */
-			res->status = nfs4_Fattr_To_FSAL_attr(
-				&sattr,
-				&arg->openhow.openflag4_u.how.createhow4_u.
-				  createattrs,
-				data);
-
-			if (res->status != NFS4_OK)
-				return res->status;
-
-			sattr_provided = true;
-
-		}
-	} else if (arg->openhow.openflag4_u.how.mode == EXCLUSIVE4_1) {
-		/**
-		 * @note EXCLUSIVE4_1 gets its own attribute check,
-		 * because they're stored in a different spot.
-		 *
-		 * @todo ACE: This can be refactored later.
-		 */
-		if (!nfs4_Fattr_Supported(
-		     &arg->openhow.openflag4_u.how.createhow4_u.ch_createboth.
-		     cva_attrs)) {
-			return NFS4ERR_ATTRNOTSUPP;
-		}
-		if (!nfs4_Fattr_Check_Access(
-		     &arg->openhow.openflag4_u.how.createhow4_u.ch_createboth.
-			cva_attrs,
-		     FATTR4_ATTR_WRITE)) {
-			return NFS4ERR_INVAL;
-		}
-		if (arg->openhow.openflag4_u.how.createhow4_u.ch_createboth.
-		    cva_attrs.attrmask.bitmap4_len != 0) {
-			/* Convert fattr4 so nfs4_sattr */
-			res->status = nfs4_Fattr_To_FSAL_attr(
-				&sattr,
-				&arg->openhow.openflag4_u.how.createhow4_u.
-				  ch_createboth.cva_attrs,
-				data);
-
-			if (res->status != NFS4_OK)
-				return res->status;
-
-			sattr_provided = true;
-		}
-		if (sattr_provided
-		    && ((FSAL_TEST_MASK(sattr.valid_mask, ATTR_ATIME))
-		    || (FSAL_TEST_MASK(sattr.valid_mask, ATTR_MTIME)))) {
-			res->status = NFS4ERR_INVAL;
-			return res->status;
-		}
-	}
-
-	if ((arg->openhow.openflag4_u.how.mode == EXCLUSIVE4_1)
-	    || (arg->openhow.openflag4_u.how.mode == EXCLUSIVE4)) {
-		char *verf =
-		    (char *)((arg->openhow.openflag4_u.how.mode == EXCLUSIVE4_1)
-			     ? &arg->openhow.openflag4_u.how.createhow4_u.
-				 ch_createboth.cva_verf :
-			       &arg->openhow.openflag4_u.how.createhow4_u.
-				 createverf);
-
-		verf_provided = true;
-
-		/* If we knew all our FSALs could store a 64 bit
-		 * atime, we could just use that and there would be
-		 * no need to split the verifier up.
-		 */
-		memcpy(&verf_hi, verf, sizeof(uint32_t));
-		memcpy(&verf_lo, verf + sizeof(uint32_t), sizeof(uint32_t));
-
-		fsal_create_set_verifier(&sattr, verf_hi, verf_lo);
-	}
-
-	squash_setattr(&sattr);
-
-	if (!(sattr.valid_mask & ATTR_MODE)) {
-		/* Make sure mode is set. */
-		sattr.mode = 0600;
-		sattr.valid_mask |= ATTR_MODE;
-	}
-
-	status = fsal_create(parent,
-			     filename,
-			     REGULAR_FILE,
-			     &sattr,
-			     NULL,
-			     &obj_newfile,
-			     NULL);
-
-	/* Complete failure */
-	if ((FSAL_IS_ERROR(status))
-	    && (status.major != ERR_FSAL_EXIST)) {
-		return nfs4_Errno_status(status);
-	}
-
-	if (status.major == ERR_FSAL_EXIST) {
-		if (obj_newfile == NULL) {
-			/* File existed but was not a REGULAR_FILE,
-			 * return EEXIST error.
-			 */
-			LogDebug(COMPONENT_STATE,
-				 "Created returned EXIST, %s not regular file",
-				 filename);
-			return nfs4_Errno_status(status);
-		}
-		if (arg->openhow.openflag4_u.how.mode == GUARDED4) {
-			obj_newfile->obj_ops.put_ref(obj_newfile);
-			obj_newfile = NULL;
-			LogDebug(COMPONENT_STATE,
-				 "Created returned EXIST, %s created GUARDED4",
-				 filename);
-			return nfs4_Errno_status(status);
-		} else if (verf_provided) {
-			if (!fsal_create_verify(obj_newfile,
-						       verf_hi,
-						       verf_lo)) {
-				obj_newfile->obj_ops.put_ref(obj_newfile);
-				obj_newfile = NULL;
-				LogDebug(COMPONENT_STATE,
-					 "Created returned EXIST, %s verify failed",
-					 filename);
-				return NFS4ERR_EXIST;
-			}
-			/* The verifier matches so consider this a case
-			 * of successful creation.
-			 */
-			*created = true;
-		}
-
-		/* Clear error code */
-		status.major = 0;
-	} else {
-		/* Successful creation */
-		*created = true;
-	}
-
-	*obj = obj_newfile;
-	return nfs4_Errno_status(status);
-}
-
-/**
- * @brief Open or create a named file
- *
- * This function implements the CLAIM_NULL type, which is used to
- * create a new or open a preëxisting file.
- *
- * @param[in]     arg   OPEN4 arguments
- * @param[in,out] data  Comopund's data
- * @param[out]    res   OPEN4 rsponse
- * @param[out]    obj   Opened file
- */
-
-static nfsstat4 open4_claim_null(OPEN4args *arg, compound_data_t *data,
-				 OPEN4res *res, struct fsal_obj_handle **obj,
-				 bool *created)
-{
-	/* Parent directory in which to open the file. */
-	struct fsal_obj_handle *parent = NULL;
-	/* Status for cache_inode calls */
-	fsal_status_t fsal_status = {0, 0};
-	/* NFS Status from function calls */
-	nfsstat4 nfs_status = NFS4_OK;
-	/* The filename to create */
-	char *filename = NULL;
-
-	/* Validate and convert the utf8 filename */
-	nfs_status =
-	    nfs4_utf8string2dynamic(&arg->claim.open_claim4_u.file,
-				    UTF8_SCAN_ALL,
-				    &filename);
-
-	if (nfs_status != NFS4_OK)
-		goto out;
-
-	/* Check parent */
-	parent = data->current_obj;
-
-	/* Parent must be a directory */
-	if (parent->type != DIRECTORY) {
-		if (parent->type == SYMBOLIC_LINK) {
-			nfs_status = NFS4ERR_SYMLINK;
-			goto out;
-		} else {
-			nfs_status = NFS4ERR_NOTDIR;
-			goto out;
-		}
-	}
-
-	switch (arg->openhow.opentype) {
-	case OPEN4_CREATE:
-		nfs_status = open4_create(arg, data, res, parent, obj,
-					  filename, created);
-		break;
-
-	case OPEN4_NOCREATE:
-		fsal_status = fsal_lookup(parent, filename, obj, NULL);
-
-		if (FSAL_IS_ERROR(fsal_status))
-			nfs_status = nfs4_Errno_status(fsal_status);
-		break;
-
-	default:
-		nfs_status = NFS4ERR_INVAL;
-	}
-
- out:
-	if (filename)
 		gsh_free(filename);
 
-	return nfs_status;
+		if (obj_lookup == NULL) {
+			res_OPEN4->status = nfs4_Errno_status(status);
+			return false;
+		}
+		res_OPEN4->status = open4_create_fh(data, obj_lookup, false);
+	}
+
+	return false;
 }
 
 /**
@@ -840,10 +329,10 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 	nfsstat4 status;
 	state_t *found_state = NULL;
 
-	if (!(op_ctx->fsal_export->exp_ops.
-	      fs_supports(op_ctx->fsal_export, fso_delegations_w)
-	      || op_ctx->fsal_export->exp_ops.
-		 fs_supports(op_ctx->fsal_export, fso_delegations_r))
+	if (!(op_ctx->fsal_export->exp_ops.fs_supports(
+					op_ctx->fsal_export, fso_delegations_w)
+	      || op_ctx->fsal_export->exp_ops.fs_supports(
+					op_ctx->fsal_export, fso_delegations_r))
 	      ) {
 		LogDebug(COMPONENT_STATE,
 			 "NFS4 OPEN returning NFS4ERR_NOTSUPP for CLAIM_DELEGATE");
@@ -852,8 +341,8 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 
 	assert(claim == CLAIM_DELEGATE_CUR);
 	utfname = &arg->claim.open_claim4_u.delegate_cur_info.file;
-	rcurr_state = &arg->claim.open_claim4_u.
-			delegate_cur_info.delegate_stateid;
+	rcurr_state =
+		&arg->claim.open_claim4_u.delegate_cur_info.delegate_stateid;
 
 	LogDebug(COMPONENT_NFS_V4, "file name: %.*s",
 		 utfname->utf8string_len, utfname->utf8string_val);
@@ -935,17 +424,20 @@ static void get_delegation(compound_data_t *data, OPEN4args *args,
 	struct state_refer refer;
 	state_t *new_state = NULL;
 	struct state_hdl *ostate;
+	open_write_delegation4 *writeres =
+		&resok->delegation.open_delegation4_u.write;
+	open_read_delegation4 *readres =
+		&resok->delegation.open_delegation4_u.read;
+	open_none_delegation4 *whynone =
+		&resok->delegation.open_delegation4_u.od_whynone;
 
 	ostate = data->current_obj->state_hdl;
 	if (!ostate) {
 		LogFullDebug(COMPONENT_NFS_V4_LOCK, "Could not get file state");
+		whynone->ond_why = WND4_RESOURCE;
 		return;
 	}
 
-	/* Check if any prior OPENs conflict with granting a delegation */
-	if (state_open_deleg_conflict(ostate, open_state))
-		return;
-
 	/* Record the sequence info */
 	if (data->minorversion > 0) {
 		memcpy(refer.session,
@@ -974,16 +466,16 @@ static void get_delegation(compound_data_t *data, OPEN4args *args,
 				      data->minorversion > 0 ? &refer : NULL);
 	if (state_status != STATE_SUCCESS) {
 		LogDebug(COMPONENT_NFS_V4_LOCK,
-			 "get_delegation call failed to add state with status %s",
+			 "get delegation call failed to add state with status %s",
 			 state_err_str(state_status));
+		whynone->ond_why = WND4_RESOURCE;
 		return;
 	} else {
 		new_state->state_seqid++;
 
 		LogFullDebugOpaque(COMPONENT_STATE,
 				   "delegation state added, stateid: %s",
-				   100, new_state->stateid_other,
-				   OTHERSIZE);
+				   100, new_state->stateid_other, OTHERSIZE);
 
 		/* acquire_lease_lock() gets the delegation from FSAL */
 		state_status = acquire_lease_lock(ostate,
@@ -991,37 +483,34 @@ static void get_delegation(compound_data_t *data, OPEN4args *args,
 						  new_state);
 		if (state_status != STATE_SUCCESS) {
 			LogDebug(COMPONENT_NFS_V4_LOCK,
-				 "get_delegation call added state but failed to lock with status %s",
+				 "get delegation call added state but failed to lock with status %s",
 				 state_err_str(state_status));
 			state_del_locked(new_state);
 			dec_state_t_ref(new_state);
+			if (state_status == STATE_LOCK_CONFLICT)
+				whynone->ond_why = WND4_CONTENTION;
+			else
+				whynone->ond_why = WND4_RESOURCE;
 			return;
 		} else {
+			nfs_space_limit4 *space_limit = &writeres->space_limit;
+
 			resok->delegation.delegation_type = deleg_type;
+			ostate->file.fdeleg_stats.fds_deleg_type = deleg_type;
 			if (deleg_type == OPEN_DELEGATE_WRITE) {
-				open_write_delegation4 *writeres =
-				&resok->delegation.open_delegation4_u.write;
-				writeres->
-					space_limit.limitby = NFS_LIMIT_SIZE;
-				writeres->
-				space_limit.nfs_space_limit4_u.filesize =
+				space_limit->limitby = NFS_LIMIT_SIZE;
+				space_limit->nfs_space_limit4_u.filesize =
 						DELEG_SPACE_LIMIT_FILESZ;
 				COPY_STATEID(&writeres->stateid, new_state);
 				writeres->recall = prerecall;
 				get_deleg_perm(&writeres->permissions,
 					       deleg_type);
-				ostate->file.fdeleg_stats.fds_deleg_type =
-					OPEN_DELEGATE_WRITE;
 			} else {
 				assert(deleg_type == OPEN_DELEGATE_READ);
-				open_read_delegation4 *readres =
-				&resok->delegation.open_delegation4_u.read;
 				COPY_STATEID(&readres->stateid, new_state);
 				readres->recall = prerecall;
 				get_deleg_perm(&readres->permissions,
 					       deleg_type);
-				ostate->file.fdeleg_stats.fds_deleg_type =
-					OPEN_DELEGATE_READ;
 			}
 		}
 	}
@@ -1036,7 +525,7 @@ static void get_delegation(compound_data_t *data, OPEN4args *args,
 		display_nfs4_owner(&dspbuf2, clientowner);
 
 		LogDebug(COMPONENT_NFS_V4_LOCK,
-			 "get_delegation openowner %s clientowner %s status %s",
+			 "get delegation openowner %s clientowner %s status %s",
 			 str1, str2, state_err_str(state_status));
 	}
 
@@ -1058,12 +547,15 @@ static void do_delegation(OPEN4args *arg_OPEN4, OPEN4res *res_OPEN4,
 	}
 
 	/* This will be updated later if we actually delegate */
-	resok->delegation.delegation_type = OPEN_DELEGATE_NONE;
+	if (clientid->cid_minorversion == 0)
+		resok->delegation.delegation_type = OPEN_DELEGATE_NONE;
+	else
+		resok->delegation.delegation_type = OPEN_DELEGATE_NONE_EXT;
 
 	/* Client doesn't want a delegation. */
 	if (arg_OPEN4->share_access & OPEN4_SHARE_ACCESS_WANT_NO_DELEG) {
-		resok->delegation.open_delegation4_u.
-			od_whynone.ond_why = WND4_NOT_WANTED;
+		resok->delegation.open_delegation4_u.od_whynone.ond_why =
+								WND4_NOT_WANTED;
 		LogFullDebug(COMPONENT_STATE, "Client didn't want delegation.");
 		return;
 	}
@@ -1071,6 +563,8 @@ static void do_delegation(OPEN4args *arg_OPEN4, OPEN4res *res_OPEN4,
 	/* Check if delegations are supported */
 	if (!deleg_supported(data->current_obj, op_ctx->fsal_export,
 			     op_ctx->export_perms, arg_OPEN4->share_access)) {
+		resok->delegation.open_delegation4_u.od_whynone.ond_why =
+							WND4_NOT_SUPP_FTYPE;
 		LogFullDebug(COMPONENT_STATE, "Delegation type not supported.");
 		return;
 	}
@@ -1078,7 +572,7 @@ static void do_delegation(OPEN4args *arg_OPEN4, OPEN4res *res_OPEN4,
 	/* Decide if we should delegate, then add it. */
 	if (can_we_grant_deleg(ostate, open_state) &&
 	    should_we_grant_deleg(ostate, clientid, open_state,
-				  arg_OPEN4, owner, &prerecall)) {
+				  arg_OPEN4, resok, owner, &prerecall)) {
 		/* Update delegation open stats */
 		if (ostate->file.fdeleg_stats.fds_num_opens == 0)
 			ostate->file.fdeleg_stats.fds_first_open = time(NULL);
@@ -1393,8 +887,8 @@ static void open4_ex(OPEN4args *arg,
 		 * Delegation recalls will be scheduled if there is a conflict.
 		 */
 		if (state_deleg_conflict(file_obj,
-					 arg->share_access &
-					     (OPEN4_SHARE_ACCESS_WRITE != 0))) {
+					  (arg->share_access &
+					   OPEN4_SHARE_ACCESS_WRITE) != 0)) {
 			res_OPEN4->status = NFS4ERR_DELAY;
 			goto out;
 		}
@@ -1640,11 +1134,7 @@ static void open4_ex(OPEN4args *arg,
 			     (*file_state)->state_data.share.share_deny_prev);
 	}
 
-#if 0
-	/** todo FSF: re-enable delegation when I get more figured out. */
 	do_delegation(arg, res_OPEN4, data, owner, *file_state, clientid);
-#endif
-
  out:
 
 	/* Release the attributes (may release an inherited ACL) */
@@ -1704,8 +1194,6 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 	 * OPEN operation is invoked.
 	 */
 	struct fsal_obj_handle *obj_change = NULL;
-	/* Open flags to be passed to the FSAL */
-	fsal_openflags_t openflags;
 	/* The found client record */
 	nfs_client_id_t *clientid = NULL;
 	/* The found or created state owner for this open */
@@ -1717,7 +1205,6 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 	/* True if the state was newly created */
 	bool new_state = false;
 	int retval;
-	bool created = false;
 
 	LogDebug(COMPONENT_STATE,
 		 "Entering NFS v4 OPEN handler -----------------------------");
@@ -1851,9 +1338,8 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 	if (!(arg_OPEN4->share_access & OPEN4_SHARE_ACCESS_BOTH)
 	    || (data->minorversion == 0
 		&& arg_OPEN4->share_access & ~OPEN4_SHARE_ACCESS_BOTH)
-	    || (arg_OPEN4->
-		share_access & (~OPEN4_SHARE_ACCESS_WANT_DELEG_MASK &
-				~OPEN4_SHARE_ACCESS_BOTH))
+	    || (arg_OPEN4->share_access & (~OPEN4_SHARE_ACCESS_WANT_DELEG_MASK &
+					   ~OPEN4_SHARE_ACCESS_BOTH))
 	    || (arg_OPEN4->share_deny & ~OPEN4_SHARE_DENY_BOTH)) {
 		res_OPEN4->status = NFS4ERR_INVAL;
 		LogDebug(COMPONENT_NFS_V4,
@@ -1861,137 +1347,17 @@ int nfs4_op_open(struct nfs_argop4 *op, compound_data_t *data,
 		goto out;
 	}
 
-	if (data->current_obj->fsal->m_ops.support_ex(data->current_obj)) {
-		/* Utilize the extended FSAL APU functionality to
-		 * perform the open.
-		 */
-		open4_ex(arg_OPEN4, data, res_OPEN4,
-			 clientid, owner,
-			 &file_state, &new_state);
+	/* Utilize the extended FSAL APU functionality to perform the open. */
+	open4_ex(arg_OPEN4, data, res_OPEN4, clientid,
+		 owner, &file_state, &new_state);
 
-		if (res_OPEN4->status == NFS4_OK)
-			goto success;
-		else
-			goto out;
-	}
-
-	/* Set the current entry to the file to be opened */
-	switch (claim) {
-	case CLAIM_NULL:
-		{
-			struct fsal_obj_handle *obj = NULL;
-			res_OPEN4->status = open4_claim_null(arg_OPEN4,
-							     data,
-							     res_OPEN4,
-							     &obj,
-							     &created);
-			if (res_OPEN4->status == NFS4_OK) {
-				res_OPEN4->status = open4_create_fh(data, obj,
-								    false);
-			}
-		}
-		break;
-
-		/* Both of these just use the current filehandle. */
-	case CLAIM_PREVIOUS:
-		owner->so_owner.so_nfs4_owner.so_confirmed = true;
-		if (!nfs4_check_deleg_reclaim(clientid, &data->currentFH)) {
-			/* It must have been revoked. Can't reclaim.*/
-			LogInfo(COMPONENT_NFS_V4, "Can't reclaim delegation");
-			res_OPEN4->status = NFS4ERR_RECLAIM_BAD;
-			goto out;
-		}
-		break;
-
-	case CLAIM_FH:
-		break;
-
-	case CLAIM_DELEGATE_PREV:
-		/* FIXME: Remove this when we have full support
-		 * for CLAIM_DELEGATE_PREV and delegpurge operations
-		 */
-		res_OPEN4->status = NFS4ERR_NOTSUPP;
-		break;
-
-	case CLAIM_DELEGATE_CUR:
-		res_OPEN4->status = open4_claim_deleg(arg_OPEN4, data);
-		break;
-
-	default:
-		LogFatal(COMPONENT_STATE,
-			 "Programming error.  Invalid claim after check.");
-		break;
-	}
-
-	if (res_OPEN4->status != NFS4_OK) {
-		LogDebug(COMPONENT_NFS_V4, "general failure");
+	if (res_OPEN4->status != NFS4_OK)
 		goto out;
-	}
-
-	/* OPEN4 is to be done on a file */
-	if (data->current_filetype != REGULAR_FILE) {
-		LogDebug(COMPONENT_NFS_V4,
-			 "Wrong file type expected REGULAR_FILE actual %s",
-			 object_file_type_to_str(data->current_filetype));
-
-		if (data->current_filetype == DIRECTORY) {
-			res_OPEN4->status = NFS4ERR_ISDIR;
-		} else {
-			/* All special nodes must return NFS4ERR_SYMLINK for
-			 * proper client behavior per this linux-nfs post:
-			 * http://marc.info/?l=linux-nfs&m=131342421825436&w=2
-			 */
-			res_OPEN4->status = NFS4ERR_SYMLINK;
-		}
-
-		goto out;
-	}
-
-	/* Set openflags. */
-	switch (arg_OPEN4->share_access & OPEN4_SHARE_ACCESS_BOTH) {
-	case OPEN4_SHARE_ACCESS_READ:
-		openflags = FSAL_O_READ;
-		break;
-	case OPEN4_SHARE_ACCESS_WRITE:
-		/* Clients may read as well due to buffer cache constraints.
-		 * This is based on text on page 112 of RFC 7530.
-		 */
-		/* Fallthrough */
-	case OPEN4_SHARE_ACCESS_BOTH:
-	default:
-		openflags = FSAL_O_RDWR;
-		break;
-	}
-
-	if (arg_OPEN4->claim.claim)
-		openflags |= FSAL_O_RECLAIM;
-
-	PTHREAD_RWLOCK_wrlock(&data->current_obj->state_hdl->state_lock);
-
-	res_OPEN4->status = open4_do_open(op,
-					  data,
-					  owner,
-					  &file_state,
-					  &new_state,
-					  openflags,
-					  created);
-
-	if (res_OPEN4->status == NFS4_OK)
-		do_delegation(arg_OPEN4, res_OPEN4, data, owner, file_state,
-			      clientid);
-
-	PTHREAD_RWLOCK_unlock(&data->current_obj->state_hdl->state_lock);
-
-	if (res_OPEN4->status != NFS4_OK) {
-		LogDebug(COMPONENT_NFS_V4, "open4_do_open failed");
-		goto out;
-	}
-
- success:
 
 	memset(&res_OPEN4->OPEN4res_u.resok4.attrset,
 	       0,
 	       sizeof(struct bitmap4));
+
 	if (arg_OPEN4->openhow.openflag4_u.how.mode == EXCLUSIVE4 ||
 	    arg_OPEN4->openhow.openflag4_u.how.mode == EXCLUSIVE4_1) {
 		struct bitmap4 *bits = &res_OPEN4->OPEN4res_u.resok4.attrset;
