@@ -179,6 +179,7 @@ static int nfs4_mds_putfh(compound_data_t *data)
 
 	/* update _ctx fields needed by nfs4_export_check_access */
 	op_ctx->ctx_export = exporting;
+	op_ctx->fsal_export = export = exporting->fsal_export;
 
 	if (changed) {
 		int status;
@@ -192,18 +193,17 @@ static int nfs4_mds_putfh(compound_data_t *data)
 		}
 	}
 
-	op_ctx->fsal_export = export = exporting->fsal_export;
 	fh_desc.len = v4_handle->fs_len;
 	fh_desc.addr = &v4_handle->fsopaque;
 
 	/* adjust the handle opaque into a cache key */
-	fsal_status = export->exp_ops.extract_handle(export,
-						     FSAL_DIGEST_NFSV4,
-						     &fh_desc,
-						     v4_handle->fhflags1);
+	fsal_status = export->exp_ops.wire_to_host(export,
+						   FSAL_DIGEST_NFSV4,
+						   &fh_desc,
+						   v4_handle->fhflags1);
 	if (FSAL_IS_ERROR(fsal_status)) {
 		LogFullDebug(COMPONENT_FILEHANDLE,
-			     "extract_handle failed %s",
+			     "wire_to_host failed %s",
 			     msg_fsal_err(fsal_status.major));
 		return nfs4_Errno_status(fsal_status);
 	}
