@@ -146,10 +146,10 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 		 * attributes.
 		 */
 		LogDebug(COMPONENT_EXPORT,
-			 "Offspring DIR %s is a junction Export_id %d Path %s",
+			 "Offspring DIR %s is a junction Export_id %d Pseudo %s",
 			 cb_parms->name,
 			 obj->state_hdl->dir.junction_export->export_id,
-			 obj->state_hdl->dir.junction_export->fullpath);
+			 obj->state_hdl->dir.junction_export->pseudopath);
 
 		/* Get a reference to the export and stash it in
 		 * compound data.
@@ -180,9 +180,9 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 			 * skip the export.
 			 */
 			LogDebug(COMPONENT_EXPORT,
-				 "NFS4ERR_ACCESS Skipping Export_Id %d Path %s",
+				 "NFS4ERR_ACCESS Skipping Export_Id %d Pseudo %s",
 				 op_ctx->ctx_export->export_id,
-				 op_ctx->ctx_export->fullpath);
+				 op_ctx->ctx_export->pseudopath);
 
 			/* Restore export and creds */
 			restore_data(tracker);
@@ -212,7 +212,7 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 				LogDebug(COMPONENT_EXPORT,
 					 "Ignoring NFS4ERR_WRONGSEC (only asked for MOUNTED_IN_FILEID) On ReadDir Export_Id %d Path %s",
 					 op_ctx->ctx_export->export_id,
-					 op_ctx->ctx_export->fullpath);
+					 op_ctx->ctx_export->pseudopath);
 
 				/* Because we are not asking for any attributes
 				 * which are a property of the exported file
@@ -236,9 +236,9 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 				 * the name into the entry.
 				 */
 				LogDebug(COMPONENT_EXPORT,
-					 "NFS4ERR_WRONGSEC On ReadDir Export_Id %d Path %s",
+					 "NFS4ERR_WRONGSEC On ReadDir Export_Id %d Pseudo %s",
 					 op_ctx->ctx_export->export_id,
-					 op_ctx->ctx_export->fullpath);
+					 op_ctx->ctx_export->pseudopath);
 			}
 		} else if (rdattr_error == NFS4_OK) {
 			/* Now we must traverse the junction to get the
@@ -250,9 +250,9 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 			 * across the junction.
 			 */
 			LogDebug(COMPONENT_EXPORT,
-				 "Need to cross junction to Export_Id %d Path %s",
+				 "Need to cross junction to Export_Id %d Pseudo %s",
 				op_ctx->ctx_export->export_id,
-				op_ctx->ctx_export->fullpath);
+				op_ctx->ctx_export->pseudopath);
 			PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 			return ERR_FSAL_CROSS_JUNCTION;
 		}
@@ -263,9 +263,9 @@ fsal_errors_t nfs4_readdir_callback(void *opaque,
 		 * Restore export and creds.
 		 */
 		LogDebug(COMPONENT_EXPORT,
-			 "Need to report error for junction to Export_Id %d Path %s",
+			 "Need to report error for junction to Export_Id %d Pseudo %s",
 			 op_ctx->ctx_export->export_id,
-			 op_ctx->ctx_export->fullpath);
+			 op_ctx->ctx_export->pseudopath);
 		restore_data(tracker);
 	}
 
@@ -347,9 +347,9 @@ not_junction:
 	fsal_status = obj->obj_ops.test_access(obj, access_mask_attr, NULL,
 					       NULL, false);
 	if (FSAL_IS_ERROR(fsal_status)) {
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "permission check for attributes status=%s",
-			     msg_fsal_err(fsal_status.major));
+		LogDebug(COMPONENT_NFS_READDIR,
+			 "permission check for attributes status=%s",
+			 msg_fsal_err(fsal_status.major));
 
 		rdattr_error = nfs4_Errno_status(fsal_status);
 		LogDebug(COMPONENT_NFS_READDIR,
@@ -521,18 +521,17 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 	estimated_num_entries = 50;
 	tracker.total_entries = estimated_num_entries;
 
-	LogFullDebug(COMPONENT_NFS_READDIR,
-		     "dircount=%lu maxcount=%lu cookie=%" PRIu64
-		     " estimated_num_entries=%u",
-		     dircount, maxcount, cookie, estimated_num_entries);
+	LogDebug(COMPONENT_NFS_READDIR,
+		 "dircount=%lu maxcount=%lu cookie=%" PRIu64
+		 " estimated_num_entries=%u",
+		 dircount, maxcount, cookie, estimated_num_entries);
 
 	/* Since we never send a cookie of 1 or 2, we shouldn't ever get
 	 * them back.
 	 */
 	if (cookie == 1 || cookie == 2) {
 		res_READDIR4->status = NFS4ERR_BAD_COOKIE;
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Bad cookie");
+		LogDebug(COMPONENT_NFS_READDIR, "Bad cookie");
 		goto out;
 	}
 
@@ -540,8 +539,7 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 	if (!nfs4_Fattr_Check_Access_Bitmap
 	    (&arg_READDIR4->attr_request, FATTR4_ATTR_READ)) {
 		res_READDIR4->status = NFS4ERR_INVAL;
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Requested invalid attributes");
+		LogDebug(COMPONENT_NFS_READDIR, "Requested invalid attributes");
 		goto out;
 	}
 
@@ -550,8 +548,7 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 	 */
 	if (maxcount < 14 || estimated_num_entries == 0) {
 		res_READDIR4->status = NFS4ERR_TOOSMALL;
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Response too small");
+		LogDebug(COMPONENT_NFS_READDIR, "Response too small");
 		goto out;
 	}
 
@@ -573,9 +570,9 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 
 		if (FSAL_IS_ERROR(fsal_status)) {
 			res_READDIR4->status = nfs4_Errno_status(fsal_status);
-			LogFullDebug(COMPONENT_NFS_READDIR,
-				     "getattrs returned %s",
-				     msg_fsal_err(fsal_status.major));
+			LogDebug(COMPONENT_NFS_READDIR,
+				 "getattrs returned %s",
+				 msg_fsal_err(fsal_status.major));
 			goto out;
 		}
 
@@ -602,8 +599,7 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 			   arg_READDIR4->cookieverf,
 			   NFS4_VERIFIER_SIZE) != 0) {
 			res_READDIR4->status = NFS4ERR_BAD_COOKIE;
-			LogFullDebug(COMPONENT_NFS_READDIR,
-				     "Bad cookie");
+			LogDebug(COMPONENT_NFS_READDIR, "Bad cookie");
 			goto out;
 		}
 	}
@@ -638,21 +634,20 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res_READDIR4->status = nfs4_Errno_status(fsal_status);
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "fsal_readdir returned %s",
-			     msg_fsal_err(fsal_status.major));
+		LogDebug(COMPONENT_NFS_READDIR,
+			 "fsal_readdir returned %s",
+			 msg_fsal_err(fsal_status.major));
 		goto out;
 	}
 
-	LogFullDebug(COMPONENT_NFS_READDIR,
-		     "fsal_readdir returned %s",
-		     msg_fsal_err(fsal_status.major));
+	LogDebug(COMPONENT_NFS_READDIR,
+		 "fsal_readdir returned %s",
+		 msg_fsal_err(fsal_status.major));
 
 	res_READDIR4->status = tracker.error;
 
 	if (res_READDIR4->status != NFS4_OK) {
-		LogFullDebug(COMPONENT_NFS_READDIR,
-			     "Tracker error");
+		LogDebug(COMPONENT_NFS_READDIR, "Tracker error");
 		goto out;
 	}
 
@@ -688,9 +683,9 @@ int nfs4_op_readdir(struct nfs_argop4 *op, compound_data_t *data,
 	if ((res_READDIR4->status != NFS4_OK) && (entries != NULL))
 		free_entries(entries);
 
-	LogFullDebug(COMPONENT_NFS_READDIR,
-		     "Returning %s",
-		     nfsstat4_to_str(res_READDIR4->status));
+	LogDebug(COMPONENT_NFS_READDIR,
+		 "Returning %s",
+		 nfsstat4_to_str(res_READDIR4->status));
 
 	return res_READDIR4->status;
 }				/* nfs4_op_readdir */
