@@ -46,13 +46,13 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	nlm4_shareargs *arg = &args->arg_nlm4_share;
 	struct fsal_obj_handle *obj;
 	state_status_t state_status = STATE_SUCCESS;
-	char buffer[MAXNETOBJ_SZ * 2];
+	char buffer[MAXNETOBJ_SZ * 2] = "\0";
 	state_nsm_client_t *nsm_client;
 	state_nlm_client_t *nlm_client;
 	state_owner_t *nlm_owner;
 	state_t *nlm_state;
 	int rc;
-	int grace = nfs_in_grace();
+	bool grace = nfs_in_grace();
 
 	/* NLM doesn't have a BADHANDLE error, nor can rpc_execute deal with
 	 * responding to an NLM_*_MSG call, so we check here if the export is
@@ -60,7 +60,7 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	 */
 	if (op_ctx->ctx_export == NULL) {
 		res->res_nlm4share.stat = NLM4_STALE_FH;
-		LogInfo(COMPONENT_NLM, "INVALID HANDLE: nlm4_Unshare");
+		LogInfo(COMPONENT_NLM, "INVALID HANDLE: NLM4_UNSHARE");
 		return NFS_REQ_OK;
 	}
 
@@ -69,7 +69,7 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	netobj_to_string(&arg->cookie, buffer, 1024);
 
 	LogDebug(COMPONENT_NLM,
-		 "REQUEST PROCESSING: Calling nlm4_Unshare cookie=%s reclaim=%s",
+		 "REQUEST PROCESSING: Calling NLM4_UNSHARE cookie=%s reclaim=%s",
 		 buffer,
 		 arg->reclaim ? "yes" : "no");
 
@@ -84,7 +84,7 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	if ((grace && !arg->reclaim) || (!grace && arg->reclaim)) {
 		res->res_nlm4share.stat = NLM4_DENIED_GRACE_PERIOD;
 		LogDebug(COMPONENT_NLM,
-			 "REQUEST RESULT: nlm4_Unshare %s",
+			 "REQUEST RESULT: NLM4_UNSHARE %s",
 			 lock_result_str(res->res_nlm4share.stat));
 		return NFS_REQ_OK;
 	}
@@ -103,16 +103,18 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 		/* Present the error back to the client */
 		res->res_nlm4share.stat = (nlm4_stats) rc;
 		LogDebug(COMPONENT_NLM,
-			 "REQUEST RESULT: nlm4_Unshare %s",
+			 "REQUEST RESULT: NLM4_UNSHARE %s",
 			 lock_result_str(res->res_nlm4share.stat));
 		return NFS_REQ_OK;
 	}
 
-	state_status = state_nlm_unshare(obj,
-					 arg->share.access,
-					 arg->share.mode,
-					 nlm_owner,
-					 nlm_state);
+	state_status = state_nlm_share(obj,
+				       arg->share.access,
+				       arg->share.mode,
+				       nlm_owner,
+				       nlm_state,
+				       false,
+				       true);
 
 	if (state_status != STATE_SUCCESS) {
 		res->res_nlm4share.stat =
@@ -128,7 +130,7 @@ int nlm4_Unshare(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	obj->obj_ops.put_ref(obj);
 	dec_nlm_state_ref(nlm_state);
 
-	LogDebug(COMPONENT_NLM, "REQUEST RESULT: nlm4_Unshare %s",
+	LogDebug(COMPONENT_NLM, "REQUEST RESULT: NLM4_UNSHARE %s",
 		 lock_result_str(res->res_nlm4share.stat));
 
 	return NFS_REQ_OK;
