@@ -160,31 +160,33 @@ void free_grant_arg(state_async_queue_t *arg)
 static void nlm4_send_grant_msg(state_async_queue_t *arg)
 {
 	int retval;
-	char buffer[1024];
+	char buffer[1024] = "\0";
 	state_status_t state_status = STATE_SUCCESS;
 	state_cookie_entry_t *cookie_entry;
 	state_nlm_async_data_t *nlm_arg =
 	    &arg->state_async_data.state_nlm_async_data;
 	struct root_op_context root_op_context;
 	struct gsh_export *export;
+	nlm4_testargs *nlm_async_grant;
+
+	nlm_async_grant = &nlm_arg->nlm_async_args.nlm_async_grant;
 
 	if (isDebug(COMPONENT_NLM)) {
-		netobj_to_string(&nlm_arg->nlm_async_args.nlm_async_grant.
-				 cookie, buffer, sizeof(buffer));
+		netobj_to_string(&nlm_async_grant->cookie,
+				 buffer, sizeof(buffer));
 
 		LogDebug(COMPONENT_NLM,
-			 "Sending GRANTED for arg=%p svid=%d start=%llx len=%llx cookie=%s",
+			 "Sending GRANTED for arg=%p svid=%d start=%" PRIx64
+			 " len=%" PRIx64 " cookie=%s",
 			 arg,
-			 nlm_arg->nlm_async_args.nlm_async_grant.alock.svid,
-			 (unsigned long long)nlm_arg->nlm_async_args.
-			 nlm_async_grant.alock.l_offset,
-			 (unsigned long long)nlm_arg->nlm_async_args.
-			 nlm_async_grant.alock.l_len, buffer);
+			 nlm_async_grant->alock.svid,
+			 nlm_async_grant->alock.l_offset,
+			 nlm_async_grant->alock.l_len, buffer);
 	}
 
 	retval = nlm_send_async(NLMPROC4_GRANTED_MSG,
 				nlm_arg->nlm_async_host,
-				&nlm_arg->nlm_async_args.nlm_async_grant,
+				nlm_async_grant,
 				nlm_arg->nlm_async_key);
 
 	dec_nlm_client_ref(nlm_arg->nlm_async_host);
@@ -202,8 +204,8 @@ static void nlm4_send_grant_msg(state_async_queue_t *arg)
 		 retval);
 
 	state_status = state_find_grant(
-			nlm_arg->nlm_async_args.nlm_async_grant.cookie.n_bytes,
-			nlm_arg->nlm_async_args.nlm_async_grant.cookie.n_len,
+			nlm_async_grant->cookie.n_bytes,
+			nlm_async_grant->cookie.n_len,
 			&cookie_entry);
 
 	if (state_status != STATE_SUCCESS) {
@@ -255,7 +257,6 @@ int nlm_process_parameters(struct svc_req *req, bool exclusive,
 			   state_nlm_client_t **ppnlm_client,
 			   state_owner_t **ppowner,
 			   state_block_data_t **block_data,
-			   bool nsm_state_applies,
 			   int32_t nsm_state,
 			   state_t **state)
 {
@@ -332,7 +333,7 @@ int nlm_process_parameters(struct svc_req *req, bool exclusive,
 		rc = get_nlm_state(STATE_TYPE_NLM_LOCK,
 				   *ppobj,
 				   *ppowner,
-				   nsm_state_applies,
+				   care,
 				   nsm_state,
 				   state);
 
@@ -469,7 +470,7 @@ int nlm_process_share_parms(struct svc_req *req, nlm4_share *share,
 		rc = get_nlm_state(STATE_TYPE_NLM_SHARE,
 				   *ppobj,
 				   *ppowner,
-				   false,
+				   care,
 				   0,
 				   state);
 
@@ -582,6 +583,7 @@ state_status_t nlm_granted_callback(struct fsal_obj_handle *obj,
 	state_cookie_entry_t *cookie_entry = NULL;
 	state_async_queue_t *arg;
 	nlm4_testargs *inarg;
+	state_nlm_async_data_t *nlm_async_data;
 	state_nlm_owner_t *nlm_grant_owner =
 	    &lock_entry->sle_owner->so_owner.so_nlm_owner;
 	state_nlm_client_t *nlm_grant_client = nlm_grant_owner->so_client;
@@ -590,6 +592,7 @@ state_status_t nlm_granted_callback(struct fsal_obj_handle *obj,
 	state_status_t state_status_int;
 
 	arg = gsh_calloc(1, sizeof(*arg));
+	nlm_async_data = &arg->state_async_data.state_nlm_async_data;
 
 	/* Get a cookie to use for this grant */
 	next_granted_cookie(&nlm_grant_cookie);
@@ -612,11 +615,9 @@ state_status_t nlm_granted_callback(struct fsal_obj_handle *obj,
 	/* Fill in the arguments for the NLMPROC4_GRANTED_MSG call */
 	inc_nlm_client_ref(nlm_grant_client);
 	arg->state_async_func = nlm4_send_grant_msg;
-	arg->state_async_data.state_nlm_async_data.nlm_async_host =
-	    nlm_grant_client;
-	arg->state_async_data.state_nlm_async_data.nlm_async_key = cookie_entry;
-	inarg = &arg->state_async_data.state_nlm_async_data.nlm_async_args.
-		nlm_async_grant;
+	nlm_async_data->nlm_async_host = nlm_grant_client;
+	nlm_async_data->nlm_async_key = cookie_entry;
+	inarg = &nlm_async_data->nlm_async_args.nlm_async_grant;
 
 	copy_netobj(&inarg->alock.fh, &nlm_block_data->sbd_nlm_fh);
 
@@ -637,7 +638,7 @@ state_status_t nlm_granted_callback(struct fsal_obj_handle *obj,
 	inarg->alock.l_len = lock_entry->sle_lock.lock_length;
 
 	if (isDebug(COMPONENT_NLM)) {
-		char buffer[1024];
+		char buffer[1024] = "\0";
 
 		netobj_to_string(&inarg->cookie, buffer, sizeof(buffer));
 
