@@ -48,13 +48,14 @@
 #include "city.h"
 #include "abstract_mem.h"
 #include "gsh_intrinsic.h"
-#include "wait_queue.h"
+#include "gsh_wait_queue.h"
 
-#define DUPREQ_BAD_ADDR1 0x01	/* safe for marked pointers, etc */
 #define DUPREQ_NOCACHE   0x02
-
 #define DUPREQ_MAX_RETRIES 5
 
+#define NFS_pcp nfs_param.core_param
+#define NFS_program NFS_pcp.program
+
 pool_t *dupreq_pool;
 pool_t *nfs_res_pool;
 pool_t *tcp_drc_pool;		/* pool of per-connection DRC objects */
@@ -73,6 +74,36 @@ const char *dupreq_state_table[] = {
 	"DUPREQ_DELETED",
 };
 
+/* drc_t holds the request/response cache. There is a single drc_t for
+ * all udp connections. There is a drc_t for each tcp connection (aka
+ * socket). Since a client could close a socket and reconnect, we would
+ * like to use the same drc cache for the reconnection. For this reason,
+ * we don't want to free the drc as soon as the tcp connection gets
+ * closed, but rather keep them in a recycle list for sometime.
+ *
+ * The life of tcp drc: it gets allocated when we process the first
+ * request on the connection. It is put into rbtree (tcp_drc_recycle_t).
+ * drc cache maintains a ref count. Every request as well as the xprt
+ * holds a ref count. Its ref count should go to zero when the
+ * connection's xprt gets freed (all requests should be completed on the
+ * xprt by this time). When the ref count goes to zero, it is also put
+ * into a recycle queue (tcp_drc_recycle_q). When a reconnection
+ * happens, we hope to find the same drc that was used before, and the
+ * ref count goes up again. At the same time, the drc will be removed
+ * from the recycle queue. Only drc's with ref count zero end up in the
+ * recycle queue. If a reconnection doesn't happen in time, the drc gets
+ * freed by drc_free_expired() after some period of inactivety.
+ *
+ * Most ref count methods assume that a ref count doesn't go up from
+ * zero, so a thread that decrements the ref count to zero would be the
+ * only one acting on it, and it could do so without any locks!  Since
+ * the drc ref count could go up from zero, care must be taken. The
+ * thread that decrements the ref count to zero will have to put the drc
+ * into the recycle queue. It will do so only after dropping the lock in
+ * the current implementation. If we let nfs_dupreq_get_drc() reuse the
+ * drc before it gets into recycle queue, we could end up with multiple
+ * threads that decrement the ref count to zero.
+ */
 struct drc_st {
 	pthread_mutex_t mtx;
 	drc_t udp_drc;		/* shared DRC */
@@ -184,7 +215,7 @@ static inline int dupreq_tcp_cmpf(const struct opr_rbtree_node *lhs,
 
 	if (lk->hin.tcp.rq_xid == rk->hin.tcp.rq_xid) {
 		LogDebug(COMPONENT_DUPREQ,
-			 "xids eq (%u), ck1 %" PRIu64 " ck2 %" PRIu64,
+			 "xids eq %" PRIu32 ", ck1 %" PRIu64 " ck2 %" PRIu64,
 			 lk->hin.tcp.rq_xid, lk->hk, rk->hk);
 		return uint64_cmpf(lk->hk, rk->hk);
 	}
@@ -297,6 +328,7 @@ static inline unsigned int get_ipproto_by_xprt(SVCXPRT *xprt)
 {
 	switch (xprt->xp_type) {
 	case XPRT_UDP:
+	case XPRT_UDP_RENDEZVOUS:
 		return IPPROTO_UDP;
 	case XPRT_TCP:
 	case XPRT_TCP_RENDEZVOUS:
@@ -319,7 +351,7 @@ static inline enum drc_type get_drc_type(struct svc_req *req)
 	if (get_ipproto_by_xprt(req->rq_xprt) == IPPROTO_UDP)
 		return DRC_UDP_V234;
 	else {
-		if (req->rq_vers == 4)
+		if (req->rq_msg.cb_vers == 4)
 			return DRC_TCP_V4;
 	}
 	return DRC_TCP_V3;
@@ -347,7 +379,7 @@ static inline drc_t *alloc_tcp_drc(enum drc_type dtype)
 	drc->maxsize = nfs_param.core_param.drc.tcp.size;
 	drc->cachesz = nfs_param.core_param.drc.tcp.cachesz;
 	drc->npart = nfs_param.core_param.drc.tcp.npart;
-	drc->hiwat = nfs_param.core_param.drc.udp.hiwat;
+	drc->hiwat = nfs_param.core_param.drc.tcp.hiwat;
 
 	PTHREAD_MUTEX_init(&drc->mtx, NULL);
 
@@ -445,7 +477,10 @@ static inline void drc_free_expired(void)
 		drc = TAILQ_FIRST(&drc_st->tcp_drc_recycle_q);
 		if (drc && (drc->d_u.tcp.recycle_time > 0)
 		    && ((now - drc->d_u.tcp.recycle_time) >
-			drc_st->expire_delta) && (drc->refcnt == 0)) {
+			drc_st->expire_delta)) {
+
+			assert(drc->refcnt == 0);
+
 			LogFullDebug(COMPONENT_DUPREQ,
 				     "remove expired drc %p from recycle queue",
 				     drc);
@@ -458,23 +493,14 @@ static inline void drc_free_expired(void)
 				LogCrit(COMPONENT_DUPREQ,
 					"BUG: asked to dequeue DRC not on queue");
 			} else {
-				(void)opr_rbtree_remove(&t->t,
-							&drc->d_u.tcp.
-							recycle_k);
+				(void)opr_rbtree_remove(
+						&t->t, &drc->d_u.tcp.recycle_k);
 			}
 			TAILQ_REMOVE(&drc_st->tcp_drc_recycle_q, drc,
 				     d_u.tcp.recycle_q);
 			--(drc_st->tcp_drc_recycle_qlen);
-			/* expect DRC to be reachable from some xprt(s) */
-			PTHREAD_MUTEX_lock(&drc->mtx);
-			drc->flags &= ~DRC_FLAG_RECYCLE;
-			/* but if not, dispose it */
-			if (drc->refcnt == 0) {
-				PTHREAD_MUTEX_unlock(&drc->mtx);
-				free_tcp_drc(drc);
-				continue;
-			}
-			PTHREAD_MUTEX_unlock(&drc->mtx);
+
+			free_tcp_drc(drc);
 		} else {
 			LogFullDebug(COMPONENT_DUPREQ,
 				     "unexpired drc %p in recycle queue expire check (nothing happens)",
@@ -511,6 +537,7 @@ nfs_dupreq_get_drc(struct svc_req *req)
 		(void)nfs_dupreq_ref_drc(drc);
 		DRC_ST_UNLOCK();
 		goto out;
+retry:
 	case DRC_TCP_V4:
 	case DRC_TCP_V3:
 		/* Idempotent address, no need for lock;
@@ -552,15 +579,33 @@ nfs_dupreq_get_drc(struct svc_req *req)
 			t = rbtx_partition_of_scalar(&drc_st->tcp_drc_recycle_t,
 						     drc_k.d_u.tcp.hk);
 			DRC_ST_LOCK();
+
+			/* Avoid double reference of drc,
+			 * rechecking xp_u2 after DRC_ST_LOCK */
+			if (req->rq_xprt->xp_u2) {
+				DRC_ST_UNLOCK();
+				goto retry;
+			}
+
 			ndrc =
 			    opr_rbtree_lookup(&t->t, &drc_k.d_u.tcp.recycle_k);
 			if (ndrc) {
 				/* reuse old DRC */
-				tdrc =
-				    opr_containerof(ndrc, drc_t,
-						    d_u.tcp.recycle_k);
+				tdrc = opr_containerof(ndrc, drc_t,
+						       d_u.tcp.recycle_k);
 				PTHREAD_MUTEX_lock(&tdrc->mtx);	/* LOCKED */
-				if (tdrc->flags & DRC_FLAG_RECYCLE) {
+
+				/* If the refcnt is zero and it is not
+				 * in the recycle queue, wait for the
+				 * other thread to put it in the queue.
+				 */
+				if (tdrc->refcnt == 0) {
+					if (!(tdrc->flags & DRC_FLAG_RECYCLE)) {
+						PTHREAD_MUTEX_unlock(
+								&tdrc->mtx);
+						DRC_ST_UNLOCK();
+						goto retry;
+					}
 					TAILQ_REMOVE(&drc_st->tcp_drc_recycle_q,
 						     tdrc, d_u.tcp.recycle_q);
 					--(drc_st->tcp_drc_recycle_qlen);
@@ -571,6 +616,7 @@ nfs_dupreq_get_drc(struct svc_req *req)
 					     "recycle TCP DRC=%p for xprt=%p",
 					     tdrc, req->rq_xprt);
 			}
+
 			if (!drc) {
 				drc = alloc_tcp_drc(dtype);
 				LogFullDebug(COMPONENT_DUPREQ,
@@ -582,17 +628,19 @@ nfs_dupreq_get_drc(struct svc_req *req)
 				/* assign already-computed hash */
 				drc->d_u.tcp.hk = drc_k.d_u.tcp.hk;
 				PTHREAD_MUTEX_lock(&drc->mtx);	/* LOCKED */
-				/* xprt ref */
-				drc->refcnt = 1;
 				/* insert dict */
 				opr_rbtree_insert(&t->t,
 						  &drc->d_u.tcp.recycle_k);
 			}
+
+			/* Avoid double reference of drc,
+			 * setting xp_u2 under DRC_ST_LOCK */
+			req->rq_xprt->xp_u2 = (void *)drc;
+			(void)nfs_dupreq_ref_drc(drc);  /* xprt ref */
+
 			DRC_ST_UNLOCK();
 			drc->d_u.tcp.recycle_time = 0;
 
-			(void)nfs_dupreq_ref_drc(drc);	/* xprt ref */
-
 			/* try to expire unused DRCs somewhat in proportion to
 			 * new connection arrivals */
 			drc_check_expired = true;
@@ -600,13 +648,6 @@ nfs_dupreq_get_drc(struct svc_req *req)
 			LogFullDebug(COMPONENT_DUPREQ,
 				     "after ref drc %p refcnt==%u ", drc,
 				     drc->refcnt);
-
-			/* Idempotent address, no need for lock;
-			 * set once here, never changes.
-			 * No other fields are modified.
-			 * Assumes address stores are atomic.
-			 */
-			req->rq_xprt->xp_u2 = (void *)drc;
 		}
 		break;
 	default:
@@ -631,11 +672,10 @@ out:
  * Release previously-ref'd DRC.  If its refcnt drops to 0, the DRC
  * is queued for later recycling.
  *
- * @param[in] xprt  The SVCXPRT associated with DRC, if applicable
  * @param[in] drc   The DRC
  * @param[in] flags Control flags
  */
-void nfs_dupreq_put_drc(SVCXPRT *xprt, drc_t *drc, uint32_t flags)
+void nfs_dupreq_put_drc(drc_t *drc, uint32_t flags)
 {
 	if (!(flags & DRC_FLAG_LOCKED))
 		PTHREAD_MUTEX_lock(&drc->mtx);
@@ -657,23 +697,33 @@ void nfs_dupreq_put_drc(SVCXPRT *xprt, drc_t *drc, uint32_t flags)
 		break;
 	case DRC_TCP_V4:
 	case DRC_TCP_V3:
-		if (drc->refcnt == 0) {
-			if (!(drc->flags & DRC_FLAG_RECYCLE)) {
-				/* note t's lock order wrt drc->mtx is
-				 * the opposite of drc->xt[*].lock */
-				drc->d_u.tcp.recycle_time = time(NULL);
-				drc->flags |= DRC_FLAG_RECYCLE;
-				PTHREAD_MUTEX_unlock(&drc->mtx); /* !LOCKED */
-				DRC_ST_LOCK();
-				TAILQ_INSERT_TAIL(&drc_st->tcp_drc_recycle_q,
-						  drc, d_u.tcp.recycle_q);
-				++(drc_st->tcp_drc_recycle_qlen);
-				LogFullDebug(COMPONENT_DUPREQ,
-					     "enqueue drc %p for recycle", drc);
-				DRC_ST_UNLOCK();
-				return;
-			}
+		if (drc->refcnt != 0) /* quick path */
+			break;
+
+		/* note t's lock order wrt drc->mtx is the opposite of
+		 * drc->xt[*].lock. Drop and reacquire locks in correct
+		 * order.
+		 */
+		PTHREAD_MUTEX_unlock(&drc->mtx);
+		DRC_ST_LOCK();
+		PTHREAD_MUTEX_lock(&drc->mtx);
+
+		/* Since we dropped and reacquired the drc lock for the
+		 * correct lock order, we need to recheck the drc fields
+		 * again!
+		 */
+		if (drc->refcnt == 0 && !(drc->flags & DRC_FLAG_RECYCLE)) {
+			drc->d_u.tcp.recycle_time = time(NULL);
+			drc->flags |= DRC_FLAG_RECYCLE;
+			TAILQ_INSERT_TAIL(&drc_st->tcp_drc_recycle_q,
+					  drc, d_u.tcp.recycle_q);
+			++(drc_st->tcp_drc_recycle_qlen);
+			LogFullDebug(COMPONENT_DUPREQ,
+				     "enqueue drc %p for recycle", drc);
 		}
+		DRC_ST_UNLOCK();
+		break;
+
 	default:
 		break;
 	};
@@ -692,7 +742,7 @@ static inline const nfs_function_desc_t *nfs_dupreq_func(dupreq_entry_t *dv)
 {
 	const nfs_function_desc_t *func = NULL;
 
-	if (dv->hin.rq_prog == nfs_param.core_param.program[P_NFS]) {
+	if (dv->hin.rq_prog == NFS_program[P_NFS]) {
 		switch (dv->hin.rq_vers) {
 #ifdef _USE_NFS3
 		case NFS_V3:
@@ -705,10 +755,10 @@ static inline const nfs_function_desc_t *nfs_dupreq_func(dupreq_entry_t *dv)
 		default:
 			/* not reached */
 			LogMajor(COMPONENT_DUPREQ,
-				 "NFS Protocol version %d unknown",
-				 (int)dv->hin.rq_vers);
+				 "NFS Protocol version %" PRIu32 " unknown",
+				 dv->hin.rq_vers);
 		}
-	} else if (dv->hin.rq_prog == nfs_param.core_param.program[P_MNT]) {
+	} else if (dv->hin.rq_prog == NFS_program[P_MNT]) {
 		switch (dv->hin.rq_vers) {
 		case MOUNT_V1:
 			func = &mnt1_func_desc[dv->hin.rq_proc];
@@ -719,19 +769,19 @@ static inline const nfs_function_desc_t *nfs_dupreq_func(dupreq_entry_t *dv)
 		default:
 			/* not reached */
 			LogMajor(COMPONENT_DUPREQ,
-				 "MOUNT Protocol version %d unknown",
-				 (int)dv->hin.rq_vers);
+				 "MOUNT Protocol version %" PRIu32 " unknown",
+				 dv->hin.rq_vers);
 			break;
 		}
 #ifdef _USE_NLM
-	} else if (dv->hin.rq_prog == nfs_param.core_param.program[P_NLM]) {
+	} else if (dv->hin.rq_prog == NFS_program[P_NLM]) {
 		switch (dv->hin.rq_vers) {
 		case NLM4_VERS:
 			func = &nlm4_func_desc[dv->hin.rq_proc];
 			break;
 		}
 #endif /* _USE_NLM */
-	} else if (dv->hin.rq_prog == nfs_param.core_param.program[P_RQUOTA]) {
+	} else if (dv->hin.rq_prog == NFS_program[P_RQUOTA]) {
 		switch (dv->hin.rq_vers) {
 		case RQUOTAVERS:
 			func = &rquota1_func_desc[dv->hin.rq_proc];
@@ -742,8 +792,9 @@ static inline const nfs_function_desc_t *nfs_dupreq_func(dupreq_entry_t *dv)
 		}
 	} else {
 		/* not reached */
-		LogMajor(COMPONENT_DUPREQ, "protocol %d is not managed",
-			 (int)dv->hin.rq_prog);
+		LogMajor(COMPONENT_DUPREQ,
+			 "protocol %" PRIu32 " is not managed",
+			 dv->hin.rq_prog);
 	}
 
 	return func;
@@ -780,6 +831,13 @@ static inline void nfs_dupreq_free_dupreq(dupreq_entry_t *dv)
 {
 	const nfs_function_desc_t *func;
 
+	assert(dv->refcnt == 0);
+
+	LogDebug(COMPONENT_DUPREQ,
+		 "freeing dupreq entry dv=%p, dv xid=%" PRIu32
+		 " cksum %" PRIu64 " state=%s",
+		 dv, dv->hin.tcp.rq_xid, dv->hk,
+		 dupreq_state_table[dv->state]);
 	if (dv->res) {
 		func = nfs_dupreq_func(dv);
 		func->free_function(dv->res);
@@ -789,6 +847,33 @@ static inline void nfs_dupreq_free_dupreq(dupreq_entry_t *dv)
 	pool_free(dupreq_pool, dv);
 }
 
+/**
+ * @brief get a ref count on dupreq_entry_t
+ */
+static inline void dupreq_entry_get(dupreq_entry_t *dv)
+{
+	(void)atomic_inc_uint32_t(&dv->refcnt);
+}
+
+/**
+ * @brief release a ref count on dupreq_entry_t
+ *
+ * The caller must not access dv any more after this call as it could be
+ * freed here.
+ */
+static inline void dupreq_entry_put(dupreq_entry_t *dv)
+{
+	int32_t refcnt;
+
+	refcnt = atomic_dec_uint32_t(&dv->refcnt);
+
+	/* If ref count is zero, no one should be accessing it other
+	 * than us.  so no lock is needed.
+	 */
+	if (refcnt == 0)
+		nfs_dupreq_free_dupreq(dv);
+}
+
 /**
  * @page DRC_RETIRE DRC request retire heuristic.
  *
@@ -896,29 +981,12 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs,
 {
 	dupreq_status_t status = DUPREQ_SUCCESS;
 	dupreq_entry_t *dv = NULL, *dk = NULL;
-	bool release_dk = true;
-	nfs_res_t *res = NULL;
 	drc_t *drc;
 	enum drc_type dtype = get_drc_type(req);
 
-	/* Disabled? */
-	if (nfs_param.core_param.drc.disabled) {
-		req->rq_u1 = (void *)DUPREQ_NOCACHE;
-		res = alloc_nfs_res();
-		goto out;
-	}
+	if (nfs_param.core_param.drc.disabled)
+		goto no_cache;
 
-	req->rq_u1 = (void *)DUPREQ_BAD_ADDR1;
-	req->rq_u2 = (void *)DUPREQ_BAD_ADDR1;
-
-	drc = nfs_dupreq_get_drc(req);
-	if (!drc) {
-		status = DUPREQ_INSERT_MALLOC_ERROR;
-		goto out;
-	}
-
-	/* We use dtype instead of drc->type because this DRC may have been
-	 * used for NFSv3 and/or NFSv4 clients. We can't trust the drc type. */
 	switch (dtype) {
 	case DRC_TCP_V4:
 		if (reqnfs->funcdesc->service_function == nfs4_Compound) {
@@ -927,55 +995,51 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs,
 				 * the request through for later
 				 * cleanup--all v41 caching is handled
 				 * by the v41 slot reply cache */
-				req->rq_u1 = (void *)DUPREQ_NOCACHE;
-				res = alloc_nfs_res();
-				goto out;
+				goto no_cache;
 			}
 		}
 		break;
 	default:
 		/* likewise for other protocol requests we may not or choose not
 		 * to cache */
-		if (!(reqnfs->funcdesc->dispatch_behaviour & CAN_BE_DUP)) {
-			req->rq_u1 = (void *)DUPREQ_NOCACHE;
-			res = alloc_nfs_res();
-			goto out;
-		}
+		if (!(reqnfs->funcdesc->dispatch_behaviour & CAN_BE_DUP))
+			goto no_cache;
 		break;
 	}
 
-	dk = alloc_dupreq();
 
+	drc = nfs_dupreq_get_drc(req);
+	dk = alloc_dupreq();
 	dk->hin.drc = drc;	/* trans. call path ref to dv */
 
 	switch (drc->type) {
 	case DRC_TCP_V4:
 	case DRC_TCP_V3:
-		dk->hin.tcp.rq_xid = req->rq_xid;
+		dk->hin.tcp.rq_xid = req->rq_msg.rm_xid;
 		/* XXX needed? */
-		dk->hin.rq_prog = req->rq_prog;
-		dk->hin.rq_vers = req->rq_vers;
-		dk->hin.rq_proc = req->rq_proc;
+		dk->hin.rq_prog = req->rq_msg.cb_prog;
+		dk->hin.rq_vers = req->rq_msg.cb_vers;
+		dk->hin.rq_proc = req->rq_msg.cb_proc;
 		break;
 	case DRC_UDP_V234:
-		dk->hin.tcp.rq_xid = req->rq_xid;
+		dk->hin.tcp.rq_xid = req->rq_msg.rm_xid;
 		if (unlikely(!copy_xprt_addr(&dk->hin.addr, req->rq_xprt))) {
-			status = DUPREQ_INSERT_MALLOC_ERROR;
-			goto release_dk;
+			nfs_dupreq_put_drc(drc, DRC_FLAG_NONE);
+			nfs_dupreq_free_dupreq(dk);
+			return DUPREQ_INSERT_MALLOC_ERROR;
 		}
-		dk->hin.rq_prog = req->rq_prog;
-		dk->hin.rq_vers = req->rq_vers;
-		dk->hin.rq_proc = req->rq_proc;
+		dk->hin.rq_prog = req->rq_msg.cb_prog;
+		dk->hin.rq_vers = req->rq_msg.cb_vers;
+		dk->hin.rq_proc = req->rq_msg.cb_proc;
 		break;
 	default:
-		/* error */
-		status = DUPREQ_ERROR;
-		goto release_dk;
+		/* @todo: should this be an assert? */
+		nfs_dupreq_put_drc(drc, DRC_FLAG_NONE);
+		nfs_dupreq_free_dupreq(dk);
+		return DUPREQ_INSERT_MALLOC_ERROR;
 	}
 
-	/* TI-RPC computed checksum */
-	dk->hk = req->rq_cksum;
-
+	dk->hk = req->rq_cksum; /* TI-RPC computed checksum */
 	dk->state = DUPREQ_START;
 	dk->timestamp = time(NULL);
 
@@ -987,6 +1051,7 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs,
 		nv = rbtree_x_cached_lookup(&drc->xt, t, &dk->rbt_k, dk->hk);
 		if (nv) {
 			/* cached request */
+			nfs_dupreq_free_dupreq(dk);
 			dv = opr_containerof(nv, dupreq_entry_t, rbt_k);
 			PTHREAD_MUTEX_lock(&dv->mtx);
 			if (unlikely(dv->state == DUPREQ_START)) {
@@ -994,57 +1059,62 @@ dupreq_status_t nfs_dupreq_start(nfs_request_t *reqnfs,
 			} else {
 				/* satisfy req from the DRC, incref,
 				   extend window */
-				res = dv->res;
+				req->rq_u1 = dv;
+				reqnfs->res_nfs = req->rq_u2 = dv->res;
+				status = DUPREQ_EXISTS;
+				dupreq_entry_get(dv);
+			}
+			PTHREAD_MUTEX_unlock(&dv->mtx);
+
+			if (status == DUPREQ_EXISTS) {
 				PTHREAD_MUTEX_lock(&drc->mtx);
 				drc_inc_retwnd(drc);
 				PTHREAD_MUTEX_unlock(&drc->mtx);
-				status = DUPREQ_EXISTS;
-				(dv->refcnt)++;
 			}
+
 			LogDebug(COMPONENT_DUPREQ,
-				 "dupreq hit dk=%p, dk xid=%u cksum %" PRIu64
-				 " state=%s", dk, dk->hin.tcp.rq_xid, dk->hk,
-				 dupreq_state_table[dk->state]);
-			req->rq_u1 = dv;
-			PTHREAD_MUTEX_unlock(&dv->mtx);
+				 "dupreq hit dv=%p, dv xid=%" PRIu32
+				 " cksum %" PRIu64 " state=%s",
+				 dv, dv->hin.tcp.rq_xid, dv->hk,
+				 dupreq_state_table[dv->state]);
 		} else {
 			/* new request */
-			res = dk->res = req->rq_u2 = alloc_nfs_res();
 			req->rq_u1 = dk;
-			release_dk = false;
-			dv = dk;
+			dk->res = alloc_nfs_res();
+			reqnfs->res_nfs = req->rq_u2 = dk->res;
 
 			/* cache--can exceed drc->maxsize */
 			(void)rbtree_x_cached_insert(&drc->xt, t,
 						&dk->rbt_k, dk->hk);
-			dk->refcnt = 1;
+
+			/* dupreq ref count starts with 2; one for the caller
+			 * and another for staying in the hash table.
+			 */
+			dk->refcnt = 2;
 
 			/* add to q tail */
 			PTHREAD_MUTEX_lock(&drc->mtx);
 			TAILQ_INSERT_TAIL(&drc->dupreq_q, dk, fifo_q);
 			++(drc->size);
 			PTHREAD_MUTEX_unlock(&drc->mtx);
+
+			LogFullDebug(COMPONENT_DUPREQ,
+				     "starting dk=%p xid=%" PRIu32
+				     " on DRC=%p state=%s, status=%s, refcnt=%d, drc->size=%d",
+				     dk, dk->hin.tcp.rq_xid, drc,
+				     dupreq_state_table[dk->state],
+				     dupreq_status_table[status],
+				     dk->refcnt, drc->size);
 		}
 		PTHREAD_MUTEX_unlock(&t->mtx);
 	}
 
-	LogFullDebug(COMPONENT_DUPREQ,
-		"starting dv=%p xid=%u on DRC=%p state=%s, status=%s, refcnt=%d, drc->size=%d",
-		dv, dk->hin.tcp.rq_xid, drc,
-		dupreq_state_table[dv->state], dupreq_status_table[status],
-		(dv) ? dv->refcnt : 0, drc->size);
-
- release_dk:
-	if (release_dk)
-		nfs_dupreq_free_dupreq(dk);
-
-	nfs_dupreq_put_drc(req->rq_xprt, drc, DRC_FLAG_NONE);	/* dk ref */
-
- out:
-	if (res)
-		reqnfs->res_nfs = req->rq_u2 = res;
-
 	return status;
+
+no_cache:
+	req->rq_u1 = (void *)DUPREQ_NOCACHE;
+	reqnfs->res_nfs = req->rq_u2 = alloc_nfs_res();
+	return DUPREQ_SUCCESS;
 }
 
 /**
@@ -1085,10 +1155,6 @@ dupreq_status_t nfs_dupreq_finish(struct svc_req *req, nfs_res_t *res_nfs)
 	if (dv == (void *)DUPREQ_NOCACHE)
 		goto out;
 
-	/* do nothing if nfs_dupreq_start failed completely */
-	if (dv == (void *)DUPREQ_BAD_ADDR1)
-		goto out;
-
 	PTHREAD_MUTEX_lock(&dv->mtx);
 	dv->res = res_nfs;
 	dv->timestamp = time(NULL);
@@ -1100,7 +1166,8 @@ dupreq_status_t nfs_dupreq_finish(struct svc_req *req, nfs_res_t *res_nfs)
 	PTHREAD_MUTEX_lock(&drc->mtx);
 
 	LogFullDebug(COMPONENT_DUPREQ,
-		     "completing dv=%p xid=%u on DRC=%p state=%s, status=%s, refcnt=%d, drc->size=%d",
+		     "completing dv=%p xid=%" PRIu32
+		     " on DRC=%p state=%s, status=%s, refcnt=%d, drc->size=%d",
 		dv, dv->hin.tcp.rq_xid, drc,
 		dupreq_state_table[dv->state], dupreq_status_table[status],
 		dv->refcnt, drc->size);
@@ -1112,41 +1179,53 @@ dupreq_status_t nfs_dupreq_finish(struct svc_req *req, nfs_res_t *res_nfs)
 dq_again:
 	if (drc_should_retire(drc)) {
 		ov = TAILQ_FIRST(&drc->dupreq_q);
-dq_retry:
 		if (likely(ov)) {
-			/* quick check without partition lock */
-			if (unlikely(ov->refcnt > 0)) {
-				/* ov still in use, apparently */
-				if (cnt++ < DUPREQ_MAX_RETRIES) {
-					ov = TAILQ_NEXT(ov, fifo_q);
-					goto dq_retry;
-				}
-				LogWarn(COMPONENT_DUPREQ,
-					"DRC retire entries: unable to find reclaimable dupreq LRU entry after %d tries on DRC=%p, drc->size=%d",
-					DUPREQ_MAX_RETRIES, drc,
-					drc->size);
+			/* remove dict entry */
+			t = rbtx_partition_of_scalar(&drc->xt, ov->hk);
+			uint64_t ov_hk = ov->hk;
+
+			/* Need to acquire partition lock, but the lock
+			 * order is partition lock followed by drc lock.
+			 * Drop drc lock and reacquire it!
+			 */
+			PTHREAD_MUTEX_unlock(&drc->mtx);
+			PTHREAD_MUTEX_lock(&t->mtx);	/* partition lock */
+			PTHREAD_MUTEX_lock(&drc->mtx);
+
+			/* Since we dropped drc lock and reacquired it,
+			 * the drc dupreq list may have changed. Get the
+			 * dupreq entry from the list again.
+			 */
+			ov = TAILQ_FIRST(&drc->dupreq_q);
+
+			/* Make sure that we are removing the entry we
+			 * expected (imperfect, but harmless).
+			 */
+			if (ov == NULL || ov->hk != ov_hk) {
+				PTHREAD_MUTEX_unlock(&t->mtx);
 				goto unlock;
 			}
+
 			/* remove q entry */
 			TAILQ_REMOVE(&drc->dupreq_q, ov, fifo_q);
 			--(drc->size);
+			/* release dv's ref */
+			nfs_dupreq_put_drc(drc, DRC_FLAG_LOCKED);
+			/* drc->mtx gets unlocked in the above call! */
 
-			/* remove dict entry */
-			t = rbtx_partition_of_scalar(&drc->xt, ov->hk);
-			/* interlock */
-			PTHREAD_MUTEX_unlock(&drc->mtx);
-			PTHREAD_MUTEX_lock(&t->mtx);	/* partition lock */
 			rbtree_x_cached_remove(&drc->xt, t, &ov->rbt_k, ov->hk);
+
 			PTHREAD_MUTEX_unlock(&t->mtx);
 
 			LogDebug(COMPONENT_DUPREQ,
-				 "retiring ov=%p xid=%u on DRC=%p state=%s, status=%s, refcnt=%d",
+				 "retiring ov=%p xid=%" PRIu32
+				 " on DRC=%p state=%s, status=%s, refcnt=%d",
 				 ov, ov->hin.tcp.rq_xid,
 				 ov->hin.drc, dupreq_state_table[dv->state],
 				 dupreq_status_table[status], ov->refcnt);
 
-			/* deep free ov */
-			nfs_dupreq_free_dupreq(ov);
+			/* release hashtable ref count */
+			dupreq_entry_put(ov);
 
 			/* conditionally retire another */
 			if (cnt++ < DUPREQ_MAX_RETRIES) {
@@ -1168,8 +1247,8 @@ dq_retry:
  *
  * @brief Remove an entry (request) from a duplicate request cache.
  *
- * The expected pattern is that nfs_rpc_execute shall delete requests only
- * in error conditions.  The refcnt of the corresponding duplicate request
+ * The expected pattern is that nfs_rpc_process_request shall delete requests
+ * only in error conditions.  The refcnt of the corresponding duplicate request
  * entry is unchanged (ie., the caller must still call nfs_dupreq_rele).
  *
  * We assert req->rq_u1 now points to the corresonding duplicate request
@@ -1191,17 +1270,14 @@ dupreq_status_t nfs_dupreq_delete(struct svc_req *req)
 	if (dv == (void *)DUPREQ_NOCACHE)
 		goto out;
 
-	/* do nothing if nfs_dupreq_start failed completely */
-	if (dv == (void *)DUPREQ_BAD_ADDR1)
-		goto out;
-
 	PTHREAD_MUTEX_lock(&dv->mtx);
 	drc = dv->hin.drc;
 	dv->state = DUPREQ_DELETED;
 	PTHREAD_MUTEX_unlock(&dv->mtx);
 
 	LogFullDebug(COMPONENT_DUPREQ,
-		     "deleting dv=%p xid=%u on DRC=%p state=%s, status=%s, refcnt=%d",
+		     "deleting dv=%p xid=%" PRIu32
+		     " on DRC=%p state=%s, status=%s, refcnt=%d",
 		     dv, dv->hin.tcp.rq_xid, drc,
 		     dupreq_state_table[dv->state], dupreq_status_table[status],
 		     dv->refcnt);
@@ -1211,18 +1287,20 @@ dupreq_status_t nfs_dupreq_delete(struct svc_req *req)
 
 	PTHREAD_MUTEX_lock(&t->mtx);
 	rbtree_x_cached_remove(&drc->xt, t, &dv->rbt_k, dv->hk);
-
 	PTHREAD_MUTEX_unlock(&t->mtx);
+
 	PTHREAD_MUTEX_lock(&drc->mtx);
 
-	if (TAILQ_IS_ENQUEUED(dv, fifo_q))
-		TAILQ_REMOVE(&drc->dupreq_q, dv, fifo_q);
+	TAILQ_REMOVE(&drc->dupreq_q, dv, fifo_q);
 	--(drc->size);
 
-	/* release dv's ref and unlock */
-	nfs_dupreq_put_drc(req->rq_xprt, drc, DRC_FLAG_LOCKED);
+	/* release dv's ref on drc and unlock */
+	nfs_dupreq_put_drc(drc, DRC_FLAG_LOCKED);
 	/* !LOCKED */
 
+	/* we removed the dupreq from hashtable, release a ref */
+	dupreq_entry_put(dv);
+
  out:
 	return status;
 }
@@ -1233,10 +1311,6 @@ dupreq_status_t nfs_dupreq_delete(struct svc_req *req)
  * We assert req->rq_u1 now points to the corresonding duplicate request
  * cache entry (dv).
  *
- * In the common case, a refcnt of 0 indicates that dv is cached.  If
- * also dv->state == DUPREQ_DELETED, the request entry has been discarded
- * and should be destroyed here.
- *
  * @param[in] req  The svc_req structure.
  * @param[in] func The function descriptor for this request type
  */
@@ -1253,30 +1327,18 @@ void nfs_dupreq_rele(struct svc_req *req, const nfs_function_desc_t *func)
 		goto out;
 	}
 
-	PTHREAD_MUTEX_lock(&dv->mtx);
-
 	LogFullDebug(COMPONENT_DUPREQ,
-		     "releasing dv=%p xid=%u on DRC=%p state=%s, refcnt=%d",
+		     "releasing dv=%p xid=%" PRIu32
+		     " on DRC=%p state=%s, refcnt=%d",
 		     dv, dv->hin.tcp.rq_xid, dv->hin.drc,
 		     dupreq_state_table[dv->state], dv->refcnt);
 
-	(dv->refcnt)--;
-	if (dv->refcnt == 0) {
-		if (dv->state == DUPREQ_DELETED) {
-			PTHREAD_MUTEX_unlock(&dv->mtx);
-			/* deep free */
-			nfs_dupreq_free_dupreq(dv);
-			return;
-		}
-	}
-	PTHREAD_MUTEX_unlock(&dv->mtx);
+	dupreq_entry_put(dv);
 
  out:
 	/* dispose RPC header */
 	if (req->rq_auth)
-		SVCAUTH_RELEASE(req->rq_auth, req);
-
-	(void)free_rpc_msg(req->rq_msg);
+		SVCAUTH_RELEASE(req);
 }
 
 /**
