@@ -43,6 +43,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#include <sched.h>
 #include "nfs4.h"
 #include "fsal.h"
 #include "sal_functions.h"
@@ -322,6 +323,7 @@ void free_client_id(nfs_client_id_t *clientid)
 #ifdef _HAVE_GSSAPI
 	if (clientid->cid_credential.flavor == RPCSEC_GSS) {
 		struct svc_rpc_gss_data *gd;
+
 		gd = clientid->cid_credential.auth_union.auth_gss.gd;
 		unref_svc_rpc_gss_data(gd, 0);
 	}
@@ -340,10 +342,8 @@ void free_client_id(nfs_client_id_t *clientid)
 		}
 	}
 
-	if (clientid->cid_recov_dir) {
-		gsh_free(clientid->cid_recov_dir);
-		clientid->cid_recov_dir = NULL;
-	}
+	gsh_free(clientid->cid_recov_tag);
+	clientid->cid_recov_tag = NULL;
 
 	PTHREAD_MUTEX_destroy(&clientid->cid_mutex);
 	PTHREAD_MUTEX_destroy(&clientid->cid_owner.so_mutex);
@@ -560,6 +560,7 @@ nfs_client_id_t *create_client_id(clientid4 clientid,
 #ifdef _HAVE_GSSAPI
 	if (credential->flavor == RPCSEC_GSS) {
 		struct svc_rpc_gss_data *gd;
+
 		gd = credential->auth_union.auth_gss.gd;
 		(void)atomic_inc_uint32_t(&gd->refcnt);
 	}
@@ -872,7 +873,7 @@ bool clientid_has_state(nfs_client_id_t *clientid)
  */
 bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 {
-	int rc;
+	int rc, held;
 	struct gsh_buffdesc buffkey;
 	struct gsh_buffdesc old_key;
 	struct gsh_buffdesc old_value;
@@ -988,10 +989,19 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 			       &owner->so_owner.so_nfs4_owner.so_perclient);
 
 		/* Hold a reference to the owner while we drop the cid_mutex. */
-		inc_state_owner_ref(owner);
+		held = hold_state_owner(owner);
 
 		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
 
+		/* If this owner is in the process of being freed, skip
+		 * and work on the next owner. We also do yield for the
+		 * other thread to complete freeing this owner!
+		 */
+		if (!held) {
+			sched_yield();
+			continue;
+		}
+
 		state_nfs4_owner_unlock_all(owner);
 
 		if (isFullDebug(COMPONENT_CLIENTID)) {
@@ -1045,10 +1055,19 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 			       &owner->so_owner.so_nfs4_owner.so_perclient);
 
 		/* Hold a reference to the owner while we drop the cid_mutex. */
-		inc_state_owner_ref(owner);
+		held = hold_state_owner(owner);
 
 		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
 
+		/* If this owner is in the process of being freed, skip
+		 * and work on the next owner. We also do yield for the
+		 * other thread to complete freeing this owner!
+		 */
+		if (!held) {
+			sched_yield();
+			continue;
+		}
+
 		release_openstate(owner);
 
 		if (isFullDebug(COMPONENT_CLIENTID)) {
@@ -1099,10 +1118,10 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 		}
 	}
 
-	if (clientid->cid_recov_dir != NULL && !make_stale) {
-		nfs4_rm_clid(clientid->cid_recov_dir, v4_recov_dir, 0);
-		gsh_free(clientid->cid_recov_dir);
-		clientid->cid_recov_dir = NULL;
+	if (clientid->cid_recov_tag != NULL && !make_stale) {
+		nfs4_rm_clid(clientid);
+		gsh_free(clientid->cid_recov_tag);
+		clientid->cid_recov_tag = NULL;
 	}
 
 	if (isDebug(COMPONENT_CLIENTID)) {
@@ -1728,8 +1747,7 @@ static void client_cb(struct fridgethr_context *ctx)
 	cb_arg = ctx->arg;
 	cb_arg->cb(cb_arg->pclientid, cb_arg->state);
 	dec_client_id_ref(cb_arg->pclientid);
-	if (cb_arg->state)
-		gsh_free(cb_arg->state);
+	gsh_free(cb_arg->state);
 	gsh_free(cb_arg);
 }
 
