@@ -52,6 +52,9 @@
 #include "fsal_up.h"
 #include "nfs_file_handle.h"
 #include "nfs_proto_tools.h"
+#ifdef USE_LTTNG
+#include "gsh_lttng/state.h"
+#endif
 
 #ifdef DEBUG_SAL
 struct glist_head state_v4_all = GLIST_HEAD_INIT(state_v4_all);
@@ -79,15 +82,16 @@ pthread_mutex_t all_state_v4_mutex = PTHREAD_MUTEX_INITIALIZER;
  *
  * @return Operation status
  */
-state_status_t state_add_impl(struct fsal_obj_handle *obj,
-			      enum state_type state_type,
-			      union state_data *state_data,
-			      state_owner_t *owner_input, state_t **state,
-			      struct state_refer *refer)
+state_status_t _state_add_impl(struct fsal_obj_handle *obj,
+			       enum state_type state_type,
+			       union state_data *state_data,
+			       state_owner_t *owner_input, state_t **state,
+			       struct state_refer *refer,
+			       const char *func, int line)
 {
 	state_t *pnew_state = *state;
 	struct state_hdl *ostate = obj->state_hdl;
-	char str[DISPLAY_STATEID_OTHER_SIZE];
+	char str[DISPLAY_STATEID_OTHER_SIZE] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
 	bool got_export_ref = false;
@@ -130,8 +134,9 @@ state_status_t state_add_impl(struct fsal_obj_handle *obj,
 	mutex_init = true;
 
 	/* Add the stateid.other, this will increment cid_stateid_counter */
-	nfs4_BuildStateId_Other(owner_input->so_owner.so_nfs4_owner.
-				so_clientrec, pnew_state->stateid_other);
+	nfs4_BuildStateId_Other(
+			owner_input->so_owner.so_nfs4_owner.so_clientrec,
+			pnew_state->stateid_other);
 
 	/* Set the type and data for this state */
 	memcpy(&(pnew_state->state_data), state_data, sizeof(*state_data));
@@ -164,7 +169,11 @@ state_status_t state_add_impl(struct fsal_obj_handle *obj,
 	pnew_state->state_obj = obj;
 
 	/* Add the state to the related hashtable */
-	if (!nfs4_State_Set(pnew_state)) {
+	status = nfs4_State_Set(pnew_state);
+	switch (status) {
+	case STATE_SUCCESS:
+		break;
+	default:
 		if (!str_valid)
 			display_stateid_other(&dspbuf,
 					      pnew_state->stateid_other);
@@ -173,10 +182,6 @@ state_status_t state_add_impl(struct fsal_obj_handle *obj,
 			"Can't create a new state id %s for the obj %p (F)",
 			str, obj);
 
-		/* Return STATE_MALLOC_ERROR since most likely the
-		 * nfs4_State_Set failed to allocate memory.
-		 */
-		status = STATE_MALLOC_ERROR;
 		goto errout;
 	}
 
@@ -202,6 +207,10 @@ state_status_t state_add_impl(struct fsal_obj_handle *obj,
 	obj->obj_ops.get_ref(obj);
 	PTHREAD_MUTEX_unlock(&pnew_state->state_mutex);
 
+#ifdef USE_LTTNG
+	tracepoint(state, add, func, line, obj, pnew_state);
+#endif
+
 	/* Add state to list for owner */
 	PTHREAD_MUTEX_lock(&owner_input->so_mutex);
 	PTHREAD_MUTEX_lock(&pnew_state->state_mutex);
@@ -273,11 +282,12 @@ errout:
  *
  * @return Operation status
  */
-state_status_t state_add(struct fsal_obj_handle *obj,
-			 enum state_type state_type,
-			 union state_data *state_data,
-			 state_owner_t *owner_input,
-			 state_t **state, struct state_refer *refer)
+state_status_t _state_add(struct fsal_obj_handle *obj,
+			  enum state_type state_type,
+			  union state_data *state_data,
+			  state_owner_t *owner_input,
+			  state_t **state, struct state_refer *refer,
+			  const char *func, int line)
 {
 	state_status_t status = 0;
 
@@ -297,8 +307,8 @@ state_status_t state_add(struct fsal_obj_handle *obj,
 
 	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 	status =
-	    state_add_impl(obj, state_type, state_data, owner_input, state,
-			   refer);
+	    _state_add_impl(obj, state_type, state_data, owner_input, state,
+			    refer, func, line);
 	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
 	return status;
@@ -313,9 +323,9 @@ state_status_t state_add(struct fsal_obj_handle *obj,
  *
  */
 
-void state_del_locked(state_t *state)
+void _state_del_locked(state_t *state, const char *func, int line)
 {
-	char str[LOG_BUFF_LEN];
+	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
 	struct fsal_obj_handle *obj;
@@ -346,10 +356,11 @@ void state_del_locked(state_t *state)
 	 * this far, and thus if the refereces were non-NULL, they must still
 	 * be good. Holding the mutex is not strictly necessary for this
 	 * reason, however, static and dynamic code analysis have no way of
-	 * knowing this reference is safe.
+	 * knowing this reference is safe.  In addition, get_state_obj_ref()
+	 * would have taken the mutex anyway.
 	 */
 	PTHREAD_MUTEX_lock(&state->state_mutex);
-	obj = get_state_obj_ref(state);
+	obj = get_state_obj_ref_locked(state);
 
 	if (obj == NULL) {
 		LogDebug(COMPONENT_STATE,
@@ -358,6 +369,10 @@ void state_del_locked(state_t *state)
 		return;
 	}
 
+#ifdef USE_LTTNG
+	tracepoint(state, delete, func, line, obj, state);
+#endif
+
 	export = state->state_export;
 	owner = state->state_owner;
 	PTHREAD_MUTEX_unlock(&state->state_mutex);
@@ -375,14 +390,12 @@ void state_del_locked(state_t *state)
 	state->state_obj = NULL;
 	PTHREAD_MUTEX_unlock(&state->state_mutex);
 
-	if (obj->fsal->m_ops.support_ex(obj)) {
-		/* We need to close the state at this point. The state will
-		 * eventually be freed and it must be closed before free. This
-		 * is the last point we have a valid reference to the object
-		 * handle.
-		 */
-		(void) obj->obj_ops.close2(obj, state);
-	}
+	/* We need to close the state at this point. The state will
+	 * eventually be freed and it must be closed before free. This
+	 * is the last point we have a valid reference to the object
+	 * handle.
+	 */
+	(void) obj->obj_ops.close2(obj, state);
 
 	if (owner != NULL) {
 		bool owner_retain = false;
@@ -401,9 +414,14 @@ void state_del_locked(state_t *state)
 
 		/* If we are dropping the last open state from an open
 		 * owner, we will want to retain a refcount and let the
-		 * reaper thread clean up with owner. */
+		 * reaper thread clean up with owner.
+		 *
+		 * @todo: should we make the following glist_null check
+		 * an assert or remove it altogether?
+		 */
 		owner_retain = owner->so_type == STATE_OPEN_OWNER_NFSV4 &&
-		    glist_empty(&nfs4_owner->so_state_list);
+		    glist_empty(&nfs4_owner->so_state_list) &&
+		    glist_null(&nfs4_owner->so_cache_entry);
 
 		PTHREAD_MUTEX_unlock(&state->state_mutex);
 
@@ -413,14 +431,14 @@ void state_del_locked(state_t *state)
 			 */
 			PTHREAD_MUTEX_lock(&cached_open_owners_lock);
 
-			atomic_store_time_t(&nfs4_owner->cache_expire,
+			atomic_store_time_t(&nfs4_owner->so_cache_expire,
 					    nfs_param.nfsv4_param.lease_lifetime
 						+ time(NULL));
 			glist_add_tail(&cached_open_owners,
-				       &nfs4_owner->so_state_list);
+				       &nfs4_owner->so_cache_entry);
 
 			if (isFullDebug(COMPONENT_STATE)) {
-				char str[LOG_BUFF_LEN];
+				char str[LOG_BUFF_LEN] = "\0";
 				struct display_buffer dspbuf = {
 						sizeof(str), str, str};
 
@@ -549,7 +567,7 @@ bool get_state_obj_export_owner_refs(state_t *state,
 		     state->state_owner);
 
 	if (obj != NULL) {
-		*obj = get_state_obj_ref(state);
+		*obj = get_state_obj_ref_locked(state);
 		if ((*obj) == NULL)
 			goto fail;
 	}
@@ -639,7 +657,7 @@ enum nfsstat4 release_lock_owner(state_owner_t *owner)
 	}
 
 	if (isDebug(COMPONENT_STATE)) {
-		char str[LOG_BUFF_LEN];
+		char str[LOG_BUFF_LEN] = "\0";
 		struct display_buffer dspbuf = {sizeof(str), str, str};
 
 		display_owner(&dspbuf, owner);
@@ -649,6 +667,7 @@ enum nfsstat4 release_lock_owner(state_owner_t *owner)
 	while (true) {
 		state_t *state;
 		struct fsal_export *save_exp;
+		struct gsh_export *save_export;
 
 		state = glist_first_entry(&owner->so_owner.so_nfs4_owner
 								.so_state_list,
@@ -668,12 +687,15 @@ enum nfsstat4 release_lock_owner(state_owner_t *owner)
 		/* Set the fsal_export properly, since this can be called from
 		 * ops that don't do a putfh */
 		save_exp = op_ctx->fsal_export;
+		save_export = op_ctx->ctx_export;
 		op_ctx->fsal_export = state->state_exp;
+		op_ctx->ctx_export = state->state_export;
 
 		state_del(state);
 
-		/* Restore fsal_export */
+		/* Restore export */
 		op_ctx->fsal_export = save_exp;
+		op_ctx->ctx_export = save_export;
 
 		dec_state_t_ref(state);
 
@@ -688,13 +710,12 @@ enum nfsstat4 release_lock_owner(state_owner_t *owner)
  */
 void release_openstate(state_owner_t *owner)
 {
-	state_status_t state_status;
 	int errcnt = 0;
 	bool ok;
 	struct state_nfs4_owner_t *nfs4_owner = &owner->so_owner.so_nfs4_owner;
 
 	if (isFullDebug(COMPONENT_STATE)) {
-		char str[LOG_BUFF_LEN];
+		char str[LOG_BUFF_LEN] = "\0";
 		struct display_buffer dspbuf = {sizeof(str), str, str};
 
 		display_owner(&dspbuf, owner);
@@ -710,7 +731,7 @@ void release_openstate(state_owner_t *owner)
 
 		PTHREAD_MUTEX_lock(&owner->so_mutex);
 
-		if (atomic_fetch_time_t(&nfs4_owner->cache_expire) != 0) {
+		if (atomic_fetch_time_t(&nfs4_owner->so_cache_expire) != 0) {
 			/* This owner has no state, it is a cached open owner.
 			 * Take cached_open_owners_lock and verify.
 			 *
@@ -719,7 +740,7 @@ void release_openstate(state_owner_t *owner)
 			 */
 			PTHREAD_MUTEX_lock(&cached_open_owners_lock);
 
-			if (atomic_fetch_time_t(&nfs4_owner->cache_expire)
+			if (atomic_fetch_time_t(&nfs4_owner->so_cache_expire)
 			    != 0) {
 				/* We aren't racing with the reaper thread or
 				 * with get_state_owner.
@@ -784,20 +805,15 @@ void release_openstate(state_owner_t *owner)
 
 		PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 
-		if (state->state_type == STATE_TYPE_SHARE) {
-			op_ctx->ctx_export = export;
-			op_ctx->fsal_export = export->fsal_export;
-
-			state_status = state_share_remove(obj, owner, state);
-
-			if (!state_unlock_err_ok(state_status)) {
-				errcnt++;
-				LogEvent(COMPONENT_CLIENTID,
-					 "EXPIRY failed to release share stateid error %s",
-					 state_err_str(state_status));
-			}
+		/* In case op_ctx->export is not NULL... */
+		if (op_ctx->ctx_export != NULL) {
+			put_gsh_export(op_ctx->ctx_export);
 		}
 
+		/* op_ctx may be used by state_del_locked and others */
+		op_ctx->ctx_export = export;
+		op_ctx->fsal_export = export->fsal_export;
+
 		/* If FSAL supports extended operations, file will be closed by
 		 * state_del_locked.
 		 */
@@ -806,19 +822,17 @@ void release_openstate(state_owner_t *owner)
 
 		dec_state_t_ref(state);
 
-		if (!obj->fsal->m_ops.support_ex(obj)) {
-			/* Close the file in FSAL */
-			fsal_close(obj);
-		}
-
 		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
-		/* Release ref we held during state_del */
+		/* Release refs we held during state_del */
 		obj->obj_ops.put_ref(obj);
+		put_gsh_export(op_ctx->ctx_export);
+		op_ctx->ctx_export = NULL;
+		op_ctx->fsal_export = NULL;
 	}
 
 	if (errcnt == STATE_ERR_MAX) {
-		char str[LOG_BUFF_LEN];
+		char str[LOG_BUFF_LEN] = "\0";
 		struct display_buffer dspbuf = {sizeof(str), str, str};
 
 		display_owner(&dspbuf, owner);
@@ -840,6 +854,9 @@ void revoke_owner_delegs(state_owner_t *client_owner)
 	state_t *state, *first;
 	struct fsal_obj_handle *obj;
 	bool so_mutex_held;
+	struct gsh_export *export = NULL;
+	struct root_op_context ctx;
+	bool ok;
 
  again:
 	first = NULL;
@@ -877,9 +894,10 @@ void revoke_owner_delegs(state_owner_t *client_owner)
 		 * even after state_deleg_revoke releases the reference it
 		 * holds.
 		 */
-		obj = get_state_obj_ref(state);
+		ok = get_state_obj_export_owner_refs(state, &obj, &export,
+						     NULL);
 
-		if (obj == NULL) {
+		if (!ok || obj == NULL) {
 			LogDebug(COMPONENT_STATE,
 				 "Stale state or file");
 			continue;
@@ -892,13 +910,22 @@ void revoke_owner_delegs(state_owner_t *client_owner)
 		 * state_del_locked which is called from deleg_revoke.
 		 */
 		PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
-		state_deleg_revoke(obj, state);
-		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
-		if (!obj->fsal->m_ops.support_ex(obj)) {
-			/* Close the file in FSAL */
-			obj->obj_ops.close(obj);
-		}
+		/* Initialize req_ctx */
+		init_root_op_context(&ctx, export, export->fsal_export,
+				     0, 0, UNKNOWN_REQUEST);
+
+		state_deleg_revoke(obj, state);
+
+		/* Release refs we held */
+		obj->obj_ops.put_ref(obj);
+		put_gsh_export(op_ctx->ctx_export);
+		op_ctx->ctx_export = NULL;
+		op_ctx->fsal_export = NULL;
+
+		release_root_op_context();
+
+		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
 
 		/* Since we dropped so_mutex, we must restart the loop. */
 		goto again;
@@ -917,7 +944,6 @@ void state_export_release_nfs4_state(void)
 {
 	state_t *state;
 	state_t *first;
-	state_status_t state_status;
 	int errcnt = 0;
 	struct glist_head *glist, *glistn;
 	bool hold_export_lock;
@@ -1057,25 +1083,6 @@ void state_export_release_nfs4_state(void)
 
 		PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
 
-		if (state->state_type == STATE_TYPE_SHARE) {
-			state_status = state_share_remove(obj, owner, state);
-
-			if (!state_unlock_err_ok(state_status)) {
-
-				LogEvent(COMPONENT_CLIENTID,
-					 "EXPIRY failed to release share stateid error %s",
-					 state_err_str(state_status));
-				errcnt++;
-
-				/* Release the references taken above */
-				dec_state_owner_ref(owner);
-				dec_state_t_ref(state);
-				PTHREAD_RWLOCK_unlock(
-						&obj->state_hdl->state_lock);
-				continue;
-			}
-		}
-
 		if (state->state_type == STATE_TYPE_DELEG) {
 			/* this deletes the state too */
 			state_deleg_revoke(obj, state);
@@ -1097,7 +1104,7 @@ void state_export_release_nfs4_state(void)
 	if (errcnt == STATE_ERR_MAX) {
 		LogFatal(COMPONENT_STATE,
 			 "Could not complete cleanup of layouts for export %s",
-			 op_ctx->ctx_export->fullpath);
+			 op_ctx->ctx_export->pseudopath);
 	}
 }
 
@@ -1118,8 +1125,8 @@ void dump_all_states(void)
 		LogFullDebug(COMPONENT_STATE, " =State List= ");
 
 		glist_for_each(glist, &state_v4_all) {
-			char str1[LOG_BUFF_LEN / 2];
-			char str2[LOG_BUFF_LEN / 2];
+			char str1[LOG_BUFF_LEN / 2] = "\0";
+			char str2[LOG_BUFF_LEN / 2] = "\0";
 			struct display_buffer dspbuf1 = {
 						sizeof(str1), str1, str1};
 			struct display_buffer dspbuf2 = {
