@@ -10,8 +10,8 @@
 #include "bsd-base64.h"
 #include "client_mgr.h"
 #include "fsal.h"
+#include "recovery_fs.h"
 
-#define NFS_V4_RECOV_DIR "v4recov"
 #define NFS_V4_OLD_DIR "v4old"
 
 char v4_recov_dir[PATH_MAX];
@@ -304,7 +304,11 @@ static void fs_rm_clid_impl(char *recov_dir, char *parent_path, int position)
 
 void fs_rm_clid(nfs_client_id_t *clientid)
 {
-	fs_rm_clid_impl(clientid->cid_recov_tag, v4_recov_dir, 0);
+	char *recov_tag = clientid->cid_recov_tag;
+
+	clientid->cid_recov_tag = NULL;
+	fs_rm_clid_impl(recov_tag, v4_recov_dir, 0);
+	gsh_free(recov_tag);
 }
 
 /**
@@ -332,7 +336,6 @@ static void fs_cp_pop_revoked_delegs(clid_entry_t *clid_ent,
 	DIR *dp;
 	rdel_fh_t *new_ent;
 
-	glist_init(&clid_ent->cl_rfh_list);
 	/* Read the contents from recov dir of this clientid. */
 	dp = opendir(path);
 	if (dp == NULL) {
@@ -491,37 +494,23 @@ static int fs_read_recov_clids_impl(const char *parent_path,
 					 new_path, errno);
 			}
 		}
-		/* keep building the clientid str by cursively */
+		/* keep building the clientid str by recursively */
 		/* reading the directory structure */
+		total_clid_len = segment_len + 1;
 		if (clid_str)
-			total_clid_len = segment_len + 1 +
-					 strlen(clid_str);
-		else
-			total_clid_len = segment_len + 1;
-		build_clid = gsh_malloc(total_clid_len);
-
-		memset(build_clid, 0, total_clid_len);
+			total_clid_len += strlen(clid_str);
+		build_clid = gsh_calloc(1, total_clid_len);
 		if (clid_str)
 			strcpy(build_clid, clid_str);
 		strncat(build_clid, dentp->d_name, segment_len);
 
-		if (tgtdir)
-			rc = fs_read_recov_clids_impl(sub_path,
-						      build_clid,
-						      new_path,
-						      takeover,
-						      add_clid_entry,
-						      add_rfh_entry);
-		else
-			rc = fs_read_recov_clids_impl(sub_path,
-						      build_clid,
-						      NULL,
-						      takeover,
-						      add_clid_entry,
-						      add_rfh_entry);
-
-		if (new_path)
-			gsh_free(new_path);
+		rc = fs_read_recov_clids_impl(sub_path,
+					      build_clid,
+					      new_path,
+					      takeover,
+					      add_clid_entry,
+					      add_rfh_entry);
+		gsh_free(new_path);
 
 		/* after recursion, if the subdir has no non-hidden
 		 * directory this is the end of this clientid str. Add
@@ -607,8 +596,8 @@ static int fs_read_recov_clids_impl(const char *parent_path,
 	return num;
 }
 
-void fs_read_recov_clids_recover(add_clid_entry_hook add_clid_entry,
-				 add_rfh_entry_hook add_rfh_entry)
+static void fs_read_recov_clids_recover(add_clid_entry_hook add_clid_entry,
+					add_rfh_entry_hook add_rfh_entry)
 {
 	int rc;
 
@@ -645,6 +634,11 @@ void fs_read_recov_clids_takeover(nfs_grace_start_t *gsp,
 	int rc;
 	char path[PATH_MAX];
 
+	if (!gsp) {
+		fs_read_recov_clids_recover(add_clid_entry, add_rfh_entry);
+		return;
+	}
+
 	switch (gsp->event) {
 	case EVENT_UPDATE_CLIENTS:
 		snprintf(path, sizeof(path), "%s", v4_recov_dir);
@@ -677,7 +671,7 @@ void fs_read_recov_clids_takeover(nfs_grace_start_t *gsp,
 	}
 }
 
-static void fs_clean_old_recov_dir_impl(char *parent_path)
+void fs_clean_old_recov_dir_impl(char *parent_path)
 {
 	DIR *dp;
 	struct dirent *dentp;
@@ -783,28 +777,13 @@ void fs_add_revoke_fh(nfs_client_id_t *delr_clid, nfs_fh4 *delr_handle)
 	}
 }
 
-bool fs_check_clid(nfs_client_id_t *clientid, clid_entry_t *clid_ent)
-{
-	LogDebug(COMPONENT_CLIENTID, "compare %s to %s",
-		 clientid->cid_recov_tag, clid_ent->cl_name);
-
-	if (!clientid->cid_recov_tag)
-		return false;
-	if (!strncmp(clientid->cid_recov_tag,
-		     clid_ent->cl_name, PATH_MAX))
-		return true;
-	return false;
-}
-
 struct nfs4_recovery_backend fs_backend = {
 	.recovery_init = fs_create_recov_dir,
 	.recovery_cleanup = fs_clean_old_recov_dir,
-	.recovery_read_clids_recover = fs_read_recov_clids_recover,
-	.recovery_read_clids_takeover = fs_read_recov_clids_takeover,
+	.recovery_read_clids = fs_read_recov_clids_takeover,
 	.add_clid = fs_add_clid,
 	.rm_clid = fs_rm_clid,
 	.add_revoke_fh = fs_add_revoke_fh,
-	.check_clid = fs_check_clid,
 };
 
 void fs_backend_init(struct nfs4_recovery_backend **backend)
