@@ -50,6 +50,7 @@ config_file_t config_ParseFile(char *file_path,
 	struct config_root *root;
 	int rc;
 
+	glist_init(&all_blocks);
 	memset(&st, 0, sizeof(struct parser_state));
 	st.err_type = err_type;
 	rc = ganeshun_yy_init_parser(file_path, &st);
@@ -70,6 +71,24 @@ config_file_t config_ParseFile(char *file_path,
 	return (config_file_t)root;
 }
 
+/**
+ *  Return the first node in the global config block list with
+ *  name == block_name
+ */
+void *config_GetBlockNode(const char *block_name)
+{
+	struct glist_head *glh;
+	struct config_node *node;
+
+	glist_for_each(glh, &all_blocks) {
+		node = glist_entry(glh, struct config_node, blocks);
+		if (!strcasecmp(node->u.nterm.name, block_name)) {
+			return node;
+		}
+	}
+	return NULL;
+}
+
 /**
  * config_Print:
  * Print the content of the syntax tree
@@ -568,12 +587,9 @@ static void convert_inet_addr(struct config_node *node,
 	struct addrinfo *res = NULL;
 	int rc;
 
-	if (node->u.term.type == TERM_V4ADDR ||
-	    node->u.term.type == TERM_V4_ANY)
-		hints.ai_family = AF_INET;
-	else if (node->u.term.type == TERM_V6ADDR)
-		hints.ai_family = AF_INET6;
-	else {
+	if (node->u.term.type != TERM_V4ADDR &&
+	    node->u.term.type != TERM_V4_ANY &&
+	    node->u.term.type != TERM_V6ADDR) {
 		config_proc_error(node, err_type,
 				  "Expected an IP address, got a %s",
 				  config_term_desc(node->u.term.type));
@@ -581,11 +597,22 @@ static void convert_inet_addr(struct config_node *node,
 		err_type->errors++;
 		return;
 	}
-	hints.ai_flags = AI_ADDRCONFIG;
+
+	/* Try IPv6 (with mapping) first.  If this fails, fall back on IPv4, if
+	 * a v4 address was given. */
+	hints.ai_family = AF_INET6;
+	hints.ai_flags = AI_ADDRCONFIG | AI_V4MAPPED;
 	hints.ai_socktype = 0;
 	hints.ai_protocol = 0;
 	rc = getaddrinfo(node->u.term.varvalue, NULL,
 			 &hints, &res);
+
+	if (rc != 0 && (node->u.term.type == TERM_V4ADDR ||
+			node->u.term.type == TERM_V4_ANY)) {
+		hints.ai_family = AF_INET;
+		rc = getaddrinfo(node->u.term.varvalue, NULL,
+				 &hints, &res);
+	}
 	if (rc == 0) {
 		memcpy(sock, res->ai_addr, res->ai_addrlen);
 	} else {
@@ -733,8 +760,6 @@ static const char *config_type_str(enum config_type type)
 		return "CONFIG_BOOLBIT";
 	case CONFIG_IP_ADDR:
 		return "CONFIG_IP_ADDR";
-	case CONFIG_INET_PORT:
-		return "CONFIG_INET_PORT";
 	case CONFIG_BLOCK:
 		return "CONFIG_BLOCK";
 	case CONFIG_PROC:
@@ -751,6 +776,8 @@ static bool do_block_init(struct config_node *blk_node,
 	struct config_item *item;
 	void *param_addr;
 	sockaddr_t *sock;
+	struct addrinfo hints;
+	struct addrinfo *res = NULL;
 	int rc;
 	int errors = 0;
 
@@ -833,27 +860,22 @@ static bool do_block_init(struct config_node *blk_node,
 			sock = (sockaddr_t *)param_addr;
 			memset(sock, 0, sizeof(sockaddr_t));
 			errno = 0;
-			if (index(item->u.ip.def, ':') != NULL) {
-				sock->ss_family = AF_INET6;
-				rc = inet_pton(AF_INET6,
-					       item->u.ip.def,
-					       &((struct sockaddr_in6 *)sock)
-					       ->sin6_addr);
-			} else if (index(item->u.ip.def, '.') != NULL) {
-				sock->ss_family = AF_INET;
-				rc = inet_pton(AF_INET,
-					       item->u.ip.def,
-					       &((struct sockaddr_in *)sock)
-					       ->sin_addr);
-			} else {
-				config_proc_error(blk_node, err_type,
-						  "Bad default for %s: %s",
-						  item->name,
-						  item->u.ip.def);
-				errors++;
-				break;
+			/* Try IPv6 (with mapping) first.  If this fails, fall
+			 * back on IPv4, if a v4 address was given. */
+			hints.ai_family = AF_INET6;
+			hints.ai_flags = AI_ADDRCONFIG | AI_V4MAPPED;
+			hints.ai_socktype = 0;
+			hints.ai_protocol = 0;
+			rc = getaddrinfo(item->u.ip.def, NULL, &hints, &res);
+
+			if (rc != 0) {
+				hints.ai_family = AF_INET;
+				rc = getaddrinfo(item->u.ip.def, NULL, &hints,
+						 &res);
 			}
-			if (rc <= 0) {
+			if (rc == 0) {
+				memcpy(sock, res->ai_addr, res->ai_addrlen);
+			} else {
 				config_proc_error(blk_node, err_type,
 						  "Cannot set IP default for %s to %s because %s",
 						  item->name,
@@ -861,9 +883,8 @@ static bool do_block_init(struct config_node *blk_node,
 						  strerror(errno));
 				errors++;
 			}
-			break;
-		case CONFIG_INET_PORT:
-			*(uint16_t *)param_addr = htons(item->u.ui16.def);
+			if (res != NULL)
+				freeaddrinfo(res);
 			break;
 		case CONFIG_BLOCK:
 			(void) item->u.blk.init(NULL, param_addr);
@@ -959,12 +980,12 @@ static int do_block_load(struct config_node *blk,
 	for (item = params; item->name != NULL; item++) {
 		uint64_t num64;
 		bool bool_val;
-		uint32_t num32;
+		uint32_t num32 = 0;
 
 		node = lookup_node(&blk->u.nterm.sub_nodes, item->name);
 		if ((item->flags & CONFIG_MANDATORY) && (node == NULL)) {
 			err_type->missing = true;
-			err_type->errors++;
+			errors = ++err_type->errors;
 			config_proc_error(blk, err_type,
 					  "Mandatory field, %s is missing from block (%s)",
 					  item->name, blk->u.nterm.name);
@@ -979,7 +1000,7 @@ static int do_block_load(struct config_node *blk,
 						  "Parameter %s set more than once",
 						  next_node->u.nterm.name);
 				err_type->unique = true;
-				err_type->errors++;
+				errors = ++err_type->errors;
 				node = next_node;
 				continue;
 			}
@@ -1009,7 +1030,7 @@ static int do_block_load(struct config_node *blk,
 						  node->u.nterm.name,
 						  term_node->u.term.varvalue);
 				err_type->invalid = true;
-				err_type->errors++;
+				errors = ++err_type->errors;
 				node = next_node;
 				continue;
 			}
@@ -1053,8 +1074,18 @@ static int do_block_load(struct config_node *blk,
 				break;
 			case CONFIG_UINT64:
 				if (convert_number(term_node, item,
-						   &num64, err_type))
+						   &num64, err_type)) {
 					*(uint64_t *)param_addr = num64;
+					if (item->flags & CONFIG_MARK_SET) {
+						void *mask_addr;
+
+						mask_addr =
+						((char *)param_struct
+						 + item->u.ui64.set_off);
+						*(uint32_t *)mask_addr
+						|= item->u.ui64.bit;
+					}
+				}
 				break;
 			case CONFIG_ANON_ID:
 				if (convert_number(term_node, item,
@@ -1185,12 +1216,6 @@ static int do_block_load(struct config_node *blk,
 						  (sockaddr_t *)param_addr,
 						  err_type);
 				break;
-			case CONFIG_INET_PORT:
-				if (convert_number(term_node, item,
-						   &num64, err_type))
-					*(uint16_t *)param_addr =
-						htons((uint16_t)num64);
-				break;
 			case CONFIG_BLOCK:
 				if (!proc_block(node, item, param_addr,
 						err_type))
@@ -1206,7 +1231,7 @@ static int do_block_load(struct config_node *blk,
 						  "Cannot set value for type(%d) yet",
 						  item->type);
 				err_type->internal = true;
-				err_type->errors++;
+				errors = ++err_type->errors;
 				break;
 				}
 			node = next_node;
@@ -1358,6 +1383,7 @@ static bool proc_block(struct config_node *node,
 
 err_out:
 	(void)item->u.blk.init(link_mem, param_struct);
+	err_type->dispose = false;
 	return false;
 }
 
@@ -1946,4 +1972,3 @@ int load_config_from_parse(config_file_t config,
 	}
 	return found;
 }
-
