@@ -131,6 +131,21 @@ struct gsh_export {
 	bool has_pnfs_ds;		/*< id_servers matches export_id */
 };
 
+/* Use macro to define this to get around include file order. */
+#define export_path(export) \
+	((nfs_param.core_param.mount_path_pseudo) \
+		? ((export)->pseudopath) \
+		: ((export)->fullpath))
+
+/* If op_ctx request is NFS_V4 always use pseudopath, otherwise use fullpath
+ * for export.
+ */
+#define op_ctx_export_path(export) \
+	((op_ctx->nfs_vers == NFS_V4) || \
+	 (nfs_param.core_param.mount_path_pseudo) \
+		? ((export)->pseudopath) \
+		: ((export)->fullpath))
+
 static inline bool op_ctx_export_has_option(uint32_t option)
 {
 	return atomic_fetch_uint32_t(&op_ctx->ctx_export->options) & option;
@@ -157,10 +172,9 @@ struct gsh_export *get_gsh_export_by_pseudo_locked(char *path,
 						   bool exact_match);
 struct gsh_export *get_gsh_export_by_tag(char *tag);
 bool mount_gsh_export(struct gsh_export *exp);
-void put_gsh_export(struct gsh_export *a_export);
 void remove_gsh_export(uint16_t export_id);
 bool foreach_gsh_export(bool(*cb) (struct gsh_export *exp, void *state),
-			void *state);
+			bool wrlock, void *state);
 
 /**
  * @brief Advisory check of export readiness.
@@ -180,10 +194,19 @@ static inline bool export_ready(struct gsh_export *a_export)
 	return a_export->export_status == EXPORT_READY;
 }
 
-static inline void get_gsh_export_ref(struct gsh_export *a_export)
-{
-	(void) atomic_inc_int64_t(&a_export->refcnt);
-}
+void _get_gsh_export_ref(struct gsh_export *a_export,
+			 char *file, int line, char *function);
+
+#define get_gsh_export_ref(a_export) \
+	_get_gsh_export_ref(a_export, \
+	(char *) __FILE__, __LINE__, (char *) __func__)
+
+void _put_gsh_export(struct gsh_export *a_export,
+		     char *file, int line, char *function);
+
+#define put_gsh_export(a_export) \
+	_put_gsh_export(a_export, \
+	(char *) __FILE__, __LINE__, (char *) __func__)
 
 void export_revert(struct gsh_export *a_export);
 void export_add_to_mount_work(struct gsh_export *a_export);
