@@ -145,12 +145,6 @@ static inline void release_root_op_context(void)
 	op_ctx = ctx->old_op_ctx;
 }
 
-/**
- * @brief init_complete used to indicate if ganesha is during
- * startup or not
- */
-extern bool init_complete;
-
 /******************************************************
  *                Structure used to define a fsal
  ******************************************************/
@@ -244,9 +238,13 @@ void log_attrlist(log_components_t component, log_levels_t level,
 		  const char *reason, struct attrlist *attr, bool is_obj,
 		  char *file, int line, char *function);
 
-#define LogAttrlist(component, level, reason, attr, is_obj) \
-	log_attrlist(component, level, reason, attr, is_obj, \
-		     (char *) __FILE__, __LINE__, (char *) __func__)
+#define LogAttrlist(component, level, reason, attr, is_obj)                  \
+	do {                                                                 \
+		if (unlikely(isLevel(component, level)))                     \
+			log_attrlist(component, level, reason, attr, is_obj, \
+				     (char *) __FILE__, __LINE__,            \
+				     (char *) __func__);                     \
+	} while (0)
 
 const char *msg_fsal_err(fsal_errors_t fsal_err);
 #define fsal_err_txt(s) msg_fsal_err((s).major)
@@ -261,7 +259,7 @@ enum cb_state {
 	CB_PROBLEM,
 };
 
-typedef fsal_errors_t (*fsal_getattr_cb_t)
+typedef fsal_errors_t (*helper_readdir_cb)
 	(void *opaque,
 	 struct fsal_obj_handle *obj,
 	 const struct attrlist *attr,
@@ -305,10 +303,27 @@ struct fsal_readdir_cb_parms {
 fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 			   struct state_t *state, struct attrlist *attr);
 
+/**
+ *
+ * @brief Checks the permissions on an object
+ *
+ * This function returns success if the supplied credentials possess
+ * permission required to meet the specified access.
+ *
+ * @param[in]  obj         The object to be checked
+ * @param[in]  access_type The kind of access to be checked
+ *
+ * @return FSAL status
+ *
+ */
+static inline
 fsal_status_t fsal_access(struct fsal_obj_handle *obj,
-			  fsal_accessflags_t access_type,
-			  fsal_accessflags_t *allowed,
-			  fsal_accessflags_t *denied);
+			  fsal_accessflags_t access_type)
+{
+	return
+	    obj->obj_ops.test_access(obj, access_type, NULL, NULL, false);
+}
+
 fsal_status_t fsal_link(struct fsal_obj_handle *obj,
 			struct fsal_obj_handle *dest_dir,
 			const char *name);
@@ -351,23 +366,15 @@ fsal_status_t fsal_write2(struct fsal_obj_handle *obj,
 			  void *buffer,
 			  bool *sync,
 			  struct io_info *info);
-fsal_status_t fsal_rdwr(struct fsal_obj_handle *obj,
-		      fsal_io_direction_t io_direction,
-		      uint64_t offset, size_t io_size,
-		      size_t *bytes_moved, void *buffer,
-		      bool *eof,
-		      bool *sync, struct io_info *info);
 fsal_status_t fsal_readdir(struct fsal_obj_handle *directory, uint64_t cookie,
 			   unsigned int *nbfound, bool *eod_met,
-			   attrmask_t attrmask, fsal_getattr_cb_t cb,
+			   attrmask_t attrmask, helper_readdir_cb cb,
 			   void *opaque);
 fsal_status_t fsal_remove(struct fsal_obj_handle *parent, const char *name);
 fsal_status_t fsal_rename(struct fsal_obj_handle *dir_src,
 			  const char *oldname,
 			  struct fsal_obj_handle *dir_dest,
 			  const char *newname);
-fsal_status_t fsal_open(struct fsal_obj_handle *obj_hdl,
-			fsal_openflags_t openflags);
 fsal_status_t fsal_open2(struct fsal_obj_handle *in_obj,
 			 struct state_t *state,
 			 fsal_openflags_t openflags,
@@ -381,6 +388,8 @@ fsal_status_t fsal_reopen2(struct fsal_obj_handle *obj,
 			   struct state_t *state,
 			   fsal_openflags_t openflags,
 			   bool check_permission);
+fsal_status_t get_optional_attrs(struct fsal_obj_handle *obj_hdl,
+				 struct attrlist *attrs_out);
 /**
  * @brief Close a file
  *
@@ -394,40 +403,48 @@ fsal_status_t fsal_reopen2(struct fsal_obj_handle *obj,
  * @param[in] obj	File to close
  * @return FSAL status
  */
-
 static inline fsal_status_t fsal_close(struct fsal_obj_handle *obj_hdl)
 {
-	bool support_ex;
-
 	if (obj_hdl->type != REGULAR_FILE) {
 		/* Can only close a regular file */
 		return fsalstat(ERR_FSAL_NO_ERROR, 0);
 	}
 
-	support_ex = obj_hdl->fsal->m_ops.support_ex(obj_hdl);
-
-	if (!support_ex && obj_hdl->obj_ops.status(obj_hdl) == FSAL_O_CLOSED) {
-		/* If not support_ex and the file isn't open, return no error.
-		 */
-		return fsalstat(ERR_FSAL_NO_ERROR, 0);
-	}
-
-	/* Otherwise, return the result of close method. */
+	/* Return the result of close method. */
 	fsal_status_t status = obj_hdl->obj_ops.close(obj_hdl);
 
-	if (!FSAL_IS_ERROR(status) && !support_ex)
+	if (status.major != ERR_FSAL_NOT_OPENED) {
 		(void) atomic_dec_size_t(&open_fd_count);
+	} else {
+		/* Wasn't open.  Not an error, but shouldn't decrement */
+		status = fsalstat(ERR_FSAL_NO_ERROR, 0);
+	}
 
 	return status;
 }
 
 fsal_status_t fsal_statfs(struct fsal_obj_handle *obj,
 			  fsal_dynamicfsinfo_t *dynamicinfo);
-fsal_status_t fsal_commit(struct fsal_obj_handle *obj_hdl, off_t offset,
-			 size_t len);
+
+/**
+ * @brief Commit a section of a file to storage
+ *
+ * @param[in] obj	File to commit
+ * @param[in] offset	Offset for start of commit
+ * @param[in] len	Length of commit
+ * @return FSAL status
+ */
+static inline
+fsal_status_t fsal_commit(struct fsal_obj_handle *obj, off_t offset,
+			 size_t len)
+{
+	if ((uint64_t) len > ~(uint64_t) offset)
+		return fsalstat(ERR_FSAL_INVAL, 0);
+
+	return obj->obj_ops.commit2(obj, offset, len);
+}
 fsal_status_t fsal_verify2(struct fsal_obj_handle *obj,
 			   fsal_verifier_t verifier);
-bool fsal_is_open(struct fsal_obj_handle *obj);
 
 /**
  * @brief Pepare an attrlist for fetching attributes.
@@ -438,10 +455,10 @@ bool fsal_is_open(struct fsal_obj_handle *obj);
  */
 
 static inline void fsal_prepare_attrs(struct attrlist *attrs,
-				      attrmask_t mask)
+				      attrmask_t request_mask)
 {
 	memset(attrs, 0, sizeof(*attrs));
-	attrs->mask = mask;
+	attrs->request_mask = request_mask;
 }
 
 /**
@@ -465,15 +482,15 @@ static inline void fsal_release_attrs(struct attrlist *attrs)
 
 		/* Poison the acl since we no longer hold a reference. */
 		attrs->acl = NULL;
-		attrs->mask &= ~ATTR_ACL;
+		attrs->valid_mask &= ~ATTR_ACL;
 	}
 }
 
 /**
  * @brief Copy a set of attributes
  *
- * If ACL is requested in dest->mask, then ACL reference is acquired, otherwise
- * acl pointer is set to NULL.
+ * If ACL is requested in dest->request_mask, then ACL reference is acquired,
+ * otherwise acl pointer is set to NULL.
  *
  * @param[in,out] dest       The attrlist to receive the copy (mask must be set)
  * @param[in]     src        The attrlist to make a copy of
@@ -485,17 +502,19 @@ static inline void fsal_copy_attrs(struct attrlist *dest,
 				   struct attrlist *src,
 				   bool pass_refs)
 {
-	bool acl_asked = dest->mask & ATTR_ACL;
+	attrmask_t save_request_mask = dest->request_mask;
 
+	/* Copy source to dest, but retain dest->request_mask */
 	*dest = *src;
+	dest->request_mask = save_request_mask;
 
-	if (pass_refs) {
+	if (pass_refs && ((save_request_mask & ATTR_ACL) != 0)) {
 		/* Pass any ACL reference to the dest, so remove from src
 		 * without adjusting the refcount.
 		 */
 		src->acl = NULL;
-		src->mask &= ~ATTR_ACL;
-	} else if (dest->acl != NULL && acl_asked) {
+		src->valid_mask &= ~ATTR_ACL;
+	} else if (dest->acl != NULL && ((save_request_mask & ATTR_ACL) != 0)) {
 		/* Take reference on ACL if necessary */
 		nfs4_acl_entry_inc_ref(dest->acl);
 	} else {
@@ -504,7 +523,7 @@ static inline void fsal_copy_attrs(struct attrlist *dest,
 		 * release the ACL reference).
 		 */
 		dest->acl = NULL;
-		dest->mask &= ~ATTR_ACL;
+		dest->valid_mask &= ~ATTR_ACL;
 	}
 }
 
@@ -557,9 +576,6 @@ enum fsal_create_mode nfs3_createmode_to_fsal(createmode3 createmode)
  * The caller may pass FSAL_O_ANY to indicate any mode of open (RDONLY,
  * WRONLY, or RDWR is useable - often just to fetch attributes or something).
  *
- * Note that FSAL_O_SYNC is considered by this function, so the caller
- * expects the fd to be considered not usable if O_SYNC doesn't match.
- *
  * @param[in] fd_openflags The openflags describing the fd
  * @param[in] to_openflags The openflags describing the desired mode
  */
@@ -601,5 +617,22 @@ static inline bool open_correct(fsal_openflags_t fd_openflags,
 					== (to_openflags & FSAL_O_RDWR));
 }
 
+/**
+ * @brief "fsal_op_stats" struct useful for all the fsals which are going to
+ * implement support for FSAL specific statistics
+ */
+struct fsal_op_stats {
+	uint16_t op_code;
+	uint64_t resp_time;
+	uint64_t num_ops;
+	uint64_t resp_time_max;
+	uint64_t resp_time_min;
+};
+
+struct fsal_stats {
+	uint16_t total_ops;
+	struct fsal_op_stats *op_stats;
+};
+
 #endif				/* !FSAL_H */
 /** @} */
