@@ -40,6 +40,7 @@
 #include <dirent.h>		/* for MAXNAMLEN */
 
 #include "uid2grp.h"
+#include "nfsv41.h"
 
 /* Cookie to be used in FSAL_ListXAttrs() to bypass RO xattr */
 #define FSAL_XATTR_RW_COOKIE (~0)
@@ -338,6 +339,9 @@ typedef struct fsal_acl_data__ {
  */
 typedef uint64_t attrmask_t;
 
+/** For stackable FSALs that just pass through dealings with attributes */
+#define ALL_ATTRIBUTES UINT64_MAX
+
 /**
  * Attribute masks.
  */
@@ -415,6 +419,9 @@ typedef uint64_t attrmask_t;
 #define ATTRS_SET_TIME (ATTR_ATIME | ATTR_MTIME | \
 			ATTR_ATIME_SERVER | ATTR_MTIME_SERVER)
 
+/** Define the set of attributes contained or derrived from struct stat that
+ *  are supplied by posix2fsal_attributes.
+ */
 #define ATTRS_POSIX (ATTR_TYPE | ATTR_SIZE | ATTR_FSID | ATTR_FILEID |     \
 		     ATTR_MODE | ATTR_NUMLINKS | ATTR_OWNER | ATTR_GROUP | \
 		     ATTR_ATIME | ATTR_CTIME | ATTR_MTIME | ATTR_CHGTIME | \
@@ -428,6 +435,8 @@ struct attrlist {
 	attrmask_t request_mask; /*< Indicates the requested from the FSAL. */
 	attrmask_t valid_mask;	/*< Indicates the attributes to be set or
 				   that have been filled in by the FSAL. */
+	attrmask_t supported;	/*< Indicates which attributes the FSAL
+				    supports. */
 	object_file_type_t type;	/*< Type of this object */
 	uint64_t filesize;	/*< Logical size (amount of data that can be
 				   read) */
@@ -533,12 +542,14 @@ typedef enum {
 static inline fsal_accessflags_t FSAL_MODE_MASK(fsal_accessflags_t access)
 {
 	unsigned long acc = access & FSAL_MODE_BIT_MASK;
+
 	return (fsal_accessflags_t) acc;
 }
 
 static inline fsal_accessflags_t FSAL_ACE4_MASK(fsal_accessflags_t access)
 {
 	unsigned long acc = access & FSAL_ACE4_BIT_MASK;
+
 	return (fsal_accessflags_t) acc;
 }
 
@@ -583,7 +594,6 @@ typedef uint16_t fsal_openflags_t;
 						     * explicitly or'd together
 						     * so that FSAL_O_RDWR can
 						     * be used as a mask */
-#define FSAL_O_SYNC            0x0004  /* sync */
 #define FSAL_O_RECLAIM         0x0008  /* open reclaim */
 #define FSAL_O_REOPEN          0x0010  /* re-open */
 #define FSAL_O_ANY             0x0020  /* any open file descriptor is usable */
@@ -615,7 +625,6 @@ typedef enum enum_fsal_fsinfo_options {
 	fso_link_support,
 	fso_symlink_support,
 	fso_lock_support,
-	fso_lock_support_owner,
 	fso_lock_support_async_block,
 	fso_named_attr,
 	fso_unique_handles,
@@ -624,14 +633,13 @@ typedef enum enum_fsal_fsinfo_options {
 	fso_auth_exportpath_xdev,
 	fso_delegations_r,
 	fso_delegations_w,
-	fso_share_support,
-	fso_share_support_owner,
 	fso_pnfs_ds_supported,
 	fso_pnfs_mds_supported,
-	fso_reopen_method,
 	fso_grace_method,
 	fso_link_supports_permission_checks,
 	fso_rename_changes_key,
+	fso_compute_readdir_cookie,
+	fso_whence_is_name,
 } fsal_fsinfo_options_t;
 
 /* The largest maxread and maxwrite value */
@@ -649,7 +657,6 @@ typedef struct fsal_staticfsinfo_t {
 	bool link_support;	/*< FS supports hardlinks? */
 	bool symlink_support;	/*< FS supports symlinks? */
 	bool lock_support;	/*< FS supports file locking? */
-	bool lock_support_owner;	/*< FS supports lock owners? */
 	bool lock_support_async_block;	/*< FS supports blocking locks? */
 	bool named_attr;	/*< FS supports named attributes. */
 	bool unique_handles;	/*< Handles are unique and persistent. */
@@ -672,17 +679,15 @@ typedef struct fsal_staticfsinfo_t {
 
 	uint32_t xattr_access_rights;	/*< This indicates who is allowed
 					   to read/modify xattrs value. */
-	bool share_support;	/*< FS supports share reservation? */
-	bool share_support_owner;	/*< FS supports share reservation
-					   with open owners ? */
 	uint32_t delegations;	/*< fsal supports delegations */
 	bool pnfs_mds;		/*< fsal supports file pnfs MDS */
 	bool pnfs_ds;		/*< fsal supports file pnfs DS */
-	bool reopen_method;	/* fsal supports reopen method */
 	bool fsal_trace;	/*< fsal trace supports */
 	bool fsal_grace;	/*< fsal will handle grace */
 	bool link_supports_permission_checks;
 	bool rename_changes_key;/*< Handle key is changed across rename */
+	bool compute_readdir_cookie;
+	bool whence_is_name;
 } fsal_staticfsinfo_t;
 
 /**
@@ -777,8 +782,8 @@ static inline fsal_status_t fsalstat(fsal_errors_t major, int minor)
  *     printf("ERROR status = %d, %d\n", status.major,status.minor);
  *  }
  */
-#define FSAL_IS_ERROR(_status_) \
-	(!((_status_).major == ERR_FSAL_NO_ERROR))
+#define FSAL_IS_SUCCESS(_status_) ((_status_).major == ERR_FSAL_NO_ERROR)
+#define FSAL_IS_ERROR(_status_) (!FSAL_IS_SUCCESS(_status_))
 
 /**
  * @brief File system dynamic info.
@@ -791,6 +796,8 @@ typedef struct fsal_dynamicfsinfo__ {
 	uint64_t total_files;
 	uint64_t free_files;
 	uint64_t avail_files;
+	uint64_t maxread;
+	uint64_t maxwrite;
 	struct timespec time_delta;
 } fsal_dynamicfsinfo_t;
 
@@ -859,7 +866,13 @@ typedef struct fsal_share_param_t {
 	bool share_reclaim;
 } fsal_share_param_t;
 
-typedef char fsal_verifier_t[8];
+typedef enum {
+	FSAL_DELEG_NONE,
+	FSAL_DELEG_RD,
+	FSAL_DELEG_WR
+} fsal_deleg_t;
+
+typedef char fsal_verifier_t[NFS4_VERIFIER_SIZE];
 
 /**
  * @brief Generic file handle.
