@@ -38,7 +38,7 @@
 #include "nfs23.h"
 #include "nfs4.h"
 #include "mount.h"
-#include "cache_inode.h"
+#include "fsal.h"
 
 char *nfsstat3_to_str(nfsstat3 code);
 char *nfsstat4_to_str(nfsstat4 code);
@@ -49,9 +49,13 @@ uint64_t nfs_htonl64(uint64_t);
 uint64_t nfs_ntohl64(uint64_t);
 
 /* Error conversion routines */
-nfsstat4 nfs4_Errno_verbose(cache_inode_status_t, const char *);
-nfsstat3 nfs3_Errno_verbose(cache_inode_status_t, const char *);
+nfsstat4 nfs4_Errno_verbose(fsal_errors_t, const char *);
 #define nfs4_Errno(e) nfs4_Errno_verbose(e, __func__)
+#define nfs4_Errno_status(e) nfs4_Errno_verbose(e.major, __func__)
+#ifdef _USE_NFS3
+nfsstat3 nfs3_Errno_verbose(fsal_errors_t, const char *);
 #define nfs3_Errno(e) nfs3_Errno_verbose(e, __func__)
+#define nfs3_Errno_status(e) nfs3_Errno_verbose(e.major, __func__)
+#endif
 
 #endif				/* _NFS_CONVERT_H */
