@@ -40,6 +40,7 @@
 
 #include "sal_data.h"
 #include "gsh_config.h"
+#include "gsh_wait_queue.h"
 
 #ifdef _USE_9P
 #include "9p.h"
@@ -71,31 +72,23 @@ typedef struct __nfs4_compound {
 	} v_u;
 } nfs4_compound_t;
 
-/* RPC callback processing */
-typedef enum rpc_call_hook {
-	RPC_CALL_COMPLETE,
-	RPC_CALL_ABORT,
-} rpc_call_hook;
-
 typedef struct _rpc_call rpc_call_t;
 
-typedef int32_t(*rpc_call_func) (rpc_call_t *call, rpc_call_hook hook,
-				 void *arg, uint32_t flags);
+typedef void (*rpc_call_func) (rpc_call_t *call);
 
 #ifdef _HAVE_GSSAPI
 extern gss_OID_desc krb5oid;
 #endif /* _HAVE_GSSAPI */
 
 struct _rpc_call {
+	struct clnt_req call_req;
 	rpc_call_channel_t *chan;
 	rpc_call_func call_hook;
+	void *call_arg;
+	void *call_user_data[2];
 	nfs4_compound_t cbt;
-	struct wait_entry we;
-	enum clnt_stat stat;
 	uint32_t states;
 	uint32_t flags;
-	void *u_data[2];
-	void *completion_arg;
 };
 
 typedef enum request_type {
@@ -108,12 +101,6 @@ typedef enum request_type {
 } request_type_t;
 
 typedef struct request_data {
-	struct glist_head req_q;	/* chaining of pending requests */
-	struct timespec time_queued;	/*< The time at which a request was
-					 *  added to the worker thread queue.
-					 */
-	request_type_t rtype;
-
 	union request_content {
 		rpc_call_t call;
 		nfs_request_t req;
@@ -121,10 +108,26 @@ typedef struct request_data {
 		struct _9p_request_data _9p;
 #endif
 	} r_u;
+
+	struct glist_head req_q;	/* chaining of pending requests */
+	struct timespec time_queued;	/*< The time at which a request was
+					 *  added to the worker thread queue.
+					 */
+	request_type_t rtype;
 } request_data_t;
 
+/* in nfs_init.c */
+
 extern pool_t *request_pool;
 
+struct _nfs_health {
+	uint64_t enqueued_reqs;
+	uint64_t dequeued_reqs;
+};
+
+extern struct _nfs_health health;
+bool nfs_health(void);
+
 /* ServerEpoch is ServerBootTime unless overriden by -E command line option */
 extern struct timespec ServerBootTime;
 extern time_t ServerEpoch;
@@ -145,6 +148,8 @@ void _9p_tcp_process_request(struct _9p_request_data *req9p);
 int _9p_process_buffer(struct _9p_request_data *req9p, char *replydata,
 		       u32 *poutlen);
 
+int _9p_worker_init(void);
+int _9p_worker_shutdown(void);
 void DispatchWork9P(request_data_t *req);
 #endif
 
@@ -158,22 +163,8 @@ void _9p_rdma_cleanup_conn(msk_trans_t *trans);
 
 void Clean_RPC(void);
 void nfs_Init_svc(void);
-void nfs_rpc_dispatch_threads(pthread_attr_t *attr_thr);
 void nfs_rpc_dispatch_stop(void);
 
-request_data_t *nfs_rpc_dequeue_req(nfs_worker_data_t *worker);
-void nfs_rpc_enqueue_req(request_data_t *req);
-uint32_t get_dequeue_count(void);
-uint32_t get_enqueue_count(void);
-
-/* in nfs_worker_thread.c */
-
-void nfs_rpc_execute(request_data_t *req);
-const nfs_function_desc_t *nfs_rpc_get_funcdesc(nfs_request_t *);
-
-int worker_init(void);
-int worker_shutdown(void);
-
 /* Config parsing routines */
 extern config_file_t config_struct;
 extern struct config_block nfs_core;
@@ -203,7 +194,6 @@ unsigned int nfs_core_select_worker_queue(unsigned int avoid_index);
 int nfs_Init_ip_name(void);
 
 void nfs_rpc_destroy_chan(rpc_call_channel_t *chan);
-int32_t nfs_rpc_dispatch_call(rpc_call_t *call, uint32_t flags);
 
 int reaper_init(void);
 int reaper_shutdown(void);
