@@ -174,6 +174,13 @@ struct export_stats {
 	.direction = "out"   \
 }
 
+#define FSAL_OPS_REPLY      \
+{                            \
+	.name = "op",        \
+	.type = "(qa(sq(tdtt)))",     \
+	.direction = "out"   \
+}
+
 #define LAYOUTS_REPLY		\
 {				\
 	.name = "getdevinfo",	\
@@ -236,7 +243,7 @@ struct export_stats {
 }
 
 
-void server_stats_summary(DBusMessageIter *iter, struct gsh_stats *st);
+void server_stats_summary(DBusMessageIter * iter, struct gsh_stats *st);
 void server_dbus_v3_iostats(struct nfsv3_stats *v3p, DBusMessageIter *iter);
 void server_dbus_v40_iostats(struct nfsv40_stats *v40p, DBusMessageIter *iter);
 void server_dbus_v41_iostats(struct nfsv41_stats *v41p, DBusMessageIter *iter);
@@ -251,6 +258,10 @@ void server_dbus_total_ops(struct export_stats *export_st,
 void global_dbus_total_ops(DBusMessageIter *iter);
 void server_dbus_fast_ops(DBusMessageIter *iter);
 void mdcache_dbus_show(DBusMessageIter *iter);
+void server_reset_stats(DBusMessageIter *iter);
+void reset_export_stats(void);
+void reset_client_stats(void);
+void reset_gsh_stats(struct gsh_stats *st);
 
 #ifdef _USE_9P
 void server_dbus_9p_iostats(struct _9p_stats *_9pp, DBusMessageIter *iter);
@@ -261,6 +272,8 @@ void server_dbus_9p_opstats(struct _9p_stats *_9pp, u8 opcode,
 			    DBusMessageIter *iter);
 #endif
 
+extern struct glist_head fsal_list;
+
 #endif				/* USE_DBUS */
 
 void server_stats_free(struct gsh_stats *statsp);
