@@ -40,12 +40,10 @@
 #include <string.h>
 #include <signal.h>
 #include <libgen.h>
-#include <execinfo.h>
 #include <sys/resource.h>
 
 #include "log.h"
 #include "gsh_list.h"
-#include "rpc/rpc.h"
 #include "gsh_rpc.h"
 #include "common_utils.h"
 #include "abstract_mem.h"
@@ -231,8 +229,8 @@ static struct log_facility *default_facility;
 
 log_header_t max_headers = LH_COMPONENT;
 
-char const_log_str[LOG_BUFF_LEN];
-char date_time_fmt[MAX_TD_FMT_LEN];
+char const_log_str[LOG_BUFF_LEN] = "\0";
+char date_time_fmt[MAX_TD_FMT_LEN] = "\0";
 
 typedef struct loglev {
 	char *str;
@@ -270,7 +268,7 @@ static int syslog_opened;
  * Variables specifiques aux threads.
  */
 
-__thread char thread_name[16];
+__thread char thread_name[32];
 __thread char log_buffer[LOG_BUFF_LEN + 1];
 __thread char *clientip = NULL;
 
@@ -288,9 +286,9 @@ __thread char *clientip = NULL;
 						 ## args); \
 	} while (0)
 
-cleanup_list_element *cleanup_list = NULL;
+struct cleanup_list_element *cleanup_list;
 
-void RegisterCleanup(cleanup_list_element *clean)
+void RegisterCleanup(struct cleanup_list_element *clean)
 {
 	clean->next = cleanup_list;
 	cleanup_list = clean;
@@ -298,7 +296,7 @@ void RegisterCleanup(cleanup_list_element *clean)
 
 void Cleanup(void)
 {
-	cleanup_list_element *c = cleanup_list;
+	struct cleanup_list_element *c = cleanup_list;
 
 	while (c != NULL) {
 		c->clean();
@@ -467,6 +465,42 @@ static void SetLevelDebug(int level_to_set)
 		   ReturnLevelInt(component_log_level[COMPONENT_ALL]));
 }
 
+static void SetNTIRPCLogLevel(int level_to_set)
+{
+	switch (level_to_set) {
+	case NIV_NULL:
+	case NIV_FATAL:
+		ntirpc_pp.debug_flags = 0; /* disable all flags */
+		break;
+	case NIV_CRIT:
+	case NIV_MAJ:
+		ntirpc_pp.debug_flags = TIRPC_DEBUG_FLAG_ERROR;
+		break;
+	case NIV_WARN:
+		ntirpc_pp.debug_flags = TIRPC_DEBUG_FLAG_ERROR |
+					TIRPC_DEBUG_FLAG_WARN;
+		break;
+	case NIV_EVENT:
+		ntirpc_pp.debug_flags = TIRPC_DEBUG_FLAG_ERROR |
+					TIRPC_DEBUG_FLAG_WARN |
+					TIRPC_DEBUG_FLAG_EVENT;
+		break;
+	case NIV_DEBUG:
+	case NIV_MID_DEBUG:
+		/* set by log_conf_commit() */
+		break;
+	case NIV_FULL_DEBUG:
+		ntirpc_pp.debug_flags = 0xFFFFFFFF; /* enable all flags */
+		break;
+	default:
+		ntirpc_pp.debug_flags = TIRPC_DEBUG_FLAG_DEFAULT;
+		break;
+	}
+
+	if (!tirpc_control(TIRPC_SET_DEBUG_FLAGS, &ntirpc_pp.debug_flags))
+		LogCrit(COMPONENT_CONFIG, "Setting nTI-RPC debug_flags failed");
+}
+
 void SetComponentLogLevel(log_components_t component, int level_to_set)
 {
 
@@ -492,29 +526,8 @@ void SetComponentLogLevel(log_components_t component, int level_to_set)
 		   ReturnLevelInt(level_to_set));
 	component_log_level[component] = level_to_set;
 
-	if (component != COMPONENT_TIRPC)
-		return;
-
-	/* If the admin configured libntirpc debug flags to anything
-	 * other than the default, preserve those and don't modify the
-	 * flags.
-	 */
-	if (nfs_param.core_param.rpc.debug_flags != TIRPC_DEBUG_FLAGS)
-		return;
-
-	/* push TIRPC component log levels into libntirpc.
-	 *
-	 * Implement only FULL_DEBUG for now
-	 */
-	switch (level_to_set) {
-	case NIV_FULL_DEBUG:
-		ntirpc_pp.debug_flags = 0xFFFFFFFF; /* enable all flags */
-		break;
-	default:
-		ntirpc_pp.debug_flags = 0; /* disable all flags */
-		break;
-	}
-	(void)tirpc_control(TIRPC_PUT_PARAMETERS, &ntirpc_pp);
+	if (component == COMPONENT_TIRPC)
+		SetNTIRPCLogLevel(level_to_set);
 }
 
 static inline int ReturnLevelDebug(void)
@@ -1038,7 +1051,7 @@ int set_log_destination(const char *name, char *dest)
 	} else {
 		PTHREAD_RWLOCK_unlock(&log_rwlock);
 		LogCrit(COMPONENT_LOG,
-			 "Log facility %s destination is not changable",
+			 "Log facility %s destination is not changeable",
 			facility->lf_name);
 		return -EINVAL;
 	}
@@ -1207,7 +1220,7 @@ static int log_to_file(log_header_t headers, void *private,
 	buffer->b_start[len] = '\n';
 	buffer->b_start[len + 1] = '\0';
 
-	fd = open(path, O_WRONLY | O_SYNC | O_APPEND | O_CREAT, log_mask);
+	fd = open(path, O_WRONLY | O_APPEND | O_CREAT, log_mask);
 
 	if (fd != -1) {
 		rc = write(fd, buffer->b_start, len + 1);
@@ -1707,13 +1720,14 @@ void rpc_warnx(char *fmt, ...)
 {
 	va_list ap;
 
-	if (component_log_level[COMPONENT_TIRPC] < NIV_DEBUG)
+	if (component_log_level[COMPONENT_TIRPC] <= NIV_FATAL)
 		return;
 
 	va_start(ap, fmt);
 
 	display_log_component_level(COMPONENT_TIRPC, "<no-file>", 0, "rpc",
-				    NIV_DEBUG, fmt, ap);
+				    component_log_level[COMPONENT_TIRPC],
+				    fmt, ap);
 
 	va_end(ap);
 
@@ -1909,10 +1923,11 @@ struct facility_config {
  */
 
 struct logger_config {
-	log_levels_t default_level;
 	struct glist_head facility_list;
 	struct logfields *logfields;
 	log_levels_t *comp_log_level;
+	log_levels_t default_level;
+	uint32_t rpc_debug_flags;
 };
 
 /**
@@ -2168,7 +2183,7 @@ static struct config_item component_levels[] = {
 			 COMPONENT_FSAL_UP, int),
 	CONF_INDEX_TOKEN("DBUS", NB_LOG_LEVEL, log_levels,
 			 COMPONENT_DBUS, int),
-	CONF_INDEX_TOKEN("COMPONENT_NFS_MSK", NB_LOG_LEVEL, log_levels,
+	CONF_INDEX_TOKEN("NFS_MSK", NB_LOG_LEVEL, log_levels,
 			 COMPONENT_NFS_MSK, int),
 	CONFIG_EOL
 };
@@ -2326,7 +2341,7 @@ static int facility_commit(void *node, void *link_mem, void *self_struct,
 			"No facility name given");
 		err_type->missing = true;
 		errcnt++;
-		goto out;
+		return errcnt;
 	}
 	if (conf->dest != NULL) {
 		if (strcasecmp(conf->dest, "stderr") == 0) {
@@ -2355,7 +2370,7 @@ static int facility_commit(void *node, void *link_mem, void *self_struct,
 			conf->facility_name);
 		err_type->missing = true;
 		errcnt++;
-		goto out;
+		return errcnt;
 	}
 	if (conf->func != log_to_syslog && conf->headers < LH_ALL)
 		LogWarn(COMPONENT_CONFIG,
@@ -2366,10 +2381,6 @@ static int facility_commit(void *node, void *link_mem, void *self_struct,
 	fac_list = link_mem;
 	glist_add_tail(fac_list, &conf->fac_list);
 	return 0;
-
-out:
-	gsh_free(self_struct); /* got the bits, be done with it */
-	return errcnt;
 }
 
 static void *log_conf_init(void *link_mem, void *self_struct)
@@ -2533,6 +2544,8 @@ static int log_conf_commit(void *node, void *link_mem, void *self_struct,
 				gsh_free(component_log_level);
 			component_log_level = logger->comp_log_level;
 		}
+		ntirpc_pp.debug_flags = logger->rpc_debug_flags;
+		SetNTIRPCLogLevel(component_log_level[COMPONENT_TIRPC]);
 	} else {
 		if (logger->logfields != NULL) {
 			struct logfields *lf = logger->logfields;
@@ -2554,6 +2567,9 @@ static int log_conf_commit(void *node, void *link_mem, void *self_struct,
 static struct config_item logging_params[] = {
 	CONF_ITEM_TOKEN("Default_log_level", NB_LOG_LEVEL, log_levels,
 			 logger_config, default_level),
+	CONF_ITEM_UI32("RPC_Debug_Flags", 0, UINT32_MAX,
+		       TIRPC_DEBUG_FLAG_DEFAULT,
+		       logger_config, rpc_debug_flags),
 	CONF_ITEM_BLOCK("Facility", facility_params,
 			facility_init, facility_commit,
 			logger_config, facility_list),
