@@ -198,16 +198,16 @@ int nfs_rpc_req2client_cred(struct svc_req *req, nfs_client_cred_t *pcred)
 	struct svc_rpc_gss_data *gd = NULL;
 #endif
 
-	pcred->flavor = req->rq_cred.oa_flavor;
-	pcred->length = req->rq_cred.oa_length;
+	pcred->length = req->rq_msg.cb_cred.oa_length;
+	pcred->flavor = req->rq_msg.cb_cred.oa_flavor;
 
-	switch (req->rq_cred.oa_flavor) {
+	switch (req->rq_msg.cb_cred.oa_flavor) {
 	case AUTH_NONE:
 		/* Do nothing... */
 		break;
 
 	case AUTH_UNIX:
-		aup = (struct authunix_parms *)(req->rq_clntcred);
+		aup = (struct authunix_parms *)req->rq_msg.rq_cred_body;
 
 		pcred->auth_union.auth_unix.aup_uid = aup->aup_uid;
 		pcred->auth_union.auth_unix.aup_gid = aup->aup_gid;
@@ -264,7 +264,7 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 	 */
 	op_ctx->cred_flags &= CREDS_LOADED | CREDS_ANON;
 
-	switch (req->rq_cred.oa_flavor) {
+	switch (req->rq_msg.cb_cred.oa_flavor) {
 	case AUTH_NONE:
 		/* Nothing to be done here... */
 		op_ctx->cred_flags |= CREDS_LOADED | CREDS_ANON;
@@ -273,10 +273,10 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 
 	case AUTH_SYS:
 		if ((op_ctx->cred_flags & CREDS_LOADED) == 0) {
-			struct authunix_parms *creds = NULL;
+			struct authunix_parms *creds = (struct authunix_parms *)
+				req->rq_msg.rq_cred_body;
 
 			/* We map the rq_cred to Authunix_parms */
-			creds = (struct authunix_parms *) req->rq_clntcred;
 			op_ctx->original_creds.caller_uid = creds->aup_uid;
 			op_ctx->original_creds.caller_gid = creds->aup_gid;
 			op_ctx->original_creds.caller_glen = creds->aup_len;
@@ -347,8 +347,10 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 
 	default:
 		LogMidDebug(COMPONENT_DISPATCH,
-			     "FAILURE: Request xid=%u, has unsupported authentication %d",
-			     req->rq_xid, req->rq_cred.oa_flavor);
+			     "FAILURE: Request xid=%" PRIu32
+			     ", has unsupported authentication %" PRIu32,
+			     req->rq_msg.rm_xid,
+			     req->rq_msg.cb_cred.oa_flavor);
 		/* Reject the request for weak authentication and
 		 * return to worker
 		 */
@@ -364,7 +366,8 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 	    ((op_ctx->export_perms->options &
 	      EXPORT_OPTION_ALL_ANONYMOUS) != 0) ||
 	    ((op_ctx->export_perms->options & EXPORT_OPTION_ROOT_SQUASH) != 0 &&
-	      op_ctx->original_creds.caller_uid == 0)) {
+	      op_ctx->fsal_export->exp_ops.is_superuser(op_ctx->fsal_export,
+					      &op_ctx->original_creds))) {
 		/* Squash uid, gid, and discard groups */
 		op_ctx->creds->caller_uid =
 					op_ctx->export_perms->anonymous_uid;
@@ -380,7 +383,8 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 		return NFS4_OK;
 	} else if ((op_ctx->export_perms->options &
 		    EXPORT_OPTION_ROOT_ID_SQUASH) != 0 &&
-		   op_ctx->original_creds.caller_uid == 0) {
+		   op_ctx->fsal_export->exp_ops.is_superuser(
+			op_ctx->fsal_export, &op_ctx->original_creds)) {
 		/* Only squash root id, leave gid and groups alone for now */
 		op_ctx->creds->caller_uid =
 					op_ctx->export_perms->anonymous_uid;
@@ -415,7 +419,7 @@ nfsstat4 nfs_req_creds(struct svc_req *req)
 		    !uid2grp(op_ctx->original_creds.caller_uid,
 			     &op_ctx->caller_gdata)) {
 			/** @todo: do we really want to bail here? */
-			LogCrit(COMPONENT_DISPATCH,
+			LogInfo(COMPONENT_DISPATCH,
 				"Attempt to fetch managed_gids failed");
 			return NFS4ERR_ACCESS;
 		}
@@ -548,7 +552,7 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 	int port = get_port(op_ctx->caller_addr);
 
 	LogMidDebugAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
-		    "nfs4_export_check_access about to call export_check_access");
+		       "about to call export_check_access");
 	export_check_access();
 
 	/* Check if any access at all */
@@ -557,7 +561,7 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"Access not allowed on Export_Id %d %s for client %s",
 			op_ctx->ctx_export->export_id,
-			op_ctx->ctx_export->fullpath,
+			op_ctx->ctx_export->pseudopath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -569,7 +573,7 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"NFS4 not allowed on Export_Id %d %s for client %s",
 			op_ctx->ctx_export->export_id,
-			op_ctx->ctx_export->fullpath,
+			op_ctx->ctx_export->pseudopath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -588,7 +592,7 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 			"NFS4 over %s not allowed on Export_Id %d %s for client %s",
 			xprt_type_to_str(xprt_type),
 			op_ctx->ctx_export->export_id,
-			op_ctx->ctx_export->fullpath,
+			op_ctx->ctx_export->pseudopath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -602,7 +606,7 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"Non-reserved Port %d is not allowed on Export_Id %d %s for client %s",
 			port, op_ctx->ctx_export->export_id,
-			op_ctx->ctx_export->fullpath,
+			op_ctx->ctx_export->pseudopath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
@@ -614,7 +618,7 @@ nfsstat4 nfs4_export_check_access(struct svc_req *req)
 		LogInfoAlt(COMPONENT_NFS_V4, COMPONENT_EXPORT,
 			"NFS4 auth not allowed on Export_Id %d %s for client %s",
 			op_ctx->ctx_export->export_id,
-			op_ctx->ctx_export->fullpath,
+			op_ctx->ctx_export->pseudopath,
 			op_ctx->client
 				? op_ctx->client->hostaddr_str
 				: "unknown client");
