@@ -88,11 +88,11 @@ struct fsal_obj_handle *nfs3_FhandleToCache(nfs_fh3 *fh3,
 	fh_desc.len = v3_handle->fs_len;
 	fh_desc.addr = &v3_handle->fsopaque;
 
-	/* adjust the handle opaque into a cache key */
+	/* adjust the wire handle opaque into a host-handle */
 	fsal_status =
-	    export->exp_ops.extract_handle(export, FSAL_DIGEST_NFSV3,
-					   &fh_desc,
-					   v3_handle->fhflags1);
+	    export->exp_ops.wire_to_host(export, FSAL_DIGEST_NFSV3,
+					 &fh_desc,
+					 v3_handle->fhflags1);
 
 	if (!FSAL_IS_ERROR(fsal_status))
 		fsal_status = export->exp_ops.create_handle(export, &fh_desc,
@@ -140,11 +140,11 @@ bool nfs4_FSALToFhandle(bool allocate,
 	fh_desc.addr = &file_handle->fsopaque;
 	fh_desc.len = fh4->nfs_fh4_len - offsetof(file_handle_v4_t, fsopaque);
 
-	if (FSAL_IS_ERROR(fsalhandle->obj_ops.handle_digest(fsalhandle,
+	if (FSAL_IS_ERROR(fsalhandle->obj_ops.handle_to_wire(fsalhandle,
 							    FSAL_DIGEST_NFSV4,
 							    &fh_desc))) {
 		LogDebug(COMPONENT_FILEHANDLE,
-			 "handle_digest FSAL_DIGEST_NFSV4 failed");
+			 "handle_to_wire FSAL_DIGEST_NFSV4 failed");
 		if (allocate)
 			nfs4_freeFH(fh4);
 		return false;
@@ -204,11 +204,11 @@ bool nfs3_FSALToFhandle(bool allocate,
 	fh_desc.addr = &file_handle->fsopaque;
 	fh_desc.len = NFS3_FHSIZE - offsetof(file_handle_v3_t, fsopaque);
 
-	if (FSAL_IS_ERROR(fsalhandle->obj_ops.handle_digest(fsalhandle,
+	if (FSAL_IS_ERROR(fsalhandle->obj_ops.handle_to_wire(fsalhandle,
 							    FSAL_DIGEST_NFSV3,
 							    &fh_desc))) {
 		LogDebug(COMPONENT_FILEHANDLE,
-			 "handle_digest FSAL_DIGEST_NFSV3 failed");
+			 "handle_to_wire FSAL_DIGEST_NFSV3 failed");
 		if (allocate)
 			nfs3_freeFH(fh3);
 		return false;
@@ -594,28 +594,26 @@ nfsstat4 nfs4_sanity_check_FH(compound_data_t *data,
 
 
 	/* Check for the correct file type */
-	if (required_type != NO_FILE_TYPE) {
-		if (data->current_filetype != required_type) {
-			LogDebug(COMPONENT_NFS_V4,
-				 "Wrong file type expected %s actual %s",
-				 object_file_type_to_str(required_type),
-				 object_file_type_to_str(data->
-							 current_filetype));
+	if (required_type != NO_FILE_TYPE &&
+	    data->current_filetype != required_type) {
+		LogDebug(COMPONENT_NFS_V4,
+			 "Wrong file type expected %s actual %s",
+			 object_file_type_to_str(required_type),
+			 object_file_type_to_str(data->current_filetype));
 
-			if (required_type == DIRECTORY) {
-				if (data->current_filetype == SYMBOLIC_LINK)
-					return NFS4ERR_SYMLINK;
-				else
-					return NFS4ERR_NOTDIR;
-			} else if (required_type == SYMBOLIC_LINK)
-				return NFS4ERR_INVAL;
+		if (required_type == DIRECTORY) {
+			if (data->current_filetype == SYMBOLIC_LINK)
+				return NFS4ERR_SYMLINK;
+			else
+				return NFS4ERR_NOTDIR;
+		} else if (required_type == SYMBOLIC_LINK)
+			return NFS4ERR_INVAL;
 
-			switch (data->current_filetype) {
-			case DIRECTORY:
-				return NFS4ERR_ISDIR;
-			default:
-				return NFS4ERR_INVAL;
-			}
+		switch (data->current_filetype) {
+		case DIRECTORY:
+			return NFS4ERR_ISDIR;
+		default:
+			return NFS4ERR_INVAL;
 		}
 	}
 
@@ -668,38 +666,36 @@ nfsstat4 nfs4_sanity_check_saved_FH(compound_data_t *data, int required_type,
 		if (-required_type == data->saved_filetype) {
 			LogDebug(COMPONENT_NFS_V4,
 				 "Wrong file type expected not to be %s was %s",
-				 object_file_type_to_str((object_file_type_t) -
-							 required_type),
-				 object_file_type_to_str(data->
-							 current_filetype));
+				 object_file_type_to_str((object_file_type_t)
+							 -required_type),
+				 object_file_type_to_str(
+						data->current_filetype));
 			if (-required_type == DIRECTORY) {
 				return NFS4ERR_ISDIR;
 				return NFS4ERR_INVAL;
 			}
 		}
-	} else if (required_type != NO_FILE_TYPE) {
-		if (data->saved_filetype != required_type) {
-			LogDebug(COMPONENT_NFS_V4,
-				 "Wrong file type expected %s was %s",
-				 object_file_type_to_str((object_file_type_t)
-							 required_type),
-				 object_file_type_to_str(data->
-							 current_filetype));
+	} else if (required_type != NO_FILE_TYPE &&
+		   data->saved_filetype != required_type) {
+		LogDebug(COMPONENT_NFS_V4,
+			 "Wrong file type expected %s was %s",
+			 object_file_type_to_str((object_file_type_t)
+						 required_type),
+			 object_file_type_to_str(data->current_filetype));
 
-			if (required_type == DIRECTORY) {
-				if (data->current_filetype == SYMBOLIC_LINK)
-					return NFS4ERR_SYMLINK;
-				else
-					return NFS4ERR_NOTDIR;
-			} else if (required_type == SYMBOLIC_LINK)
-				return NFS4ERR_INVAL;
+		if (required_type == DIRECTORY) {
+			if (data->current_filetype == SYMBOLIC_LINK)
+				return NFS4ERR_SYMLINK;
+			else
+				return NFS4ERR_NOTDIR;
+		} else if (required_type == SYMBOLIC_LINK)
+			return NFS4ERR_INVAL;
 
-			switch (data->saved_filetype) {
-			case DIRECTORY:
-				return NFS4ERR_ISDIR;
-			default:
-				return NFS4ERR_INVAL;
-			}
+		switch (data->saved_filetype) {
+		case DIRECTORY:
+			return NFS4ERR_ISDIR;
+		default:
+			return NFS4ERR_INVAL;
 		}
 	}
 
