@@ -72,6 +72,9 @@
 #define NFS_V42_NB_OPERATION (NFS4_OP_WRITE_SAME + 1)
 #define _9P_NB_COMMAND 33
 
+#define NFS_pcp nfs_param.core_param
+#define NFS_program NFS_pcp.program
+
 #ifdef USE_DBUS
 
 struct op_name {
@@ -693,6 +696,182 @@ static void record_op(struct proto_op *op, nsecs_elapsed_t request_time,
 	record_latency(op, request_time, qwait_time, dup);
 }
 
+#ifdef USE_DBUS
+/**
+ *  @brief reset the counts for protocol operation
+ *  Use atomic ops to avoid locks.
+ *  @param op           [IN] pointer to specific protocol struct
+ */
+
+static void reset_op(struct proto_op *op)
+{
+	(void)atomic_store_uint64_t(&op->total, 0);
+	(void)atomic_store_uint64_t(&op->errors, 0);
+	(void)atomic_store_uint64_t(&op->dups, 0);
+	/* reset latency related counters */
+	(void)atomic_store_uint64_t(&op->latency.latency, 0);
+	(void)atomic_store_uint64_t(&op->latency.min, 0);
+	(void)atomic_store_uint64_t(&op->latency.max, 0);
+	(void)atomic_store_uint64_t(&op->dup_latency.latency, 0);
+	(void)atomic_store_uint64_t(&op->dup_latency.min, 0);
+	(void)atomic_store_uint64_t(&op->dup_latency.max, 0);
+	(void)atomic_store_uint64_t(&op->queue_latency.latency, 0);
+	(void)atomic_store_uint64_t(&op->queue_latency.min, 0);
+	(void)atomic_store_uint64_t(&op->queue_latency.max, 0);
+}
+
+/**
+ *  @brief reset the counts for xfer protocol operation
+ *  Use atomic ops to avoid locks.
+ *  @param xfer           [IN] pointer to specific xfer protocol struct
+ */
+
+static void reset_xfer_op(struct xfer_op *xfer)
+{
+	reset_op(&xfer->cmd);
+	(void)atomic_store_uint64_t(&xfer->requested, 0);
+	(void)atomic_store_uint64_t(&xfer->transferred, 0);
+}
+
+/**
+ * @brief reset the counts related to layout
+ * Use atomic ops to avoid locks.
+ * @param lo           [IN] pointer to specific layout struct
+ */
+
+static void reset_layout_op(struct layout_op *lo)
+{
+	(void)atomic_store_uint64_t(&lo->total, 0);
+	(void)atomic_store_uint64_t(&lo->errors, 0);
+	(void)atomic_store_uint64_t(&lo->delays, 0);
+}
+
+/**
+ * @brief reset the counts nfsv3_stats
+ * Use atomic ops to avoid locks.
+ * @param nfsv3           [IN] pointer to nfsv3_stats struct
+ */
+
+static void reset_nfsv3_stats(struct nfsv3_stats *nfsv3)
+{
+	/* Reset stats counter for nfsv3 protocol */
+	reset_op(&nfsv3->cmds);
+	reset_xfer_op(&nfsv3->read);
+	reset_xfer_op(&nfsv3->write);
+}
+
+/**
+ * @brief reset the counts nfsv40_stats
+ * Use atomic ops to avoid locks.
+ * @param nfsv40           [IN] pointer to nfsv40_stats struct
+ */
+
+static void reset_nfsv40_stats(struct nfsv40_stats *nfsv40)
+{
+	/* Reset stats counter for nfsv4 protocol */
+	reset_op(&nfsv40->compounds);
+	(void)atomic_store_uint64_t(&nfsv40->ops_per_compound, 0);
+	reset_xfer_op(&nfsv40->read);
+	reset_xfer_op(&nfsv40->write);
+}
+
+
+/**
+ * @brief reset the counts nfsv41_stats
+ * Use atomic ops to avoid locks.
+ * @param nfsv41           [IN] pointer to nfsv41_stats struct
+ */
+
+static void reset_nfsv41_stats(struct nfsv41_stats *nfsv41)
+{
+	/* Reset stats counter for nfsv41 protocol */
+	reset_op(&nfsv41->compounds);
+	(void)atomic_store_uint64_t(&nfsv41->ops_per_compound, 0);
+	reset_xfer_op(&nfsv41->read);
+	reset_xfer_op(&nfsv41->write);
+	reset_layout_op(&nfsv41->getdevinfo);
+	reset_layout_op(&nfsv41->layout_get);
+	reset_layout_op(&nfsv41->layout_commit);
+	reset_layout_op(&nfsv41->layout_return);
+	reset_layout_op(&nfsv41->recall);
+}
+
+
+/**
+ * @brief reset the counts mnt_stats
+ * Use atomic ops to avoid locks.
+ * @param mnt           [IN] pointer to mnt_stats struct
+ */
+
+static void reset_mnt_stats(struct mnt_stats *mnt)
+{
+	/* Reset stats counter for mount protocol */
+	reset_op(&mnt->v1_ops);
+	reset_op(&mnt->v3_ops);
+}
+
+/**
+ * @brief reset the counts rquota_stats
+ * Use atomic ops to avoid locks.
+ * @param rquota           [IN] pointer to rquota_stats struct
+ */
+
+static void reset_rquota_stats(struct rquota_stats *rquota)
+{
+	/* Reset stats counter for quota */
+	reset_op(&rquota->ops);
+	reset_op(&rquota->ext_ops);
+}
+
+/**
+ * @brief reset the counts nlmv4_stats
+ * Use atomic ops to avoid locks.
+ * @param nlmv4           [IN] pointer to nlmv4_stats struct
+ */
+
+static void reset_nlmv4_stats(struct nlmv4_stats *nlmv4)
+{
+	/* Reset stats counter for nlmv4 */
+	reset_op(&nlmv4->ops);
+}
+
+/**
+ * @brief reset the counts nlmv4_stats
+ * Use atomic ops to avoid locks.
+ * @param nlmv4           [IN] pointer to nlmv4_stats struct
+ */
+
+static void reset_deleg_stats(struct deleg_stats *deleg)
+{
+	/* Reset stats counter for deleg */
+	(void)atomic_store_uint32_t(&deleg->curr_deleg_grants, 0);
+	(void)atomic_store_uint32_t(&deleg->tot_recalls, 0);
+	(void)atomic_store_uint32_t(&deleg->failed_recalls, 0);
+	(void)atomic_store_uint32_t(&deleg->num_revokes, 0);
+}
+
+#ifdef _USE_9P
+static void reset__9P_stats(struct _9p_stats *_9p)
+{
+	u8 opc;
+
+	reset_op(&_9p->cmds);
+	reset_xfer_op(&_9p->read);
+	reset_xfer_op(&_9p->write);
+	(void)atomic_store_uint64_t(&_9p->trans.rx_bytes, 0);
+	(void)atomic_store_uint64_t(&_9p->trans.rx_pkt, 0);
+	(void)atomic_store_uint64_t(&_9p->trans.rx_err, 0);
+	(void)atomic_store_uint64_t(&_9p->trans.tx_bytes, 0);
+	(void)atomic_store_uint64_t(&_9p->trans.tx_pkt, 0);
+	(void)atomic_store_uint64_t(&_9p->trans.tx_err, 0);
+	for (opc = 0; opc <= _9P_RWSTAT; opc++) {
+		if (_9p->opcodes[opc] != NULL)
+			reset_op(_9p->opcodes[opc]);
+	}
+}
+#endif
+#endif		/* USE_DBUS */
+
 /**
  * @brief record V4.1 layout op stats
  *
@@ -850,12 +1029,13 @@ static void record_stats(struct gsh_stats *gsh_st, pthread_rwlock_t *lock,
 			 bool global)
 {
 	struct svc_req *req = &reqdata->r_u.req.svc;
-	uint32_t proto_op = req->rq_proc;
+	uint32_t proto_op = req->rq_msg.cb_proc;
+	uint32_t program_op = req->rq_msg.cb_prog;
 
-	if (req->rq_prog == nfs_param.core_param.program[P_NFS]) {
+	if (program_op == NFS_program[P_NFS]) {
 		if (proto_op == 0)
 			return;	/* we don't count NULL ops */
-		if (req->rq_vers == NFS_V3) {
+		if (req->rq_msg.cb_vers == NFS_V3) {
 			struct nfsv3_stats *sp = get_v3(gsh_st, lock);
 
 			/* record stuff */
@@ -879,10 +1059,10 @@ static void record_stats(struct gsh_stats *gsh_st, pthread_rwlock_t *lock,
 			/* We don't do V4 here and V2 is toast */
 			return;
 		}
-	} else if (req->rq_prog == nfs_param.core_param.program[P_MNT]) {
+	} else if (program_op == NFS_program[P_MNT]) {
 		struct mnt_stats *sp = get_mnt(gsh_st, lock);
 
-		if (global && req->rq_vers == MOUNT_V1)
+		if (global && req->rq_msg.cb_vers == MOUNT_V1)
 			record_op(&global_st.mnt.v1_ops, request_time,
 				  qwait_time, success, dup);
 		else if (global)
@@ -890,13 +1070,13 @@ static void record_stats(struct gsh_stats *gsh_st, pthread_rwlock_t *lock,
 				  qwait_time, success, dup);
 
 		/* record stuff */
-		if (req->rq_vers == MOUNT_V1)
+		if (req->rq_msg.cb_vers == MOUNT_V1)
 			record_op(&sp->v1_ops, request_time, qwait_time,
 				  success, dup);
 		else
 			record_op(&sp->v3_ops, request_time, qwait_time,
 				  success, dup);
-	} else if (req->rq_prog == nfs_param.core_param.program[P_NLM]) {
+	} else if (program_op == NFS_program[P_NLM]) {
 		struct nlmv4_stats *sp = get_nlm4(gsh_st, lock);
 
 		if (global)
@@ -904,14 +1084,14 @@ static void record_stats(struct gsh_stats *gsh_st, pthread_rwlock_t *lock,
 				  qwait_time, success, dup);
 		/* record stuff */
 		record_op(&sp->ops, request_time, qwait_time, success, dup);
-	} else if (req->rq_prog == nfs_param.core_param.program[P_RQUOTA]) {
+	} else if (program_op == NFS_program[P_RQUOTA]) {
 		struct rquota_stats *sp = get_rquota(gsh_st, lock);
 
 		if (global)
 			record_op(&global_st.rquota.ops, request_time,
 				  qwait_time, success, dup);
 		/* record stuff */
-		if (req->rq_vers == RQUOTAVERS)
+		if (req->rq_msg.cb_vers == RQUOTAVERS)
 			record_op(&sp->ops, request_time, qwait_time, success,
 				  dup);
 		else
@@ -1004,7 +1184,7 @@ void server_stats_9p_done(u8 opc, struct _9p_request_data *req9p)
 /**
  * @brief record NFS op finished
  *
- * Called from nfs_rpc_execute at operation/command completion
+ * Called from nfs_rpc_process_request at operation/command completion
  */
 
 void server_stats_nfs_done(request_data_t *reqdata, int rc, bool dup)
@@ -1013,15 +1193,18 @@ void server_stats_nfs_done(request_data_t *reqdata, int rc, bool dup)
 	struct timespec current_time;
 	nsecs_elapsed_t stop_time;
 	struct svc_req *req = &reqdata->r_u.req.svc;
-	uint32_t proto_op = req->rq_proc;
+	uint32_t proto_op = req->rq_msg.cb_proc;
+	uint32_t program_op = req->rq_msg.cb_prog;
 
-	if (req->rq_prog == NFS_PROGRAM && op_ctx->nfs_vers == NFS_V3)
-			global_st.v3.op[proto_op]++;
-	else if (req->rq_prog == nfs_param.core_param.program[P_NLM])
+	if (!nfs_param.core_param.enable_NFSSTATS)
+		return;
+	if (program_op == NFS_PROGRAM && op_ctx->nfs_vers == NFS_V3)
+		global_st.v3.op[proto_op]++;
+	else if (program_op == NFS_program[P_NLM])
 		global_st.lm.op[proto_op]++;
-	else if (req->rq_prog == nfs_param.core_param.program[P_MNT])
+	else if (program_op == NFS_program[P_MNT])
 		global_st.mn.op[proto_op]++;
-	else if (req->rq_prog == nfs_param.core_param.program[P_RQUOTA])
+	else if (program_op == NFS_program[P_RQUOTA])
 		global_st.qt.op[proto_op]++;
 
 	if (nfs_param.core_param.enable_FASTSTATS)
@@ -1066,6 +1249,8 @@ void server_stats_nfsv4_op_done(int proto_op,
 	struct timespec current_time;
 	nsecs_elapsed_t stop_time;
 
+	if (!nfs_param.core_param.enable_NFSSTATS)
+		return;
 	if (op_ctx->nfs_vers == NFS_V4)
 		global_st.v4.op[proto_op]++;
 
@@ -1122,6 +1307,8 @@ void server_stats_compound_done(int num_ops, int status)
 	struct timespec current_time;
 	nsecs_elapsed_t stop_time;
 
+	if (!nfs_param.core_param.enable_NFSSTATS)
+		return;
 	now(&current_time);
 	stop_time = timespec_diff(&ServerBootTime, &current_time);
 	if (client != NULL) {
@@ -1159,6 +1346,8 @@ void server_stats_compound_done(int num_ops, int status)
 void server_stats_io_done(size_t requested,
 			  size_t transferred, bool success, bool is_write)
 {
+	if (!nfs_param.core_param.enable_NFSSTATS)
+		return;
 	if (op_ctx->client != NULL) {
 		struct server_stats *server_st;
 
@@ -1676,6 +1865,69 @@ void server_dbus_all_iostats(struct export_stats *export_statistics,
 	}
 }
 
+void reset_gsh_stats(struct gsh_stats *st)
+{
+	if (st->nfsv3)
+		reset_nfsv3_stats(st->nfsv3);
+	if (st->nfsv40)
+		reset_nfsv40_stats(st->nfsv40);
+	if (st->nfsv41)
+		reset_nfsv41_stats(st->nfsv41);
+	if (st->nfsv42)
+		reset_nfsv41_stats(st->nfsv42); /* Uses v41 stats */
+	if (st->mnt)
+		reset_mnt_stats(st->mnt);
+	if (st->rquota)
+		reset_rquota_stats(st->rquota);
+	if (st->nlm4)
+		reset_nlmv4_stats(st->nlm4);
+	if (st->deleg)
+		reset_deleg_stats(st->deleg);
+#ifdef _USE_9P
+	if (st->_9p)
+		reset__9P_stats(st->_9p);
+#endif
+}
+
+void reset_global_stats(void)
+{
+	int i;
+	/* Reset all ops counters of nfsv3 */
+	for (i = 0; i < NFSPROC3_COMMIT; i++) {
+		(void)atomic_store_uint64_t(&global_st.v3.op[i], 0);
+	}
+	/* Reset all ops counters of nfsv4 */
+	for (i = 0; i < NFS4_OP_LAST_ONE; i++) {
+		(void)atomic_store_uint64_t(&global_st.v4.op[i], 0);
+	}
+	/* Reset all ops counters of lock manager */
+	for (i = 0; i < NLM4_FAILED; i++) {
+		(void)atomic_store_uint64_t(&global_st.lm.op[i], 0);
+	}
+	/* Reset all ops counters of mountd */
+	for (i = 0; i < MOUNTPROC3_EXPORT; i++) {
+		(void)atomic_store_uint64_t(&global_st.mn.op[i], 0);
+	}
+	/* Reset all ops counters of rquotad */
+	for (i = 0; i < RQUOTAPROC_SETACTIVEQUOTA; i++) {
+		(void)atomic_store_uint64_t(&global_st.qt.op[i], 0);
+	}
+	reset_nfsv3_stats(&global_st.nfsv3);
+	reset_nfsv40_stats(&global_st.nfsv40);
+	reset_nfsv41_stats(&global_st.nfsv41);
+	reset_nfsv41_stats(&global_st.nfsv42);  /* Uses v41 stats */
+	reset_mnt_stats(&global_st.mnt);
+	reset_rquota_stats(&global_st.rquota);
+	reset_nlmv4_stats(&global_st.nlm4);
+}
+
+void global_dbus_reset_stats(DBusMessageIter *iter)
+{
+	reset_global_stats();
+	reset_export_stats();
+	reset_client_stats();
+}
+
 void server_dbus_total_ops(struct export_stats *export_st,
 			   DBusMessageIter *iter)
 {
@@ -1704,6 +1956,14 @@ void global_dbus_total_ops(DBusMessageIter *iter)
 	global_dbus_total(iter);
 }
 
+void server_reset_stats(DBusMessageIter *iter)
+{
+	struct timespec timestamp;
+
+	now(&timestamp);
+	dbus_append_timestamp(iter, &timestamp);
+	global_dbus_reset_stats(iter);
+}
 
 #ifdef _USE_9P
 void server_dbus_9p_iostats(struct _9p_stats *_9pp, DBusMessageIter *iter)
